<?php

namespace WPEP\ACL;

use WPEP\ACL\User as WPEP_ACL_User;
use WPEP\ACL\Post as WPEP_ACL_Post;

class API {

  public static function manage_members( $request_data ) {
    return current_user_can( WPEP_CAP_MANAGE_MEMBERS );
  }

  public static function manage_settings( $request_data ) {
    return current_user_can( WPEP_CAP_MANAGE_SETTINGS );
  }

  public static function analytics_view_user_id( $request_data ) {
    if( !isset( $request_data[ 'user_id' ] ) )
      return false;

    return WPEP_ACL_User::analytics( $request_data[ 'user_id' ] );
  }

  public static function analytics_view_content_type( $request_data ) {
    if( current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      return true;

    if( isset( $request_data[ 'post_id' ] ) ) {
      if( isset( $request_data[ 'post_ids' ] ) || isset( $request_data[ 'user_ids' ] ) )
        return false;

      if( WPEP_ACL_Post::analytics( $request_data[ 'post_id' ], 'view' ) )
        return true;

      if( isset( $request_data[ 'user_id' ] ) )
        return WPEP_ACL_User::analytics( $request_data[ 'user_id' ], 'view' );

      return false;
    }

    if( isset( $request_data[ 'post_ids' ] ) ) {
      if( !is_array( $request_data[ 'post_ids' ] ) )
        return false;

      foreach( $request_data[ 'post_ids' ] as $post_id_index => $post_id ) {
        if( !WPEP_ACL_Post::analytics( $post_id, 'view' ) )
          unset( $request_data[ 'post_ids' ][ $post_id_index ] );
      }

      if( empty( $request_data[ 'post_ids' ] ) )
        return false;

      // No User IDS need to be checked.
      if( !isset( $request_data[ 'user_ids' ] ) )
        return true;
    }


    if( isset( $request_data[ 'user_ids' ] ) ) {
      if( !is_array( $request_data[ 'user_ids' ] ) )
        return false;

      foreach( $request_data[ 'user_ids' ] as $user_id_index => $user_id ) {
        if( !WPEP_ACL_User::analytics( $user_id, 'view' ) )
          unset( $request_data[ 'user_ids' ][ $user_id_index ] );
      }

      if( empty( $request_data[ 'user_ids' ] ) )
        return false;

      return true;
    }

    if( isset( $request_data[ 'user_id' ] ) )
      return WPEP_ACL_User::analytics( $request_data[ 'user_id' ], 'view' );

    return false;
  }

  public static function analytics_view_post_id( $request_data ) {
    if( !isset( $request_data[ 'post_id' ] ) )
      return false;

    if( WPEP_ACL_Post::analytics( $request_data[ 'post_id' ], 'view' ) )
      return true;

    if( !isset( $request_data[ 'user_ids' ] ) )
      return false;

    if( !is_array( $request_data[ 'user_ids' ] ) || empty( $request_data[ 'user_ids' ] ) )
      return false;

    // Minimum 1 invalid user_id, we automatically restrict statistics.
    foreach( $request_data[ 'user_ids' ] as $user_id )
      if( !WPEP_ACL_User::analytics( $user_id ) )
        return false;

    return true;
  }

  public static function analytics_view_user_id_post_id( $request_data ) {
    if( !isset( $request_data[ 'post_id' ] ) || !isset( $request_data[ 'user_id' ] ) )
      return false;

    if( WPEP_ACL_Post::analytics( $request_data[ 'post_id' ], 'view' ) )
      return true;

    return WPEP_ACL_User::analytics( $request_data[ 'user_id' ] );
  }

}