<?php

namespace WPEP\API;

class Misc {

  public static function format_timestamps_to_datetime( $request_data ) {
    if( !isset( $request_data[ 'timestamps' ] ) )
      return false;

    $response = [
      'status'  => 'ok',
      'entries' => []
    ];

    foreach( $request_data[ 'timestamps' ] as $timestamp ) {
      $timestamp = intval( $timestamp );

      $response[ 'entries' ][ $timestamp ] = wpep_format_date_with_time( $timestamp );
    }

    return $response;
  }

  public static function allowed_mime_types() {
    return [
      'status'  => 'ok',
      'map'     => get_allowed_mime_types()
    ];
  }

  public static function icons() {
    return wpep_get_admin_icon_selection_list();
  }

  public static function date_modify_drip_options() {
    return wpep_learning_journey_drip_options();
  }

  public static function content_type_search_results( $request_data ) {
    if( !isset( $request_data[ 'field_value' ] ))
      return [
        'status'    => 'error',
        'response'  => '<p class="wpep-notification wpep-notification-danger">' . __( "Search term not included", "wpep" ) . '</p>'
      ];

    return [
      'status'   => 'ok',
      'entries'  => wpep_controller()->content->search->get_as_array(
        $request_data['field_value'],
        ( isset( $request_data['wpep_search_post_types'] ) ? $request_data['wpep_search_post_types'] : [] ),
        wpep_get_setting( 'search-results-number' ),
        [
          'include_categories'  => ( isset( $request_data['wpep_search_include_categories'] ) ? $request_data['wpep_search_include_categories'] : '' ),
          'exclude_categories'  => ( isset( $request_data['wpep_search_exclude_categories'] ) ? $request_data['wpep_search_exclude_categories'] : '' )
        ]
      )
    ];
  }

}