<?php

namespace WPEP\API\System;

use WPEP\API\Utils\Report as API_Utils_Report;
use WPEP\API\Utils\User as API_Utils_User;
use WPEP\Entity\Discussion as Entity_Discussion;

class Discussion {

  public static function activity_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    $time_slots = $request_data[ 'report_time_slots' ];

    $discussion_integrations = wpep_controller()->discussion->all();

    if( empty( $discussion_integrations ) )
      return [ 'status' => 'ok', 'map' => [] ] ;

    $response = [];

    foreach( $time_slots as $time_slot_from => $time_slot_to ) {
      $response[ $time_slot_from ] = [];

      foreach( $discussion_integrations as $discussion_integration ) {
        $comment_count = absint( Entity_Discussion::instance()->get_comment_count_by_args( [
          'time_from' => $time_slot_from,
          'time_to'   => $time_slot_to,
          'parent'    => 0,
          'namespace' => $discussion_integration->get_namespace()
        ] ) );

        $comment_reply_count = absint( Entity_Discussion::instance()->get_comment_reply_count_by_args( [
          'time_from' => $time_slot_from,
          'time_to'   => $time_slot_to,
          'namespace' => $discussion_integration->get_namespace()
        ] ) );


        if( $comment_count != 0 )
          $response[ $time_slot_from ][ sprintf( __( '%s - Comment', "wpep" ), $discussion_integration->get_namespace_title() ) ] = $comment_count;

        if( $comment_reply_count != 0 )
          $response[ $time_slot_from ][ sprintf( __( '%s - Comment Reply', "wpep" ), $discussion_integration->get_namespace_title() ) ] = $comment_reply_count;

        if( wpep_get_setting_flag( 'comments-reactions-status' ) ) {
          $comment_reaction_count = absint( Entity_Discussion::instance()->get_comment_data_count_by_args( [
            'time_from' => $time_slot_from,
            'time_to'   => $time_slot_to,
            'key'       => WPEP_DISCUSSION_COMMENT_USER_DATA_REACTION,
            'namespace' => $discussion_integration->get_namespace()
          ] ) );

          if( $comment_reaction_count != 0 )
            $response[ $time_slot_from ]
                     [ sprintf( __( '%s - Comment Reaction', "wpep" ), $discussion_integration->get_namespace_title() ) ] = $comment_reaction_count;
        }

        if( wpep_get_setting_flag( 'comments-mentions-status' ) ) {
          $comment_mention_count = absint( Entity_Discussion::instance()->get_comment_data_count_by_args( [
            'time_from' => $time_slot_from,
            'time_to'   => $time_slot_to,
            'key'       => WPEP_DISCUSSION_COMMENT_USER_DATA_MENTION,
            'namespace' => $discussion_integration->get_namespace()
          ] ) );

          if( $comment_mention_count != 0 )
            $response[ $time_slot_from ]
                     [ sprintf( __( '%s - Comment Mention', "wpep" ), $discussion_integration->get_namespace_title() ) ] = $comment_mention_count;
        }
      }
    }

    return [ 'status' => 'ok', 'map' => $response ] ;
  }

}