<?php

namespace WPEP\API\System;

class PlatformHealth {

  public static function export() {
    return [
      'status'             => 'ok',
      'report'             => wpep_platform_health()->report->get(),
      'service_report'     => wpep_platform_health()->service->get_report(),
      'maintenance_report' => wpep_platform_health()->maintenance->get_report()
    ];
  }

  public static function export_plain() {
    $response = '';

    $response .= '---------- ' . __( "Service Status Report", "wpep" ) . ' ----------' . "\n\n";

    foreach( wpep_platform_health()->service->get_report() as $service_key => $service_information ) {
      $response .=  $service_key . ' : ' . $service_information['status'] . "\n";
    }

    $response .= "\n" . '---------- ' . __( "Maintenance Report", "wpep" ) . ' ----------' . "\n\n";

    foreach( wpep_platform_health()->maintenance->get_report() as $maintenance_key => $maintenance_information ) {
      $response .=  $maintenance_key . ' : ' . $maintenance_information['status'] . "\n";
    }

    foreach( wpep_platform_health()->report->get() as $report_key => $report_group ) {
      $response .=  "\n";

      $response .=  '---------- ' . $report_group['name'] . ' ----------' . "\n\n";

      foreach( $report_group['entries'] as $report_entry_key => $report_entry_data ) {
        $response .=  ( is_array( $report_entry_data ) && isset( $report_entry_data[ 'title' ] ) ? $report_entry_data[ 'title' ] : $report_entry_key );
        $response .= ' : ';
        $response .= ( is_array( $report_entry_data ) ? ( isset( $report_entry_data[ 'value' ] ) ? $report_entry_data[ 'value' ] : '' ) : $report_entry_data );
        $response .= "\n";
      }
    }

    return [
      'status'   => 'ok',
      'content'  => $response,
      'alias'    => wpep_label_to_alias( __( "Platform Health", "wpep" ) . '_' . wpep_format_date() ),
      'lang'     => [
        'copy'       => __( "Copy to Clipboard", "wpep" ),
        'copied'     => __( "Copied to Clipboard", "wpep" ),
        'download'   => __( "Download", "wpep" ),
      ]
    ];
  }

}