<?php

namespace WPEP\API\System;

use WPEP\API\Utils\Report as API_Utils_Report;

class Post {

  public static function content_activity_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    $time_slots = $request_data[ 'report_time_slots' ];

    $response = [];

    foreach( $time_slots as $time_slot_from => $time_slot_to ) {
      $current_time_slot_data = [];

      foreach( wpep_content_type()->contentTypes as $contentType ) {
        $sql = 'SELECT COUNT( * ) FROM `' . wpep_wpdb()->posts . '` 
                               WHERE post_type = "' . $contentType->get_post_type() . '"
                                 AND post_status IN ("' . implode( '","', wpep_post_monitored_post_status_list() ) . '")
                                 AND post_date >= "' . date( "Y-m-d H:i:s", $time_slot_from ) . '" AND post_date < "' . date( "Y-m-d H:i:s", $time_slot_to ) . '"';

        $count = absint( wpep_wpdb()->get_var( $sql ) );

        if( $count !== 0 )
          $current_time_slot_data[ sprintf( __( 'Added - %s', "wpep" ), $contentType->get_plural_name() ) ] = $count;
      }

      $response[ $time_slot_from ] = $current_time_slot_data;
    }

    return [ 'status' => 'ok', 'map' => $response ] ;
  }

  public static function search_map( $request_data ) {
    if( !isset( $request_data[ 'search' ] ) )
      return [
        'status'  => 'error',
        'error'   => __( "Missing search term.", "wpep" )
      ];

    $search     = sanitize_text_field( $request_data[ 'search' ] );
    $post_types = ( isset( $request_data[ 'post_type' ] ) ? $request_data[ 'post_type' ] : array_keys( wpep_content_type()->contentTypes ) );

    if( !is_array( $post_types ) )
      $post_types = [ $post_types ];

    foreach( $post_types as $post_type_key => $post_type ) {
      if( !isset( wpep_content_type()->contentTypes[ $post_type ] ) && !in_array( $post_type, [ 'page', 'post' ] ) )
        unset( $post_types[ $post_type_key ] );
    }

    if( empty( $post_types ) )
      return [
        'status'  => 'error',
        'error'   => __( "Invalid post_type provided.", "wpep" )
      ];

    $search_results = [];
    $has_multiple_post_types = count( $post_types ) >= 2;

    foreach( $post_types as $post_type ) {
      $post_list = wpep_get_post_list_assoc( $post_type, 'ID', 'post_title', 'post_title', [], $search );

      if( empty( $post_list ) )
        continue;

      foreach( $post_list as $post_id => $post_title )
        $search_results[] = [
          'text'  => $post_title . ( $has_multiple_post_types && isset( wpep_content_type()->contentTypes[ $post_type ] ) ? ' - ' . wpep_content_type()->contentTypes[ $post_type ]->get_singular_name() : '' ) . ' #' . $post_id,
          'id'    => $post_id
        ];
    }

    return [
      'status'  => 'ok',
      'results' => $search_results
    ];
  }

  public static function search_learning_journey_map( $request_data ) {
    if( !isset( $request_data[ 'search' ] ) )
      return [
        'status'  => 'error',
        'error'   => __( "Missing search term.", "wpep" )
      ];

    $search     = sanitize_text_field( $request_data[ 'search' ] );
    $post_types = array_keys( wpep_content_type()->get_learning_journey_instances() );

    $post_list = wpep_get_post_list( $post_types, 'post_title', [], $search );

    if( empty( $post_list ) )
      return [
        'status'  => 'ok',
        'results' => []
      ];

    $search_results = [];

    foreach( $post_list as $post_information ) {
      $text = '';

      if( isset( wpep_content_type()->contentTypes[ $post_information->post_type ] ) )
        $text .= wpep_content_type()->contentTypes[ $post_information->post_type ]->get_singular_name() . ' - ';

      $text .= $post_information->post_title . ' #' . $post_information->ID;

      $search_results[] = [
        'text'  => $text,
        'id'    => $post_information->ID
      ];
    }

    return [
      'status'  => 'ok',
      'results' => $search_results
    ];
  }

  public static function save( $request_data ) {
    if( !current_user_can( WPEP_CAP_EDIT_COURSE ) && !current_user_can( WPEP_CAP_EDIT_ASSESSMENT ) )
      return [
        'status'  => 'error',
        'error'   => sprintf( __( "Current user cannot %s.", "wpep" ), WPEP_CAP_EDIT_COURSE . ' or ' . WPEP_CAP_EDIT_ASSESSMENT )
      ];

    if( !isset( $request_data['post_id'] ) )
      return false;

    $post_id = intval( $request_data['post_id'] );

    $post = get_post( $post_id );

    if( !isset( wpep_content_type()->contentTypes[ $post->post_type ] ) )
      return false;

    if( $post->post_author != get_current_user_id() && !current_user_can( 'edit_others_' . wpep_content_type()->contentTypes[ $post->post_type ]->get_cap_name() . 's' ) )
      return false;

    $entity_id_map = ( isset( $request_data['wpep_entity_id_map'] ) ? $request_data['wpep_entity_id_map'] : [] );

    if( !is_array( $entity_id_map ) )
      $entity_id_map = [];

    $entity_id_map = apply_filters( 'wpep_admin_save_post_ajax', $entity_id_map, $post_id );

    return [
      'status'          => 'ok',
      'entity_id_map'   => $entity_id_map
    ];
  }

}