<?php

namespace WPEP\API\System;

use WPEP\Controller as WPEP_Controller;

class Setting {

  public static function search( $request_data ) {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return [
        'status'  => 'error',
        'error'   => sprintf( __( "Current user cannot %s.", "wpep" ), WPEP_CAP_MANAGE_SETTINGS )
      ];

    $search = sanitize_text_field( $request_data[ 'search' ] );

    return [
      'status'  => 'ok',
      'content' => WPEP_Controller::instance()->adminController->settings->get_data_map_search_content( $search, ( isset( $request_data[ 'refresh' ] ) && $request_data[ 'refresh' ] ) )
    ];
  }

  public static function search_by_index( $request_data ) {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return [
        'status'  => 'error',
        'error'   => sprintf( __( "Current user cannot %s.", "wpep" ), WPEP_CAP_MANAGE_SETTINGS )
      ];

    $search = sanitize_text_field( $request_data[ 'search' ] );

    return [
      'status'  => 'ok',
      'content' => WPEP_Controller::instance()->adminController->settings->get_data_map_search_by_index_content( $search )
    ];
  }

  public static function save( $request_data ) {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return [
        'status'  => 'error',
        'error'   => sprintf( __( "Current user cannot %s.", "wpep" ), WPEP_CAP_MANAGE_SETTINGS )
      ];

    WPEP_Controller::instance()->adminController->settings->save_settings( $request_data );

    return [
      'status'        => 'ok',
      'notification'  => __( "Searched settings have been updated, if this message is visible for a long period of time, there's a display error.", "wpep" ) . ' <a href="' . wpep_admin_url_settings() . '">' . __( "Refresh Settings Page", "wpep" ) . '</a>'
    ];
  }

  public static function disable_legacy_comments( $request_data ) {
    if( !wp_verify_nonce( $request_data[ 'nonce' ], 'wpep_disable_legacy_comments' ) )
      return [
        'status'  => 'error',
        'error'   => __( "Invalid Nonce.", "wpep" )
      ];

    wpep_set_setting( 'enable-course-comments', 0 );

    return [
      'status'  => 'success',
    ];
  }

}