<?php

namespace WPEP\API\System;

use WPEP\API\Utils\Report as API_Utils_Report;

class StatisticsCurrentUser {

  public static function configuration_report( $request_data ) {
    $analyze_activity_posts = [];
    $list_activity_posts = [];

    $user_id = ( isset( $request_data[ 'scu_user_id' ] ) && current_user_can( WPEP_CAP_MANAGE_MEMBERS ) ? intval( $request_data[ 'scu_user_id' ] ) : intval( get_current_user_id() ) );

    $user_ids = [ $user_id ];

    if( user_can( $user_id, WPEP_CAP_SUPERVISE_STUDENTS ) )
      $user_ids = array_merge( wpep_wpdb()->get_col( 'SELECT user_id FROM `' . wpep_wpdb()->usermeta . '` WHERE `meta_key` = "' . WPEP_USER_META_SUPERVISOR_USER_ID . '" AND `meta_value` = "' . $user_id . '"' ) );

    $user_ids = array_merge( $user_ids, wpep_controller()->contentLibrary->get_sub_account_receiver_user_ids_from_provider_user_id( $user_id ) );

    if( !is_array( $user_ids ) )
      return false;

    $user_ids = array_unique( $user_ids );

    foreach( wpep_content_type()->contentTypes as $contentType ) {
      $analyze_activity_posts[ 'activity-' . $contentType->get_post_type() ] = [
        'title'            =>  $contentType->get_plural_name(),
        'type'             => 'data-view-analyze',
        'data_action'      => 'wpep/system/user/post-type-activity-report',
        'data_action_args' => [
          'post_type' => $contentType->get_post_type(),
          'user_ids'  => $user_ids
        ]
      ];

      $list_activity_posts[ 'list-' . $contentType->get_post_type() ] = [
        'title'            =>  $contentType->get_plural_name(),
        'type'             => 'data-view-list',
        'data_action'      => 'wpep/system/user/post-type-activity-list',
        'data_action_args' => [
          'post_type' => $contentType->get_post_type(),
          'user_ids'  => $user_ids
        ]
      ];
    }

    $report_data_sources = [
      'activity-list' => [
        'title'       => __( "Activity List", "wpep" ),
        'type'        => 'group',
        'group'       => [
            'user-list' => [
              'title'            => __( "User Activity", "wpep" ),
              'type'             => 'data-view-list',
              'data_action'      => 'wpep/system/user/entries',
              'data_action_args' => [
                'user_ids'  => $user_ids
              ]
            ],
          ] + $list_activity_posts
      ],
      'activity-metrics' => [
        'title'       => __( "Activity Metrics", "wpep" ),
        'type'        => 'group',
        'group'       => [
            'overview' => [
              'title'            =>  __( "General", "wpep" ),
              'type'             => 'data-view-analyze',
              'data_action'      => 'wpep/system/user/activity-report',
              'data_action_args' => [
                'user_ids'  => $user_ids
              ]
            ],
            'authentication' => [
              'title'            =>  __( "Authentication", "wpep" ),
              'type'             => 'data-view-analyze',
              'data_action'      => 'wpep/system/user/authentication-activity-report',
              'data_action_args' => [
                'user_ids'  => $user_ids
              ]
            ],
          ] + $analyze_activity_posts
      ]
    ];

    return [
        'status'              => 'ok',
        'report_data_sources' => $report_data_sources,
        'user_ids'            => $user_ids
      ] + API_Utils_Report::configuration_for_statistics( $request_data );
  }

}