<?php

namespace WPEP\API\Utils;

use WPEP\ACL\User as WPEP_ACL_User;

class User {

  /**
   * @param [] $response
   * @param [] $request_data
   * @param \WP_User[] $entries_user_objects
   * @return mixed
   */
  public static function entries( $response, $request_data, $entries_user_objects ) {
    if( empty( $entries_user_objects ) )
      return $response;

    $public_profiles_enabled = wpep_public_profiles_enabled();

    $has_user_profile_url = false;
    $has_sid_support = wpep_has_sid_support();
    $has_all_information = current_user_can( WPEP_CAP_MANAGE_MEMBERS );
    $user_role_assoc = wpep_get_user_role_list_array_assoc( [] );

    foreach( $response[ 'entries' ] as $entry_index => $entry ) {
      $profile_picture_src = wpep_user_profile_picture_url( $entry[ 'user_id' ], 48 );
      $profile_name = wpep_content_user_profile_format_name( $entries_user_objects[ $entry[ 'user_id' ] ] );

      $attach_start = [
        'picture'    => [
          'type'   => 'img',
          'src'    => $profile_picture_src,
          'width'  => 48,
          'height' => 48
        ],
        'name'   => $profile_name,
      ];

      if( $has_all_information ) {
        $attach_start[ 'email_address' ] = $entries_user_objects[ $entry[ 'user_id' ] ]->user_email;

        $attach_start[ 'user_role_string' ] = '';

        foreach( $entries_user_objects[ $entry[ 'user_id' ] ]->roles as $role )
          $attach_start[ 'user_role_string' ] .= ( $attach_start[ 'user_role_string' ] === '' ? '' : ',' ) . $user_role_assoc[ $role ];
      }

      if( $has_sid_support )
        $attach_start[ 'student_identifier' ] = get_user_meta( $entry[ 'user_id' ], WPEP_USER_META_STUDENT_IDENTIFICATION, true );

      $response[ 'entries' ][ $entry_index ] = $attach_start + $response[ 'entries' ][ $entry_index ];

      if( $public_profiles_enabled ) {
        if( WPEP_ACL_User::profile( $entry[ 'user_id' ] ) )
          $response[ 'entries' ][ $entry_index ][ 'user_profile_url' ] = [
            'type'  => 'link',
            'link'  => wpep_user_profile_public_link( $entry[ 'user_id' ] )
          ];
        else
          $response[ 'entries' ][ $entry_index ][ 'user_profile_url' ] = '';

        $has_user_profile_url = true;
      }

      if( WPEP_ACL_User::analytics( $entry[ 'user_id' ] ) ) {
        $response[ 'entries' ][ $entry_index ][ 'name' ] = [
          'label'          => $response[ 'entries' ][ $entry_index ][ 'name' ],
          'type'           => 'component_trigger',
          'trigger'        => 'analytics_attach',
          'args' => [
            'component'       => 'analytics-panel',
            'component_title' => sprintf( __( "User : %s", "wpep" ), $profile_name ),
            'component_img'   => $profile_picture_src,
            'action'          => 'wpep/system/statistics/user-id-configuration-report',
            'user_id'         => $entry[ 'user_id' ]
          ]
        ];
      }

    }

    $response = apply_filters( "wpep_api_utils_user_entries", $response, $request_data, $entries_user_objects );

    $append_entries_header = [
        'picture'                 => '',
        'name'                    => __( "Name", "wpep" ),
      ] + (
      $has_user_profile_url ? [
        'user_profile_url'  => __( "User Profile", "wpep" )
      ] : []
      ) + (
      $has_all_information ? [
        'email_address'    => __( "Email", "wpep" ),
        'user_role_string' => __( "User Role", "wpep" )
      ] : []
      ) + (
      $has_sid_support ? [ 'student_identifier' => __( "SID", "wpep" ) ] : [] );

    $response[ 'entries_header' ] = $append_entries_header + $response[ 'entries_header' ];

    return $response;
  }

}