<?php
namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;

/**
 * Class AddOnsSection
 * @package WPEP\Admin\Coordinator
 */
class AddOnsSection {

  public function setup() {
    add_action( 'admin_notices', [ $this, 'admin_notices' ], 15 );
  }

  public function admin_notices() {
    if( empty( WPEP_Controller::instance()->addOns->addOns_missing_dependencies )
        || !is_array( WPEP_Controller::instance()->addOns->addOns_missing_dependencies ) )
      return;

    $admin_notice_list = $this->_get_admin_notice_list();

    if( $admin_notice_list == '' )
      return;

    echo '<div class="error">';
    echo    '<h2>' . sprintf( __( "%s AddOn Integrations Notice.", "wpep" ), WPEP_PLUGIN_NAME ) . '</h2>';
    echo    '<p>' . __( "The following Integrations have not been loaded due to missing their integrated plugin, make sure it's active.", "wpep" ) . '</p>';
    echo    '<ul>';
    echo      $admin_notice_list;
    echo    '</ul>';
    echo '</div>';
  }

  private function _get_admin_notice_list() {
    $resp = '';

    foreach( WPEP_Controller::instance()->addOns->addOns_missing_dependencies as $addOnAlias ) {
      $addOn = WPEP_Controller::instance()->addOns->get_add_on( $addOnAlias );

      $requirements_response = $addOn->has_requirements_met();

      if( !is_array( $requirements_response ) )
        continue;

      $requirements_list = '';

      foreach( $requirements_response as $plugin_name => $plugin_url )
        $requirements_list .= '<a href="' . $plugin_url . '" target="_blank">' . $plugin_name . '</a>';

      $resp .= '<li>' . sprintf(
                  __( '%s %s requires : %s ', 'wpep' ),
            WPEP_PLUGIN_NAME,
                  $addOn->get_name(),
                  $requirements_list
                ) . '</li>';
    }

    return $resp;
  }

  public function _register_menu() {
    if( !is_main_site() )
      return;

    wpep_admin_register_menu(
      'Add-ons',
      'Add-ons',
      WPEP_CAP_MANAGE_SETTINGS,
      'wpep-addons',
      [ $this, 'administration_page' ]
    );
  }

  public function administration_page() {
    echo '<div class="wrap" data-wpep-page="addons">';
    echo    '<h2 class="wpep-title-page">' . sprintf( __('%s Add-ons', "wpep" ), WPEP_PLUGIN_NAME ) . '</h2>';
    echo    '<div data-ec-component="plugin-management"></div>';
    echo '</div>';
  }
}