<?php

namespace WPEP\Admin\Coordinator;

class Assets {

  public function setup() {
    add_action( 'admin_enqueue_scripts', [ $this, 'admin_assets' ], 5 );

    if( wpep_get_setting_flag( 'admin-library-conflict-select-2-status' ) )
      add_action( 'admin_enqueue_scripts', [ $this, '_admin_deque_conflicting_assets' ], 9999 );
  }

  public function _admin_deque_conflicting_assets( $hook ) {
    $wp_scripts = wp_scripts();

    global $post;

    if( strpos( $hook, 'wpep-' ) === false
        && !( in_array( $hook, [ "post-new.php", "post.php" ] ) && isset( $post->post_type ) && ( isset( wpep_content_type()->contentTypes[ $post->post_type ] ) ) ) )
      return;

    if( wpep_get_setting_flag( 'admin-library-conflict-select-2-status' ) ) {
      foreach( $wp_scripts->registered as $script_key => $script ) {
        if( strpos( $script_key, 'select2' ) === false )
          continue;

        wp_deregister_script( $script_key );
      }


      wp_enqueue_style('select2', WPEP_BASE_URL_PATH . 'interface/third-party-library/select2/select2.min.css', [], '4.0.7' );
      wp_enqueue_script('select2', WPEP_BASE_URL_PATH . 'interface/third-party-library/select2/select2.full.min.js', [ 'jquery' ], '4.0.7' , true);
    }
  }

  public function admin_assets($hook) {
    $wpep_admin_dependency  = [ 'jquery', 'elearncommerce', 'wp-color-picker', 'jquery-ui-sortable', 'jquery-ui-resizable', 'clipboard' ];
    $wpep_course_dependency = [ 'jquery', 'wpep-admin-script' ];

    if( wpep_get_setting_flag( 'admin-editor-integration' ) ) {
      add_action( "admin_head", [ $this, '_tiny_mce_admin_head' ], 50 );
      add_filter( 'mce_buttons', [ $this, '_tiny_mce_buttons' ], 50 );
      add_filter( 'mce_external_plugins', [ $this, '_tiny_mce_extensions' ], 50 );
    }

    wp_register_style('wpep-admin-style', WPEP_BASE_URL_PATH . 'assets/admin/style.css', [], WPEP_VERSION );
    wp_enqueue_style('wpep-admin-style');

    wp_enqueue_style( 'wp-color-picker' );

    wp_enqueue_script('clipboard', WPEP_BASE_URL_PATH . 'assets/clipboard.min.js', [], '2.0.0' , true);

    wpep_register_universal_interface();

    /* enqueue Script. */
    wp_enqueue_script( 'wpep-admin-script', WPEP_BASE_URL_PATH . 'assets/admin/primary-min.js', $wpep_admin_dependency, WPEP_VERSION , true);
    wp_localize_script( 'wpep-admin-script', 'wpep_vars', apply_filters( "wpep_admin_script_variables", [
      'plugin_name'                   => WPEP_PLUGIN_NAME,
      'external_embed_helper_script'  => WPEP_BASE_URL_PATH . 'assets/front/embed-support-min.js',
      'can_manage_settings'           => intval( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) ),
      'post_type_courses'             => WPEP_POST_TYPE_COURSE,
      'wpep_settings_url'             => wpep_admin_url_settings(),
      'is_network_admin'              => intval( is_network_admin() )
    ], $hook ) );
    wp_enqueue_script( 'wpep-admin-script' );

    if( strpos( $hook, 'wpep-' ) !== false
        || ( in_array( $hook, [ 'edit-tags.php', 'term.php' ] ) && isset( $_GET[ 'taxonomy' ] ) && $_GET[ 'taxonomy' ] == WPEP_TAXONOMY_CATEGORY ) )
      $this->_enqueu_media();

    if ( in_array( $hook, [ "post-new.php", "post.php" ] ) && wpep_content_type()->is_enabled( get_post_type() ) ) {
      $this->_enqueu_media();

      if( wpep_content_type()->contentTypes[ get_post_type() ]->get_builder_instance() !== false ) {

        wp_register_script('wpep-course-edit', WPEP_BASE_URL_PATH . 'assets/admin/builder-min.js', $wpep_course_dependency, WPEP_VERSION , true);
        wp_localize_script( 'wpep-course-edit', 'wpep_settings', [
          'course_save_alert' => wpep_get_setting_flag( 'admin-course-save-alert' ),
          'max_input_vars'    => intval( @ini_get( 'max_input_vars' ) )
        ] );
        wp_enqueue_script( 'wpep-course-edit' );

        if( wpep_get_setting_flag( 'admin-editor-integration' ) ) {
          add_filter('mce_css', [$this, '_tiny_mce_css_within_builder'], 500);
          add_filter('mce_buttons', [$this, '_tiny_mce_buttons_within_builder'], 500);
        }

        add_action( 'admin_footer', [ $this, '_page_loader' ], 500);
      }
    } else if( isset( $_GET['page'] ) && $_GET['page'] == 'wpep-settings' && wpep_get_setting_flag( 'admin-editor-integration' ) ) {
      add_filter('mce_css', [$this, '_tiny_mce_css_within_builder'], 500);
    }
  }

  private function _enqueu_media() {
    wp_enqueue_media();
    wp_enqueue_script( 'media-grid' );
    wp_enqueue_script( 'media' );
    wp_enqueue_style( 'media');
    wp_enqueue_script( 'media-upload' );
    wp_enqueue_script( 'thickbox' );
  }

  public function _tiny_mce_admin_head() {
    echo "<script type='text/javascript'>
            /* <![CDATA[ */
            var wpep_toolkit_icon = '" . WPEP_BASE_URL_PATH . "assets/img/logo-white-background.png';
                wpep_toolkit_list = " . json_encode( wpep_toolkit_list_map(), JSON_HEX_TAG ) . ";
            /* ]]> */
          </script>";
  }

  public function _tiny_mce_buttons( $buttons ) {
    array_push( $buttons, 'separator', 'wpep_toolkit', 'separator' );

    return $buttons;
  }

  public function _tiny_mce_extensions( $extensions ) {
    $extensions['wpep_toolkit'] = WPEP_BASE_URL_PATH . 'assets/admin/tinymce-toolkit-min.js?ver=' . WPEP_VERSION;

    return $extensions;
  }

  public function _tiny_mce_css_within_builder( $mce_css ) {
    $mce_css .= ',' . WPEP_BASE_URL_PATH . 'assets/admin/wp-editor.css?ver=' . WPEP_VERSION;

    return $mce_css;
  }

  public function _tiny_mce_buttons_within_builder( $buttons ) {
    $options = [ 'fontselect', 'fontsizeselect', 'styleselect' ];

    foreach( $options as $option )
      if( !in_array( $option, $buttons ) )
        $buttons[] = $option;

    $buttons = array_diff( $buttons, ['wp_more', 'fullscreen'] );

    return $buttons;
  }

  public function _page_loader() {
    $message = sprintf( __( "Loading %s Builder", "wpep" ), WPEP_PLUGIN_NAME );

    global $post;

    if( isset( $post->post_type ) && wpep_content_type()->is_enabled( $post->post_type ) )
      $message = sprintf( __( "Loading the %s Builder", "wpep" ), wpep_content_type()->contentTypes[ $post->post_type ]->get_singular_name() );

    echo '<div id="wpep-loader-container">';
    echo    wpep_spinner_loader( $message );
    echo '</div>';
  }

}