<?php

namespace WPEP\Admin\Coordinator\DataSystem;

use WPEP\Controller as WPEP_Controller;

class ContentTypes {

  private $_post_types                      = [];
  private $_post_table_fields               = [];
  private $_post_utility_fields             = [];
  private $_post_meta_keys_map_by_type      = [];
  private $_post_meta_map_by_type_transient = 'wpep_ds_ct_meta_transient';

  public function setup() {
    add_action( 'wp_ajax_wpep_data_system_export_content_types_list', [ $this, 'export_list' ] );
  }

  public function export_list() {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS )  )
      exit( '<div class="wpep-notification wpep-notification-danger">' . __( "Your account is not allowed to complete this operation", "wpep" ) . '</div>');

    $this->_setup_data();

    if( isset( $_POST[ 'wpep_cte_post_type' ] ) && in_array( $_POST[ 'wpep_cte_post_type' ], $this->_post_types) ) {
      if( isset( $_POST[ 'wpep_cte_keys_post_table'] ) || isset( $_POST[ 'wpep_cte_keys_post_meta'] ) || isset( $_POST[ 'wpep_cte_keys_utility_information' ] ) ) {
        $request_post_table_information    = ( isset( $_POST[ 'wpep_cte_keys_post_table'] )          ? $_POST[ 'wpep_cte_keys_post_table']          : [] );
        $request_meta_information          = ( isset( $_POST[ 'wpep_cte_keys_post_meta'] )           ? $_POST[ 'wpep_cte_keys_post_meta']           : [] );
        $request_utility_information       = ( isset( $_POST[ 'wpep_cte_keys_utility_information'] ) ? $_POST[ 'wpep_cte_keys_utility_information'] : [] );

        $posts = get_posts( [
          'numberposts' => -1,
          'post_type'   => $_POST[ 'wpep_cte_post_type' ],
          'post_status' => wpep_post_monitored_post_status_list()
        ] );

        $table_information_head = [];
        $table_information_body = [];

        foreach( $request_post_table_information as $key )
          $table_information_head[ $key ] = sanitize_text_field( $key );

        foreach( $request_meta_information as $key )
          $table_information_head[ 'meta_' . $key ] = sanitize_text_field( $key );

        foreach( $request_utility_information as $key )
          $table_information_head[ 'utility_' . $key ] = sanitize_text_field( $key );

        if( in_array( 'permalink', $request_utility_information ) ) {
          if( \wpep_controller()->content->library->gridItem === null )
            \wpep_controller()->content->library->_setup();

          remove_filter( 'wpep_primary_content_item_link',  [ \wpep_controller()->content->library->gridItem, 'wpep_post_link' ], WPEP_ACTION_AND_FILTER_PRIORITY_LAST );
        }

        foreach( $posts as $post ) {
          $current_table_body = [];

          foreach( $request_post_table_information as $post_key )
            $current_table_body[ $post_key ] = $post->$post_key;

          foreach( $request_meta_information as $post_meta_key )
            $current_table_body[ 'meta_' . $post_meta_key ] = implode( "\n", get_post_meta( $post->ID, $post_meta_key, false ) );

          if( in_array( 'permalink', $request_utility_information ) )
            $current_table_body[ 'utility_permalink' ] = apply_filters( 'wpep_primary_content_item_link', get_permalink( $post->ID ), $post->ID );

          $table_information_body[] = $current_table_body;
        }

        echo '<div class="wpep-title-container">';
        echo    '<h3 class="wpep-title">' . __( "Export Completed, a preview has been generated bellow", "wpep" ) . '</h3>';
        echo    '<div class="wpep-import-operations-container">';
        echo    '<input class="wpep-button wpep-button-secondary" 
                        type="submit" 
                        data-switch-sync-step="dynamic"
                        value="' . __( "Start Over", "wpep" ) . '"/> ';
        echo    '<a class="wpep-button wpep-button-primary" 
                    href="data:text/csv;base64,' . base64_encode( WPEP_Controller::instance()->hooks->get_csv_content( $table_information_head, $table_information_body) ). '"
                    download="' . $_POST[ 'wpep_cte_post_type' ] . '-'  . wpep_label_to_alias( wpep_format_date_with_time() ) . '-export.csv"';
        echo    '>';
        echo        __( "Download", "wpep" );
        echo    '</a> ';
        echo   '</div>';
        echo '</div>';

        wpep_admin_display_table(
          false,
          $table_information_head,
          array_slice( $table_information_body, 0, 5 )
        );

        exit;
      }

      $this->_display_field_selection( $_POST[ 'wpep_cte_post_type' ] );
      exit;
    }

    $this->_display_dashboard();

    exit;
  }

  private function _display_dashboard() {
    echo '<div class="wpep-title-container">';
    echo    '<h3 class="wpep-title">' . __( "Step 1 : Select Export Content Type", "wpep" ) . '</h3>';
    echo    '<input class="wpep-button wpep-button-primary" 
                    type="submit" 
                    data-switch-sync-step="dynamic"
                    value="' . __( "Next", "wpep" ) . '"/>';
    echo '</div>';

    echo  '<div class="wpep-utility-data-sync-row">';
    echo    '<label for="wpep-ph-ds-cte-source">' . __( "Export", "wpep" ) . '</label>';
    echo    '<select id="wpep-ph-ds-cte-source" name="wpep_cte_post_type">';
    foreach( wpep_content_type()->standardContentTypes as $standardContentType )
      echo      '<option value="' . $standardContentType->get_post_type() . '">' . $standardContentType->get_plural_name() . '</option>';
    echo    '</select>';
    echo    '<div class="wpep-description">' . __( "You'll get an CSV Export of the selected Content Type above, one entity per line, in the next step you'll choose the post table fields you desire to export, and meta information.", "wpep" ) . '</div>';
    echo  '</div>';
  }

  private function _display_field_selection( $post_type ) {
    echo '<div class="wpep-title-container">';
    echo    '<h3 class="wpep-title">';
    echo      sprintf( __( "Step 2 : Select %s Export Fields", "wpep" ),
                 '<strong>' . wpep_content_type()->contentTypes[ $post_type ]->get_plural_name() . '</strong>'
              );
    echo    '</h3>';
    echo    '<input class="wpep-button wpep-button-primary" 
                    type="submit" 
                    data-switch-sync-step="dynamic"
                    value="' . __( "Export", "wpep" ) . '"/>';
    echo '</div>';
    echo '<input type="hidden" name="wpep_cte_post_type" value="' . $post_type . '"/>';

    echo  '<h3>' . __( "Post Table", "wpep" ) . '</h3>';

    foreach( $this->_post_table_fields as $field_key ) {
      echo  '<div class="wpep-utility-data-sync-row wpep-utility-data-sync-row-wide-label">';
      echo    '<label for="wpep-ph-ds-cte-post-' . $field_key . '">' . $field_key . '</label>';
      echo    '<input id="wpep-ph-ds-cte-post-' . $field_key . '" type="checkbox" name="wpep_cte_keys_post_table[]" value="' . $field_key . '"/>';
      echo  '</div>';
    }

    echo  '<h3>' . __( "Utility Information", "wpep" ) . '</h3>';

    foreach( $this->_post_utility_fields as $field_key => $field_information ) {
      echo  '<div class="wpep-utility-data-sync-row wpep-utility-data-sync-row-wide-label">';
      echo    '<label for="wpep-ph-ds-cte-post-' . $field_key . '">' . ( isset( $field_information['title'] ) ? $field_information['title'] : ( is_string( $field_information ) ? $field_information : $field_key ) ) . '</label>';
      echo    '<input id="wpep-ph-ds-cte-post-' . $field_key . '" type="checkbox" name="wpep_cte_keys_utility_information[]" value="' . $field_key . '"/>';

      if( isset( $field_information['description'] ) )
        echo    '<div class="wpep-description">' . $field_information['description'] . '</div>';
      echo  '</div>';
    }

    if( !isset( $this->_post_meta_keys_map_by_type[ $post_type ] ) )
      return;

    foreach( $this->_post_meta_keys_map_by_type[ $post_type ] as $post_meta_keys_map ) {
      echo '<h3>' . $post_meta_keys_map['title'] . '</h3>';

      foreach( $post_meta_keys_map[ 'options' ] as $field_key => $field_title ) {
        echo  '<div class="wpep-utility-data-sync-row wpep-utility-data-sync-row-wide-label">';
        echo    '<label for="wpep-ph-ds-cte-meta-' . $field_key . '">' . $field_title . '</label>';
        echo    '<input id="wpep-ph-ds-cte-meta-' . $field_key . '" type="checkbox" name="wpep_cte_keys_post_meta[]" value="' . $field_key . '"/>';
        echo    '<div class="wpep-description">' . $field_key . '</div>';
        echo  '</div>';
      }
    }
  }

  private function _setup_data() {
    $this->_post_types = array_keys( wpep_content_type()->standardContentTypes );

    $this->_post_table_fields = [
      'ID',
      'post_name',
      'post_title',
      'post_excerpt',
      'post_content',
      'guid',
      'post_author',
      'post_status',
      'post_date',
      'post_date_gmt',
      'post_modified',
      'post_modified_gmt',
      'post_parent',
    ];

    $this->_post_utility_fields = [
      'permalink' => [
        'title'       => __( "Link", "wpep" ),
        'description' => __( "The same link as you'd click on an Index Page ( all link alterations will be applied, with the exception of options such as the Coming Soon that eliminate the link entirely ).", "wpep" )
      ]
    ];

    $this->_post_meta_keys_map_by_type = get_transient( $this->_post_meta_map_by_type_transient );

    if( !is_array( $this->_post_meta_keys_map_by_type ) ) {
      $meta_boxes_information = WPEP_Controller::instance()->adminController->fieldLibrary->post->get_meta_boxes();

      usort($meta_boxes_information, function($a, $b){ return strcmp($a["title"], $b["title"]); });

      $this->_post_meta_keys_map_by_type = [];

      foreach( wpep_content_type()->standardContentTypes as $contentType ) {
        $this->_post_meta_keys_map_by_type[ $contentType->get_post_type() ] = [];

        $standard_post_meta_fields = $contentType->standard_get_post_meta_fields();

        if( !empty( $standard_post_meta_fields ) ) {

          $this->_post_meta_keys_map_by_type[ $contentType->get_post_type() ][ 'base_post_meta_box' ] = [
            'title'   => __( "General Options", "wpep" ),
            'options' => []
          ];

          foreach( $standard_post_meta_fields as $field_information ) {
            if( !isset( $field_information['id'] ) )
              continue;

            $this->_post_meta_keys_map_by_type
            [ $contentType->get_post_type() ]
            [ 'base_post_meta_box' ]
            [ 'options' ]
            [ $field_information['id'] ] = (
            isset( $field_information[ 'name' ] )
              ? $field_information['name']
              : $field_information['id']
            );
          }

        }

        foreach( $meta_boxes_information as $meta_box_information ) {
          if( isset( $meta_box_information['pages'] ) && !isset( $meta_box_information['post_types'] ) )
            $meta_box_information['post_types'] = $meta_box_information['pages'];

          if( !in_array( $contentType->get_post_type(), $meta_box_information[ 'post_types' ] ) )
            continue;

          $this->_post_meta_keys_map_by_type[ $contentType->get_post_type() ][ $meta_box_information['id'] ] = [
            'title'   => $meta_box_information[ 'title' ],
            'options' => []
          ];

          foreach( $meta_box_information[ 'fields' ] as $meta_box_field_information ) {
            if( !isset( $meta_box_field_information['id'] ) )
              continue;

            $this->_post_meta_keys_map_by_type
            [ $contentType->get_post_type() ]
            [ $meta_box_information['id'] ]
            [ 'options' ]
            [ $meta_box_field_information['id'] ] = (
            isset( $meta_box_field_information[ 'name' ] )
              ? $meta_box_field_information['name']
              : $meta_box_field_information['id']
            );
          }
        }
      }

      set_transient( $this->_post_meta_map_by_type_transient, $this->_post_meta_keys_map_by_type, HOUR_IN_SECONDS );
    }
  }

}
