<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;

class Menu {

  public function setup() {
    add_action( 'admin_head-nav-menus.php', [ $this, '_register_navigation_menu_meta_box' ] );
    add_action( 'wp_update_nav_menu_item', [ $this, '_navigation_menu_item_save' ], 100, 3 );
  }

  public function _register_navigation_menu_meta_box() {
    add_meta_box(
      'wpep_navigation_menu',
      sprintf( __( '%s Toolkit', 'wpep' ), WPEP_PLUGIN_NAME ),
      [ $this, '_navigation_menu_meta_box_callback' ],
      'nav-menus',
      'side',
      'default'
    );
  }

  public function _navigation_menu_item_save( $menu_id, $_actual_db_id, $args ) {
    if( !isset( $args[ 'menu-item-url' ] ) )
      return;

    if( strpos($args[ 'menu-item-url' ], '#' . WPEP_MENU_PREFIX ) !== 0 )
      return;

    update_post_meta( $_actual_db_id, '_menu_item_url', $args[ 'menu-item-url' ] );
  }

  public function _navigation_menu_meta_box_callback( $information_object ) {
    if ( !class_exists('\WPEP_Menu_Meta_Box_Item') ) {
      @include_once WPEP_BASE_PATH . '/lib/_internal/Menu_Meta_Box_Item.php';
    }

    if ( !class_exists('\WPEP_Menu_Meta_Box_Item' ) )
      return;

    echo '<div id="wpep-navigation-menu-item-list" class="wpep-navigation-menu-item-list-container">';
    echo    '<div id="tabs-panel-wpep-navigation-menu-item-list-all" class="tabs-panel tabs-panel-view-all tabs-panel-active">';
    echo      '<ul id="wpep-navigation-menu-item-listchecklist" class="list:wpep-navigation-menu-item-list categorychecklist form-no-clear">';
    echo          $this->_navigation_menu_meta_box_tree();
    echo      '</ul>';
    echo    '</div>';
    echo    '<p class="button-controls">';
    echo      '<span class="add-to-menu">';
    echo        '<input type="submit" class="button-secondary submit-add-to-menu right"';
    echo        ' value="' . esc_attr( 'Add to Menu', 'wpep' ) . '"';
    echo        ' name="wpep_navigation_menu_item_add"';
    echo        ' id="submit-wpep-navigation-menu-item-list"';
    echo        '/>';
    echo        '<span class="spinner"></span>';
    echo      '</span>';
    echo    '</p>';
    echo '</div>';
  }

  public function _navigation_menu_meta_box_tree() {
    $element_list = [];

    foreach( WPEP_Controller::instance()->menu->get_items() as $menu_item_index => $menu_item_information ) {
      $element_list[ $menu_item_information[ 'name' ] ]            = new \WPEP_Menu_Meta_Box_Item();
      $element_list[ $menu_item_information[ 'name' ] ]->object_id = esc_attr( '#' . WPEP_MENU_PREFIX . $menu_item_index . '#' );
      $element_list[ $menu_item_information[ 'name' ] ]->title		 = esc_attr( $menu_item_information[ 'name' ] );
      $element_list[ $menu_item_information[ 'name' ] ]->url			 = esc_attr( '#' . WPEP_MENU_PREFIX . $menu_item_index . '#' );
    }

    $walker = new \Walker_Nav_Menu_Checklist( [] );

    return walk_nav_menu_tree(
      array_map('wp_setup_nav_menu_item', $element_list ),
      0,
      (object) [
        'walker' => $walker
      ]
    );
  }

  public function _register_menu() {
    if( WPEP_ADMIN_BASE_MENU_SLUG === true )
      return;

    add_menu_page(
      WPEP_PLUGIN_NAME,
      WPEP_MENU_NAME,
      WPEP_CAP_LEARNING_CENTER,
      WPEP_ADMIN_BASE_MENU_SLUG,
      false,
      '',
      intval( wpep_get_setting( 'menu-position' ) )
    );
  }

}