<?php

namespace WPEP\Admin\Coordinator;

class PlatformHealth {

  public $tabs;

  public function _register_menu() {
    $menu_title = __( "Platform Health", 'wpep');

    wpep_admin_register_menu(
      sprintf( __("%s Platform Health", 'wpep'), WPEP_PLUGIN_NAME ),
      $menu_title,
      WPEP_CAP_MANAGE_SETTINGS,
      "wpep-platform-health",
      [ $this, "display" ]
    );
  }

  public function _init() {
    if( !is_admin() )
      return;

    if( intval( get_option( 'wpep_platform_health_report_pushed' ) ) + DAY_IN_SECONDS < time()
        && get_option( 'wpep_platform_health_report_pushing', false ) === false )
      $this->_push_platform_health_report_to_store();
  }

  public function display() {
    echo '<div class="wrap" data-wpep-page="platform-health">';
    echo '<h2 class="wpep-title-page">' . sprintf( __("%s - Platform Health", 'wpep'), WPEP_PLUGIN_NAME ) . '</h2>';

    $current_platform_health_page = $this->get_pages( $this->get_current_tab() );

    $this->show_tabs( $this->get_current_tab() );

    $current_platform_health_page->display();

    echo '</div>';
  }

  public function show_tabs( $current = 'homepage' ) {
    echo '<h2 class="wpep-navigation wpep-navigation-primary" data-wpep-navigation="platform-health">';
    foreach( $this->get_pages() as $tab => $setting_page ){
      $class = ( $tab == $current ) ? ' wpep-navigation-item-active' : '';
      echo '<a data-wpep-tab="' . $tab . '" class="wpep-navigation-item' . $class . '" href="' . wpep_admin_url_platform_health( [ 'tab' => $tab ] ). '">';
      echo    '<span>' . $setting_page->get_name() . '</span>';
      echo '</a>';
    }

    echo '</h2>';

      echo '<div class="wpep-clearfix"></div>';
  }

  public function get_current_tab() {
    $statistics_pages = $this->get_pages();

    $current_tab = ( isset( $_GET['tab'] ) ? $_GET['tab'] : key( $statistics_pages ) );

    if( !isset( $statistics_pages[ $current_tab ] ) )
      $current_tab = key( $statistics_pages );

    return $current_tab;
  }

  public function get_pages( $index = false ) {
    if( !is_array( $this->tabs ) )
      $this->_setup_pages();

    return ( $index == false ? $this->tabs : $this->tabs[$index] );
  }

  private function _setup_pages() {
    $this->tabs = [];

    foreach( [
               new PlatformHealth\Dashboard(),
               new PlatformHealth\DataSystem(),
               new PlatformHealth\SystemLog(),
               new PlatformHealth\Tools(),
             ] as $current_instance )
      $this->tabs[ $current_instance->get_alias() ] = $current_instance;

    $this->tabs = apply_filters( 'wpep_administration_platform_health_pages', $this->tabs );
  }

  public function _push_platform_health_report_to_store() {
    update_option( 'wpep_platform_health_report_pushing', time() );

    $response = wp_remote_post(
      WPEP_STORE_LINK,
      [
        'body'  => [
          'platform_data_request'   => 'platform_health',
          'platform_link'           => get_option( 'home' ),
          'platform_version'        => WPEP_VERSION,
          'platform_software'       => WPEP_MENU_NAME,
          'platform_health_summary' => wpep_platform_health()->service->get_report_data(),
          'platform_health_maintenance' => wpep_platform_health()->maintenance->get_report_data(),
          'platform_health_report'  => wpep_platform_health()->report->get()
        ]
      ]
    );

    if ( is_wp_error($response) ) {
      delete_option( 'wpep_platform_health_report_pushing' );

      return;
    }

    $response = json_decode(wp_remote_retrieve_body( $response ), true );

    delete_option( 'wpep_platform_health_report_pushing' );

    if( !is_array( $response ) )
      return;

    if( $response[ 'status' ] !== 'ok' )
      return;

    update_option( 'wpep_platform_health_report_pushed', time() );
  }

}
