<?php

namespace WPEP\Admin\Coordinator\PlatformHealth;

use WPEP\Container\FormFields as WPEP_FormFields;
use WPEP\Entity\SystemLog as WPEP_Entity_SystemLog;

class DataSystem extends Skeleton {

  use WPEP_FormFields;

  public function __construct() {
    $this->set_field_prefix( 'wpep_data_system_settings' );

    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return;

    if( isset( $_GET['download-backup'] ) ) {
      $attachment_id   = intval( $_GET['download-backup' ] );
      $attachment_file = get_attached_file( $attachment_id );

      if( $attachment_file != false && wpep_string_ends_with( $attachment_file, '.zip' ) ) {
        @set_time_limit(0);
        @ini_set('memory_limit','512M');

        header("Content-type: application/zip");
        header("Content-Disposition: attachment; filename=" . basename( $attachment_file ) );
        header("Content-length: " . filesize($attachment_file));
        header("Pragma: no-cache");
        header("Expires: 0");
        readfile( $attachment_file );
        exit;
      }
    } elseif( isset( $_GET[ 'wpep-create-backup' ] ) && wp_verify_nonce( $_GET[ 'wpep-create-backup' ], 'wpep-create-backup' ) != false ) {

      $system_log = WPEP_Entity_SystemLog::instance( 'admin', 'manual_backup', time() );
      $system_log->save();

      wpep_data_system()->create_backup();

      $system_log->completed_at = time();
      $system_log->save();



      wpep_redirect( wpep_admin_url_platform_health( [ 'tab' => 'data-system', 'backup-created' => 1 ] ) );
      exit;
    } elseif( isset( $_POST[ 'wpep-data-system-save-settings' ] ) ) {
      wpep_settings_instance()->update_settings( $_POST['wpep_data_system_settings'] );

      wpep_redirect( wpep_admin_url_platform_health( [ 'tab' => 'data-system', 'settings-updated' => 1 ] ) );
      exit;
    }
  }

  public function get_name() {
    return __( "Data System", "wpep" );
  }

  public function get_alias() {
    return 'data-system';
  }

  public function display() {
    wpep_template_load( 'admin/platform-health/data-system-dashboard.php', [
      'settings_updated'      => ( isset( $_GET[ 'settings-updated' ] ) ? true : false ),
      'form_fields_instance'  => $this
    ] );
  }

  public function get_option($field_key, $field_default = '') {
    return wpep_get_setting( $field_key, $field_default );
  }

}