<?php

namespace WPEP\Admin\Coordinator\Settings;

use WPEP\Entity\SystemCache as Entity_SystemCache;
use WPEP\API as WPEP_API;

class Data {

  protected $_map = [];

  public function get_map_by_args( $args ) {
    if( !isset( $args[ 'search' ] ) || strlen( $args[ 'search' ] ) < 3 )
      return [
        'map'        => [],
        'updated_at' => date( "Y-m-d H:i:s", time() )
      ];

    $args[ 'search' ] = sanitize_text_field( $args[ 'search' ] );

    $sc_settings_search_map = Entity_SystemCache::instance( [
      'namespace' => WPEP_SYSTEM_CACHE_NAMESPACE_SETTINGS_SEARCH,
      'segment'   => 'search_' . wpep_label_to_alias( $args[ 'search' ] )
    ] );

    if( $sc_settings_search_map->is_configuration_required() || ( isset( $args['refresh_cache'] ) && $args['refresh_cache'] ) ) {
      $navigation_data = $this->get_map_search_navigation_data( ( isset( $args['refresh_cache'] ) && $args['refresh_cache'] ) );

      foreach( $navigation_data[ 'map' ] as $navigation_data_map_key => $navigation_data_map ) {
        if( !isset( $navigation_data_map['search_terms'] ) || empty( $navigation_data_map['search_terms'] ) ) {
          unset( $navigation_data[ 'map' ][ $navigation_data_map_key ] );
          continue;
        }

        // If the search term exists, then we skip the unset.
        foreach( $navigation_data_map['search_terms'] as $search_term ) {
          if( is_array( $search_term ) )
            continue;

          if( stripos( $search_term, $args[ 'search' ] ) !== false )
            continue 2;
        }

        unset( $navigation_data[ 'map' ][ $navigation_data_map_key ] );
      }

      foreach( $navigation_data[ 'map' ] as $navigation_data_map_key => $navigation_data_map )
        unset( $navigation_data[ 'map' ][ $navigation_data_map_key ][ 'search_terms' ] );

      $navigation_data_simple_map = [];

      foreach( $navigation_data[ 'map' ] as $navigation_data_map ) {
        if( !isset( $navigation_data_simple_map[ $navigation_data_map[ 'top_level' ] ] ) ) {
          $navigation_data_simple_map[ $navigation_data_map[ 'top_level' ] ] = [
            $navigation_data_map[ 'second_level' ] => [
              $navigation_data_map[ 'third_level' ] => [ $navigation_data_map[ 'forth_level' ] ]
            ]
          ];

          continue;
        }

        if( !isset( $navigation_data_simple_map[ $navigation_data_map[ 'top_level' ] ][ $navigation_data_map[ 'second_level' ] ] ) ) {
          $navigation_data_simple_map[ $navigation_data_map[ 'top_level' ] ][ $navigation_data_map[ 'second_level' ] ] = [
            $navigation_data_map[ 'third_level' ] => [ $navigation_data_map[ 'forth_level' ] ]
          ];

          continue;
        }

        if( !isset( $navigation_data_simple_map[ $navigation_data_map[ 'top_level' ] ][ $navigation_data_map[ 'second_level' ] ][ $navigation_data_map[ 'third_level' ] ] ) ) {
          $navigation_data_simple_map[ $navigation_data_map[ 'top_level' ] ][ $navigation_data_map[ 'second_level' ] ][ $navigation_data_map[ 'third_level' ] ] = [ $navigation_data_map[ 'forth_level' ] ];

          continue;
        }

        $navigation_data_simple_map[ $navigation_data_map[ 'top_level' ] ][ $navigation_data_map[ 'second_level' ] ][ $navigation_data_map[ 'third_level' ] ][] = $navigation_data_map[ 'forth_level' ];
      }

      $sc_settings_search_map->content = $navigation_data_simple_map;
      $sc_settings_search_map->expires_at = date( "Y-m-d H:i:s", time() + 3600 );

      $sc_settings_search_map->save();
    }

    $map = $this->get_map();

    foreach( $map as $map_key => $map_data ) {
      if( !isset( $sc_settings_search_map->content[ $map_key ] ) ) {
        unset( $map[ $map_key ] );

        continue;
      }

      foreach( $map_data[ 'sections' ] as $map_section_key => $map_section ) {
        if( !isset( $sc_settings_search_map->content[ $map_key ][ $map_section_key ] ) ) {
          unset( $map[ $map_key ][ 'sections' ][ $map_section_key ] );

          continue;
        }

        foreach( $map_section[ 'field_map' ] as $map_field_map_key => $map_field_map ) {
          if( !isset( $sc_settings_search_map->content[ $map_key ][ $map_section_key ][ $map_field_map_key ] ) ) {
            unset( $map[ $map_key ][ 'sections' ][ $map_section_key ][ 'field_map' ][ $map_field_map_key ] );

            continue;
          }

          foreach( $map_field_map[ 'fields' ] as $map_field_key => $map_field ) {
            if( !in_array( $map_field_key, $sc_settings_search_map->content[ $map_key ][ $map_section_key ][ $map_field_map_key ] ) ) {
              unset( $map[ $map_key ][ 'sections' ][ $map_section_key ][ 'field_map' ][ $map_field_map_key ][ 'fields' ][ $map_field_key ] );

              continue;
            }
          }
        }
      }
    }

    return [
      'map'        => $map,
      'updated_at' => $sc_settings_search_map->updated_at
    ];
  }

  public function get_map() {
    if( empty( $this->_map ) )
      $this->_setup_map();

    return $this->_map;
  }

  public function get_map_top_level_navigation_data() {
    if( empty( $this->_map ) )
      $this->_setup_map();

    $response = [];

    foreach( $this->_map as $map_key => $map_data )
      $response[ $map_data[ 'alias' ] ] = $map_data[ 'name' ];

    return $response;
  }

  public function get_map_secondary_level_navigation_data( $key ) {
    if( empty( $this->_map ) )
      $this->_setup_map();

    $response = [];

    foreach( $this->_map[ $key ][ 'sections' ] as $map_key => $map_data )
      $response[ $map_data[ 'alias' ] ] = $map_data[ 'name' ];

    return $response;
  }

  public function get_map_search_navigation_data( $refresh_cache = false ) {
    $sc_settings_search_map = Entity_SystemCache::instance( [
      'namespace' => WPEP_SYSTEM_CACHE_NAMESPACE_SETTINGS_SEARCH,
      'segment'   => 'map'
    ] );

    if( $sc_settings_search_map->is_configuration_required() || $refresh_cache ) {
      $sc_settings_search_map_content = [];
      $map = $this->get_map();

      foreach( $map as $map_top_level_key => $map_top_level ) {
        foreach( $map_top_level[ 'sections' ] as $map_second_level_key => $map_second_level ) {
          if( isset( $map_second_level[ 'allow_in_search' ] ) && !$map_second_level[ 'allow_in_search' ] )
            continue;

          foreach( $map_second_level[ 'field_map' ] as $map_third_level_key => $map_third_level ) {
            if( !is_array( $map_third_level[ 'fields' ] ) && is_callable( $map_third_level[ 'fields' ] ) )
              $map_third_level[ 'fields' ] = call_user_func( $map_third_level[ 'fields' ] );

            foreach( $map_third_level['fields'] as $map_forth_level_key => $map_forth_level ) {
              $search_terms = [
                $map_top_level[ 'name' ],
                $map_top_level[ 'alias' ],
                $map_second_level[ 'name' ],
                $map_second_level[ 'alias' ]
              ];

              if( isset( $map_third_level[ 'name' ] ) )
                $search_terms[] = $map_third_level[ 'name' ];

              if( isset( $map_third_level[ 'alias' ] ) )
                $search_terms[] = $map_third_level[ 'alias' ];

              if( isset( $map_forth_level[ 'name' ] ) )
                $search_terms[] = $map_forth_level[ 'name' ];

              if( isset( $map_forth_level[ 'alias' ] ) )
                $search_terms[] = $map_forth_level[ 'alias' ];

              if( isset( $map_forth_level[ 'description' ] ) )
                $search_terms[] = ( is_callable( $map_forth_level[ 'description' ] ) ? call_user_func( $map_forth_level[ 'description' ] ) : $map_forth_level[ 'description' ] );

              if( isset( $map_forth_level[ 'alias' ] ) && isset( wpep_settings_instance()->defaultOptions[ $map_forth_level[ 'alias' ] ] ) ) {
                $option_value = wpep_settings_instance()->get_option( $map_forth_level[ 'alias' ] );

                if( !is_numeric( $option_value ) && !is_array( $option_value ) ) {
                  $search_terms[] = $option_value;

                  $option_value_translated = wpep_settings_instance()->get_option_translated( $map_forth_level[ 'alias' ] );

                  if( $option_value !== $option_value_translated )
                    $search_terms[] = $option_value_translated;
                }
              }

              $sc_settings_search_map_content[] = [
                'top_level'     => $map_top_level_key,
                'second_level'  => $map_second_level_key,
                'third_level'   => $map_third_level_key,
                'forth_level'   => $map_forth_level_key,
                'search_terms'  => $search_terms
              ];
            }
          }
        }
      }

      $sc_settings_search_map->content = $sc_settings_search_map_content;

      $sc_settings_search_map->save();
    }

    return [
      'map'        => $sc_settings_search_map->content,
      'updated_at' => $sc_settings_search_map->updated_at
    ];
  }

  public function get_map_dashboard_data() {
    if( empty( $this->_map ) )
      $this->_setup_map();

    $response = [];

    foreach( $this->_map as $map_key => $map_data ) {
      $map_section_data = [];

      foreach( $map_data[ 'sections' ] as $current_section_key => $current_section_data ) {
        unset( $current_section_data[ 'field_map' ] );

        $map_section_data[ $current_section_key ] = $current_section_data;
      }

      unset( $map_data[ 'sections'] );

      $response[ $map_key ] = $map_data;

      $response[ $map_key ][ 'sections' ] = $map_section_data;
    }

    return $response;
  }

  public function get_map_field_settings( $key ) {
    if( empty( $this->_map ) )
      $this->_setup_map();

    return ( isset( $this->_map[ $key ][ 'field_settings' ] ) ? $this->_map[ $key ][ 'field_settings' ] : [] );
  }

  public function get_map_for_management( $key, $inner_key ) {
    if( empty( $this->_map ) )
      $this->_setup_map();

    return ( isset( $this->_map[ $key ][ 'sections' ][ $inner_key ] ) ? $this->_map[ $key ][ 'sections' ][ $inner_key ] : false );
  }

  public function get_map_item_count() {
    if( empty( $this->_map ) )
      $this->_setup_map();

    return count( $this->_map );
  }

  public function has_map_key( $key ) {
    if( empty( $this->_map ) )
      $this->_setup_map();

    return isset( $this->_map[ $key ] );
  }

  public function has_map_inner_key( $top_level_key, $inner_key ) {
    if( empty( $this->_map ) )
      $this->_setup_map();

    return isset( $this->_map[ $top_level_key ][ 'sections' ][ $inner_key ] );
  }

  public function get_map_first_inner_key( $top_level_key ) {
    if( empty( $this->_map ) )
      $this->_setup_map();

    return key( $this->_map[ $top_level_key ][ 'sections' ] );
  }

  private function _setup_map() {
    $index_page_library_content_type_settings = [];

    if( is_array( wpep_content_type()->libraryContentTypes ) && !empty( wpep_content_type()->libraryContentTypes ) ) {
      foreach( wpep_content_type()->libraryContentTypes as $post_type => $contentType ) {
        $field_map = $contentType->get_index_page_settings_field_map();

        if( empty( $field_map ) )
          continue;

        $index_page_library_content_type_settings[ $contentType->get_post_type() ] = [
          'name'      => sprintf( __( '%s Options', "wpep" ), $contentType->get_singular_name() ),
          'alias'     => $contentType->get_post_type(),
          'field_map' => $field_map
        ];
      }
    }

    $this->_map = [
      'general' =>  [
        'name'      => __( "General", "wpep" ),
        'alias'     => 'general',
        'sections'  => [
          'design'      => $this->_get_general_design_options(),
          'content'     => $this->_get_general_content_options(),
          'auth'        => $this->_get_general_auth_options(),
          'auth-emails' => $this->_get_general_auth_emails(),
          'advanced'    => $this->_get_general_advanced_options(),
          'url'         => $this->_get_general_url_settings(),
          'search'      => $this->_get_general_search_settings(),
          'api'         => $this->_get_general_api_settings()
        ]
      ],
      'index_page' =>  [
        'name'      => __( "Index Page", "wpep" ),
        'alias'     => 'index_page',
        'sections'  => [
          'general' => $this->_get_index_page_general_options()
        ] + $index_page_library_content_type_settings + [
          'grid'  => $this->_get_index_page_grid_options()
          ]
      ],
      'learner_profile' =>  [
        'name'      => __( "Learner Profile", "wpep" ),
        'alias'     => 'learner_profile',
        'sections'  => [
          'general'            => $this->_get_learner_profile_general_options(),
          'tab_management'     => $this->_get_learner_profile_tab_management(),
          'public'             => $this->_get_learner_profile_public_profiles(),
          'social-networks'    => $this->_get_learner_profile_social_networks(),
          'student-supervisor' => $this->_get_learner_profile_student_supervisor()
        ]
      ],
      'learner_engagement' => [
        'name'      => __( 'Learner Engagement', "wpep" ),
        'alias'     => 'learner_engagement',
        'sections'  => [
          'data_privacy'      => $this->_get_learner_engagement_data_privacy(),
          'delivery_settings' => $this->_get_learner_engagement_delivery_settings()
        ] + (
          wpep_controller()->emailMarketing->has_text_based_tag_integrations() ? [
            'onboarding' => $this->_get_learner_engagement_onboarding()
          ] : []
        ) + [
          'weekly_recap'     => $this->_get_learner_engagement_weekly_recap(),
          'discussions'      => $this->_get_learner_engagement_discussions(),
          'auto-login-links' => $this->_get_learner_engagement_auto_login_links()
        ] + (
          wpep_has_embed_support() ? [
            'share_embed' => $this->_get_learner_engagement_share_embed()
          ] : []
          )
      ]
    ];

    $this->_map = apply_filters( 'wpep_administration_settings_map', $this->_map );
  }

  private function _get_general_design_options() {
    return [
      'name'             => __( 'Design Options', "wpep" ),
      'alias'            => 'design',
      'has_live_preview' => true,
      'field_map'        => [
        'general' => [
          'name'      => __( "General Options", "wpep" ),
          'has_title' => false,
          'alias'     => 'general',
          'fields'    => [
            'wpep-theme'  => [
              'name'    =>  __( 'Theme', "wpep" ),
              'alias'   => 'wpep-theme',
              'type'    => 'select',
              'options' => wpep_utility_map_to_array_assoc( wpep_controller()->templateStyle->get_theme_list(), 'alias', 'name' )
            ],
            'wpep-custom-font-size'  => [
              'name'    =>  __( 'Base Font Size', "wpep" ),
              'alias'   => 'wpep-custom-font-size',
              'type'    => 'select',
              'options' => function() {
                $default_font_size   = ( isset( wpep_controller()->templateStyle->get_current_theme()['defaults']['font_size'] ) ? wpep_controller()->templateStyle->get_current_theme()['defaults']['font_size'] . 'px' : ' Error ' );

                return [
                    ''   => 'Default - ' . $default_font_size,
                  ] + wpep_get_indexed_number_list(10, 20, __( "%value%px", "wpep" ) );
              }
            ],
            'wpep-custom-font-family'  => [
              'name'    =>  __( 'Font Family', "wpep" ),
              'alias'   => 'wpep-custom-font-family',
              'type'    => 'select',
              'class'   => 'wpep-select-field',
              'options' => function() {
                $default_font_family = ( isset( wpep_controller()->templateStyle->get_current_theme()['defaults']['font_family'] ) ? wpep_controller()->templateStyle->get_current_theme()['defaults']['font_family']  : ' Error ' );

                $google_font_list = wpep_get_google_font_list();
                $native_font_list = wpep_get_native_font_list();

                return [
                  ''              => 'Default - ' . $default_font_family,
                  'inherit'       => 'Inherit Theme Fonts',
                  'native_fonts'  => [
                    'group_title'   => __( "Native Fonts", "wpep" ),
                    'group_options' => array_combine( $native_font_list, $native_font_list )
                  ],
                  'google_fonts'  => [
                    'group_title'   => __( "Google Fonts", "wpep" ),
                    'group_options' => array_combine( $google_font_list, $google_font_list )
                  ]
                ];
              },
              'attributes'  => [
                'data-ec-library' => "select2"
              ]
            ],
            'wpep-custom-color' => [
              'name'    =>  __( 'Primary Color', "wpep" ),
              'alias'   => 'wpep-custom-color',
              'type'    => 'color'
            ],
            'wpep-custom-color-secondary' => [
              'name'        =>  __( 'Secondary Color', "wpep" ),
              'alias'       => 'wpep-custom-color-secondary',
              'type'        => 'color',
              'description' => __( 'This should be a darker or lighter color similar to the primary one', "wpep" )
            ],
          ]
        ],
        'wrapper' => [
          'name'      => __( "Wrapper Options", "wpep" ),
          'has_title' => false,
          'fields'    => [
            'wrapper_options' => [
              'type'      => 'wrapper_options'
            ]
          ]
        ],
        'other_options' => [
          'name'      => __( "Other Options", "wpep" ),
          'fields'    => [
            'wpep-placeholder-image'  => [
              'name'         => __( "Default Image", "wpep" ),
              'alias'        => 'wpep-placeholder-image',
              'type'         => 'image',
              'description'  => sprintf( __( "The default image used within %s where one is required, this includes the default Course Image.", "wpep" ), WPEP_PLUGIN_NAME )
            ]
          ]
        ],
      ]
    ];
  }

  private function _get_general_content_options() {
    $response = [
      'name'      => __( 'Content Options', "wpep" ),
      'alias'     => 'content',
      'field_map' => [
        'content_types' => [
          'name'    => __( "Content Types", "wpep" ),
          'alias'   => 'content_types',
          'fields'  => []
        ]
      ]
    ];

    foreach( wpep_content_type()->switchableContentTypes as $contentType )
      $response[ 'field_map' ][ 'content_types' ][ 'fields' ][ 'content-type-' . $contentType->get_post_type() . '-enabled' ] = [
        'name'  => $contentType->get_plural_name(),
        'alias' => 'content-type-' . $contentType->get_post_type() . '-enabled',
        'type'  => 'checkbox',
      ];

    $response[ 'field_map' ] += [
      'content_modules' => [
        'name'    => __( "Content Modules", "wpep" ),
        'alias'   => 'content_modules',
        'fields'  => [
          'enable-post-author-field'  => [
            'name'        =>  __( 'Enable Author Field', "wpep" ),
            'alias'       => 'enable-post-author-field',
            'type'        => 'checkbox',
            'description' => __( "Practical when you use multiple Individual Content Managers.", "wpep" )
          ],
          'post-list-type-by-individual-author' => [
            'name'    =>  __( 'Post List For Individual Authors', "wpep" ),
            'alias'   => 'post-list-type-by-individual-author',
            'type'    => 'select',
            'options' => [
              'all'   => __( "See all content created.", "wpep" ),
              'mine'  => __( "See content created by them.", "wpep" )
            ]
          ],
          'register-default-categories'  => [
            'name'        =>  __( 'Enable Standard Categories', "wpep" ),
            'alias'       => 'register-default-categories',
            'type'        => 'checkbox',
            'description' => __( "Practical when your membership plugin cannot protect Custom Taxonomies, and you want to create Course Bundles trough them.", "wpep" )
          ],
        ]
      ],
      'member_magnet' => [
        'name'    => __( "Member Magnet", "wpep" ),
        'alias'   => 'member_magnet',
        'fields'  => [
          'enable-post-member-magnet-option'  => [
            'name'        =>  __( 'Enable', "wpep" ),
            'alias'       => 'enable-post-member-magnet-option',
            'type'        => 'checkbox'
          ],
          'enable-post-member-magnet-interaction' => [
            'name'        =>  __( 'Allow Preview', "wpep"),
            'alias'       => 'enable-post-member-magnet-interaction',
            'type'        => 'checkbox',
            'description' => __( "For Execution Plan, Execution Plan Unfolded and Learn Map Course Types Enable Registration or Login upon Content Tab button click action.", "wpep" )
          ],
          'post-member-magnet-type'  => [
            'name'    =>  __( 'Type', "wpep" ),
            'alias'   => 'post-member-magnet-type',
            'type'    => 'select',
            'options' => [
              WPEP_MEMBER_MAGNET_TYPE_STANDARD => __( "Standard", "wpep" ),
              WPEP_MEMBER_MAGNET_TYPE_QUICK    => __( "Quick, requests Username / Email Address / Password", "wpep" )
            ]
          ],
        ]
      ],
      'categories_options' => [
        'name'    => __( "Categories Options", "wpep" ),
        'alias'   => 'categories_options',
        'fields'  => [
          'index-page-category-presentation-labels'  => [
            'name'        =>  __( 'Presentation Card Labels', "wpep"),
            'alias'       => 'index-page-category-presentation-labels',
            'type'        => 'checkbox',
            'description' => __( "Enabling this option will allow you to choose the status of a Category to Show as Tag, apart from all the standard functionality, it will also display a tag card on the Presentation Card.", "wpep" )
          ]
        ]
      ]
    ];

    return $response;
  }

  private function _get_general_auth_options() {
    return [
      'name'             => __( 'Auth Options', "wpep" ),
      'alias'            => 'auth',
      'field_map'        => [
        'general' => [
          'name'      => __( "General Settings", "wpep" ),
          'has_title' => true,
          'alias'     => 'general',
          'fields'    => [
            'user-auth-status-level'  => [
              'name'        =>  __( 'Internal Auth Status', "wpep"),
              'alias'       => 'user-auth-status-level',
              'type'        => 'select',
              'options'     => wpep_get_auth_status_level_options(),
              'description' => sprintf( __( 'By default %s does not enforce users to login. Your WP Membership plugin will usually take care of this..', "wpep" ), WPEP_PLUGIN_NAME )
            ],
            'user-auth-enable-registration'  => [
              'name'        =>  __( 'Registration Status', "wpep"),
              'alias'       => 'user-auth-enable-registration',
              'type'        => 'select',
              'options'     => [
                0 => __( 'Inactive', "wpep" ),
                1 => __( 'Active', "wpep" ),
                2 => __( "Active - Display Registration instead of login", "wpep" )
              ]
            ],
            'user-auth-sidebar-marketing-status'  => [
              'name'        =>  __( 'Enable Marketing Content', "wpep"),
              'alias'       => 'user-auth-sidebar-marketing-status',
              'type'        => 'select',
              'options'     => [
                0 => __( "Disabled", "wpep" ),
                1 => __( "Enabled", "wpep" ),
                2 => __( "Enabled, hidden on register", "wpep" )
              ],
              'description' => sprintf( __( "Enable a Marketing text on the right side of the Authentication Login Page. This applies only for the authentication page that %s creates for you..", "wpep" ), WPEP_PLUGIN_NAME )
            ],
            'user-auth-sidebar-marketing-content'  => [
              'name'        =>  __( "Marketing Content", "wpep" ),
              'alias'       => 'user-auth-sidebar-marketing-content',
              'type'        => 'editor'
            ]
          ]
        ],
        'form_fields_and_actions' => [
          'name'      => __( "Form Fields & Actions", "wpep" ),
          'has_title' => true,
          'alias'     => 'form_fields_and_actions',
          'fields'    => [
            'user-auth-field-login-identifier'  => [
              'name'        =>  __( "Login Identifier", "wpep" ),
              'alias'       => 'user-auth-field-login-identifier',
              'type'        => 'text_translated',
              'description' => __( "Username or Email Address", "wpep" )
            ],
            'user-auth-field-username'  => [
              'name'        =>  __( "Username", "wpep" ),
              'alias'       => 'user-auth-field-username',
              'type'        => 'text_translated'
            ],
            'user-auth-field-email-address'  => [
              'name'        =>  __( "Email Address", "wpep" ),
              'alias'       => 'user-auth-field-email-address',
              'type'        => 'text_translated'
            ],
            'user-auth-field-first-name'  => [
              'name'        =>  __( "First Name", "wpep" ),
              'alias'       => 'user-auth-field-first-name',
              'type'        => 'text_translated'
            ],
            'user-auth-field-last-name'  => [
              'name'        =>  __( "Last Name", "wpep" ),
              'alias'       => 'user-auth-field-last-name',
              'type'        => 'text_translated'
            ],
            'user-auth-field-password'  => [
              'name'        =>  __( "Password", "wpep" ),
              'alias'       => 'user-auth-field-password',
              'type'        => 'text_translated'
            ],
            'user-auth-field-password-confirm'  => [
              'name'        =>  __( "Confirm Password", "wpep" ),
              'alias'       => 'user-auth-field-password-confirm',
              'type'        => 'text_translated'
            ],
            'user-auth-field-password-old'  => [
              'name'        =>  __( "Existent Password", "wpep" ),
              'alias'       => 'user-auth-field-password-old',
              'type'        => 'text_translated'
            ],
            'user-auth-field-remember-me'  => [
              'name'        =>  __( "Remember me", "wpep" ),
              'alias'       => 'user-auth-field-remember-me',
              'type'        => 'text_translated'
            ],
            'user-auth-open-tab-register'  => [
              'name'        =>  __( "Register Account", "wpep" ),
              'alias'       => 'user-auth-open-tab-register',
              'type'        => 'text_translated'
            ],
            'user-auth-open-tab-reset-password'  => [
              'name'        =>  __( "Reset Password", "wpep" ),
              'alias'       => 'user-auth-open-tab-reset-password',
              'type'        => 'text_translated'
            ],
            'user-auth-open-tab-login'  => [
              'name'        =>  __( "I already have an Account", "wpep" ),
              'alias'       => 'user-auth-open-tab-login',
              'type'        => 'text_translated'
            ],
            'user-auth-return-to-login'  => [
              'name'        =>  __( "Return to Login", "wpep" ),
              'alias'       => 'user-auth-return-to-login',
              'type'        => 'text_translated'
            ],
            'user-auth-operation-login'  => [
              'name'        =>  __( "Login", "wpep" ),
              'alias'       => 'user-auth-operation-login',
              'type'        => 'text_translated'
            ],
            'user-auth-operation-register'  => [
              'name'        =>  __( "Register", "wpep" ),
              'alias'       => 'user-auth-operation-register',
              'type'        => 'text_translated'
            ],
            'user-auth-operation-reset-password'  => [
              'name'        =>  __( "Reset Password", "wpep" ),
              'alias'       => 'user-auth-operation-reset-password',
              'type'        => 'text_translated'
            ],
            'user-auth-operation-express' => [
              'name'        =>  __( "Express Login", "wpep" ),
              'alias'       => 'user-auth-operation-express',
              'type'        => 'text_translated'
            ]
          ]
        ],
        'other_fields' => [
          'name'      => __( "Other Fields", "wpep" ),
          'has_title' => true,
          'alias'     => 'other_fields',
          'fields'    => [
            'user-auth-field-sid'  => [
              'name'        =>  __( "SID", "wpep" ),
              'alias'       => 'user-auth-field-sid',
              'type'        => 'text_translated',
              'description' => __( "Used when the Student Identification Option is enabled.", "wpep" )
            ],
            'user-auth-field-location'  => [
              'name'        =>  __( "Location", "wpep" ),
              'alias'       => 'user-auth-field-location',
              'type'        => 'text_translated'
            ],
            'user-auth-field-website'  => [
              'name'        =>  __( "Website", "wpep" ),
              'alias'       => 'user-auth-field-website',
              'type'        => 'text_translated'
            ],
          ]
        ],
        'form_notifications' => [
          'name'      => __( "Form Notifications", "wpep" ),
          'has_title' => true,
          'alias'     => 'form_notifications',
          'fields'    => [
            'user-auth-notification-please-register'  => [
              'name'        =>  __( "Registration Notice", "wpep" ),
              'alias'       => 'user-auth-notification-please-register',
              'type'        => 'text_translated',
              'description' => __( "Displays this message when the form defaults to register.", "wpep" )
            ],
            'user-auth-notification-login-details-required'  => [
              'name'        =>  __( "Login Details Required", "wpep" ),
              'alias'       => 'user-auth-notification-login-details-required',
              'type'        => 'text_translated'
            ],
            'user-auth-notification-password-incorrect'  => [
              'name'        =>  __( "Incorrect Password", "wpep" ),
              'alias'       => 'user-auth-notification-password-incorrect',
              'type'        => 'text_translated'
            ],
            'user-auth-notification-logged-in'  => [
              'name'        =>  __( "Successfully Logged In", "wpep" ),
              'alias'       => 'user-auth-notification-logged-in',
              'type'        => 'text_translated'
            ],
            'user-auth-notification-check-email' => [
              'name'        =>  __( "Token Check Email", "wpep" ),
              'alias'       => 'user-auth-notification-check-email',
              'type'        => 'text_translated'
            ],
            'user-auth-notification-account-registered'  => [
              'name'        =>  __( "Account Registered", "wpep" ),
              'alias'       => 'user-auth-notification-account-registered',
              'type'        => 'text_translated'
            ],
            'user-auth-notification-password-reset'  => [
              'name'        =>  __( "Password Reset", "wpep" ),
              'alias'       => 'user-auth-notification-password-reset',
              'type'        => 'text_translated'
            ],
            'user-auth-notification-recovery-not-registered'  => [
              'name'        =>  __( "Password Reset Not Registered", "wpep" ),
              'alias'       => 'user-auth-notification-recovery-not-registered',
              'type'        => 'text_translated'
            ],
            'user-auth-notification-already-logged-in'  => [
              'name'        =>  __( "Already Logged In", "wpep" ),
              'alias'       => 'user-auth-notification-already-logged-in',
              'type'        => 'text_translated'
            ],
            'user-auth-notification-page-expired'  => [
              'name'        =>  __( "Notification Page Expired", "wpep" ),
              'alias'       => 'user-auth-notification-page-expired',
              'type'        => 'text_translated'
            ],
          ]
        ],
        'other_options' => [
          'name'      => __( "Other Options", "wpep" ),
          'has_title' => true,
          'alias'     => 'other_options',
          'fields'    => [
            'user-auth-form-instructions'  => [
              'name'        =>  __( "Form Instructions", "wpep" ),
              'alias'       => 'user-auth-form-instructions',
              'type'        => 'editor',
              'description' => __( "Optionally display instructions at the top of the login & registration form.", "wpep" )
            ]
          ]
        ],
      ]
    ];
  }

  private function _get_general_auth_emails() {
    return [
      'name'      => __( 'Auth Emails', "wpep" ),
      'alias'     => 'auth-emails',
      'field_map' => [
        'password_reset_email' => [
          'name'      => __( "Password Reset Email", "wpep" ),
          'has_title' => true,
          'alias'     => 'password_reset_email',
          'fields'    => [
            'email-reset-password-override-filter'  => [
              'name'        =>  __( "Enable Globally", "wpep" ),
              'alias'       => 'email-reset-password-override-filter',
              'type'        => 'checkbox',
              'description' => sprintf( __( "By default only the %s Authentication Form Password reset will use this email content.", "wpep" ), WPEP_PLUGIN_NAME )
            ],
            'email-reset-password-use-template'  => [
              'name'        =>  __( "Use Delivery Template", "wpep" ),
              'alias'       => 'email-reset-password-use-template',
              'type'        => 'checkbox',
              'description' => sprintf( __( "Enable the %s Email Template when sending the password reset email notification - this applies for the global setting above, reset emails from the authentication form will always use the template.", "wpep" ), WPEP_PLUGIN_NAME )
            ],
            'email-reset-password-title'  => [
              'name'        =>  __( "Email Title", "wpep" ),
              'alias'       => 'email-reset-password-title',
              'type'        => 'text_translated',
              'description' => __( "%s will be replaced with the website name, using it is strictly optional, and is there to make the email personalized by default", "wpep" )
            ],
            'email-reset-password-content'  => [
              'name'        =>  __( "Email Content", "wpep" ),
              'alias'       => 'email-reset-password-content',
              'type'        => 'editor_translated',
              'description' => '<h2>' . __( "Available ShortCodes", "wpep" ) . '</h2>' .
                               '<ul>' .
                               '<li>[site_url] - ' . __( "The Site Link", "wpep" ) . '</li>' .
                               '<li>[username] - ' . __( "The username of the student that has requested a password reset", "wpep" ) . '</li>' .
                               '<li>[password_reset] - ' . __( "The password reset link", "wpep" ) . '</li>' .
                               '</ul>'
            ]
          ]
        ],
        'express_authentication_login_email' => [
          'name'      => __( "Express Authentication - Login", "wpep" ),
          'has_title' => true,
          'alias'     => 'express_authentication_login_email',
          'fields'    => [
            'email-express-authentication-login-title'  => [
              'name'        =>  __( "Email Title", "wpep" ),
              'alias'       => 'email-express-authentication-login-title',
              'type'        => 'text_translated',
              'description' => __( "%s will be replaced with the website name, using it is strictly optional, and is there to make the email personalized by default", "wpep" )
            ],
            'email-express-authentication-login-content'  => [
              'name'        =>  __( "Email Content", "wpep" ),
              'alias'       => 'email-express-authentication-login-content',
              'type'        => 'editor_translated',
              'description' => '<h2>' . __( "Available ShortCodes", "wpep" ) . '</h2>' .
                '<ul>' .
                '<li>[site_url] - ' . __( "The Site Link", "wpep" ) . '</li>' .
                '<li>[email_address] - ' . __( "The email address to which this email was delivered", "wpep" ) . '</li>' .
                '<li>[login_link] - ' . __( "The account login link", "wpep" ) . '</li>' .
                '</ul>'
            ],
          ]
        ],
        'express_authentication_register_email' => [
          'name'      => __( "Express Authentication - Register", "wpep" ),
          'has_title' => true,
          'alias'     => 'express_authentication_register_email',
          'fields'    => [
            'email-express-authentication-register-title'  => [
              'name'        =>  __( "Email Title", "wpep" ),
              'alias'       => 'email-express-authentication-register-title',
              'type'        => 'text_translated',
              'description' => __( "%s will be replaced with the website name, using it is strictly optional, and is there to make the email personalized by default", "wpep" )
            ],
            'email-express-authentication-register-content'  => [
              'name'        =>  __( "Email Content", "wpep" ),
              'alias'       => 'email-express-authentication-register-content',
              'type'        => 'editor_translated',
              'description' => '<h2>' . __( "Available ShortCodes", "wpep" ) . '</h2>' .
                '<ul>' .
                '<li>[site_url] - ' . __( "The Site Link", "wpep" ) . '</li>' .
                '<li>[email_address] - ' . __( "The email address to which this email was delivered", "wpep" ) . '</li>' .
                '<li>[password] - ' . __( "The email address to which this email was delivered", "wpep" ) . '</li>' .
                '<li>[register_link] - ' . __( "The account register link", "wpep" ) . '</li>' .
                '</ul>'
            ],
          ]
        ]
      ]
    ];
  }

  private function _get_general_advanced_options() {
    return [
      'name'      => __( 'Advanced Options', "wpep" ),
      'alias'     => 'advanced',
      'field_map' => [
        'frontend'  => [
          'name'   => __( "Frontend", "wpep" ),
          'alias'  => 'frontend',
          'fields' => [
            'disable-admin-bar-for-students' => [
              'name'        => __( 'Disable Admin Bar', "wpep" ),
              'alias'       => 'disable-admin-bar-for-students',
              'type'        => 'checkbox',
              'description' => __( "Disable the admin bar on the frontend for non-admin users, Administrators & Platform Managers will continue to see it.", "wpep" )
            ],
            'disable-xmlrpc' => [
              'name'        => __( 'Disable XMLRpc', "wpep" ),
              'alias'       => 'disable-xmlrpc',
              'type'        => 'checkbox',
              'description' => sprintf( __( "%s can disable the xmlrpc functionality from WordPress, this may reduce server load dramatically as it can stop common brute-force attacks.", "wpep" ), WPEP_PLUGIN_NAME )
            ],
            'wpep-custom-css-by-file' => [
              'name'        => __( 'Custom CSS File', "wpep" ),
              'alias'       => 'wpep-custom-css-by-file',
              'type'        => 'checkbox',
              'description' => sprintf( __( "%s will generate a Custom CSS file to deliver the Color Variation & Custom CSS, disable this option to embed them directly on page if you encounter CSS problems.", "wpep" ), WPEP_PLUGIN_NAME )
            ]
          ]
        ],
        'administration'  => [
          'name'   => __( "Administration", "wpep" ),
          'alias'  => 'administration',
          'fields' => [
            'menu-position' => [
              'name'        => __( 'Menu Position', "wpep" ),
              'alias'       => 'menu-position',
              'type'        => 'select',
              'options'     => [
                3 => "Default, at the top",
                2 => "Under Dashboard",
                4 => "Under Dashboard Separator",
                5 => "Under Posts",
                10 => "Under Media",
                15 => "Under Links",
                20 => "Under Pages",
                25 => "Under Comments",
                59 => "Under Separator",
                60 => "Under Appearance",
                65 => "Under Plugins",
                70 => "Under Users",
                75 => "Under Tools",
                80 => "Under Settings",
                99 => "Under Bottom Separator",
              ]
            ],
            'admin-editor-integration' => [
              'name'        => __('Editor Integration', "wpep"),
              'alias'       => 'admin-editor-integration',
              'type'        => 'checkbox',
              'description' => sprintf( __('Check this to enable the %s editor improvements and shortcode builder.', "wpep"), WPEP_PLUGIN_NAME )
            ],
            'wpep-editor-setup' => [
              'name'        => __('Text Editor', "wpep"),
              'alias'       => 'wpep-editor-setup',
              'type'        => 'select',
              'options'     => [
                'default'  => 'Default WP Editor - Visual',
                'visual'   => 'Visual WP Editor',
                'html'     => 'Default WP Editor - HTML',
                'textarea' => 'Textarea Only'
              ],
              'description' => sprintf( __( 'Chose the editor that will be displayed everywhere within %s.', "wpep"), WPEP_PLUGIN_NAME )
            ]
          ]
        ],
        'system_settings'  => [
          'name'   => __( "System Settings", "wpep" ),
          'alias'  => 'system_settings',
          'fields' => [
            'data-system-daily-maintenance' => [
              'name'        => __( "Background Maintenance", "wpep" ),
              'alias'       => 'data-system-daily-maintenance',
              'type'        => 'multiple_select',
              'options'     => [
                'danger'  => __( "Critical", "wpep" ),
                'warning' => __( "Warnings", "wpep" )
              ],
              'description' => sprintf( __( "%s maintenance operations will run in the background, once per day.", "wpep" ), WPEP_PLUGIN_NAME )
            ],
            'data-system-api-tracking' => [
              'name'        => __( "API Tracking", "wpep" ),
              'alias'       => 'data-system-api-tracking',
              'type'        => 'checkbox',
              'description' => sprintf( __( "API Requests will be tracked for debugging purposes, enabling this option will degrade general performance while tracking %s API / Ajax Requests.", "wpep" ), WPEP_PLUGIN_NAME )
            ],
            'data-system-settings-tracking' => [
              'name'        => __( "Settings Tracking", "wpep" ),
              'alias'       => 'data-system-settings-tracking',
              'type'        => 'checkbox',
              'description' => sprintf( __( "%s Setting changes will be tracked, making debugging & replicating steps easily..", "wpep" ), WPEP_PLUGIN_NAME )
            ],
          ]
        ],
        'platform_backup' => [
          'name'   => __( "Platform Backup", "wpep" ),
          'alias'  => 'platform_backup',
          'fields' => [
            'data-system-enable-background-backups' => [
              'name'        => __( "Background Backups", "wpep" ),
              'alias'       => 'data-system-enable-background-backups',
              'type'        => 'checkbox',
              'description' => sprintf( __( "We'll create daily backups of all your content created in the %s Platform.", "wpep" ), WPEP_PLUGIN_NAME )
            ],
            'data-system-backups-cap' => [
              'name'        => __( "Export Cleanup", "wpep" ),
              'alias'       => 'data-system-backups-cap',
              'type'        => 'numeric',
              'description' => __( "An backup expires after the number of days chosen above, the most recent backup will be kept, even if it's expired.", "wpep" )
            ],
          ]
        ],
        'utilities'  => [
          'name'   => __( "Utilities", "wpep" ),
          'alias'  => 'utilities',
          'fields' => [
            'admin-library-conflict-select-2-status' => [
              'name'        => __( 'Prevent Select2 Conflict', "wpep" ),
              'alias'       => 'admin-library-conflict-select-2-status',
              'type'        => 'checkbox',
              'description' => sprintf( __( "Disables 3-rd party loaded select2 libraries from the %s administration pages.", "wpep" ), WPEP_PLUGIN_NAME )
            ],
            'admin-library-datetimepicker-simple-format' => [
              'name'        => __( 'Simple DateTimePicker', "wpep" ),
              'alias'       => 'admin-library-datetimepicker-simple-format',
              'type'        => 'checkbox',
              'description' =>'<p>' . sprintf( __( "Will format the DateTimePicker fields to the standard %s format instead of the WordPress Setting.", "wpep" ), '<strong>Y-m-d</strong>' ) . '</p>'
            ],
            'reporting-start-date'  => [
              'name'        => __( 'Reporting Start Date', "wpep" ),
              'alias'       => 'reporting-start-date',
              'type'        => 'date',
              'description' => __( "Used for reporting data maps, if empty automatically calculated to the first registered user date.", "wpep" )
            ]
          ]
        ],
        'backward_compatibility'  => [
          'name'   => __( "Backward Compatibility", "wpep" ),
          'alias'  => 'backward_compatibility',
          'fields' => [
            'enable-backward-compatibility-short-codes' => [
              'name'        => __( 'ShortCodes', "wpep" ),
              'alias'       => 'enable-backward-compatibility-short-codes',
              'type'        => 'checkbox',
              'description' => sprintf(
                __( '%s shortcodes start with %s, before they have started with wpep_, check this option to have the old shortcodes continue working.', "wpep" ),
                WPEP_PLUGIN_NAME,
                WPEP_SHORT_CODE_PREFIX
              )
            ],
          ]
        ]
      ]
    ];
  }

  private function _get_general_url_settings() {
    $response = [
      'name'      => __( 'URL Settings', "wpep" ),
      'alias'     => 'url',
      'field_map' => [
        'permalink_name'  => [
          'name'   => __( "Permalink Name", "wpep" ),
          'alias'  => 'permalink_name',
          'fields' => []
        ],
        'archive_page'  => [
          'name'   => __( "Archive Page", "wpep" ),
          'alias'  => 'archive_page',
          'fields' => []
        ]
      ]
    ];

    foreach( wpep_content_type()->libraryContentTypes as $libraryContentType ) {
      if( !$libraryContentType->is_publicly_queryable() )
        continue;

      $response[ 'field_map' ][ 'permalink_name' ][ 'fields' ][ 'content-type-' . $libraryContentType->get_post_type() . '-rewrite-slug' ] = [
        'name'        => $libraryContentType->get_plural_name(),
        'alias'       => 'content-type-' . $libraryContentType->get_post_type() . '-rewrite-slug',
        'type'        => 'text',
        'description' => sprintf( __( "The permalink prefix goes here, by default %s", "wpep" ), $libraryContentType->get_post_type() )
      ];

      $response[ 'field_map' ][ 'archive_page' ][ 'fields' ][ 'content-type-' . $libraryContentType->get_post_type() . '-enable-archive' ] = [
        'name'        => $libraryContentType->get_plural_name(),
        'alias'       => 'content-type-' . $libraryContentType->get_post_type() . '-enable-archive',
        'type'        => 'checkbox',
        'description' => sprintf( __( "By default WordPress will create the %s Archive Page under /%s/ you can disable this here", "wpep" ), $libraryContentType->get_plural_name(), $libraryContentType->get_post_type() )
      ];
    }

    $response[ 'field_map' ][ 'permalink_name' ][ 'fields' ][ 'rewrite-slug-categories' ] = [
      'name'        => __( 'Categories', "wpep" ),
      'alias'       => 'rewrite-slug-categories',
      'type'        => 'text',
      'description' => __( "The permalink prefix goes here, by default course_category", "wpep" )
    ];


    return $response;
  }

  private function _get_general_search_settings() {
    return [
      'name'      => __( 'Search Settings', "wpep" ),
      'alias'     => 'search',
      'field_map' => [
        'general' => [
          'name'      => __( "General Settings", "wpep" ),
          'has_title' => false,
          'alias'     => 'general',
          'fields'    => [
            'search-image-ribbon'  => [
              'name'        =>  __( "Image Ribbon", "wpep" ),
              'alias'       => 'search-image-ribbon',
              'type'        => 'checkbox',
              'description' => __( "Enable this option to display the presentation card ribbon text on the image.", "wpep" )
            ],
            'search-results-number'  => [
              'name'        =>  __( "Max Results", "wpep" ),
              'alias'       => 'search-results-number',
              'type'        => 'text',
              'description' => __( "The maximum number of results displayed when a search is made.", "wpep" )
            ],
            'search-no-results-found'  => [
              'name'        =>  __( "No Results Found", "wpep" ),
              'alias'       => 'search-no-results-found',
              'type'        => 'text_translated'
            ],
            'search-input-placeholder'  => [
              'name'        =>  __( "Search Placeholder", "wpep" ),
              'alias'       => 'search-input-placeholder',
              'type'        => 'text_translated'
            ],
          ]
        ]
      ]
    ];
  }

  private function _get_general_api_settings() {
    return [
      'name'      => __( 'API Settings', "wpep" ),
      'alias'     => 'api',
      'field_map' => [
        'api-response-scu-crd' => [
          'name'      => __( "User Statistics", "wpep" ),
          'has_title' => true,
          'alias'     => 'api-response-scu-crd',
          'fields'    => function() {
            $api_response = WPEP_API::instance()->get_request_response_by_key( 'system/statistics-current-user/configuration-report', [
              '_ignore_crd_sources' => 1
            ] );
            //
            if( !isset( $api_response[ 'report_data_sources' ] ) || empty( $api_response[ 'report_data_sources' ] ) )
              return [];

            $response = [
              'api-response-scu-crd-status' => [
                'name'        => __( "Enable Filter", "wpep" ),
                'alias'       => 'api-response-scu-crd-status',
                'type'        => 'checkbox',
                'description' => __( "Only the options selected bellow will be available in the learner statistics, this includes the Student Supervisor View & Sub-Account Manager view, at least one-view should be available.", "wpep" ),
              ]
            ];

            foreach( $api_response[ 'report_data_sources' ] as $report_data_source_key => $report_data_source ) {

              if( $report_data_source[ 'type' ] === 'group' && is_array( $report_data_source[ 'group' ] ) ) {
                $options = [];

                foreach( $report_data_source[ 'group' ] as $g_key => $g_item )
                  $options[ $g_key ] = $g_item[ 'title' ];

                $response[ 'api-response-scu-crd-sources[' . $report_data_source_key . ']' ] = [
                  'name'        =>  $report_data_source[ 'title' ],
                  'alias'       => 'api-response-scu-crd-sources[' . $report_data_source_key . ']',
                  'type'        => 'multiple_select',
                  'options'     => $options,
                  'description' => __( "Select the available statistics in this folder.", "wpep" )
                ];

                continue;
              }

              $response[ 'api-response-scu-crd-sources[' . $report_data_source_key . ']' ] = [
                'name'        => $report_data_source[ 'title' ],
                'alias'       => 'api-response-scu-crd-sources[' . $report_data_source_key . ']',
                'type'        => 'checkbox'
              ];
            }

            return $response;
          }
        ]
      ]
    ];
  }

  private function _get_index_page_general_options() {
    return [
      'name'      => __( 'General', "wpep" ),
      'alias'     => 'general',
      'field_map' => [
        'general' => [
          'name'      => __( "General", "wpep" ),
          'has_title' => false,
          'alias'     => 'general',
          'fields'    => [
            'opt-course-index'  => [
              'name'        =>  __( "Index Page", "wpep" ),
              'alias'       => 'opt-course-index',
              'type'        => 'select',
              'options'     => function() {
                return [ __( "Ignore This Feature", "wpep" ) ] + wpep_get_post_list_assoc( 'page', 'ID', 'post_title' );
              },
              'description' => '<p>' . sprintf(
                  __( 'You can also use %s to display the course grid on any page.', "wpep" ),
                  wpep_admin_shortcode_builder(
                    WPEP_SHORT_CODE_PREFIX . 'index',
                    [ 'wpep_admin_shortcode_builder_wpep_index_options' ],
                    '[' . WPEP_SHORT_CODE_PREFIX . 'index]',
                    false
                  )
                ) . '</p>'
            ],
            'index-page-categories-per-row'  => [
              'name'        =>  __( 'Categories Per Row', "wpep" ),
              'alias'       => 'index-page-categories-per-row',
              'type'        => 'select',
              'options'     => wpep_content_categories_row_options(),
            ],
            'opt-course-secondary-filter'  => [
              'name'        =>  __( 'Secondary Filter', "wpep" ),
              'alias'       => 'opt-course-secondary-filter',
              'type'        => 'text_translated',
              'description' => __( 'Personalize the text of the secondary category level filter.', "wpep" )
            ],
            'index-page-single-category-primary-view'  => [
              'name'        => __( "Single Category View", "wpep" ),
              'alias'       => 'index-page-single-category-primary-view',
              'type'        => 'select',
              'options'     => [
                0 => __( "Show secondary category navigation" ),
                1 => __( "Show primary category navigation" )
              ]
            ]
          ]
        ],
        'all_categories_filter' => [
          'name'      => __( "All Categories Filter", "wpep" ),
          'has_title' => true,
          'alias'     => 'all_categories_filter',
          'fields'    => [
            'index-page-enable-filter-all-categories'  => [
              'name'        =>  __( "Status", "wpep" ),
              'alias'       => 'index-page-enable-filter-all-categories',
              'type'        => 'checkbox',
              'description' => __( "Add an Category Filter tab that will show content from all categories.", "wpep" )
            ],
            'index-page-filter-all-categories-title'  => [
              'name'        =>  __( 'Title', "wpep" ),
              'alias'       => 'index-page-filter-all-categories-title',
              'type'        => 'text_translated'
            ],
            'index-page-filter-all-categories-image'  => [
              'name'        => __( "Image", "wpep" ),
              'alias'       => 'index-page-filter-all-categories-image',
              'type'        => 'image',
            ]
          ]
        ],
        'introduction_text' => [
          'name'      => __( "Introduction Text", "wpep" ),
          'has_title' => true,
          'alias'     => 'introduction_text',
          'fields'    => [
            'opt-course-heading'  => [
              'name'        =>  __( 'Main heading', "wpep" ),
              'alias'       => 'opt-course-heading',
              'type'        => 'text',
              'description' => __( 'Enter the main heading of your course index page.', "wpep" )
            ],
            'opt-course-sub-heading'  => [
              'name'        =>  __( 'Sub Heading', "wpep" ),
              'alias'       => 'opt-course-sub-heading',
              'type'        => 'editor',
              'description' => __('Enter the sub heading of your course index page.', "wpep" )
            ]
          ]
        ],
        'filtering' => [
          'name'      => __( "Filtering", "wpep" ),
          'has_title' => true,
          'alias'     => 'filtering',
          'fields'    => [
            'index-page-filter-search'  => [
              'name'        =>  __( 'Search', "wpep" ),
              'alias'       => 'index-page-filter-search',
              'type'        => 'text_translated'
            ],
            'index-page-categories-filter-require-all'  => [
              'name'        =>  __( 'Require All Categories', "wpep"),
              'alias'       => 'index-page-categories-filter-require-all',
              'type'        => 'checkbox',
              'description' => __( "When filtering by categories, the content must be placed in all categories to be displayed", "wpep" )
            ]
          ]
        ],
        'pagination' => [
          'name'      => __( "Pagination", "wpep" ),
          'has_title' => true,
          'alias'     => 'pagination',
          'fields'    => [
            'index-page-enable-pagination'  => [
              'name'        =>  __( 'Status', "wpep" ),
              'alias'       => 'index-page-enable-pagination',
              'type'        => 'select',
              'options'     => [
                0 => __( 'Inactive', "wpep" ),
                1 => __( 'Active', "wpep" )
              ],
              'description' => __( "Enabling Pagination will speed up the Index Page load time dramatically.", "wpep" ),
              'attributes'  => [
                'data-wpep-conditional-display' => 'data-wpep-ip-enable-pagination'
              ]
            ],
            'index-page-entries-per-page'  => [
              'name'        =>  __( 'Entries Per Page', "wpep"),
              'alias'       => 'index-page-entries-per-page',
              'type'        => 'text',
              'row_data_attributes' => [
                'wpep-ip-enable-pagination'  => 1
              ]
            ]
          ]
        ]
      ]
    ];
  }

  private function _get_index_page_grid_options() {
    return [
      'name'             => __( 'Grid Options', "wpep" ),
      'alias'            => 'grid',
      'allow_in_search'  => false,
      'field_map'        => [
        'general' => [
          'name'      => __( "General", "wpep" ),
          'has_title' => false,
          'alias'     => 'general',
          'fields'    => [
            'index-page-enforce-ordering'  => [
              'name'        =>  __( 'Enforce Ordering', "wpep" ),
              'alias'       => 'index-page-enforce-ordering',
              'type'        => 'checkbox',
              'description' => __('Check this to resolve ordering issues on the Index Page', "wpep" )
            ],
            'index_page_content_type_ordering' => [
              'name'    => __( "Content Ordering", "wpep" ),
              'type'    => 'content_ordering',
              'options' => function() {
                return wpep_content_type()->library_post_type_list() + [
                    WPEP_TAXONOMY_CATEGORY => __( "Categories", "wpep" )
                  ];
              },
              'tab_base_url'  => wpep_admin_url_settings( [ 'tab' => 'index_page', 'tab_inner' => 'grid' ] ),
              'tab_index'     => 'tab_inner_section',
              'field_prefix'  => [
                'post'     => 'wpep_settings_post_order',
                'taxonomy' => 'wpep_settings_taxonomy_order'
              ]
            ]
          ]
        ]
      ]
    ];
  }

  private function _get_learner_profile_general_options() {
    return [
      'name'             => __( 'General', "wpep" ),
      'alias'            => 'general',
      'field_map'        => [
        'general' => [
          'name'      => __( "General", "wpep" ),
          'has_title' => false,
          'alias'     => 'general',
          'fields'    => [
            'user-profile-courses-per-row'  => [
              'name'        =>  __( 'Courses Per Row', "wpep" ),
              'alias'       => 'user-profile-courses-per-row',
              'type'        => 'select',
              'options'     => function() {
                return wpep_content_primary_row_options( WPEP_POST_TYPE_COURSE );
              },
              'description' => __( "How many courses per row will be displayed", "wpep" )
            ],
            'user-profile-cover-picture-status'  => [
              'name'        =>  __( "Cover Picture", "wpep" ),
              'alias'       => 'user-profile-cover-picture-status',
              'type'        => 'select',
              'options'     => wpep_flag_setting_field_options(),
              'attributes' => [
                'data-wpep-conditional-display' => 'data-wpep-lp-enable-cover-picture'
              ]
            ],
            'user-profile-change-picture-button'  => [
              'name'        =>  __( "Change Picture", "wpep" ),
              'alias'       => 'user-profile-change-picture-button',
              'type'        => 'text_translated'
            ],
            'user-profile-change-picture'  => [
              'name'        =>  __('Change Profile Picture', "wpep"),
              'alias'       => 'user-profile-change-picture',
              'type'        => 'text_translated',
              'description' => __( "Displayed during the upload & file selection modal / popup", "wpep" )
            ],
            'user-profile-change-picture-cover'  => [
              'name'        =>  __('Change Cover Picture', "wpep"),
              'alias'       => 'user-profile-change-picture-cover',
              'type'        => 'text_translated',
              'description' => __( "Displayed during the upload & file selection modal / popup", "wpep" ),
              'row_data_attributes' => [
                'wpep-lp-enable-cover-picture'  => 1
              ]
            ],
            'user-profile-update-button'  => [
              'name'        =>  __( 'Update Button', "wpep" ),
              'alias'       => 'user-profile-update-button',
              'type'        => 'text_translated'
            ],
          ]
        ],
        'change_password' => [
          'name'      => __( "Change Password", "wpep" ),
          'alias'     => 'change_password',
          'fields'    => [
            'user-profile-change-password-status'  => [
              'name'        =>  __( "Status", "wpep" ),
              'alias'       => 'user-profile-change-password-status',
              'type'        => 'select',
              'options'     => wpep_flag_setting_field_options(),
              'attributes' => [
                'data-wpep-conditional-display' => 'data-wpep-lp-enable-change-password'
              ]
            ],
            'user-profile-change-password-open-popup'  => [
              'name'        =>  __( "Button Text", "wpep" ),
              'alias'       => 'user-profile-change-password-open-popup',
              'type'        => 'text_translated',
              'description' => __( "The button that will open the popup which will allow to change the password", "wpep" ),
              'row_data_attributes' => [
                'wpep-lp-enable-change-password'  => 1
              ]
            ],
            'user-profile-change-password-popup-title'  => [
              'name'        =>  __( "Popup Title", "wpep" ),
              'alias'       => 'user-profile-change-password-popup-title',
              'type'        => 'text_translated',
              'row_data_attributes' => [
                'wpep-lp-enable-change-password'  => 1
              ]
            ],
            'user-profile-change-password-popup-button'  => [
              'name'        =>  __( "Popup Button Update", "wpep" ),
              'alias'       => 'user-profile-change-password-popup-button',
              'type'        => 'text_translated',
              'row_data_attributes' => [
                'wpep-lp-enable-change-password'  => 1
              ]
            ],
            'user-profile-change-password-popup-cancel'  => [
              'name'        =>  __( "Popup Button Cancel", "wpep" ),
              'alias'       => 'user-profile-change-password-popup-cancel',
              'type'        => 'text_translated',
              'row_data_attributes' => [
                'wpep-lp-enable-change-password'  => 1
              ]
            ],
            'user-profile-change-password-notification-wrong-existent'  => [
              'name'        =>  __( "Wrong Old Password", "wpep" ),
              'alias'       => 'user-profile-change-password-notification-wrong-existent',
              'type'        => 'text_translated',
              'description' => __( "Notification displayed if the old password is wrong.", "wpep" ),
              'row_data_attributes' => [
                'wpep-lp-enable-change-password'  => 1
              ]
            ],
            'user-profile-change-password-notification-not-match'  => [
              'name'        =>  __( "Passwords do not match", "wpep" ),
              'alias'       => 'user-profile-change-password-notification-not-match',
              'type'        => 'text_translated',
              'description' => __( "Notification displayed if the new password is doesn't match with new password confirmation.", "wpep" ),
              'row_data_attributes' => [
                'wpep-lp-enable-change-password'  => 1
              ]
            ],
            'user-profile-change-password-notification-success'  => [
              'name'        =>  __( "Password Updated", "wpep" ),
              'alias'       => 'user-profile-change-password-notification-success',
              'type'        => 'text_translated',
              'description' => __( "Notification displayed after successfully updating the password.", "wpep" ),
              'row_data_attributes' => [
                'wpep-lp-enable-change-password'  => 1
              ]
            ],
          ]
        ],
        'marketing_content' => [
          'name'      => __( "Marketing Content", "wpep" ),
          'alias'     => 'marketing_content',
          'fields'    => [
            'user-profile-marketing-content-status'  => [
              'name'        =>  __( "Status", "wpep" ),
              'alias'       => 'user-profile-marketing-content-status',
              'type'        => 'checkbox',
              'description' => __( "Display the Marketing Content from the Authentication Form on the left side of the Profile Page", "wpep" )
            ],
            'user-profile-marketing-content'  => [
              'name'        =>  __( "Content", "wpep" ),
              'alias'       => 'user-profile-marketing-content',
              'type'        => 'editor'
            ]
          ]
        ],
        'default_settings' => [
          'name'      => __( 'Default Settings', "wpep" ),
          'alias'     => 'default_settings',
          'fields'    => [
            'user-profile-default-profile-picture'  => [
              'name'        =>  __( "Profile Picture", "wpep" ),
              'alias'       => 'user-profile-default-profile-picture',
              'type'        => 'image',
              'image_css_attributes' => [
                'width' => '256px',
                'height'=> '256px'
              ],
              'image_allow_remove' => false
            ],
            'user-profile-default-cover-picture'  => [
              'name'        =>  __( "Cover Picture", "wpep" ),
              'alias'       => 'user-profile-default-cover-picture',
              'type'        => 'image',
              'image_css_attributes' => [
                'width'     => '984px',
                'height'    => '400px',
                'max-width' => '100%'
              ],
              'image_allow_remove' => false,
              'row_data_attributes' => [
                'wpep-lp-enable-cover-picture'  => 1
              ]
            ],
          ]
        ],
        'email_notifications' => [
          'name'      => __( "Email Notifications", "wpep" ),
          'alias'     => 'email_notifications',
          'fields'    => [
            'user-profile-email-notification-open-management' => [
              'name'        => __( 'Manage Open', "wpep" ),
              'alias'       => 'user-profile-email-notification-open-management',
              'type'        => 'text_translated',
              'description' => __( "Manage platform-wide email notification settings.", "wpep" )
            ],
            'user-profile-email-notification-management-title' => [
              'name'        => __( 'Management Title', "wpep" ),
              'alias'       => 'user-profile-email-notification-management-title',
              'type'        => 'text_translated',
              'description' => __( "Popup Title when managing email notifications.", "wpep" )
            ],
            'user-profile-email-notification-general'  => [
              'name'        =>  __( 'Platform Wide', "wpep" ),
              'alias'       => 'user-profile-email-notification-general',
              'type'        => 'text_translated',
              'description' => sprintf( __( "This setting will disable any %s email notification from being sent.", "wpep" ), WPEP_PLUGIN_NAME )
            ],
            'user-profile-email-notification-weekly-recap'  => [
              'name'        =>  __( 'Weekly Recap', "wpep" ),
              'alias'       => 'user-profile-email-notification-weekly-recap',
              'type'        => 'text_translated',
            ]
          ]
        ],
      ]
    ];
  }

  private function _get_learner_profile_tab_management() {
    return [
      'name'             => __( 'Tab Management', "wpep" ),
      'alias'            => 'tab_management',
      'field_map'        => [
        'general_options' => [
          'name'      => __( "General Options", "wpep" ),
          'alias'     => 'general_options',
          'fields'    => [
            'user-profile-tab-link-format'  => [
              'name'        =>  __( 'Tab Links', "wpep" ),
              'alias'       => 'user-profile-tab-link-format',
              'type'        => 'select',
              'options'     => [
                'label'   => __( "Tab Title", "wpep" ),
                'alias'   => __( "Tab Alias", "wpep" ),
                'disabled'=> __( "Disable Navigation", "wpep" )
              ],
              'description' => __( "The URL Format of the links generated for the tabs when clicking on them.", "wpep" )
            ],
            'user-profile-allow-tab-expanding' => [
              'name'        =>  __( 'Tab Expanding', "wpep" ),
              'alias'       => 'user-profile-allow-tab-expanding',
              'type'        => 'checkbox',
              'description' => __( "Display an option to expand the Tabs & Content on the User Profile to Full Screen.", "wpep" )
            ]
          ]
        ],
        'default_tabs' => [
          'name'      => __( "Default Tabs", "wpep" ),
          'alias'     => 'default_tabs',
          'fields'    => [
            'user-profile-tab-default-tabs-enabled'  => [
              'name'        =>  __( 'Display Default Tabs', "wpep" ),
              'alias'       => 'user-profile-tab-default-tabs-enabled',
              'type'        => 'checkbox',
              'description' => __( "This includes Progressing Courses, Completed Courses and Bookmarked Courses", "wpep" )
            ],
            'user-profile-tab-progressing-courses'  => [
              'name'        =>  __( 'Progressing Courses', "wpep" ),
              'alias'       => 'user-profile-tab-progressing-courses',
              'type'        => 'text_translated'
            ],
            'user-profile-tab-completed-courses'  => [
              'name'        =>  __( 'Completed Courses', "wpep" ),
              'alias'       => 'user-profile-tab-completed-courses',
              'type'        => 'text_translated'
            ],
            'user-profile-tab-bookmarked-courses'  => [
              'name'        =>  __( 'Bookmarked Courses', "wpep" ),
              'alias'       => 'user-profile-tab-bookmarked-courses',
              'type'        => 'text_translated'
            ]
          ]
        ],
        'author_tab' => [
          'name'      => __( "Author Tab", "wpep" ),
          'alias'     => 'author_tab',
          'fields'    => [
            'user-profile-tab-author-content-enabled'  => [
              'name'        =>  __('Display Author Tab', "wpep"),
              'alias'       => 'user-profile-tab-author-content-enabled',
              'type'        => 'checkbox',
              'description' => __( "My author tab will display an Index Page of the content created by the user.", "wpep" )
            ],
            'user-profile-tab-author-content'  => [
              'name'        =>  __('My Content', "wpep"),
              'alias'       => 'user-profile-tab-author-content',
              'type'        => 'text_translated'
            ],
          ]
        ],
        'my_content_tab' => [
          'name'      => __( "My Content Tab", "wpep" ),
          'alias'     => 'my_content_tab',
          'fields'    => [
            'user-profile-tab-my-content-enabled'  => [
              'name'        =>  __('Display My Content Tab', "wpep"),
              'alias'       => 'user-profile-tab-my-content-enabled',
              'type'        => 'checkbox',
              'description' => function() {
                $response = '<p>' . __( "My Content Tab will display an Index Page that is not paginated, that will show all the Content the Student has access to.", "wpep" ) . '</p>';

                if( !wpep_controller()->contentLibrary->is_integrated() )
                  $response .= '<div class="wpep-notification wpep-notification-info">' . __( "No Deep Membership Integration detected, this option may not work as expected.", "wpep" ) . '</div>';

                return $response;
              }
            ],
            'user-profile-tab-my-content'  => [
              'name'        =>  __('My Content', "wpep"),
              'alias'       =>'user-profile-tab-my-content',
              'type'        => 'text_translated'
            ],
          ]
        ],
        'statistics_current_user' => [
          'name'      => __( "Statistics Tab", "wpep" ),
          'alias'     => 'statistics_current_user',
          'fields'    => [
            'user-profile-tab-statistics-current-user'  => [
              'name'        =>  __('Available Statistics', "wpep"),
              'alias'       => 'user-profile-tab-statistics-current-user',
              'type'        => 'text_translated',
              'description' => __( "For Instructors & Sub-Account Managers", "wpep" )
            ],
          ]
        ],
        'tab_ordering' => [
          'name'      => __( "Tab Ordering", "wpep" ),
          'alias'     => 'tab_ordering',
          'fields'    => [
            'all_available_tabs'  => [
              'name'        =>  __('All Available Tabs', "wpep"),
              'alias'       => 'all_available_tabs',
              'type'        => 'html_separator',
              'html_container_class' => 'wpep-entity-order-functionality-container',
              'html' => function() {
                $response = '<ul class="wpep-entity-order-functionality wpep-entity-order-functionality-keys">';

                foreach( wpep_content_user_profile_tabs( 0 ) as $tab_key => $tab_information ) {
                  $response .= '<li>';
                  $response .=    '<div>';
                  $response .=      '<input type="hidden" name="wpep_settings[user-profile-tab-ordering][]" value="' . $tab_key . '">';
                  $response .=      '<span class="wpep-title">' . ( isset( $tab_information['title'] ) ? $tab_information['title'] : $tab_key ) . '</span>';
                  $response .=    '</div>';
                  $response .= '</li>';
                }

                $response .= '</ul>';

                return $response;
              }
            ],
          ]
        ],
        'custom_tabs' => [
          'name'      => __( "Custom Tabs", "wpep" ),
          'alias'     => 'custom_tabs',
          'fields'    => [
            'user-profile-custom-tabs-enabled'  => [
              'name'        =>  __( 'Status', "wpep"),
              'alias'       => 'user-profile-custom-tabs-enabled',
              'type'        => 'checkbox',
              'description' => __( "New Custom Tabs will appear in the ordering above, only after you Save Changes.", "wpep"  )
            ],
            'user-profile-custom-tabs'  => [
              'name'        =>  __( 'Configure Tabs', "wpep"),
              'alias'       => 'user-profile-custom-tabs',
              'type'        => 'attributes_content_editor',
              'default'     => [
                'Example Tab' => __( "You can add anything here, and it will be displayed within this tab.", "wpep" )
              ]
            ],
          ]
        ]
      ]
    ];
  }

  private function _get_learner_profile_public_profiles() {
    return [
      'name'             => __( 'Public Profiles', "wpep" ),
      'alias'            => 'public',
      'field_map'        => [
        'general_options' => [
          'name'      => __( "General Options", "wpep" ),
          'alias'     => 'general_options',
          'fields'    => [
            'user-profile-public-active'  => [
              'name'        =>  __( 'Enable', "wpep" ),
              'alias'       => 'user-profile-public-active',
              'type'        => 'checkbox',
              'checkbox_force_checked' => ( wpep_public_profiles_enabled() && wpep_get_setting_flag( 'user-profile-public-active' ) == false ),
              'description' => function() {
                if( wpep_public_profiles_enabled() ) {
                  return '<p>' . sprintf(
                      __( "Your Public Profile :  %s", "wpep" ),
                      '<a target="_blank" href="' . wpep_user_profile_public_link() . '">' . wpep_user_profile_public_link() . '</a>'
                    ) . '</p>' . '<p>' . __( "If the above link doesn't work, reset permalinks from the Platform Health.", "wpep" ) . '</p>';
                }

                return __( "Activating the Public Profiles will automatically provide every WordPress User with a Profile Page.", "wpep" );
              }
            ],
            'user-profile-heading'  => [
              'name'        =>  __( 'Profile Page Title', "wpep" ),
              'alias'       => 'user-profile-heading',
              'type'        => 'text_translated',
              'description' => __( "Adding the %s identifier will replace it with the First Name & Last Name of the student", "wpep" )
            ],
            'user-profile-public-slug'  => [
              'name'   =>  __('Profile Slug', "wpep"),
              'alias'  => 'user-profile-public-slug',
              'type'   => 'text'
            ]
          ]
        ],
        'profile_privacy' => [
          'name'      => __( "Profile Privacy", "wpep" ),
          'alias'     => 'profile_privacy',
          'fields'    => [
            'user-profile-privacy-setting-default'  => [
              'name'        =>  __( "Default Setting", "wpep" ),
              'alias'       => 'user-profile-privacy-setting-default',
              'type'        => 'select',
              'options'     => [
                WPEP_USER_PROFILE_PRIVACY_TYPE_PRIVATE => __( "Private, students can only see their own public profile.", "wpep" ),
                WPEP_USER_PROFILE_PRIVACY_TYPE_PUBLIC  => __( "Public, anyone with a Link can see the Public Profiles.", "wpep" )
              ],
              'description' => __( "Admin & Staff accounts will be able to see Private User Profiles while they're logged in", "wpep" )
            ],
            'user-profile-privacy-setting-status'  => [
              'name'        =>  __( 'Allow Switching', "wpep"),
              'alias'       => 'user-profile-privacy-setting-status',
              'type'        => 'checkbox',
              'description' => __( "Allow Students to choose their own Privacy Setting from their Profile, by default the one above will be applied.", "wpep" )
            ],
            'user-profile-privacy-setting-private'  => [
              'name'        =>  __( 'Private Profile', "wpep"),
              'alias'       => 'user-profile-privacy-setting-private',
              'type'        => 'text_translated'
            ],
            'user-profile-privacy-setting-public'  => [
              'name'        =>  __( 'Public Profile', "wpep"),
              'alias'       => 'user-profile-privacy-setting-public',
              'type'        => 'text_translated'
            ],
            'user-profile-privacy-setting-tooltip'  => [
              'name'        =>  __( 'Profile Types', "wpep"),
              'alias'       => 'user-profile-privacy-setting-tooltip',
              'type'        => 'textarea',
              'description' => __( "This tooltip is displayed on the right side of the profile privacy type setting box", "wpep" )
            ],
            'user-profile-course-public-email-address'  => [
              'name'        =>  __( 'Public Email Address', "wpep" ),
              'alias'       => 'user-profile-course-public-email-address',
              'type'        => 'checkbox',
              'description' => __( "Enable this option to display the Email Address of the Students on Public Profiles", "wpep" )
            ]
          ]
        ],
        'course_options' => [
          'name'      => __( "Course Options", "wpep" ),
          'alias'     => 'course_options',
          'fields'    => [
            'user-profile-course-public-available' => [
              'name'        =>  __( 'Display Courses', "wpep"),
              'alias'       => 'user-profile-course-public-available',
              'type'        => 'checkbox',
              'description' => __( "Progressing, Completed & Bookmarked Courses will be displayed on the Public Profiles if this option is enabled. Admin & Staff accounts will still see them.", "wpep" )
            ],
            'user-profile-course-public-progress-bars' => [
              'name'        =>  __( 'Display Progress Bars', "wpep"),
              'alias'       => 'user-profile-course-public-progress-bars',
              'type'        => 'checkbox'
            ]
          ]
        ]
      ]
    ];
  }

  private function _get_learner_profile_social_networks() {
    return [
      'name'             => __( 'Social Networks', "wpep" ),
      'alias'            => 'social-networks',
      'field_map'        => [
        'general_options' => [
          'name'      => __( "General Options", "wpep" ),
          'alias'     => 'general_options',
          'has_title' => false,
          'fields'    => function() {
            $response = [];

            foreach( wpep_user_profile_social_network_platform_list() as $social_platform_key => $social_platform_information ) {
              $response[ 'user-profile-enabled-social-networks-' . $social_platform_key ] = [
                'name'           => '<span data-wpep-icon-pack-type="social" data-wpep-icon-pack="' . $social_platform_information['alias'] . '"></span>' . $social_platform_information['name'],
                'alias'          => 'user-profile-enabled-social-networks',
                'type'           => 'checkbox_array',
                'checkbox_value' => $social_platform_key
              ];
            }

            return $response;
          }
        ],
      ]
    ];
  }

  private function _get_learner_profile_student_supervisor() {
    return [
      'name'             => __( "Student Supervisor", "wpep" ),
      'alias'            => 'student-supervisor',
      'field_map'        => [
        'general_options' => [
          'name'      => __( "General Options", "wpep" ),
          'alias'     => 'general_options',
          'has_title' => false,
          'fields'    => [
            'enable-student-supervisor' => [
              'name'        =>  __( 'Student Supervisor', "wpep" ),
              'alias'       => 'enable-student-supervisor',
              'type'        => 'checkbox',
              'description' => __( "Allows students to choose a supervisor from their Learner Profile.", "wpep" )
            ],
            'enable-student-supervisor-multiple' => [
              'name'        =>  __( 'Multi Supervisor', "wpep" ),
              'alias'       => 'enable-student-supervisor-multiple',
              'type'        => 'checkbox',
              'description' => __( "Allows students to have multiple student supervisors.", "wpep" )
            ]
          ]
        ],
        'student_profile' => [
          'name'      => __( "Student Profile", "wpep" ),
          'alias'     => 'student_profile',
          'fields'    => [
            'user-profile-student-supervisor-placeholder' => [
              'name'        =>  __( 'Choose Supervisor', "wpep" ),
              'alias'       => 'user-profile-student-supervisor-placeholder',
              'type'        => 'text_translated'
            ],
            'user-profile-student-supervisor-allow-change' => [
              'name'        => __( 'Allow Student Switch', "wpep"),
              'alias'       => 'user-profile-student-supervisor-allow-change',
              'type'        => 'checkbox',
              'description' => __( "Allows your students to change their supervisor from the User Profile, they can choose from all the existent Student Supervisors.", "wpep" )
            ]
          ]
        ],
      ]
    ];
  }

  private function _get_learner_engagement_data_privacy() {
    return [
      'name'             => __( "Data & Privacy", "wpep" ),
      'alias'            => 'data_privacy',
      'field_map'        => [
        'terms_of_service' => [
          'name'      => __( "Terms of Service", "wpep" ),
          'alias'     => 'terms_of_service',
          'fields'    => [
            'user-service-agreement-enabled' => [
              'name'        =>  __( "Status", "wpep" ),
              'alias'       => 'user-service-agreement-enabled',
              'type'        => 'checkbox',
              'description' => __( "Require Students to comply with the terms of service.", "wpep" )
            ],
            'user-service-agreement-title' => [
              'name'        =>  __( 'Title', "wpep" ),
              'alias'       => 'user-service-agreement-title',
              'type'        => 'text_translated'
            ],
            'user-service-agreement-content' => [
              'name'        =>  __( 'Description', "wpep" ),
              'alias'       => 'user-service-agreement-content',
              'type'        => 'editor_translated'
            ],
            'user-service-agreement-error' => [
              'name'        =>  __( 'Error Notification', "wpep" ),
              'alias'       => 'user-service-agreement-error',
              'type'        => 'text_translated'
            ],
            'user-service-agreement-last-change' => [
              'name'        =>  __( 'Last Change', "wpep" ),
              'alias'       => 'user-service-agreement-last-change',
              'type'        => 'date',
              'description'     => '<p>' . __( "If you require your students to re-confirm compliance after changes where made to the policy, change this date, anyone who approved the terms before will be asked to approve again.", "wpep" ) . '</p>' .
                                   '<p>' . __( "Leave this empty to ignore this feature", "wpep" ) . '</p>',
              'value'           => wpep_format_date_with_time( wpep_get_setting( 'user-service-agreement-last-change' ), 'datepicker' ),
            ],
          ]
        ],
        'privacy_policy' => [
          'name'      => __( "Privacy Policy", "wpep" ),
          'alias'     => 'privacy_policy',
          'fields'    => [
            'user-privacy-agreement-enabled' => [
              'name'        =>  __( "Status", "wpep" ),
              'alias'       => 'user-privacy-agreement-enabled',
              'type'        => 'checkbox',
              'description' => __( "Require Students to comply with the privacy policy terms.", "wpep" )
            ],
            'user-privacy-agreement-title' => [
              'name'        =>  __( 'Title', "wpep" ),
              'alias'       => 'user-privacy-agreement-title',
              'type'        => 'text_translated'
            ],
            'user-privacy-agreement-content' => [
              'name'        =>  __( 'Description', "wpep" ),
              'alias'       => 'user-privacy-agreement-content',
              'type'        => 'editor_translated'
            ],
            'user-privacy-agreement-error' => [
              'name'        =>  __( 'Error Notification', "wpep" ),
              'alias'       => 'user-privacy-agreement-error',
              'type'        => 'text_translated'
            ],
            'user-privacy-agreement-last-change' => [
              'name'        =>  __( 'Last Change', "wpep" ),
              'alias'       => 'user-privacy-agreement-last-change',
              'type'        => 'date',
              'description'     => '<p>' . __( "If you require your students to re-confirm compliance after changes where made to the policy, change this date, anyone who approved the terms before will be asked to approve again.", "wpep" ) . '</p>' .
                                   '<p>' . __( "Leave this empty to ignore this feature", "wpep" ) . '</p>',
              'value'           => wpep_format_date_with_time( wpep_get_setting( 'user-privacy-agreement-last-change' ), 'datepicker' )
            ],
          ]
        ],
        'general_settings' => [
          'name'      => __( 'General Settings', "wpep" ),
          'alias'     => 'general_settings',
          'fields'    => [
            'user-agreement-introduction-text' => [
              'name'        =>  __( 'Intermediate Step', "wpep" ),
              'alias'       => 'user-agreement-introduction-text',
              'type'        => 'editor_translated',
              'description'     => '<p>' . __( "If you've made changes to your Terms of Service or Privacy Policy & set the Last Change Date accordingly, the students wil be asked to confirm them again.", "wpep" ) . '</p>' .
                                   '<p>' . __( "Text displayed before the Terms of Service or Privacy Policy.", "wpep" ) . '</p>',
            ],
            'user-agreement-button-accept' => [
              'name'        =>  __( 'Accept Terms', "wpep" ),
              'alias'       => 'user-agreement-button-accept',
              'type'        => 'text_translated'
            ],
            'user-agreement-button-decline' => [
              'name'        =>  __( 'Decline Terms', "wpep" ),
              'alias'       => 'user-agreement-button-decline',
              'type'        => 'text_translated'
            ],
            'user-agreement-button-decline-page' => [
              'name'        =>  __( 'Decline Terms Redirect', "wpep" ),
              'alias'       => 'user-agreement-button-decline-page',
              'type'        => 'text',
              'description' => __( "The page where the user will be taken when he declines Terms of Service or Privacy Policy", "wpep" ),
              'placeholder' => get_site_url()
            ],
          ]
        ],
        'account_delete' => [
          'name'      => __( 'Account Delete', "wpep" ),
          'alias'     => 'account_delete',
          'fields'    => [
            'user-account-delete-enabled' => [
              'name'        =>  __( 'Status', "wpep" ),
              'alias'       => 'user-account-delete-enabled',
              'type'        => 'checkbox',
              'description' => '<p>' . __( "Allow Students to delete their account permanently from the User Profile.", "wpep" ) . '</p>' .
                               '<p>' . sprintf( __( "%s is compliant with any other plugin that deletes an User Account, and will follow-up by deleting data it stores", "wpep" ), WPEP_PLUGIN_NAME ) . '</p>'
            ],
            'user-account-delete-button' => [
              'name'        =>  __( "Button", "wpep" ),
              'alias'       => 'user-account-delete-button',
              'type'        => 'text_translated'
            ],
            'user-account-delete-confirmation' => [
              'name'        =>  __( "Confirmation Message", "wpep" ),
              'alias'       => 'user-account-delete-confirmation',
              'type'        => 'editor_translated'
            ],
            'user-account-delete-confirmation-button' => [
              'name'        =>  __( "Confirmation Button", "wpep" ),
              'alias'       => 'user-account-delete-confirmation-button',
              'type'        => 'text_translated'
            ],
            'user-account-delete-confirmation-cancel-button' => [
              'name'        =>  __( "Cancel Button", "wpep" ),
              'alias'       => 'user-account-delete-confirmation-cancel-button',
              'type'        => 'text_translated'
            ],
            'user-account-delete-success-page' => [
              'name'        =>  __( "Success Redirect", "wpep" ),
              'alias'       => 'user-account-delete-success-page',
              'type'        => 'text',
              'description' => __( "Redirect the User to a specific page after his account will be deleted, by default the homepage.", "wpep" ),
              'placeholder' => get_site_url()
            ],
          ]
        ],
        'account_export' => [
          'name'      => __( 'Account Export', "wpep" ),
          'alias'     => 'account_export',
          'fields'    => [
            'user-account-export-student-information' => [
              'name'        =>  __( "Student Information", "wpep" ),
              'alias'       => 'user-account-export-student-information',
              'type'        => 'text_translated'
            ],
          ]
        ],
      ]
    ];
  }

  private function _get_learner_engagement_delivery_settings() {
    return [
      'name'             => __( "Delivery Settings", "wpep" ),
      'alias'            => 'delivery_settings',
      'field_map'        => [
        'branding' => [
          'name'      => __( "Branding", "wpep" ),
          'alias'     => 'branding',
          'fields'    => [
            'email-logo' => [
              'name'        =>  __( "Email Logo", "wpep" ),
              'alias'       => 'email-logo',
              'type'        => 'image',
              'description' => __( 'The Logo used in the Email Template', "wpep" ),
              'image_css_attributes' => [
                'width'  => WPEP_EMAIL_LOGO_WIDTH,
                'height' => WPEP_EMAIL_LOGO_HEIGHT
              ],
            ],
            'email-company-name' => [
              'name'        =>  __( 'Company Name', "wpep" ),
              'alias'       => 'email-company-name',
              'type'        => 'text'
            ],
            'email-company-address' => [
              'name'        =>  __( 'Company Address', "wpep" ),
              'alias'       => 'email-company-address',
              'type'        => 'text'
            ],
            'email-copyright-text' => [
              'name'        =>  __( 'Copyright', "wpep" ),
              'alias'       => 'email-copyright-text',
              'type'        => 'text'
            ]
          ]
        ],
        'email_delivery' => [
          'name'      => __( "Email Delivery", "wpep" ),
          'alias'     => 'email_delivery',
          'fields'    => [
            'email-from-name' => [
              'name'        =>  __( 'From Name', "wpep" ),
              'alias'       => 'email-from-name',
              'type'        => 'text',
              'description' => __( 'The default name will be used if left empty.', "wpep" )
            ],
            'email-from-email' => [
              'name'        =>  __( 'From Email', "wpep" ),
              'alias'       => 'email-from-email',
              'type'        => 'text',
              'description' => '<p>' . __( 'The default email address will be used if left empty.', "wpep" ) . '</p>' .
                               '<p>' . __( 'Note : To avoid getting the emails flagged as spam, you must use an address from the current domain.', 'wpep' ) . '</p>'
            ],
          ]
        ],/** @todo implement unsubscribe option directly from the email
        'email_privacy' => [
          'name'      => __( "Email Privacy", "wpep" ),
          'alias'     => 'email_tracking',
          'fields'    => function() {
            $token = ( wpep_get_setting( 'email-tracking-salt' ) !== '' ? null : wpep_generate_random_token( 24 ) );

            return [
              'email-tracking-salt' => [
                'name'        =>  __( 'Encryption Token', "wpep" ),
                'alias'       => 'email-tracking-salt',
                'type'        => 'text',
                'description' => ( $token !== null ? '' : __( 'Changing this setting will invalidate all existing tracking events, existent data stays.', "wpep" ) ),
                'value'       => $token
              ]
            ];
          }
        ],**/
        'email_tracking' => [
          'name'      => __( "Email Tracking", "wpep" ),
          'alias'     => 'email_tracking',
          'fields'    => function() {
            $token = ( wpep_get_setting( 'email-tracking-salt' ) !== '' ? null : wpep_generate_random_token( 24 ) );

            return [
              'email-tracking-salt' => [
                'name'        =>  __( 'Encryption Token', "wpep" ),
                'alias'       => 'email-tracking-salt',
                'type'        => 'text',
                'description' => ( $token !== null ? '' : __( 'Changing this setting will invalidate all existing tracking events, existent data stays.', "wpep" ) ),
                'value'       => $token
              ]
            ];
          }
        ]
      ]
    ];
  }

  private function _get_learner_engagement_onboarding() {
    return [
      'name'             => __( "Onboarding", "wpep" ),
      'alias'            => 'onboarding',
      'field_map'        => [
        'general' => [
          'name'      => __( "General Options", "wpep" ),
          'alias'     => 'general',
          'has_title' => false,
          'fields'    => function() {
            $response    = [];
            $information = wpep_get_setting( 'user-register-tag-email-marketing-map' );

            foreach( wpep_controller()->emailMarketing->get_text_based_tag_integrations() as $emailMarketingIntegration ) {
              if( !$emailMarketingIntegration->is_configured() )
                continue;

              $response[ 'user-register-tag-email-marketing-map-' . $emailMarketingIntegration->get_alias() ] = [
                'name'        => $emailMarketingIntegration->get_name(),
                'alias'       => 'user-register-tag-email-marketing-map-' . $emailMarketingIntegration->get_alias(),
                'type'        => 'text',
                'field_name'  => wpep_controller()->adminController->settings->field_prefix . '[user-register-tag-email-marketing-map][' . $emailMarketingIntegration->get_alias() . ']',
                'description' => sprintf( __( "Automatically apply a tag in %s when a new user is created.", "wpep" ), $emailMarketingIntegration->get_name() ),
                'value'       => ( isset( $information[ $emailMarketingIntegration->get_alias() ] ) ? $information[ $emailMarketingIntegration->get_alias() ] : '' )
              ];
            }

            return $response;
          }
        ],
      ]
    ];
  }

  private function _get_learner_engagement_weekly_recap() {
    return [
      'name'             => __( "Weekly Recap", "wpep" ),
      'alias'            => 'weekly_recap',
      'field_map'        => [
        'general' => [
          'name'      => __( "General Options", "wpep" ),
          'alias'     => 'general',
          'has_title' => false,
          'fields'    => [
            'email-weekly-recap-status' => [
              'name'  => __( 'Enable Weekly Recap', "wpep" ),
              'alias' => 'email-weekly-recap-status',
              'type'  => 'checkbox'
            ],
            'email_weekly_recap_preview' => [
              'name'  => __( 'Preview', "wpep" ),
              'alias' => 'email_weekly_recap_preview',
              'type'  => 'html_separator',
              'html'  => function() {
                $weekly_recap_user_ids = get_option( 'wpep_weekly_recap_que', [] );

                if( in_array( get_current_user_id(), $weekly_recap_user_ids ) ) {
                  return '<div class="wpep-notification wpep-notification-success">' .
                            sprintf( __( "Weekly Recap Test has been added to the queue, you should receive an email at %s within 5 minutes.", "wpep" ), wp_get_current_user()->user_email ) .
                         '</div>';
                }

                return '<p class="wpep-operations-line-container">' .
                         '<a class="wpep-button wpep-button-primary" target="_blank" href="' . add_query_arg( 'wpep-preview-email', 'weekly-recap-browser', get_site_url() ) . '">' . __( "Preview", "wpep" ) . '</a>' .
                         '<a class="wpep-button wpep-button-secondary" href="' . add_query_arg( 'wpep-preview-email', 'weekly-recap-instant-email', get_site_url() ) . '">' . __( "Send Instant Email", "wpep" ) . '</a>' .
                         '<a class="wpep-button wpep-button-secondary" href="' . add_query_arg( 'wpep-preview-email', 'weekly-recap-email', get_site_url() ) . '">' . __( "Send Email", "wpep" ) . '</a>' .
                       '</p>';
              }
            ],
            'email-weekly-recap-title' => [
              'name'        => __( 'Title', "wpep" ),
              'alias'       => 'email-weekly-recap-title',
              'type'        => 'text_translated',
              'description' => __( 'This is the email title, you can optionally use shortcodes, for example [user_name], [date]', "wpep" )
            ],
            'email-weekly-recap-content' => [
              'name'        => __( 'Content', "wpep" ),
              'alias'       => 'email-weekly-recap-content',
              'type'        => 'editor_translated',
              'description' => function() {
                return wpep_controller()->userEngagement->get_available_shortcode_list( true );
              }
            ],
          ]
        ],
      ]
    ];
  }

  private function _get_learner_engagement_discussions() {
    $field_map = [
      'list' => [
        'name'      => __( "List", "wpep" ),
        'alias'     => 'list',
        'fields'    => [
          'comments-search-label' => [
            'name'  => __( 'Search', "wpep" ),
            'alias' => 'comments-search-label',
            'type'  => 'text_translated'
          ],
          'comments-filter-mine' => [
            'name'        => __( 'Show My Comments', "wpep" ),
            'alias'       => 'comments-filter-mine',
            'type'        => 'text_translated',
            'description' => __( "The text displayed alongside to display only the comments the current user has posted", "wpep" )
          ],
          'comments-per-page' => [
            'name'        => __( 'Comments Per Page', "wpep" ),
            'alias'       => 'comments-per-page',
            'type'        => 'text'
          ],
          'comment-replies-per-page' => [
            'name'        => __( 'Comment Replies Per Page', "wpep" ),
            'alias'       => 'comment-replies-per-page',
            'type'        => 'text',
          ],
          'comment-replies-show-more' => [
            'name'        => __( 'Show More Replies', "wpep" ),
            'alias'       => 'comment-replies-show-more',
            'type'        => 'text_translated',
            'description' => __( "%s will be replaced with the number of replies that haven't been shown already.", "wpep" )
          ],
          'comments-button-reply' => [
            'name'        => __( 'Reply', "wpep" ),
            'alias'       => 'comments-button-reply',
            'type'        => 'text_translated'
          ],
          'comments-navigation-previous' => [
            'name'        => __( 'Previous', "wpep" ),
            'alias'       => 'comments-navigation-previous',
            'type'        => 'text_translated'
          ],
          'comments-navigation-next' => [
            'name'        => __( 'Next', "wpep" ),
            'alias'       => 'comments-navigation-next',
            'type'        => 'text_translated'
          ],
          'comments-authority-badge-name' => [
            'name'        => __( 'Authority Badge', "wpep" ),
            'alias'       => 'comments-authority-badge-name',
            'type'        => 'text_translated'
          ],
        ]
      ],
      'list_admin' => [
        'name'      => __( "List Admin", "wpep" ),
        'alias'     => 'list_admin',
        'fields'    => [
          'comments-button-delete' => [
            'name'  => __( 'Delete', "wpep" ),
            'alias' => 'comments-button-delete',
            'type'  => 'text_translated'
          ],
          'comments-button-confirm-delete' => [
            'name'  => __( 'Confirm Delete', "wpep" ),
            'alias' => 'comments-button-confirm-delete',
            'type'  => 'text_translated'
          ],
          'comments-button-feature-comment' => [
            'name'  => __( 'Feature Comment', "wpep" ),
            'alias' => 'comments-button-feature-comment',
            'type'  => 'text_translated'
          ],
        ]
      ],
      'form' => [
        'name'      => __( "Form", "wpep" ),
        'alias'     => 'form',
        'fields'    => [
          'comments-notification-content-empty' => [
            'name'  => __( 'Content Required', "wpep" ),
            'alias' => 'comments-notification-content-empty',
            'type'  => 'text_translated'
          ],
          'comments-button-cancel-reply' => [
            'name'  => __( 'Cancel Reply', "wpep" ),
            'alias' => 'comments-button-cancel-reply',
            'type'  => 'text_translated'
          ],
          'comments-button-submit' => [
            'name'  => __( 'Submit Comment', "wpep" ),
            'alias' => 'comments-button-submit',
            'type'  => 'text_translated'
          ],
          'comments-button-reply-submit' => [
            'name'  => __( 'Submit Reply', "wpep" ),
            'alias' => 'comments-button-reply-submit',
            'type'  => 'text_translated'
          ],
        ]
      ],
    ];

    if( wpep_feature_has_settings_panel( 'discussion_notifications_email' ) ) {
      $field_map[ 'notifications_email' ] = [
        'name'      => __( "Notifications Email", "wpep" ),
        'alias'     => 'notifications_email',
        'fields'    => [
          'comments-email-notification-title' => [
            'name'        => __( 'Title', "wpep" ),
            'alias'       => 'comments-email-notification-title',
            'type'        => 'text_translated'
          ],
          'comments-email-notification-content' => [
            'name'        => __( 'Content', "wpep" ),
            'alias'       => 'comments-email-notification-content',
            'type'        => 'editor_translated',
            'description' => function() {
              return '<h2>' . __( "Discussion ShortCodes") . '</h2>' .
                '<ul>' .
                '<li>[discussion_activity_preview] - ' . __( "Displays a Preview of all the new Activity within Discussions.", "wpep" ) . '</li>' .
                '<li>[discussion_activity_number] - ' . __( "The Number of Discussions there's new activity within", "wpep" ) . '</li>' .
                '</ul>' .
                wpep_controller()->userEngagement->get_available_shortcode_list( false );
            }
          ],
          'comments-email-notification-frequency' => [
            'name'        => __( 'Frequency', "wpep" ),
            'alias'       => 'comments-email-notification-frequency',
            'type'        => 'select',
            'options'     => wpep_discussion_notifications_intervals(),
            'description' => __( "When there's activity within a Discussion, students will be notified maximum once per frequency cycle.", "wpep" )
          ],
          'comments-email-notification-button-view' => [
            'name'        => __( 'View Discussion', "wpep" ),
            'alias'       => 'comments-email-notification-button-view',
            'type'        => 'text_translated',
            'description' => __( "The [discussion_activity_preview] shortcode will display a button for each Discussion.", "wpep" )
          ],
        ]
      ];
    }

    if( wpep_feature_has_settings_panel( 'discussion_user_list' ) ) {
      $field_map[ 'user_list' ] = [
        'name'      => __( "User List", "wpep" ),
        'alias'     => 'user_list',
        'fields'    => [
          'comments-user-list-status' => [
            'name'        => __( "Show User List", "wpep" ),
            'alias'       => 'comments-user-list-status',
            'type'        => 'checkbox',
            'description' => __( "Display a list of students who've engaged in the discussion at the top", "wpep" )
          ],
          'comments-user-list-title' => [
            'name'        => __( 'Title', "wpep" ),
            'alias'       => 'comments-user-list-title',
            'type'        => 'text_translated'
          ],
          'comments-user-list-view-profile' => [
            'name'        => __( 'View Profile', "wpep" ),
            'alias'       => 'comments-user-list-view-profile',
            'type'        => 'text_translated',
            'description' => __( "Students need to have a Public Profile in order for other students to have the rights to view them.", "wpep" )
          ],
          'comments-user-list-show-container' => [
            'name'        => __( 'Show List', "wpep" ),
            'alias'       => 'comments-user-list-show-container',
            'type'        => 'text_translated'
          ]
        ]
      ];
    }

    if( wpep_feature_has_settings_panel( 'discussion_reactions' ) ) {
      $field_map[ 'reactions' ] = [
        'name'      => __( "Reactions", "wpep" ),
        'alias'     => 'reactions',
        'fields'    => [
          'comments-reactions-status' => [
            'name'        => __( 'Enable Reactions', "wpep" ),
            'alias'       => 'comments-reactions-status',
            'type'        => 'checkbox'
          ],
          'comments-reactions-default-reaction' => [
            'name'        => __( 'Default Reaction', "wpep" ),
            'alias'       => 'comments-reactions-default-reaction',
            'type'        => 'select',
            'options'     => function() {
              return wpep_get_emoji_list();
            },
            'description' => __( "The default reaction should be part of the Reactions bellow in order to be displayed correctly.", "wpep" ),
            'attributes'  => [
              'data-ec-library' => 'select2'
            ]
          ],
          'comments-reactions-available-emoji-list' => [
            'name'        => __( 'Reactions', "wpep" ),
            'alias'       => 'comments-reactions-available-emoji-list',
            'type'        => 'multiple_select',
            'options'     => function() {
              return wpep_get_emoji_list();
            }
          ],
          'comments-reactions-popup-all-reactions-text' => [
            'name'        => __( "Popup - All Reactions", "wpep" ),
            'alias'       => 'comments-reactions-popup-all-reactions-text',
            'type'        => 'text_translated'
          ],
          'comments-reactions-popup-view-profile' => [
            'name'        => __( "Popup - View Profile", "wpep" ),
            'alias'       => 'comments-reactions-popup-view-profile',
            'type'        => 'text_translated'
          ],
          'comments-reactions-popup-reactions-per-request' => [
            'name'        => __( "Popup - Pagination Limit", "wpep" ),
            'alias'       => 'comments-reactions-popup-reactions-per-request',
            'type'        => 'text_translated',
            'description' => __( "How many users that have reacted should be displayed at once", "wpep" )
          ],
          'comments-reactions-popup-show-more' => [
            'name'        => __( "Popup - Show More", "wpep" ),
            'alias'       => 'comments-reactions-popup-show-more',
            'type'        => 'text_translated'
          ]
        ]
      ];
    }

    if( wpep_feature_has_settings_panel( 'discussion_direct_url' ) ) {
      $field_map[ 'direct_url' ] = [
        'name'      => __( "Direct URLs", "wpep" ),
        'alias'     => 'direct_url',
        'fields'    => [
          'comments-direct-url-status' => [
            'name'        => __( 'Enable Direct URLs', "wpep" ),
            'alias'       => 'comments-direct-url-status',
            'type'        => 'checkbox',
            'description' => __( "Will create special links to each discussion directly from the URL", "wpep" ),
            'checkbox_force_checked' => ( wpep_discussion_direct_url_enabled() && wpep_get_setting_flag( 'comments-direct-url-status' ) == false ),
          ],
          'comments-direct-url-slug' => [
            'name'        => __( 'URL Slug', "wpep" ),
            'alias'       => 'comments-direct-url-slug',
            'type'        => 'text'
          ],
          'comments-direct-url-list-title' => [
            'name'        => __( 'Conversation Archive Title', "wpep" ),
            'alias'       => 'comments-direct-url-list-title',
            'type'        => 'text'
          ],
          'comments-direct-url-title' => [
            'name'        => __( 'Conversation Single Title', "wpep" ),
            'alias'       => 'comments-direct-url-title',
            'type'        => 'text_translated'
          ],
          'comments-direct-url-my-discussions' => [
            'name'        => __( 'My Discussions', "wpep" ),
            'alias'       => 'comments-direct-url-my-discussions',
            'type'        => 'text_translated'
          ],
          'comments-direct-url-expand-my-discussions' => [
            'name'        => __( 'View My Discussions', "wpep" ),
            'alias'       => 'comments-direct-url-expand-my-discussions',
            'type'        => 'text_translated',
            'description' => __( "Displayed on mobile devices to show all the discussions", "wpep" )
          ],
          'comments-direct-url-show-more' => [
            'name'        => __( 'Show More', "wpep" ),
            'alias'       => 'comments-direct-url-show-more',
            'type'        => 'text_translated'
          ],
          'comments-direct-url-no-discussions' => [
            'name'        => __( "No Discussions", "wpep" ),
            'alias'       => 'comments-direct-url-no-discussions',
            'type'        => 'editor_translated'
          ]
        ]
      ];

      $field_map[ 'direct_url_notifications' ] = [
        'name'      => __( "Direct URLs Notifications", "wpep" ),
        'alias'     => 'direct_url_notifications',
        'fields'    => [
          'comments-direct-url-menu-item-bubble' => [
            'name'        => __( 'Menu Item Bubble', "wpep" ),
            'alias'       => 'comments-direct-url-menu-item-bubble',
            'type'        => 'checkbox',
            'description' => __( "When using the Discussions Menu Item, a Bubble will appear with notifications that will show on hover, or click on mobile.", "wpep" )
          ],
          'comments-direct-url-menu-item-bubble-item-click' => [
            'name'        => __( 'Open on Item Click', "wpep" ),
            'alias'       => 'comments-direct-url-menu-item-bubble-item-click',
            'type'        => 'checkbox',
            'description' => __( "Enabling this option will open the notifications when clicking on the menu item, when disabled the notifications will show when clicking the icon.", "wpep" )
          ],
          'comments-direct-url-menu-item-notification-comment' => [
            'name'        => __( 'Comment', "wpep" ),
            'alias'       => 'comments-direct-url-menu-item-notification-comment',
            'type'        => 'text_translated',
            'description' => __( "The first %s will be replaced with the name of the person who wrote the comment, and the second with the discussion name", "wpep" )
          ],
          'comments-direct-url-menu-item-notification-comment-reply' => [
            'name'        => __( 'Comment Reply', "wpep" ),
            'alias'       => 'comments-direct-url-menu-item-notification-comment-reply',
            'type'        => 'text_translated',
            'description' => __( "The first %s will be replaced with the name of the person who replied the comment, and the second with the discussion name", "wpep" )
          ],
          'comments-direct-url-menu-item-notification-comment-reply-on-reply' => [
            'name'        => __( 'Comment Reply on Reply', "wpep" ),
            'alias'       => 'comments-direct-url-menu-item-notification-comment-reply-on-reply',
            'type'        => 'text_translated',
            'description' => __( "The first %s will be replaced with the name of the person who replied the comment the receiver of the notification also wrote on, and the second with the discussion name", "wpep" )
          ],
          'comments-direct-url-menu-item-notification-comment-reaction' => [
            'name'        => __( 'Comment Reaction', "wpep" ),
            'alias'       => 'comments-direct-url-menu-item-notification-comment-reaction',
            'type'        => 'text_translated',
            'description' => __( "The first %s will be replaced with the name of the person who reacted to the comment, and the second with the discussion name", "wpep" )
          ],
          'comments-direct-url-menu-item-notification-comment-mention' => [
            'name'        => __( 'Comment Mention', "wpep" ),
            'alias'       => 'comments-direct-url-menu-item-notification-comment-mention',
            'type'        => 'text_translated',
            'description' => __( "The first %s will be replaced with the name of the person who mentioned you in the comment, and the second with the discussion name", "wpep" )
          ]
        ]
      ];
    }

    if( wpep_feature_has_settings_panel( 'discussion_mentions' ) ) {
      $field_map[ 'mentions' ] = [
        'name'      => __( "Mentions", "wpep" ),
        'alias'     => 'mentions',
        'fields'    => [
          'comments-mentions-status' => [
            'name'        => __( 'Enable Mentions', "wpep" ),
            'alias'       => 'comments-mentions-status',
            'type'        => 'checkbox',
            'description' => __( "Only users who are part of the discussion can be mentioned in comments.", "wpep" )
          ],
          'comments-mentions-notification-title' => [
            'name'        => __( 'Email Title', "wpep" ),
            'alias'       => 'comments-mentions-notification-title',
            'type'        => 'text_translated'
          ],
          'comments-mentions-notification-content' => [
            'name'        => __( 'Email Content', "wpep" ),
            'alias'       => 'comments-mentions-notification-content',
            'type'        => 'editor_translated',
            'description' => function() {
              return '<h2>' . __( "Discussion Mentions ShortCodes", "wpep" ) . '</h2>' .
                     '<ul>' .
                        '<li>[mention_source_name] - ' . __( "The name of the user that wrote the comment where the receiver is mentioned.", "wpep" ) . '</li>' .
                        '<li>[mention_source_picture] - ' . __( "The profile picture of the user that wrote the comment where the receiver is mentioned.", "wpep" ) . '</li>' .
                        '<li>[mention_activity_preview] - ' . __( "Displays a preview of the activity around the comment.", "wpep" ) . '</li>' .
                        '<li>[discussion_name] - ' . __( "The name of the discussion.", "wpep" ) . '</li>' .
                        '<li>[discussion_link] - ' . __( "The link of the discussion.", "wpep" ) . '</li>' .
                        '<li>[discussion_comment_content] - ' . __( "The comment content, in which the user has been mentioned.", "wpep" ) . '</li>' .
                        '<li>[discussion_comment_date_time] - ' . __( "The comment date & time, in which the user has been mentioned.", "wpep" ) . '</li>' .
                     '</ul>' .
                     wpep_controller()->userEngagement->get_available_shortcode_list( false );
            }
          ]
        ]
      ];
    }

    if( wpep_feature_has_settings_panel( 'discussion_advanced_settings' ) ) {
      $field_map[ 'advanced_settings' ] = [
        'name'      => __( "Advanced Settings", "wpep" ),
        'alias'     => 'advanced_settings',
        'fields'    => [
          'comments-refresh-ms-interval' => [
            'name'        => __( "Sync Comments", "wpep" ),
            'alias'       => 'comments-refresh-ms-interval',
            'type'        => 'text',
            'description' => '<p>' . sprintf( __( "In seconds, how frequent %s should check if new comments were posted while the page is open for a student, minimum 3.", "wpep" ), WPEP_PLUGIN_NAME ) . '</p>' .
              '<p>' . __( "Pro Tip : This process is highly optimized to not overlap requests.", "wpep" ) . '</p>'
          ]
        ]
      ];
    }

    return [
      'name'             => __( "Discussions", "wpep" ),
      'alias'            => 'discussions',
      'field_map'        => $field_map
    ];
  }

  private function _get_learner_engagement_auto_login_links() {
    return [
      'name'             => __( "Auto Login Links", "wpep" ),
      'alias'            => 'auto-login-links',
      'field_map'        => [
        'general' => [
          'name'      => __( "General", "wpep" ),
          'alias'     => 'general',
          'has_title' => false,
          'fields'    => [
            'actions' => [
              'name'  => __( 'Actions', "wpep" ),
              'alias' => 'actions',
              'type'  => 'html_separator',
              'html'  => function() {
                if( wpep_get_setting_flag( 'enable-auto-login-link' ) ) {
                  $last_token_reset = wpep_get_setting( 'auto-login-link-last-generate' );

                  $response = '';

                  $response .= '<p class="wpep-operations-line-container">';
                  $response .=   '<a class="wpep-button wpep-button-primary" href="#" data-ec-simple-download="' . esc_attr( json_encode( [ 'action' => 'wpep/user/auto-login/token-entries', 'format' => 'csv' ] ) ) . '">' . __( "Prepare CSV Export", "wpep" ) . '</a>';

                  if( $last_token_reset != 0 ) {
                    $response .=   '<a class="wpep-button wpep-button-secondary" href="#" data-ec-simple-sync="' . esc_attr( json_encode( [ 'action' => 'wpep/user/auto-login/token-reset' ] ) ) . '">';
                    $response .=      __( "Delete Tokens", "wpep" );
                    $response .=   '</a>';
                    $response .=   '<span class="wpep-operation-status">' . sprintf( __( "Last Download : %s", "wpep" ), '<strong>' . wpep_format_date_with_time( $last_token_reset ) . '</strong>' ) . '</span>';
                  }
                  $response .= '</p>';

                  return $response;
                }

                return __( "Enable Dynamic Login Links and save the settings in order to enable this", "wpep" );
              }
            ],
            'enable-auto-login-link' => [
              'name'        => __( 'Dynamic Login Links', "wpep" ),
              'alias'       => 'enable-auto-login-link',
              'type'        => 'checkbox',
              'description' => function() {
                return '<p>' . __( "Dynamic Login Links are a sensitive feature, some security experts consider it dangerous, that's why we allow to enable / disable it on demand.", "wpep" ) . '</p>' .
                       '<p>' . sprintf( __( "It's also possible to regenerate all the login tokens anytime, invalidating existent URLs, all login tokens that are generated are always unique, across all %s installations & site users, it is also recommended to regenerate the tokens & grabbing a new CSV file every time you send the login links out.", "wpep" ), WPEP_PLUGIN_NAME ) . '</p>' .
                       '<p>' . __( "If you disable this feature, existent login links will not work anymore until you active it back.", "wpep" ) . '</p>';
              }
            ],
            'auto-login-link-token' => [
              'name'        => __( 'URL Slug', "wpep" ),
              'alias'       => 'auto-login-link-token',
              'type'        => 'text',
              'description' => __( "If you change this parameter all existent dynamic login links will stop working, remember to Save Settings before you export if you've changed this.", "wpep" )
            ],
            'auto-login-link-salt' => [
              'name'        => __( 'Token Salt', "wpep" ),
              'alias'       => 'auto-login-link-salt',
              'type'        => 'text',
              'description' => __( "When Login Tokens are generated, this seed is used in order to ensure they're unique.", "wpep" ),
              'value'       => ( wpep_get_setting( 'auto-login-link-salt' ) !== '' ? null : wpep_generate_random_token( 24 ) )
            ]
          ]
        ]
      ]
    ];
  }

  private function _get_learner_engagement_share_embed() {
    return [
      'name'             => __( 'Sharing & Embed', "wpep" ),
      'alias'            => 'share_embed',
      'field_map'        => [
        'general' => [
          'name'      => __( "General", "wpep" ),
          'alias'     => 'general',
          'has_title' => false,
          'fields'    => [
            'user-content-embed-box-enabled' => [
              'name'    => __( 'Allow Students to Embed Content', "wpep" ),
              'alias'   => 'user-content-embed-box-enabled',
              'type'    => 'select',
              'options' => [
                0 => __( 'Inactive', "wpep" ),
                1 => __( 'Active', "wpep" )
              ],
              'description' => __( "Enable this option to allow anyone to share your courses, keeping it inactive will allow you to limit the domains allowed to embed content.", "wpep" ),
              'attributes'  => [
                'data-wpep-conditional-display' => 'data-wpep-le-embed-box-status'
              ]
            ],
          ]
        ],
        'whitelist_domains' => [
          'name'                => __( 'Whitelist Domains', "wpep" ),
          'alias'               => 'whitelist_domains',
          'has_title'           => true,
          'row_data_attributes' => [ 'wpep-le-embed-box-status' => 0 ],
          'fields'              => [
            'user-content-embed-domain-verify' => [
              'name'    => __( 'Status', "wpep" ),
              'alias'   => 'user-content-embed-domain-verify',
              'type'    => 'select',
              'options' => [
                0 => __( "Any domain is allowed to embed content", "wpep" ),
                1 => __( "Only specific domains are allowed to embed content", "wpep" )
              ],
              'description' => __( "Enable this option to allow anyone to share your courses, keeping it inactive will allow you to limit the domains allowed to embed content.", "wpep" ),
              'attributes'  => [
                'data-wpep-conditional-display' => 'data-wpep-le-domain-verify-status'
              ]
            ],
            'user-content-embed-domain-list' => [
              'name'    => __( 'List', "wpep" ),
              'alias'   => 'user-content-embed-domain-list',
              'type'    => 'list',
              'options' => [
                0 => __( "Any domain is allowed to embed content", "wpep" ),
                1 => __( "Only specific domains are allowed to embed content", "wpep" )
              ],
              'description'         => __( "Enable this option to allow anyone to share your courses, keeping it inactive will allow you to limit the domains allowed to embed content.", "wpep" ),
              'row_data_attributes' => [ 'wpep-le-domain-verify-status' => 1],
              'default'             => [ get_site_url()]
            ],
            'user-content-embed-error-notification' => [
              'name'    => __( "Error Notification", "wpep" ),
              'alias'   => 'user-content-embed-error-notification',
              'type'    => 'editor_translated',
              'description' => __( "If a domain embeds content and is not part of the whitelist, it will be presented with this error.", "wpep" ),
              'row_data_attributes' => [ 'wpep-le-domain-verify-status' => 1 ]
            ],
          ]
        ],
        'call_to_action' => [
          'name'                => __( 'Call To Action', "wpep" ),
          'alias'               => 'call_to_action',
          'has_title'           => true,
          'row_data_attributes' => [ 'wpep-le-embed-box-status' => 1 ],
          'fields'              => [
            'user-content-embed-box-course-status' => [
              'name'    => __( 'Default Status', "wpep" ),
              'alias'   => 'user-content-embed-box-course-status',
              'type'    => 'checkbox',
              'description' => __( "The Default Setting, you can disable / enable the CTA Individually for each course.", "wpep" )
            ],
            'user-content-embed-box-title' => [
              'name'    => __( 'Title', "wpep" ),
              'alias'   => 'user-content-embed-box-title',
              'type'    => 'text_translated'
            ],
            'user-content-embed-box-description' => [
              'name'    => __( 'Description', "wpep" ),
              'alias'   => 'user-content-embed-box-description',
              'type'    => 'editor_translated'
            ],
            'user-content-embed-box-button' => [
              'name'    => __( 'Button Text', "wpep" ),
              'alias'   => 'user-content-embed-box-button',
              'type'    => 'text_translated'
            ],
            'user-content-embed-box-button-clicked' => [
              'name'    => __( 'Button Text Clicked', "wpep" ),
              'alias'   => 'user-content-embed-box-button-clicked',
              'type'    => 'text_translated'
            ],
            'user-content-embed-box-inside-embeds' => [
              'name'    => __( 'Show Inside Embeds', "wpep" ),
              'alias'   => 'user-content-embed-box-inside-embeds',
              'type'    => 'checkbox'
            ],
          ]
        ]
      ]
    ];
  }

}