<?php

namespace WPEP\Admin\Coordinator;

class Toolbar {

  public function setup() {
    add_action( 'admin_bar_menu', [ $this, '_admin_bar_menu' ], 250 );
  }

  public function _admin_bar_menu( $wp_admin_bar ) {
    $node_list = [];

    if( current_user_can( WPEP_CAP_LEARNING_CENTER )
        || current_user_can( WPEP_CAP_EDIT_COURSE ) )
      $node_list[] = [
        'id'      => 'wpep_toolbar_shortcode_builder',
        'title'   => __( "Shortcode Builder", "wpep" ),
        'parent'	=> 'wpep_toolbar',
        'href'    => wpep_admin_url_platform_health( [
          'tab'       => 'tools',
          'tab_inner' => 'shortcode-builder'
        ] )
      ];

    if( current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      $node_list[] = [
        'id'      => 'wpep_toolbar_discussions',
        'title'   =>	__( "Discussions", "wpep" ),
        'parent'	=>	'wpep_toolbar',
        'href'    => wpep_admin_url_discussions_management()
      ];

    if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) )
      $node_list[] = [
        'id'      => 'wpep_toolbar_statistics',
        'title'   =>	__( "Statistics", "wpep" ),
        'parent'	=>	'wpep_toolbar',
        'href'    => wpep_admin_url_statistics()
      ];

    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      $node_list[] = [
        'id'      => 'wpep_toolbar_platform_health',
        'title'   =>	__( "Platform Health", "wpep" ),
        'parent'	=>	'wpep_toolbar',
        'href'    => wpep_admin_url_platform_health()
      ];

    if( empty( $node_list ) )
      return;

    $wp_admin_bar->add_node( [
      'id'     => 'wpep_toolbar',
      'title'  =>	WPEP_PLUGIN_NAME,
      'meta'   => [ 'class' => 'first-toolbar-group' ],
    ] );

    foreach( $node_list as $node )
      $wp_admin_bar->add_node( $node );
  }

}
