<?php

namespace WPEP\Container;

trait DataWalker {

  /**
   * @var array
   */
  public $data_walker_url_map   = [];

  public function data_walker_setup( $replace_attachment_url_map = [] ) {
    $this->data_walker_url_map     = $replace_attachment_url_map;
  }

  /**
   * @param $content_information
   * @param $asset_map
   * @return mixed
   */
  public function data_walker_get_assets( $content_information, $asset_map ) {
    $content_information = is_array( $content_information )
      ? $content_information
      : ( is_string( $content_information ) ? json_decode( $content_information, true ) : get_object_vars( $content_information ) );

    $response = $this->_data_walker_get_assets_from_entry( $content_information, $asset_map );

    return array_unique( $response );
  }

  /**
   * @param $entries
   * @return array
   */
  public function data_walker_replace_assets_from_entries( $entries ) {
    foreach ($entries as $entry_key => $entry) {
      if( is_array( $entry ) ) {
        $entries[ $entry_key ] = $this->data_walker_replace_assets_from_entries( $entry );

        continue;
      }

      $entries[ $entry_key ] = $this->_data_walker_replace_assets_from_string( $entry );
    }

    return $entries;
  }

  /**
   * @param $content
   * @return array
   */
  private function _data_walker_replace_assets_from_string( $content ) {
    if( strpos( $content, 'http' ) === false )
      return $content;

    foreach( $this->data_walker_url_map as $previous_url => $new_url )
      $content = str_replace( $previous_url, $new_url, $content );

    return $content;
  }

  /**
   * @param $entry
   * @param $path_map
   * @return array
   */
  private function _data_walker_get_assets_from_entry( $entry, $path_map ) {
    $response = [];

    if( is_object( $path_map ) )
      $path_map = get_object_vars( $path_map );

    if( !is_array( $path_map ) )
      return $response;

    foreach( $path_map as $path_index => $path ) {
      // We got to a section we care about, we're going to extract the information now.
      if( is_numeric( $path_index ) && isset( $entry[ $path ] ) ) {

        if( is_array( $entry[ $path ] ) ) {
          foreach( $entry[ $path ] as $value )
            $response += wpep_utility_html_find_hosted_assets( $value );
        } else {
          $response += wpep_utility_html_find_hosted_assets( $entry[ $path ] );
        }

        continue;
      }

      // We don't care about this case.
      if( !isset( $entry[ $path_index ] ) )
        continue;

      // We've reached an set of multiple entries, so we navigate accordingly.
      if( is_array( $entry[ $path_index ] ) && is_numeric( key( $entry[ $path_index ] ) ) ) {
        $response += $this->_data_walker_get_assets_from_entries( $entry[$path_index], $path );

        continue;
      }

      // Moving 1 level deeper into the array.
      if( is_array( $path ) ) {
        $response += $this->_data_walker_get_assets_from_entry( $entry[ $path_index ], $path );

        continue;
      }

    }

    return $response;
  }

  /**
   * @param $entries
   * @param $path_map
   * @return array
   */
  public function _data_walker_get_assets_from_entries( $entries, $path_map ) {
    $response = [];

    foreach ($entries as $entry) {
      $response += $this->_data_walker_get_assets_from_entry( $entry, $path_map );
    }

    return $response;
  }
  
}