<?php

namespace WPEP\ContentType\Assessment;

use WPEP\API\Utils\User as API_Utils_User;
use WPEP\API\Utils\Report as API_Utils_Report;
use WPEP\API\Utils\Post as API_Utils_Post;

use WPEP\Entity\Assessment as Entity_Assessment;

class API {

  public static function template_interactive_single_content( $request_data ) {
    if( !isset( $request_data[ 'assessment_id' ] ) )
      return false;

    $assessment = get_post( intval( $request_data[ 'assessment_id' ] ) );

    if( $assessment === null )
      return false;

    $assessment_request_args = ( isset( $request_data[ 'assessment_request_args'] ) ? $request_data[ 'assessment_request_args'] : [] );

    $request = new Delivery\Request( $assessment->ID, get_current_user_id(), $assessment_request_args );
    
    $response = $request->serve( wpep_content_type_assessment_instance()->delivery->get_current_request_type( $request_data ), [
      'answers'           => isset( $request_data['wpep_assessment_answer'] ) ? $request_data['wpep_assessment_answer'] : [],
      'current_step'      => isset( $request_data['current_step'] ) ? $request_data['current_step'] : 0,
      'display_questions' => ( isset( $request_data['display-questions'] ) && $request_data['display-questions'] == 1 )
    ] );

    while( $response['type'] != 'view_assessment' ) {
      if( $response['type'] == 'switch_assessment' )
        $request = new Delivery\Request( $response[ 'assessment_id' ], get_current_user_id() );

      if( in_array( $response[ 'type' ], [ 'switch_assessment', 'refresh_assessment' ] ) ) {
        $response = $request->serve( 'view-assessment', [
          'current_step'  =>  ( isset( $response['step'] ) ? $response['step'] : 0 )
        ] );
      }
    }

    remove_all_actions( 'wpep_before_main_content' );
    remove_all_actions( 'wpep_after_main_content' );
    add_action( 'wpep_single_assessment_content', [ $request, 'display_content' ], 10 );

    $response = [];

    $response['assessment_id'] = $assessment->ID;
    $response['content']       = wpep_template_get( 'content-assessment-single.php', [
      'assessment_id'           => $assessment->ID,
      'include_header'          => false,
      'include_footer'          => false,
      'assessment_request_args' => $assessment_request_args
    ] );
    $response['user_status'] = wpep_user_assessment_status( $assessment->ID, get_current_user_id() );

    return $response;
  }

  /**
   * @param $request_data
   * @return array|bool
   */
  public static function submission_list( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    $pagination_page  = ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 );
    $pagination_limit = 20;

    $response = [
      'status'          => 'ok',
      'entries_header'  => [
        'status_label'               => __( "Status", "wpep" ),
        'submission_score_formatted' => __( "Score", "wpep" ),
        'submission_answers'         => __( "Answers", "wpep" ),
        'started_at_formatted'       => __( "Started", "wpep" ),
        'completed_at_formatted'     => __( "Submitted", "wpep" ),
      ],
      'entries'         => [],
      'pagination'      => [
        'current'       => $pagination_page,
        'total'         => 0,
        'total_entries' => 0
      ]
    ];
    $assessment_status = ( isset( $request_data[ WPEP_USER_ASSESSMENT_STATUS ] ) ? $request_data[ WPEP_USER_ASSESSMENT_STATUS ] : false );

    if( !in_array( $assessment_status, [
      false,
      WPEP_USER_ASSESSMENT_STATUS_PENDING,
      WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW,
      WPEP_USER_ASSESSMENT_STATUS_COMPLETED,
      WPEP_USER_ASSESSMENT_STATUS_FAILED
    ] ) )
      return $response;

    if( isset( $request_data[ 'user_id' ] ) ) {
      $request_data[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $request_data[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    if( isset( $request_data[ 'post_id' ] ) ) {
      $request_data[ 'post_ids' ] = intval( $request_data[ 'post_id' ] );
    } elseif( isset( $request_data[ 'post_ids' ] ) )
      $request_data[ 'post_ids' ] = ( is_array( $request_data[ 'post_ids' ] ) ? array_map( 'intval', $request_data[ 'post_ids' ] ) : intval( $request_data[ 'post_ids' ] ) );

    $submission_query_sql = 'SELECT DISTINCT response_l_0.`post_id`,  
                                    response_l_0.`user_id`
                               FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA . '` response_l_0';

    $submission_query_sql .= ' INNER JOIN `' . wpep_wpdb()->posts . '` response_p_0 
                                       ON response_p_0.`ID` = response_l_0.post_id
                                      AND response_p_0.`post_status` IN ( "' . implode( '","', wpep_post_monitored_post_status_list() ) . '" )';

    $submission_query_sql .=' WHERE response_l_0.`question_id` = 0
                                AND response_l_0.`question_answer_id` = 0
                                AND ( response_l_0.`key` = "' . WPEP_USER_ASSESSMENT_START_TIME. '" OR response_l_0.`key` = "' . WPEP_USER_ASSESSMENT_SUBMISSION_TIME. '" )';

    if( isset( $request_data[ 'report_time_from' ] ) )
      $submission_query_sql .= ' AND CAST( response_l_0.`value` AS UNSIGNED ) >= ' . intval( $request_data[ 'report_time_from' ] );

    if( isset( $request_data[ 'report_time_to' ] ) )
      $submission_query_sql .= ' AND CAST( response_l_0.`value` AS UNSIGNED ) < ' . intval( $request_data[ 'report_time_to' ] );

    if( isset( $request_data[ 'user_ids' ] ) ) {
      if( is_numeric( $request_data[ 'user_ids' ] ) )
        $submission_query_sql .= ' AND response_l_0.`user_id` = ' . intval( $request_data[ 'user_ids' ] );
      else if( is_array( $request_data[ 'user_ids' ] ) )
        $submission_query_sql .= ' AND response_l_0.`user_id` IN (' . implode( ',', $request_data[ 'user_ids' ] ) . ')';
    }

    if( isset( $request_data[ 'post_ids' ] ) ) {
      if( is_numeric( $request_data[ 'post_ids' ] ) )
        $submission_query_sql .= ' AND response_p_0.`ID` = ' . intval( $request_data[ 'post_ids' ] );
      else if( is_array( $request_data[ 'post_ids' ] ) )
        $submission_query_sql .= ' AND response_p_0.`ID` IN (' . implode( ',', $request_data[ 'post_ids' ] ) . ')';
    }

    $submission_query_sql = 'SELECT SQL_CALC_FOUND_ROWS
                                    response_inner.`post_id`,
                                    response_inner.`user_id`,
                                    IFNULL( join_post_assessment_type.`meta_value`, "' . WPEP_ASSESSMENT_TYPE_STANDARD . '" ) as assessment_type,
                                    join_ud_status.`value` as status,
                                    IFNULL( CAST( join_ud_score.`value` AS UNSIGNED ), 0 ) as score,
                                    join_ud_start_time.`value` as submission_start_time,
                                    IFNULL( join_ud_submission_time.`value`, 0 ) as submission_end_time,
                                    
                                    ( SELECT COUNT( DISTINCT inner_select_ud_answer.`question_id` ) 
                                        FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA . '` inner_select_ud_answer
                                       WHERE inner_select_ud_answer.`post_id` = response_inner.`post_id`
                                         AND inner_select_ud_answer.`user_id` = response_inner.`user_id`
                                         AND inner_select_ud_answer.`key` = "answer"
                                    ) as submission_answer_count,
                                    
                                    IFNULL( ( SELECT SUM( CAST( inner_select_ud_is_correct.`value` as SIGNED ) ) 
                                        FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA . '` inner_select_ud_is_correct
                                       WHERE inner_select_ud_is_correct.`post_id` = response_inner.`post_id`
                                         AND inner_select_ud_is_correct.`user_id` = response_inner.`user_id`
                                         AND inner_select_ud_is_correct.`key` = "is_correct"
                                    ), 0 ) as submission_answer_correct_count
                                    
                               FROM ( ' . $submission_query_sql . ' ) response_inner
                         INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA . '` join_ud_status
                                 ON join_ud_status.`post_id` = response_inner.`post_id`
                                AND join_ud_status.`user_id` = response_inner.`user_id`
                                AND join_ud_status.`key` = "' . WPEP_USER_ASSESSMENT_STATUS . '"
                         INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA . '` join_ud_start_time
                                 ON join_ud_start_time.`post_id` = response_inner.`post_id`
                                AND join_ud_start_time.`user_id` = response_inner.`user_id`
                                AND join_ud_start_time.`key` = "' . WPEP_USER_ASSESSMENT_START_TIME . '"
                          LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA . '` join_ud_submission_time
                                 ON join_ud_submission_time.`post_id` = response_inner.`post_id`
                                AND join_ud_submission_time.`user_id` = response_inner.`user_id`
                                AND join_ud_submission_time.`key` = "' . WPEP_USER_ASSESSMENT_SUBMISSION_TIME . '"
                          LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA . '` join_ud_score
                                 ON join_ud_score.`post_id` = response_inner.`post_id`
                                AND join_ud_score.`user_id` = response_inner.`user_id`
                                AND join_ud_score.`key` = "' . WPEP_USER_ASSESSMENT_SCORE . '"
                          LEFT JOIN `' . wpep_wpdb()->postmeta . '` join_post_assessment_type
                                 ON join_post_assessment_type.`post_id` = response_inner.`post_id`
                                AND join_post_assessment_type.`meta_key` = "wpep_assessment_type"';

    if( $assessment_status !== false )
      $submission_query_sql .= ' WHERE join_ud_status.`value` = "' . $assessment_status . '"';

    $submission_query_sql .= ' ORDER BY CAST( join_ud_start_time.`value` as UNSIGNED ) DESC
                                  LIMIT ' . $pagination_limit . ( $pagination_page === 1 ? '' : ' OFFSET ' . ( ( $pagination_page - 1 ) * $pagination_limit) );

    $submissions = wpep_wpdb()->get_results( $submission_query_sql );

    if( empty( $submissions ) )
      return $response;

    $response[ 'pagination' ][ 'total_entries' ] = absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) );
    $response[ 'pagination' ][ 'total' ] = ceil( $response[ 'pagination' ][ 'total_entries' ] / $pagination_limit );

    $status_to_notification = [
      WPEP_USER_ASSESSMENT_STATUS_PENDING        => 'none',
      WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW => 'warning',
      WPEP_USER_ASSESSMENT_STATUS_COMPLETED      => 'success',
      WPEP_USER_ASSESSMENT_STATUS_FAILED         => 'danger',
    ];

    foreach( $submissions as $submission )
      $response[ 'entries' ][] = [
        'user_id' => $submission->user_id,
        'post_id' => $submission->post_id,
        'assessment_status'          => $submission->status,
        'status'                     => $status_to_notification[ $submission->status ],
        'status_label'               => wpep_get_setting_translated( 'assessment-status-label-' . $submission->status ),
        'assessment_type'            => $submission->assessment_type,
        'submission_answers'         => (
          $submission->assessment_type === WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING
            ? sprintf( __( "%s answers", "wpep" ), $submission->submission_answer_count )
            : ( in_array( $submission->status, [ WPEP_USER_ASSESSMENT_STATUS_PENDING, WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW ] ) ? '-' : sprintf( __( "%s out of %s correct", "wpep" ), $submission->submission_answer_correct_count, $submission->submission_answer_count ) )
        ),
        'submission_score'           => $submission->score,
        'submission_score_formatted' => (
          $submission->assessment_type === WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING
            ? number_format( $submission->score,2 )
            : wpep_get_percentage( $submission->score )
        ),
        'started_at'                 => $submission->submission_start_time,
        'started_at_formatted'       => wpep_format_date_with_time( $submission->submission_start_time ),
        'completed_at'               => $submission->submission_end_time,
        'completed_at_formatted'     => intval( $submission->submission_end_time ) === 0 ? '-' : wpep_format_date_with_time( $submission->submission_end_time )
      ];

    $response = API_Utils_Post::entries( $response, $request_data );


    if( isset( $request_data[ '_ignore_api_utils_user' ] ) && $request_data[ '_ignore_api_utils_user' ] )
      return $response;

    if( isset( $request_data[ 'user_ids' ] ) && is_numeric( $request_data[ 'user_ids' ] ) )
      return $response;

    $user_ids = wpep_utility_map_to_array_assoc(
      $response[ 'entries' ],
      'user_id',
      'user_id'
    );

    if( empty( $user_ids ) )
      return $response;

    $entries_user_objects = ( isset( $response[ 'entries' ] ) ? wpep_utilities_map_object(
      get_users( [
        'include' => $user_ids
      ] ),
      'ID'
    ) : [] );

    return API_Utils_User::entries( $response, $request_data, $entries_user_objects );
  }

  /**
   * This is not the cleanest way to implement this feature, but regardless, it works.
   * @param $request_data
   * @return array|mixed
   */
  public static function submission_user_overview_list( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    $pagination_page  = ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 );
    $pagination_limit = 20;

    $response = [
      'status'          => 'ok',
      'entries_header'  => [
        'entry_count_' . WPEP_USER_ASSESSMENT_STATUS_PENDING        => sprintf( __( "Assessments %s", "wpep" ), wpep_get_setting_translated( 'assessment-status-label-' . WPEP_USER_ASSESSMENT_STATUS_PENDING ) ),
        'entry_count_' . WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW => sprintf( __( "Assessments %s", "wpep" ), wpep_get_setting_translated( 'assessment-status-label-' . WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW ) ),
        'entry_count_' . WPEP_USER_ASSESSMENT_STATUS_COMPLETED      => sprintf( __( "Assessments %s", "wpep" ), wpep_get_setting_translated( 'assessment-status-label-' . WPEP_USER_ASSESSMENT_STATUS_COMPLETED ) ),
        'entry_count_' . WPEP_USER_ASSESSMENT_STATUS_FAILED         => sprintf( __( "Assessments %s", "wpep" ), wpep_get_setting_translated( 'assessment-status-label-' . WPEP_USER_ASSESSMENT_STATUS_FAILED ) ),
      ],
      'entries'         => [],
      'pagination'      => [
        'current'       => $pagination_page,
        'total'         => 0,
        'total_entries' => 0
      ]
    ];

    if( isset( $request_data[ 'user_id' ] ) ) {
      $request_data[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $request_data[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    if( isset( $request_data[ 'post_id' ] ) ) {
      $request_data[ 'post_ids' ] = intval( $request_data[ 'post_id' ] );
    } elseif( isset( $request_data[ 'post_ids' ] ) )
      $request_data[ 'post_ids' ] = ( is_array( $request_data[ 'post_ids' ] ) ? array_map( 'intval', $request_data[ 'post_ids' ] ) : intval( $request_data[ 'post_ids' ] ) );

    $entries_queried_sql = 'SELECT DISTINCT response_l_0.`post_id`,  
                                    response_l_0.`user_id`
                               FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA . '` response_l_0';

    $entries_queried_sql .= ' INNER JOIN `' . wpep_wpdb()->posts . '` response_p_0 
                                       ON response_p_0.`ID` = response_l_0.post_id
                                      AND response_p_0.`post_status` IN ( "' . implode( '","', wpep_post_monitored_post_status_list() ) . '" )';

    $entries_queried_sql .=' WHERE response_l_0.`question_id` = 0
                                AND response_l_0.`question_answer_id` = 0
                                AND ( response_l_0.`key` = "' . WPEP_USER_ASSESSMENT_START_TIME. '" OR response_l_0.`key` = "' . WPEP_USER_ASSESSMENT_SUBMISSION_TIME. '" )';

    if( isset( $request_data[ 'report_time_from' ] ) )
      $entries_queried_sql .= ' AND CAST( response_l_0.`value` AS UNSIGNED ) >= ' . intval( $request_data[ 'report_time_from' ] );

    if( isset( $request_data[ 'report_time_to' ] ) )
      $entries_queried_sql .= ' AND CAST( response_l_0.`value` AS UNSIGNED ) < ' . intval( $request_data[ 'report_time_to' ] );

    if( isset( $request_data[ 'post_ids' ] ) ) {
      if( is_numeric( $request_data[ 'post_ids' ] ) )
        $entries_queried_sql .= ' AND response_p_0.`ID` = ' . intval( $request_data[ 'post_ids' ] );
      else if( is_array( $request_data[ 'post_ids' ] ) )
        $entries_queried_sql .= ' AND response_p_0.`ID` IN (' . implode( ',', $request_data[ 'post_ids' ] ) . ')';
    }

    $user_ids_queried_sql = 'SELECT SQL_CALC_FOUND_ROWS DISTINCT response_inner.`user_id` FROM ( ' . $entries_queried_sql;

    if( isset( $request_data[ 'user_ids' ] ) ) {
      if( is_numeric( $request_data[ 'user_ids' ] ) )
        $user_ids_queried_sql .= ' AND response_l_0.`user_id` = ' . intval( $request_data[ 'user_ids' ] );
      else if( is_array( $request_data[ 'user_ids' ] ) )
        $user_ids_queried_sql .= ' AND response_l_0.`user_id` IN (' . implode( ',', $request_data[ 'user_ids' ] ) . ')';
    }

    $user_ids_queried_sql .= ' ) response_inner
                         INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA . '` join_ud_start_time
                                 ON join_ud_start_time.`post_id` = response_inner.`post_id`
                                AND join_ud_start_time.`user_id` = response_inner.`user_id`
                                AND join_ud_start_time.`key` = "' . WPEP_USER_ASSESSMENT_START_TIME . '"
                          LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA . '` join_ud_submission_time
                                 ON join_ud_submission_time.`post_id` = response_inner.`post_id`
                                AND join_ud_submission_time.`user_id` = response_inner.`user_id`
                                AND join_ud_submission_time.`key` = "' . WPEP_USER_ASSESSMENT_SUBMISSION_TIME . '"';

    $user_ids_queried_sql .= ' LIMIT ' . $pagination_limit . ( $pagination_page === 1 ? '' : ' OFFSET ' . ( ( $pagination_page - 1 ) * $pagination_limit) );

    $user_ids = wpep_wpdb()->get_col( $user_ids_queried_sql );

    if( empty( $user_ids ) )
      return $response;

    $response[ 'pagination' ][ 'total_entries' ] = absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) );
    $response[ 'pagination' ][ 'total' ] = ceil( $response[ 'pagination' ][ 'total_entries' ] / $pagination_limit );

    foreach( $user_ids as $user_id ) {
      $user_assessment_post_ids = wpep_wpdb()->get_col( $entries_queried_sql . ' AND response_l_0.`user_id` = ' . $user_id );

      $status_count_map = [
        WPEP_USER_ASSESSMENT_STATUS_PENDING        => 0,
        WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW => 0,
        WPEP_USER_ASSESSMENT_STATUS_COMPLETED      => 0,
        WPEP_USER_ASSESSMENT_STATUS_FAILED         => 0,
      ];

      $submission_entries = [
        'user_id' => $user_id
      ];

      foreach( $user_assessment_post_ids as $user_assessment_post_id )
        $status_count_map[ wpep_user_assessment_status( $user_assessment_post_id, $user_id ) ]++;

      foreach( $status_count_map as $status_key => $status_count )
        $submission_entries[ 'entry_count_' . $status_key ] = '' . $status_count;

      $response['entries'][] = $submission_entries;
    }

    if( isset( $request_data[ 'user_ids' ] ) && is_numeric( $request_data[ 'user_ids' ] ) )
      return $response;

    $user_ids = wpep_utility_map_to_array_assoc(
      $response[ 'entries' ],
      'user_id',
      'user_id'
    );

    if( empty( $user_ids ) )
      return $response;

    $entries_user_objects = ( isset( $response[ 'entries' ] ) ? wpep_utilities_map_object(
      get_users( [
        'include' => $user_ids
      ] ),
      'ID'
    ) : [] );

    return API_Utils_User::entries( $response, $request_data, $entries_user_objects );
  }

  public static function post_id_submission_answers_list( $request_data ) {
    $request_data[ 'post_ids' ] = intval( $request_data[ 'post_id' ] );

    $request_data[ '_ignore_api_utils_user' ] = true;

    unset( $request_data[ 'post_id' ] );

    $response = self::submission_list( $request_data );

    $user_ids = wpep_utility_map_to_array_assoc(
      $response[ 'entries' ],
      'user_id',
      'user_id'
    );

    if( empty( $user_ids ) )
      return $response;

    $questions_mapped_by_id = wpep_utilities_map_object( Entity_Assessment::instance()->get_questions_by_assessment_id( $request_data[ 'post_ids' ], false, false ), 'id' );

    $append_entries_header = [];

    foreach( $questions_mapped_by_id as $question_index => $question )
      $append_entries_header[ 'assessment_question_' . $question->id ] = $question->title;

    $response[ 'entries_header' ] = $append_entries_header + $response[ 'entries_header' ];

    foreach( $response[ 'entries' ] as $entry_key => $entry ) {
      $user_answers = wpep_content_type_assessment_instance()->user->get_answers_text_by_assessment_id_and_user_id( $request_data[ 'post_ids' ], $entry[ 'user_id' ] );

      foreach( $questions_mapped_by_id as $question ) {
        if( !isset( $user_answers[ $question->id ] ) ) {
          $response[ 'entries' ][ $entry_key ][ 'assessment_question_' . $question->id ] = '-';
          continue;
        }

        $question_user_answers = $user_answers[ $question->id ];

        if( is_array( $question_user_answers ) ) {
          $response[ 'entries' ][ $entry_key ][ 'assessment_question_' . $question->id ] = implode( "\n", $question_user_answers );

          continue;
        }

        $response[ 'entries' ][ $entry_key ][ 'assessment_question_' . $question->id ] = $question_user_answers;
      }
    }

    $entries_user_objects = ( isset( $response[ 'entries' ] ) ? wpep_utilities_map_object(
      get_users( [
        'include' => $user_ids
      ] ),
      'ID'
    ) : [] );

    return API_Utils_User::entries( $response, $request_data, $entries_user_objects );
  }

  public static function user_id_post_id_data_view_plain( $request_data ) {
    $request_data[ 'user_id' ] = intval( $request_data[ 'user_id' ] );
    $request_data[ 'post_id' ] = intval( $request_data[ 'post_id' ] );

    $user_information = get_user_by( 'ID', $request_data[ 'user_id' ] );

    $assessment_title = get_the_title( $request_data[ 'post_id' ] );
    $assessment_type  = wpep_get_assessment_type( $request_data[ 'post_id' ] );

    $questions = Entity_Assessment::instance()->get_questions_by_assessment_id( $request_data[ 'post_id' ], true, false );

    $user_answers = wpep_content_type_assessment_instance()->user->get_answers_text_by_assessment_id_and_user_id( $request_data[ 'post_id' ], $request_data[ 'user_id' ] );

    $question_answer_status_list = wpep_utility_map_to_array_assoc( wpep_content_type_assessment_instance()->user->data->get(
      'is_correct', $request_data[ 'post_id' ], false, false, $request_data[ 'user_id' ], false
    ), 'question_id', 'value', true );

    $status = wpep_user_assessment_status( $request_data[ 'post_id' ], $request_data[ 'user_id' ] );

    $score = wpep_content_type_assessment_instance()->user->data->get_value(
      WPEP_USER_ASSESSMENT_SCORE, $request_data[ 'post_id' ], 0, 0, $request_data[ 'user_id' ]
    );

    $start_time = wpep_content_type_assessment_instance()->user->data->get_value(
      WPEP_USER_ASSESSMENT_START_TIME, $request_data[ 'post_id' ], 0, 0, $request_data[ 'user_id' ]
    );
    $submitted_time = wpep_content_type_assessment_instance()->user->data->get_value(
      WPEP_USER_ASSESSMENT_SUBMISSION_TIME, $request_data[ 'post_id' ], 0, 0, $request_data[ 'user_id' ]
    );

    return [
      'status'  => 'ok',
      'html'    => wpep_template_get( 'admin/data-view-plain-user-assessment.php', [
        'user_information' => $user_information,

        'assessment_id'    => $request_data[ 'post_id' ],
        'user_id'          => $request_data[ 'user_id' ],

        'assessment_title' => $assessment_title,
        'assessment_type'  => $assessment_type,
        'questions'        => $questions,
        'user_answers'     => $user_answers,
        'question_answer_status_list' => $question_answer_status_list,
        'status'           => $status,
        'score'            => $score,
        'start_time'       => $start_time,
        'submitted_time'   => $submitted_time
      ] )
    ];
  }
}