<?php

namespace WPEP\ContentType\Assessment\Admin;

class Settings {

  public function __construct() {
    add_filter( 'wpep_administration_settings_map', [ $this, '_admin_settings_map' ], 25 );
  }

  public function _admin_settings_map( $settings_map ) {
    if( isset( $settings_map[ 'general' ][ 'sections' ]['url'][ 'field_map' ][ 'permalink_name' ][ 'fields' ] ) ) {
      $settings_map[ 'general' ][ 'sections' ]['url'][ 'field_map' ][ 'permalink_name' ][ 'fields' ][ 'content-type-assessment-rewrite-slug' ] = [
        'name'        => __( 'Assessment', "wpep" ),
        'alias'       => 'content-type-assessment-rewrite-slug',
        'type'        => 'text',
        'description' => __( "The permalink prefix goes here, by default wpep_assessment", "wpep" )
      ];
    }

    if( !isset( $settings_map[ 'course_page' ][ 'sections' ] ) )
      return $settings_map;

    $settings_map[ 'course_page' ][ 'sections' ][ 'assessments' ] = [
      'name'      => __( "Assessments", "wpep" ),
      'alias'     => 'assessments',
      'field_map' => [
        'general' => [
          'name'      => __( "General", "wpep" ),
          'has_title' => false,
          'alias'     => 'general',
          'fields'    => [
            'assessment-course-page-embed' => [
              'name'        => __( 'Streamlined Experience', "wpep"),
              'alias'       => 'assessment-course-page-embed',
              'type'        => 'checkbox',
              'description' => __( "Students will not leave the course page in order to complete assessments", "wpep" )
            ],
            'assessment-course-protection-rules' => [
              'name'        => __('Enable Protection', "wpep"),
              'alias'       => 'assessment-course-protection-rules',
              'type'        => 'checkbox',
              'description' => __( 'Enabling this option will protect the Assessment Page as the Course Page, disable this option if you want to configure each assessment protection rules individually.', "wpep" )
            ]
          ]
        ],
        'simple_scoring' => [
          'name'      => __( "Simple Scoring", "wpep" ),
          'has_title' => true,
          'alias'     => 'simple_scoring',
          'fields'    => [
            'assessment-simple-score' => [
              'name'        => __( 'Score', "wpep"),
              'alias'       => 'assessment-simple-score',
              'type'        => 'checkbox'
            ]
          ]
        ],
        'interaction' => [
          'name'      => __( "Interaction", "wpep" ),
          'has_title' => true,
          'alias'     => 'interaction',
          'fields'    => [
            'assessment' => [
              'name'        => __( 'Assessment', "wpep"),
              'alias'       => 'assessment',
              'type'        => 'text_translated'
            ],
            'assessment-view' => [
              'name'        => __( 'View Assessment', "wpep"),
              'alias'       => 'assessment-view',
              'type'        => 'text_translated'
            ],
            'assessment-input-placeholder' => [
              'name'        => __( 'Input Placeholder', "wpep"),
              'alias'       => 'assessment-input-placeholder',
              'type'        => 'text_translated'
            ],
            'assessment-score' => [
              'name'        => __( 'Assessment Score', "wpep"),
              'alias'       => 'assessment-score',
              'type'        => 'text_translated'
            ],
            'assessment-score-requirement' => [
              'name'        => __( 'Required to Pass', "wpep"),
              'alias'       => 'assessment-score-requirement',
              'type'        => 'text_translated'
            ],
            'assessment-questions-correct' => [
              'name'        => __( 'Questions Correct', "wpep"),
              'alias'       => 'assessment-questions-correct',
              'type'        => 'text_translated'
            ],
            'assessment-file-upload' => [
              'name'        => __( "File Upload", "wpep" ),
              'alias'       => 'assessment-file-upload',
              'type'        => 'text_translated'
            ],
            'assessment-part-prev' => [
              'name'        => __( "Paginated Prev", "wpep" ),
              'alias'       => 'assessment-part-prev',
              'type'        => 'text_translated'
            ],
            'assessment-part-next' => [
              'name'        => __( "Paginated Next", "wpep" ),
              'alias'       => 'assessment-part-next',
              'type'        => 'text_translated'
            ],
            'assessment-save' => [
              'name'        => __( "Save Answers", "wpep" ),
              'alias'       => 'assessment-save',
              'type'        => 'text_translated'
            ],
            'assessment-submit' => [
              'name'        => __( "Submit Answers", "wpep" ),
              'alias'       => 'assessment-submit',
              'type'        => 'text_translated'
            ],
            'assessment-submit-singular' => [
              'name'        => __( "Submit Answer", "wpep" ),
              'alias'       => 'assessment-submit-singular',
              'type'        => 'text_translated'
            ],
            'assessment-go-to-course' => [
              'name'        => __( "Continue to Course", "wpep" ),
              'alias'       => 'assessment-go-to-course',
              'type'        => 'text_translated'
            ],
            'assessment-return-to-course' => [
              'name'        => __( "Return to Course", "wpep" ),
              'alias'       => 'assessment-return-to-course',
              'type'        => 'text_translated'
            ]
          ]
        ],
        'completed_assessment' => [
          'name'      => __( "Completed Assessment", "wpep" ),
          'has_title' => true,
          'alias'     => 'completed_assessment',
          'fields'    => [
            'assessment-success-message' => [
              'name'        => __( 'Default Completed Message', "wpep"),
              'alias'       => 'assessment-success-message',
              'type'        => 'editor'
            ],
            'assessment-questions' => [
              'name'        => __( "Questions", "wpep" ),
              'alias'       => 'assessment-questions',
              'type'        => 'text_translated'
            ],
            'assessment-answer-breakdown-separator' => [
              'name'        => __( 'Of', "wpep" ),
              'alias'       => 'assessment-answer-breakdown-separator',
              'type'        => 'text_translated'
            ],
            'assessment-view-questions' => [
              'name'        => __( "View Questions", "wpep" ),
              'alias'       => 'assessment-view-questions',
              'type'        => 'text_translated'
            ],
            'assessment-retake' => [
              'name'        => __( "Retake Assessment", "wpep" ),
              'alias'       => 'assessment-retake',
              'type'        => 'text_translated'
            ]
          ]
        ],
        'time_remaining' => [
          'name'      => __( "Time Remaining Settings", "wpep" ),
          'has_title' => true,
          'alias'     => 'time_remaining',
          'fields'    => [
            'assessment-time-allowed' => [
              'name'        => __( "Time Allowed", "wpep" ),
              'alias'       => 'assessment-time-allowed',
              'type'        => 'text_translated'
            ],
            'assessment-time-remaining' => [
              'name'        => __( "Countdown Text", "wpep" ),
              'alias'       => 'assessment-time-remaining',
              'type'        => 'text_translated'
            ],
            'assessment-time-text' => [
              'name'        => __( "Introduction Text", "wpep" ),
              'alias'       => 'assessment-time-text',
              'type'        => 'editor'
            ],
            'assessment-time-start' => [
              'name'        => __( "Start Timed Assessment", "wpep" ),
              'alias'       => 'assessment-time-start',
              'type'        => 'text_translated'
            ]
          ]
        ],
        'status' => [
          'name'      => __( "Status", "wpep" ),
          'has_title' => true,
          'alias'     => 'status',
          'fields'    => [
            'assessment-status' => [
              'name'        => __( "Status", "wpep" ),
              'alias'       => 'assessment-status',
              'type'        => 'text_translated'
            ],
            'assessment-status-label-pending' => [
              'name'        => __( "Status Pending", "wpep" ),
              'alias'       => 'assessment-status-label-pending',
              'type'        => 'text_translated'
            ],
            'assessment-status-label-pending_review' => [
              'name'        => __( "Status Pending Review", "wpep" ),
              'alias'       => 'assessment-status-label-pending_review',
              'type'        => 'text_translated'
            ],
            'assessment-status-label-completed' => [
              'name'        => __( "Status Completed", "wpep" ),
              'alias'       => 'assessment-status-label-completed',
              'type'        => 'text_translated'
            ],
            'assessment-status-label-failed' => [
              'name'        => __( "Status Failed", "wpep" ),
              'alias'       => 'assessment-status-label-failed',
              'type'        => 'text_translated'
            ],
          ]
        ],
        'status_notification' => [
          'name'      => __( "Status Notification", "wpep" ),
          'has_title' => true,
          'alias'     => 'status_notification',
          'fields'    => [
            'assessment-status-pending' => [
              'name'        => __( "Pending", "wpep" ),
              'alias'       => 'assessment-status-pending',
              'type'        => 'text_translated'
            ],
            'assessment-status-pending-review' => [
              'name'        => __( "Pending Review", "wpep" ),
              'alias'       => 'assessment-status-pending-review',
              'type'        => 'text_translated'
            ],
            'assessment-status-completed' => [
              'name'        => __( "Completed", "wpep" ),
              'alias'       => 'assessment-status-completed',
              'type'        => 'text_translated'
            ],
            'assessment-status-failed' => [
              'name'        => __( "Failed", "wpep" ),
              'alias'       => 'assessment-status-failed',
              'type'        => 'text_translated'
            ],
            'assessment-question-status-correct' => [
              'name'        => __( "Question Correct", "wpep" ),
              'alias'       => 'assessment-question-status-correct',
              'type'        => 'text_translated'
            ],
            'assessment-question-status-incorrect' => [
              'name'        => __( "Question Incorrect", "wpep" ),
              'alias'       => 'assessment-question-status-incorrect',
              'type'        => 'text_translated'
            ],
          ]
        ],
        'status_notification_assignment' => [
          'name'      => __( "Assignment Status Notification", "wpep" ),
          'has_title' => true,
          'alias'     => 'status_notification_assignment',
          'fields'    => [
            'assessment-assignment-status-pending-review' => [
              'name'        => __( "Pending Review", "wpep" ),
              'alias'       => 'assessment-status-pending-review',
              'type'        => 'text_translated'
            ],
            'assessment-assignment-status-completed' => [
              'name'        => __( "Completed", "wpep" ),
              'alias'       => 'assessment-status-completed',
              'type'        => 'text_translated'
            ],
            'assessment-assignment-status-failed' => [
              'name'        => __( "Failed", "wpep" ),
              'alias'       => 'assessment-status-failed',
              'type'        => 'text_translated'
            ],
          ]
        ],
        'manual_review_email_notification' => [
          'name'      => __( "Manual Review Email Notification", "wpep" ),
          'has_title' => true,
          'alias'     => 'manual_review_email_notification',
          'fields'    => [
            'assessment-review-completed-email-notification-status' => [
              'name'        => __( "Review Email Notification Status", "wpep" ),
              'alias'       => 'assessment-review-completed-email-notification-status',
              'type'        => 'checkbox',
              'description' => __( "This email will be sent to the student when the assessment has been manually reviewed from the administration panel.", "wpep" )
            ],
            'assessment-review-completed-email-notification-title' => [
              'name'        => __( "Email Title", "wpep" ),
              'alias'       => 'assessment-review-completed-email-notification-title',
              'type'        => 'text_translated'
            ],
            'assessment-review-completed-email-notification-content' => [
              'name'        => __( "Email Content", "wpep" ),
              'alias'       => 'assessment-review-completed-email-notification-content',
              'type'        => 'editor',
              'description' => function() {
                return wpep_controller()->userEngagement->assessment->get_available_shortcode_list() .
                       wpep_controller()->userEngagement->get_available_shortcode_list( false );
              }
            ],
          ]
        ],
        'teacher_notifications' => [
          'name'      => __( "Teacher Notifications", "wpep" ),
          'has_title' => true,
          'alias'     => 'teacher_notifications',
          'fields'    => [
            'assessment-admin-email-notification-deliver-to' => [
              'name'        => __( "Deliver To", "wpep" ),
              'alias'       => 'assessment-admin-email-notification-deliver-to',
              'type'        => 'select',
              'options'     => function() {
                return [
                    0 =>  __( "Don't send Email Notifications", "wpep" )
                  ] + wpep_get_users_by_role_as_array_assoc( [ WPEP_USER_ROLE_AUDIENCE_MANAGER, WPEP_USER_ROLE_PLATFORM_MANAGER, 'administrator' ], 'ID', 'user_email' );
              },
              'description' => __( "The Receiver must have an admin account on the site, on one of the following roles : Administrator Audience Manager or Platform Manager.", "wpep" )
            ],
            'assessment-admin-email-notification-title' => [
              'name'        => __( "Email Title", "wpep" ),
              'alias'       => 'assessment-admin-email-notification-title',
              'type'        => 'text_translated'
            ],
            'assessment-admin-email-notification-content' => [
              'name'        => __( "Email Content", "wpep" ),
              'alias'       => 'assessment-admin-email-notification-content',
              'type'        => 'editor',
              'description' => function() {
                return wpep_controller()->userEngagement->assessment->get_available_shortcode_list() .
                       wpep_controller()->userEngagement->get_available_shortcode_list( false );
              }
            ],
          ]
        ],
      ]
    ];

    return $settings_map;
  }

}