<?php

namespace WPEP\ContentType\Assessment;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Assessment as EntityAssessment;

class Delivery {

  /**
   * @var Delivery\Course;
   */
  public $course;

  /**
   * @var Delivery\Request;
   */
  public $request;

  public function __construct() {
    add_action( 'init', [ $this, '_wp_init' ] );

    $this->course = new Delivery\Course();
  }

  public function _wp_init() {
    if( wpep_get_current_post_type() != WPEP_POST_TYPE_ASSESSMENT )
      return;

    $assessment_id = wpep_get_current_post_id();

    if( $assessment_id === 0 )
      return;

    $course_id = wpep_get_assessment_course_id( $assessment_id );

    if( wpep_get_setting_flag( 'assessment-course-protection-rules' ) && $course_id != 0 ) {
      if( wpep_user_course_started_at( $course_id, get_current_user_id() ) === 0 || !WPEP_Controller::instance()->contentLibrary->has_access_to_post( $course_id ) ) {
        $course_permalink = apply_filters( 'wpep_primary_content_item_link', wpep_course_permalink( $course_id ), $course_id );

        if( $course_permalink == '' || $course_permalink == '#' )
          $course_permalink = wpep_course_permalink( $course_id );

        wp_redirect( $course_permalink );
        exit;
      }
    }

    if( $course_id != 0 && wpep_course_type_from_id( $course_id ) === WPEP_COURSE_TYPE_FOCUS ) {
      $course_permalink = apply_filters( 'wpep_primary_content_item_link', wpep_course_permalink( $course_id ), $course_id );

      if( $course_permalink == '' || $course_permalink == '#' )
        $course_permalink = wpep_course_permalink( $course_id );

      $course_permalink = add_query_arg( 'assessment', $assessment_id, $course_permalink );

      wp_redirect( $course_permalink );
      exit;
    }

    $this->request = new Delivery\Request( $assessment_id, get_current_user_id() );

    $request_serve_data = [
      'answers'           => isset( $_POST['wpep_assessment_answer'] ) ? $_POST['wpep_assessment_answer'] : [],
      'current_step'      => isset( $_GET['step'] ) ? $_GET['step'] : 0,
      'display_questions' => ( isset( $_GET['display-questions'] ) && $_GET['display-questions'] == 1 )
    ];

    if( isset( $_POST['user_start_time'] ) )
      $request_serve_data[ 'user_start_time' ] = intval( $_POST['user_start_time'] );

    $response = $this->request->serve( $this->get_current_request_type( $_POST + $_GET ), $request_serve_data );

    if( $response['type'] == 'switch_assessment' ) {
      wpep_redirect( get_permalink( $response[ 'assessment_id' ] ) );
      exit;
    } elseif( $response['type'] == 'refresh_assessment' ) {
      $permalink = get_permalink( $response[ 'assessment_id' ] );

      if( isset( $response['step'] ) && $response['step'] != 0 )
        $permalink = add_query_arg( 'step', $response['step'], $permalink );

      wpep_redirect( $permalink );
      exit;
    }

    add_action( 'wpep_single_assessment_content', [ $this->request, 'display_content' ], 10 );
  }

  public function get_current_request_type( $request_data ) {
    if( isset( $request_data['retake_assessment_force'] ) && wp_verify_nonce( $request_data['retake_assessment_force'], 'retake_assessment_force_' . get_current_user_id() ) != false )
      return 'retake-assessment';

    if( isset( $request_data['retake_assessment'] ) && wp_verify_nonce( $request_data['retake_assessment'], 'retake_assessment' ) != false )
      return 'retake-assessment';

    if( isset( $request_data['wpep_assessment_submit'] ) )
      return 'verify-assessment';

    if( isset( $request_data['wpep_start_assessment'] ) )
      return 'start-assessment';

    if( isset( $request_data['wpep_prev_part'] ) )
      return 'navigation-previous';

    if( isset( $request_data['wpep_next_part'] ) )
      return 'navigation-next';

    if( isset( $request_data['wpep_assessment_answer'] ) )
      return 'save-assessment';

    return 'view-assessment';
  }

}
