<?php

namespace WPEP\ContentType\Assessment\Delivery;

class Course {

  private $_storage = [];

  public function preload_by_user_id( $course_id, $user_id ) {
    $user_id = ( $user_id == false ? get_current_user_id() : $user_id );

    if( $user_id === 0 )
      return;

    $assessment_ids = $this->get_assessment_ids_by_user_id( $course_id, $user_id );

    if( empty( $assessment_ids ) )
      return;

    wpep_content_type_assessment_instance()->user->data->preload( WPEP_USER_ASSESSMENT_STATUS, $assessment_ids, $user_id );
  }

  /**
   * @param $course_id
   * @param $post_status_list
   * @return array
   */
  public function preload_for_post_status_list( $course_id, $post_status_list ) {
    $post_status_list = ( !is_array( $post_status_list ) ? wpep_post_monitored_post_status_list() : $post_status_list );

    return $this->get_assessment_ids( $course_id, $post_status_list );
  }

  /**
   * @param $course_id
   * @param int $section_id
   * @param int $user_id
   * @return int|mixed
   */
  public function get_assessment_id_by_user_id( $course_id, $section_id = 0, $user_id = 0 ) {
    $user_id         = ( $user_id == false ? get_current_user_id() : $user_id );

    if( $user_id == 0 )
      return 0;

    return $this->get_assessment_id( $course_id, $section_id, $this->_get_assessment_status_list_by_user_id( $user_id ) );
  }

  /**
   * @param $course_id
   * @param $user_id
   * @return array
   */
  public function get_assessment_ids_by_user_id( $course_id, $user_id ) {
    return $this->get_assessment_ids( $course_id, $this->_get_assessment_status_list_by_user_id( $user_id ) );
  }

  /**
   * @param $course_id
   * @param array|null $assessment_status
   * @return array
   */
  public function get_assessment_ids( $course_id, $assessment_status = null ) {
    $assessment_status = ( !is_array( $assessment_status ) ? wpep_post_monitored_post_status_list() : $assessment_status );
    $assessment_status_as_alias = implode( '_', $assessment_status );

    if( isset( $this->_storage[ $course_id . '_' . $assessment_status_as_alias ] ) )
      return $this->_storage[ $course_id . '_' . $assessment_status_as_alias ];

    $sql = 'SELECT ID as assessment_id,
                   ( SELECT apmc.meta_value FROM ' . wpep_wpdb()->postmeta . ' apmc WHERE post_id = apt.ID AND meta_key = "wpep_section_id" ) as section_id
                    FROM ' . wpep_wpdb()->posts . ' apt
                   WHERE post_type = "' . WPEP_POST_TYPE_ASSESSMENT . '"
                     AND post_status ' . ( is_array( $assessment_status ) && !empty( $assessment_status ) ? ' IN ("' . implode( '","', $assessment_status ). '") ' : ' = "' . $assessment_status . '"' ) . '
                     AND ( SELECT apmc.meta_value FROM ' . wpep_wpdb()->postmeta . ' apmc WHERE post_id = apt.ID AND meta_key = "wpep_course_id" ) = ' . intval( $course_id ). '
                ORDER BY post_modified DESC';

    $sql_response = wpep_wpdb()->get_results( $sql );

    if( empty( $sql_response ) ) {
      $this->_storage[ $course_id . '_' . $assessment_status_as_alias ] = [];

      return [];
    }

    $response = [];

    foreach( $sql_response as $sql_datum )
      $response[ $sql_datum->section_id ] = $sql_datum->assessment_id;

    $this->_storage[ $course_id . '_' . $assessment_status_as_alias ] = $response;

    return $response;
  }

  /**
   * @param $course_id
   * @param int $section_id
   * @param array|null $assessment_status
   * @return int|mixed
   */
  public function get_assessment_id( $course_id, $section_id = 0, $assessment_status = null ) {
    $assessment_status = ( !is_array( $assessment_status ) ? wpep_post_monitored_post_status_list() : $assessment_status );
    $assessment_status_as_alias = implode( '_', $assessment_status );

    if( isset( $this->_storage[ $course_id . '_' . $assessment_status_as_alias ] ) ) {
      if( empty( $this->_storage[ $course_id . '_' . $assessment_status_as_alias ] ) )
        return 0;

      if( isset( $this->_storage[ $course_id . '_' . $assessment_status_as_alias ][ $section_id ] ) )
        return $this->_storage[ $course_id . '_' . $assessment_status_as_alias ][ $section_id ];

      return 0;
    }

    $sql = 'SELECT ID FROM ' . wpep_wpdb()->posts . ' apt
                   WHERE post_type = "' . WPEP_POST_TYPE_ASSESSMENT . '"
                     AND post_status ' . ( is_array( $assessment_status ) && !empty( $assessment_status ) ? ' IN ("' . implode( '","', $assessment_status ). '") ' : ' = "' . $assessment_status . '"' ) . '
                     AND ( SELECT apmc.meta_value FROM ' . wpep_wpdb()->postmeta . ' apmc WHERE post_id = apt.ID AND meta_key = "wpep_course_id" ) = ' . intval( $course_id ). '
                     AND ( SELECT apma.meta_value FROM ' . wpep_wpdb()->postmeta . ' apma WHERE post_id = apt.ID AND meta_key = "wpep_section_id" ) = ' . intval( $section_id ) . '
                ORDER BY post_modified DESC LIMIT 1';


    return intval( wpep_wpdb()->get_var( $sql ) );
  }

  private function _get_assessment_status_list_by_user_id( $user_id ) {
    $assessment_status = [ 'publish' ];

    if( user_can( $user_id, WPEP_CAP_EDIT_ASSESSMENT ) )
      $assessment_status[] = 'private';

    return $assessment_status;
  }

}
