<?php

namespace WPEP\ContentType\Assessment\User;

use WPEP\Entity\ActionTracking as Entity_ActionTracking;

class Data {

  private $_storage = [];

  /**
   * @param array|string $keys
   * @param array|int $assessment_ids
   * @param int $user_id
   */
  public function preload( $keys, $assessment_ids, $user_id ) {
    $user_data_list = $this->get( $keys, $assessment_ids, false, false, $user_id, false, false );

    if( !isset( $this->_storage[ $user_id ] ) )
      $this->_storage[ $user_id ] = [];

    $keys = ( !is_array( $keys ) ? [ $keys ] : $keys );
    $assessment_ids = ( !is_array( $assessment_ids ) ? [ $assessment_ids ] : $assessment_ids );

    foreach( $assessment_ids as $assessment_id ) {
      if( !isset( $this->_storage[ $user_id ][ $assessment_id ] ) )
        $this->_storage[ $user_id ][ $assessment_id ] = [];

      foreach( $keys as $key )
        $this->_storage[ $user_id ][ $assessment_id ][ $key ] = null;
    }

    foreach( $user_data_list as $user_data )
      $this->_storage[ $user_id ][ $user_data->post_id ][ $user_data->key ] = $user_data;
  }

  public function get_value(
    $key = false,
    $post_id = false,
    $question_id = false,
    $question_answer_id = false,
    $user_id = false
  ) {
    $user_data = $this->get( $key, $post_id, $question_id, $question_answer_id, $user_id );

    return ( $user_data === null ? false : $user_data->value );
  }

  public function get(
    $key = false,
    $post_id = false,
    $question_id = false,
    $question_answer_id = false,
    $user_id = false,
    $single_row = true,
    $value = false
  ) {
    if ( $single_row ) {
      $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

      if ($user_id == 0 )
        return null;

      if( $post_id !== false && ( $question_id === false || $question_id === 0 ) && ( $question_answer_id === false || $question_answer_id === 0 ) ) {
        if( isset( $this->_storage[ $user_id ][ $post_id ] )
          && array_key_exists( $key, $this->_storage[ $user_id ][ $post_id ] ) )
          return $this->_storage[ $user_id ][ $post_id ][ $key ];
      }
    }

    $sql = 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA . ' aud WHERE 1 = 1 ';

    if( $user_id != 0 )
      $sql .= ' AND aud.user_id ' . ( is_array( $user_id ) ? ' IN (' . implode( ',', array_map( 'intval', $user_id ) ) . ')' : ' = ' . intval( $user_id ) );

    if( $value !== false )
      $sql .= ' AND aud.value = "' . esc_sql( $value ) . '"';

    if( $key !== false )
      $sql .= ' AND aud.key = "' . esc_sql( $key ) . '"';

    if( $post_id !== false )
      $sql .= ' AND aud.post_id ' . ( is_array( $post_id ) ? ' IN (' . implode( ',', array_map( 'intval', $post_id ) ) . ')' : ' = ' . intval( $post_id ) );

    if( $question_id !== false )
      $sql .= ' AND aud.question_id = ' . intval( $question_id );

    if( $question_answer_id !== false )
      $sql .= ' AND aud.question_answer_id  = ' . intval( $question_answer_id );

    if( $single_row ) {
      $response = wpep_wpdb()->get_row( $sql );

      if( $question_id === false && $question_answer_id === false ) {
        if( !isset( $this->_storage[ $user_id ] ) )
          $this->_storage[ $user_id ] = [];

        if( !isset( $this->_storage[ $user_id ][ $post_id ] ) )
          $this->_storage[ $user_id ][ $post_id ] = [];

        $this->_storage[ $user_id ][ $post_id ][ $key ] = $response;
      }

      return $response;
    }

    return wpep_wpdb()->get_results( $sql );
  }

  public function set(
    $key,
    $value,
    $post_id,
    $question_id,
    $question_answer_id,
    $user_id = false
  ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return null;

    $post_id  = intval( $post_id );
    $question_id = intval( $question_id );
    $question_answer_id  = intval( $question_answer_id );

    $value_updated = true;

    $existent_entries = $this->get( $key, $post_id, $question_id, $question_answer_id, $user_id, false );

    if( empty( $existent_entries ) ) {
      $this->add( $key, $value, $post_id, $question_id, $question_answer_id, $user_id );
    } else if( count( $existent_entries ) >= 2 ) {
      $this->delete( $key, $post_id, $question_id, $question_answer_id, $user_id );

      $this->add( $key, $value, $post_id, $question_id, $question_answer_id, $user_id );
    } else {
      $existent_entry = array_shift( $existent_entries );

      if( $existent_entry->value != $value ) {
        wpep_wpdb()->update(
          wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA,
          [ 'value' => $value ],
          [ 'id'    => $existent_entry->id ]
        );
      } else {
        $value_updated = false;
      }
    }

    if( !$value_updated )
      return;

    unset( $this->_storage[ $user_id ][ $post_id ][ $key ] );

    if( $key === 'status' ) {
      if( $value === WPEP_USER_ASSESSMENT_STATUS_PENDING )
        Entity_ActionTracking::instance()->track_event( $post_id, $user_id, WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_STARTED, '' );
      if( $value === WPEP_USER_ASSESSMENT_STATUS_COMPLETED )
        Entity_ActionTracking::instance()->track_event( $post_id, $user_id, WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_COMPLETED, '' );
      if( $value === WPEP_USER_ASSESSMENT_STATUS_FAILED )
        Entity_ActionTracking::instance()->track_event( $post_id, $user_id, WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_FAILED, '' );
      if( $value === WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW )
        Entity_ActionTracking::instance()->track_event( $post_id, $user_id, WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_REVIEW_REQUIRED, '' );
    }

    do_action( "wpep_user_set_assessment_data", $key, $value, $post_id, $question_id, $question_answer_id, $user_id, true );
    do_action( "wpep_user_set_assessment_data_" . $key, $value, $post_id, $question_id, $question_answer_id, $user_id, true );
  }

  public function add(
    $key,
    $value,
    $post_id,
    $question_id = 0,
    $question_answer_id = 0,
    $user_id = false
  ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return;

    $post_id  = intval( $post_id );
    $question_id = intval( $question_id );
    $question_answer_id  = intval( $question_answer_id );

    wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA, [
      'user_id'             => $user_id,
      'post_id'             => $post_id,
      'question_id'         => $question_id,
      'question_answer_id'  => $question_answer_id,
      'key'                 => $key,
      'value'               => $value,
      'created_at'          => date( 'Y-m-d H:i:s' )
    ] );

    do_action( "wpep_user_add_assessment_data", $key, $value, $post_id, $question_id, $question_answer_id, $user_id );
  }

  public function delete(
    $key = false,
    $post_id = false,
    $question_id = false,
    $question_answer_id = false,
    $user_id = false
  ) {
    if ($user_id === false)
      $user_id = get_current_user_id();

    if ($user_id == 0)
      return null;

    $where = [
      'user_id' => intval( $user_id )
    ];

    if( $key !== false )
      $where['key'] = esc_sql( $key );

    if( $post_id !== false )
      $where['post_id'] = intval( $post_id );

    if( $question_id !== false )
      $where['question_id'] = intval( $question_id );

    if( $question_answer_id !== false )
      $where['question_answer_id'] = intval( $question_answer_id );

    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA, $where );

    unset( $this->_storage[ $user_id ][ $post_id ][ $key ] );

    if( $key === false && $question_id === false && $question_answer_id === false && $post_id !== false && $user_id !== false )
      Entity_ActionTracking::instance()->track_event( $post_id, $user_id, WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_RESET, '' );

    do_action( "wpep_user_delete_assessment_data", $key, $post_id, $question_id, $question_answer_id, $user_id );
  }

}