<?php

namespace WPEP\ContentType;

use WPEP\API as WPEP_API;
use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;
use WPEP\ContentType\Course\Container\PostDependency as Course_PostDependency;

class Certification extends Skeleton\Standard implements Course_PostDependency {

  /**
   * @var Certification\Admin
   */
  public $admin;

  /**
   * @var Certification\Delivery
   */
  public $delivery;

  public function setup() {
    $this->_register_api_methods();
  }

  public function init() {
    $this->admin    = new Certification\Admin();
    $this->delivery = new Certification\Delivery();

    $this->delivery->init();
  }

  public function get_learning_journey_instance() {
    return false;
  }

  public function get_settings_defaults() {
    return [
      'certification-download-box-place'   => 'top',
      'certification-download-box-title'   => 'Download Your Certificate',
      'certification-download-box-button'  => 'Download Certificate',
      'certification-user-profile-title'   => 'Certifications',
      'certification-user-profile-public'  => 0,
    ];
  }

  public function has_duplicate() {
    return false;
  }

  public function get_post_supports() {
    return [ 'title' ];
  }

  public function get_post_type() {
    return WPEP_POST_TYPE_CERTIFICATE;
  }

  public function get_cap_name() {
    return WPEP_POST_CAP_CERTIFICATE;
  }

  public function get_singular_name() {
    return __( 'Certificate', "wpep" );
  }

  public function get_plural_name() {
    return __( 'Certifications', "wpep" );
  }

  public function get_label_all_items() {
    return __( 'All Certifications', "wpep" );
  }

  public function register_post_type() {
    $post_labels = [
      'name'               => _x( $this->get_singular_name(), 'Post Type General Name', "wpep"),
      'singular_name'      => _x( $this->get_singular_name(), 'Post Type Singular Name', "wpep"),
      'menu_name'          => $this->get_label_all_items(),
      'name_admin_bar'     => __( $this->get_singular_name(), "wpep"),
      'parent_item_colon'  => __('Parent ' . $this->get_singular_name() . ':', "wpep"),
      'add_new'            => __('New ' . $this->get_singular_name(), "wpep"),
      'add_new_item'       => __('Add New ' . $this->get_singular_name(), "wpep"),
      'new_item'           => __('New ' . $this->get_singular_name(), "wpep"),
      'edit_item'          => __('Edit ' . $this->get_singular_name(), "wpep"),
      'update_item'        => __('Update ' . $this->get_singular_name(), "wpep"),
      'view_item'          => __('View ' . $this->get_singular_name(), "wpep"),
      'all_items'          => $this->get_label_all_items(),
      'search_items'       => __('Search ' . $this->get_singular_name(), "wpep"),
      'not_found'          => __('No ' . $this->get_singular_name() . ' found', "wpep"),
      'not_found_in_trash' => __('No ' . $this->get_singular_name() . ' found in Trash', "wpep")
    ];

    $post_params = [
      'supports'            => $this->get_post_supports(),
      'taxonomies'          => [],
      'hierarchical'        => false,
      'public'              => true,
      'show_ui'             => true,
      'show_in_menu'        => WPEP_ADMIN_BASE_MENU_SLUG,
      'show_in_nav_menus'   => true,
      'show_in_admin_bar'   => true,
      'menu_position'       => 3,
      'menu_icon'           => '',
      'can_export'          => true,
      'has_archive'         => false,
      'exclude_from_search' => false,
      'publicly_queryable'  => true,
      'capabilities' => [
        'create_posts'          => 'do_not_allow'
      ],
      'capability_type'     => WPEP_POST_CAP_CERTIFICATE,
      'map_meta_cap'        => true,
      'rewrite' => [
        'slug'  => wpep_get_setting( 'content-type-' . WPEP_POST_TYPE_CERTIFICATE . '-rewrite-slug', WPEP_POST_TYPE_CERTIFICATE )
      ],
      'label'               => __( strtolower( $this->get_singular_name() ), "wpep" ),
      'labels'              => $post_labels
    ];

    $post_params = apply_filters( 'wpep_content_type_' . $this->get_post_type() . '_register_args', $post_params );

    register_post_type( $this->get_post_type(), $post_params);
  }

  public function is_enabled() {
    if( !wpep_get_setting( 'content-type-' . WPEP_POST_TYPE_COURSE . '-enabled', false ) )
      return false;

    return wpep_get_setting( 'enable-certifications-support', 0 );
  }

  public function data_export_array() {
    return false;
  }

  public function data_asset_map() {
    return [

    ];
  }

  public function attach_data_asset_map_for_course( $response ) {
    return $response;
  }

  public function attach_data_export_array_for_course( $response ) {
    return $response;
  }

  public function attach_data_export_array_for_course_section( $response, $course_id, $section_id = 0, $include_user_activity = true ) {
    return $response;
  }

  public function data_import_single_post_array( $post_information, $id_mapping = [] ) {}

  public function on_course_progress_reset( $course_id, $user_id ) {}

  public function on_course_save( $course_post_id, $entity_id_map ) {
    return $entity_id_map;
  }

  public function current_user_can_view_certificates_for( $user_id ) {
    if( $user_id == get_current_user_id() )
      return true;

    if( current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      return true;

    if( wpep_get_setting_flag( 'certification-user-profile-public' ) == true )
      return true;

    if( current_user_can( WPEP_CAP_SUPERVISE_STUDENTS ) )
      return in_array( get_current_user_id(), get_user_meta( $user_id, WPEP_USER_META_SUPERVISOR_USER_ID, false ) );

    return false;
  }

  private function _register_api_methods() {
    $method_map = [
      'template-user-content' => [
        'callback' => '\WPEP\ContentType\Certification\API::template_user_content',
        'type'    => WPEP_API_METHOD_TYPE_PUBLIC
      ]
    ];

    WPEP_API::instance()->register_content_type( $this->get_post_type(), $method_map );
  }

  /**
   * @param $tab_list
   * @param $builder_integration
   * @return mixed
   */
  public function attach_course_builder_tabs( $tab_list, $builder_integration ) {
    $tab_list[ 'certification' ] = [
      'name'    => __( "Certify", "wpep" ),
      'slug'    => 'course-certifications',
      'content' => [ $this->admin, 'course_builder_tab_content' ]
    ];

    return $tab_list;
  }

}