<?php

namespace WPEP\ContentType\Certification;

use WPEP\Container\FormFields as WPEP_FormFields;

class Admin {

  use WPEP_FormFields;

  public function __construct() {
    $this->set_field_prefix( 'wpep_meta_value' );

    add_filter( 'wpep_administration_settings_map', [ $this, '_admin_settings_map' ], 30 );
  }

  public function get_value( $field_key, $field_default = '', $single = true ) {
    $field_value = get_post_meta( get_the_ID(), $field_key, $single );

    return empty( $field_value ) ? $field_default : $field_value;
  }

  public function course_builder_tab_content() {
    $certification_status_options = [
      WPEP_COURSE_CERTIFICATE_STATUS_DISABLED => __( "Disabled", "wpep" ),
      WPEP_COURSE_CERTIFICATE_STATUS_ENABLED  => __( "Awarded On Course Completion", "wpep" ),
    ];

    if( wpep_assessments_enabled() )
      $certification_status_options[WPEP_COURSE_CERTIFICATE_STATUS_ENABLED_WITH_ASSESSMENTS] = __( "Awarded On Course & Assessments Completion", "wpep" );

    echo '<div class="wpep-tab-item wpep-tab-course-certifications">';

    if( get_post_type() != WPEP_POST_TYPE_COURSE ) {
      echo '<h2>' . __( "Certification Settings", "wpep" ) . '</h2>';
    }

    if( !function_exists( 'imagettfbbox' ) ) {
      echo '<div class="wpep-notification wpep-notification-danger">' . __( "PHP Function missing : imagettfbbox, please ask your Hosting Company to enable the PHP GD Library, or enable it if you're familiar with the process.", "wpep" ) . '</div>';
    }

    if( wpep_image_builder_get_font_directory_path() === false ) {
      echo '<div class="wpep-notification wpep-notification-danger">' . sprintf( __( "Missing Font Files, this is caused by a server error, %s will attempt to download the font files until they're stored on your server successfully.", "wpep" ), WPEP_PLUGIN_NAME ) . '</div>';
    }

    echo      '<div class="wpep-form-group">';

    echo        '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Status", "wpep" ) . '</label>';

    $this->display_select_field(
      'wpep_certificate_status',
      $certification_status_options
    );
    echo        '</div>';

    echo        '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Lock After", "wpep" ) . '</label>';

    $this->display_date_field(
      'wpep_certificate_reset_date',
      false,
      $this->get_value( 'wpep_certificate_reset_date', date( "Y-m-d" ) )
    );

    echo           '<div class="description">' . __( "All Certificates that have been created after the file lock day will stay as they are, even if you make template changes.", "wpep" ) . '</div>';
    echo        '</div>';

    echo      '</div>';

    echo      '<div class="wpep-form-group">';
    echo        '<div class="wpep-form-group-row">';

    $field_options = [
      '{full_name}'         => __( "Full Name", "wpep" ),
      '{first_name}'        => __( "First Name", "wpep" ),
      '{last_name}'         => __( "Last Name", "wpep" ),
      '{issue_date}'        => __( "Issue Date", "wpep" ),
      '{course_name}'       => __( "Course Name", "wpep" ),
      '{course_started}'    => __( "Course Started Date", "wpep"),
      '{course_completed}'  => __( "Course Completed Date", "wpep"),
      '{user_site_name}'    => __( "User Website", "wpep" ),
    ];

    if( wpep_assessments_enabled() ) {
      $field_options += [
        '{assessment_grading}'         => __( "Assessment Grading", "wpep" ),
        '{assessment_start_time}'      => __( "Assessment Start Time", "wpep" ),
        '{assessment_submission_time}' => __( "Assessment Submission Time", "wpep" ),
        '{assessment_status}'          => __( "Assessment Status", "wpep" )
      ];
    }

    if( wpep_has_sid_support() )
      $field_options = [ '{student_identification}' => __( "Student Identification", "wpep" ) ] + $field_options;

    $field_options['enter text here'] = __( "Custom Text", "wpep" );

    $this->display_image_builder_field(
      $field_options,
      'wpep_certificate_image',
      'wpep_certificate_content_map',
      ( WPEP_BASE_URL_PATH . 'assets/img/default-certificate.jpg' ),
      [
        [
          'content'     => 'Certificate Example',
          'left'        => '222px',
          'top'         => '150px',
          'width'       => '394px',
          'height'      => '22px',
          'font-size'   => '18px',
          'font-family' => 'Arial',
          'font-weight' => 300,
          'color'       => '#FFFFFF'
        ],
        [
          'content'     => '{full_name} has successfully completed',
          'left'        => '127px',
          'top'         => '216px',
          'width'       => '618px',
          'height'      => '27px',
          'font-size'   => '23px',
          'font-family' => 'Arial',
          'font-weight' => 300,
          'color'       => '#000000'
        ],
        [
          'content'     => '{course_name}',
          'left'        => '215px',
          'top'         => '264px',
          'width'       => '411px',
          'height'      => '31px',
          'font-size'   => '27px',
          'font-family' => 'Arial',
          'font-weight' => 300,
          'color'       => '#000000'
        ],
        [
          'content'     => '...',
          'left'        => '108px',
          'top'         => '456px',
          'width'       => '200px',
          'height'      => '22px',
          'font-size'   => '19px',
          'font-family' => 'Arial',
          'font-weight' => 300,
          'color'       => '#000000'
        ],
        [
          'content'     => '...',
          'left'        => '561px',
          'top'         => '456px',
          'width'       => '200px',
          'height'      => '22px',
          'font-size'   => '19px',
          'font-family' => 'Arial',
          'font-weight' => 300,
          'color'       => '#000000'
        ]
      ],
      wpep_image_builder_get_font_list()
    );

    echo        '</div>';

    echo      '</div>';

    echo '</div>';
  }

  public function _admin_settings_map( $settings_map ) {
    if( !isset( $settings_map[ 'course_page' ][ 'sections' ] ) )
      return $settings_map;

    $settings_map[ 'course_page' ][ 'sections' ][ 'certification' ] = [
      'name'      => __( "Certification", "wpep" ),
      'alias'     => 'certification',
      'field_map' => [
        'download_box' => [
          'name'      => __( "Download Box", "wpep" ),
          'has_title' => true,
          'alias'     => 'download_box',
          'fields'    => [
            'certification-download-box-place' => [
              'name'        => __( "Placement", "wpep" ),
              'alias'       => 'certification-download-box-place',
              'type'        => 'select',
              'options'     => [
                'top'     => __( "Before Course Content"  ),
                'bottom'  => __( "After Course Content" )
              ],
              'description' => __( "After earning an certificate, the student can download it from the Course Page, chose where to display the download section.")
            ],
            'certification-download-box-title' => [
              'name'        => __( 'Title', "wpep"),
              'alias'       => 'certification-download-box-title',
              'type'        => 'text_translated'
            ],
            'certification-download-box-button' => [
              'name'        => __( 'Button', "wpep"),
              'alias'       => 'certification-download-box-button',
              'type'        => 'text_translated'
            ],
          ]
        ],
        'user_profile' => [
          'name'      => __( "User Profile", "wpep" ),
          'has_title' => true,
          'alias'     => 'user_profile',
          'fields'    => [
            'certification-user-profile-title' => [
              'name'        => __( 'Title', "wpep"),
              'alias'       => 'certification-user-profile-title',
              'type'        => 'text_translated'
            ],
            'certification-user-profile-public' => [
              'name'        => __( "Show Public", "wpep" ),
              'alias'       => 'certification-user-profile-public',
              'type'        => 'checkbox',
              'description' => __( "By default we don't show the certificates the student has earned on the public profile link, check this if you want to.", "wpep" )
            ],
          ]
        ],
      ]
    ];

    return $settings_map;
  }
}