<?php

namespace WPEP\ContentType\Course\Block;

use WPEP\ContentType\Course\Delivery  as Course_Delivery;
use WPEP\Controller as WPEP_Controller;
use WPEP\Container\FilterProxy as FilterProxy;

class General {

  use FilterProxy;

  public function __construct() {
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'course', [ $this, 'single' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'course_preview', [ $this, 'single_preview' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'course_grid', [ $this, 'grid' ] );

    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'index_courses_recommended', [ $this, 'index_courses_recommended' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'index_courses_progressing', [ $this, 'index_courses_progressing' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'index_courses_completed', [ $this, 'index_courses_completed' ] );

    if( wpep_course_has_bookmarks() )
      add_shortcode( WPEP_SHORT_CODE_PREFIX . 'index_courses_bookmarked', [ $this, 'index_courses_bookmarked' ] );
  }

  public function single( $attributes ) {
    $post_id     = intval( isset($attributes['id']) ? $attributes['id'] : get_the_ID() );

    if( $post_id == false )
      return __( "You need to offer the id param for this shortcode. ", "wpep" );

    $post = get_post( $post_id );

    if( $post === null || get_post_type( $post ) != WPEP_POST_TYPE_COURSE ) {
      return sprintf( __( "This Course has gone missing, please verify the following course id %s ", "wpep" ), $post_id );
    }

    if( !is_user_logged_in() && in_array( 'course-shortcode', wpep_get_auth_status_page_restrictions() ) ) {
      return WPEP_Controller::instance()->authentication->get_form( $attributes );
    }

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    $coordinator_object = new Course_Delivery();

    $this->_filter_proxy_setup( $this->_single_filters( $attributes ) );

    $coordinator_object->init( $post_id );

    if( isset($attributes['read_only']) && $attributes['read_only'] !== '' )
      $coordinator_object->getCurrentDeliveryMethod()->read_only = intval( $attributes['read_only'] );

    $content = WPEP_Controller::instance()->template->get_template(
      'content-course-single-minimal.php',
      [
        'coordinator_object'  => $coordinator_object,
        'course_id'           => $post_id
      ]
    );

    $coordinator_object->close();

    $this->_filter_proxy_remove();

    unset( $coordinator_object );

    $content = wpep_format_content_autoembed( $content );

    return $content;
  }

  private function _single_filters( $attributes ) {
    $show_header              = intval( isset($attributes['show_header'])         ? $attributes['show_header'] : true );
    $disable_assessments      = intval( isset($attributes['disable_assessments']) ? $attributes['disable_assessments'] : false );
    $course_type              = ( isset( $attributes['course_type'] )                  ? $attributes['course_type'] : 'default' );
    $course_available_lessons = ( isset( $attributes['course_lessons'] )               ? explode( ",", $attributes['course_lessons'] ) : [] );

    $filter_proxy_list = [
      'wpep_single_course_prerequisites_list'      => [],
      'wpep_single_course_has_next_suggestions'    => false
    ];

    $filter_proxy_list['wpep_course_single_content_include_header'] = $show_header;

    if( $course_type != 'default' )
      $filter_proxy_list['wpep_single_content_course_type'] = function( $course_type_internal ) use ( $course_type ) {
        return $course_type;
      };

    if( !empty( $course_available_lessons ) ) {
      $filter_proxy_list['wpep_user_activity_course_sections_args'] = function( $args ) use ( $course_available_lessons ) {
        $args['available_sections'] = [];
        $args['available_lessons']  = $course_available_lessons;

        return $args;
      };
    }

    if( $disable_assessments ) {
      $filter_proxy_list['wpep_user_activity_course_sections'] = function( $course_sections ) {
        foreach( $course_sections as $course_section_id => $course_section ) {
          $course_sections[$course_section_id]['assessment_id'] = 0;
        }

        return $course_sections;
      };
    }


    return $filter_proxy_list;
  }

  public function single_preview( $attributes ) {
    $attributes['course_type']          = ( isset( $attributes['course_type'] ) && $attributes['course_type'] !== '' ? $attributes['course_type'] : WPEP_COURSE_TYPE_EXECUTION_PLAN );
    $attributes['disable_assessments']  = true;
    $attributes['read_only']            = true;
    $attributes['course_lessons']       = ( !isset( $attributes['course_lessons'] ) ? '' : $attributes['course_lessons'] );

    add_filter( 'wp_redirect', [ $this, '_return_false' ], 100 );
    add_filter( 'wpep_single_content_course_before_html', '__return_empty_string', 100 );
    add_filter( 'wpep_single_content_course_after_html', '__return_empty_string', 100 );

    $response = $this->single( $attributes );

    remove_filter( 'wp_redirect', [ $this, '_return_false' ], 100 );
    remove_filter( 'wpep_single_content_course_before_html', '__return_empty_string', 100 );
    remove_filter( 'wpep_single_content_course_after_html', '__return_empty_string', 100 );

    return $response;
  }

  public function grid( $attributes ) {
    if( !is_user_logged_in() && in_array( 'course-grid-shortcode', wpep_get_auth_status_page_restrictions() ) )
      return WPEP_Controller::instance()->authentication->get_form( $attributes );

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    $this->_filter_proxy_setup( $this->_grid_filters( $attributes ) );

    $response = WPEP_Controller::instance()->template->get_template(
      'primary-grid-courses.php',
      [
        'include_header' => false,
        'include_footer' => false
      ]
    );

    $this->_filter_proxy_remove();

    return $response;
  }

  private function _grid_filters( $attributes ) {
    $filter_proxy_list = [];

    $course_list_ids      = isset( $attributes['courses'] ) ? $attributes['courses'] : '';
    $container_class      = isset( $attributes['container_class'] ) ? $attributes['container_class'] : '';

    if( $course_list_ids !== '' ) {
      $filter_proxy_list['wpep_primary_content_' . WPEP_POST_TYPE_COURSE . '_list_args'] = function ($args) use ($course_list_ids) {
        $args['post__in'] = wp_parse_id_list( $course_list_ids );

        return $args;
      };
    }

    if( $container_class !== '' ) {
      $filter_proxy_list['wpep_primary_content_course_grid_container_class'] = function ( $class ) use ( $container_class ) {
        return $class . ' ' . $container_class;
      };
    }

    return $filter_proxy_list;
  }

  public function _return_false() {
    return false;
  }

  public function index_courses_recommended() {
    if( !is_user_logged_in() )
      return '';

    $course_ids = wpep_content_type_course_instance()->user->get_recommended_ids( get_current_user_id(), 999 );

    if( empty( $course_ids ) )
      return '';

    return wpep_content_primary_list_by_id_map( $course_ids );
  }

  public function index_courses_progressing() {
    if( !is_user_logged_in() )
      return '';

    $course_ids = wpep_content_type_course_instance()->user->get_progressing_course_list( get_current_user_id() );

    if( empty( $course_ids ) )
      return '';

    return wpep_content_primary_list_by_id_map( $course_ids );
  }

  public function index_courses_completed() {
    if( !is_user_logged_in() )
      return '';

    $course_ids = wpep_content_type_course_instance()->user->get_completed_course_list( get_current_user_id() );

    if( empty( $course_ids ) )
      return '';

    return wpep_content_primary_list_by_id_map( $course_ids );
  }

  public function index_courses_bookmarked() {
    if( !is_user_logged_in() )
      return '';

    $course_ids = wpep_content_type_course_instance()->user->get_bookmarked_course_list( get_current_user_id() );

    if( empty( $course_ids ) )
      return '';

    return wpep_content_primary_list_by_id_map( $course_ids );
  }


}