<?php

namespace WPEP\ContentType\Course\Block;

use WPEP\Entity\Course as Entity_Course;

class Misc {

  private $_recommended_course_id = null;

  public function __construct() {
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'lesson_content', [ $this, 'lesson_content' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'course_is_completed', [ $this, 'course_is_completed' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'course_not_completed', [ $this, 'course_not_completed' ] );

    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'user_has_course_recommendation', [ $this, 'user_has_course_recommendation' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'user_course_recommendation_title', [ $this, 'user_course_recommendation_title' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'user_course_recommendation_link', [ $this, 'user_course_recommendation_link' ] );
  }

  public function lesson_content( $attributes, $content ) {
    if( !isset( $attributes['lesson'] ) )
      return '<p class="wpep-notification wpep-notification-danger">' . __( "wpep_lesson_content - Missing lesson parameter", "wpep" ) . '</p>';

    $lesson = Entity_Course::instance()->get_lesson( intval( $attributes['lesson'] ) );

    if( !isset( $lesson->content ) )
      return '<p class="wpep-notification wpep-notification-danger">' . __( "wpep_lesson_content - lesson content is missing.", "wpep" ) . '</p>';


    return wpep_format_content( $lesson->content );
  }

  public function course_is_completed( $attributes, $content ) {
    if( !isset( $attributes['id'] ) )
      return '<p class="wpep-notification wpep-notification-danger">' . __( "wpep_course_is_completed - Missing id parameter", "wpep" ) . '</p>';

    if( !is_user_logged_in() )
      return '';

    $course_ids = explode( ",", $attributes['id'] );

    foreach( $course_ids as $course_id ) {
      if( !wpep_user_course_is_completed( $course_id ) )
        return '';
    }

    return do_shortcode( $content );
  }

  public function course_not_completed( $attributes, $content ) {
    if( !isset( $attributes['id'] ) )
      return '<p class="wpep-notification wpep-notification-danger">' . __( "wpep_course_is_completed - Missing id parameter", "wpep" ) . '</p>';

    if( !is_user_logged_in() )
      return $content;

    $course_ids = explode( ",", $attributes['id'] );

    foreach( $course_ids as $course_id ) {
      if( wpep_user_course_is_completed( $course_id ) )
        return '';
    }

    return do_shortcode( $content );
  }

  public function user_has_course_recommendation( $attributes, $content ) {
    if( $this->_get_recommended_content_id() == false )
      return '';

    return do_shortcode( $content );
  }

  public function user_course_recommendation_title() {
    if( $this->_get_recommended_content_id() == false )
      return '';

    return get_the_title( $this->_get_recommended_content_id() );
  }

  public function user_course_recommendation_link( $attributes, $content ) {
    if( $this->_get_recommended_content_id() == false )
      return '';

    $link = wpep_course_permalink( $this->_get_recommended_content_id() );

    if( $content !== '' )
      return '<a href="' . $link . '">' . do_shortcode( $content ) . '</a>';

    return $link;
  }

  private function _get_recommended_content_id() {
    if( $this->_recommended_course_id !== null )
      return $this->_recommended_course_id;

    if( !is_user_logged_in() ) {
      $this->_recommended_course_id = 0;

      return 0;
    }

    $ids = wpep_content_type_course_instance()->user->get_recommended_ids( get_current_user_id(), 999 );

    foreach( $ids as $id ) {
      if( wpep_post_read_only( $id ) )
        continue;

      if( !wpep_controller()->contentLibrary->has_access_to_post( $id ) )
        continue;

      $this->_recommended_course_id = $id;

      return $this->_recommended_course_id;
    }


    return 0;
  }

}