<?php

namespace WPEP\ContentType\Course\Container;

use WPEP\Entity\Course as Entity_Course;

trait Data {

  /**
   * @param $args
   * @return array|null
   */
  public function data_user_activity_report( $args ) {
    $response = parent::data_user_activity_report( $args );

    $has_progress_repeat = wpep_get_setting_flag( 'enable-progress-repeat-support' );

    $courses_started_sql      = 'SELECT COUNT( cud.id ) FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA . ' cud';
    $courses_completed_at_sql = 'SELECT COUNT( cud.id ) FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA . ' cud';
    $lessons_completed_sql    = 'SELECT COUNT( lc.id ) FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . ' lc';

    if( $has_progress_repeat )
      $course_progress_repeat_sql = 'SELECT COUNT( cud.id ) FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA . ' cud';

    $courses_started_sql      .= ' INNER JOIN ' . wpep_wpdb()->posts . ' post_join ON post_join.`ID` = cud.`course_id`';
    $courses_completed_at_sql .= ' INNER JOIN ' . wpep_wpdb()->posts . ' post_join ON post_join.`ID` = cud.`course_id`';

    $lessons_completed_sql    .= ' INNER JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . ' lesson_join ON lesson_join.`id` = lc.`lesson_id`
                                   INNER JOIN ' . wpep_wpdb()->posts . ' post_join ON post_join.`ID` = lesson_join.`post_id`';

    if( $has_progress_repeat )
      $course_progress_repeat_sql      .= ' INNER JOIN ' . wpep_wpdb()->posts . ' post_join ON post_join.`ID` = cud.`course_id`';

    $courses_started_sql      .= ' WHERE `key` = "' . WPEP_USER_COURSE_STARTED . '" 
                                     AND post_join.`post_status` IN ("' . implode( '","', wpep_post_monitored_post_status_list() ) . '")';
    $courses_completed_at_sql .= ' WHERE `key` = "' . WPEP_USER_COURSE_COMPLETED_AT . '" 
                                     AND post_join.`post_status` IN ("' . implode( '","', wpep_post_monitored_post_status_list() ) . '")';
    $lessons_completed_sql    .= ' WHERE post_join.`post_status` IN ("' . implode( '","', wpep_post_monitored_post_status_list() ) . '")';

    if( $has_progress_repeat )
      $course_progress_repeat_sql .= ' WHERE `key` = "' . WPEP_USER_COURSE_REPEAT_TRACKER . '" 
                                         AND post_join.`post_status` IN ("' . implode( '","', wpep_post_monitored_post_status_list() ) . '")';

    if( isset( $args[ 'user_ids' ] ) ) {
      if( is_numeric( $args[ 'user_ids' ] ) ) {
        $courses_started_sql .= ' AND cud.`user_id` = ' . intval( $args[ 'user_ids' ] );
        $courses_completed_at_sql .= ' AND cud.`user_id` = ' . intval( $args[ 'user_ids' ] );
        $lessons_completed_sql .= ' AND lc.`user_id` = ' . intval( $args[ 'user_ids' ] );
      } else if( is_array( $args[ 'user_ids' ] ) ) {
        $courses_started_sql .= ' AND cud.`user_id` IN (' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ')';
        $courses_completed_at_sql .= ' AND cud.`user_id` IN (' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ')';
        $lessons_completed_sql .= ' AND lc.`user_id` IN (' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ')';
      }
    }

    if( isset( $args[ 'post_ids' ] ) ) {
      if( is_numeric( $args[ 'post_ids' ] ) ) {
        $courses_started_sql .= ' AND post_join.`ID` = ' . intval( $args[ 'post_ids' ] );
        $courses_completed_at_sql .= ' AND post_join.`ID` = ' . intval( $args[ 'post_ids' ] );
        $lessons_completed_sql .= ' AND post_join.`ID` = ' . intval( $args[ 'post_ids' ] );

        if( $has_progress_repeat )
          $course_progress_repeat_sql .= ' AND post_join.`ID` = ' . intval( $args[ 'post_ids' ] );
      } else if( is_array( $args[ 'post_ids' ] ) ) {
        $courses_started_sql .= ' AND post_join.`ID` IN (' . implode( ',', array_map( 'intval', $args[ 'post_ids' ] ) ) . ')';
        $courses_completed_at_sql .= ' AND post_join.`ID` IN (' . implode( ',', array_map( 'intval', $args[ 'post_ids' ] ) ) . ')';
        $lessons_completed_sql .= ' AND post_join.`ID` IN (' . implode( ',', array_map( 'intval', $args[ 'post_ids' ] ) ) . ')';

        if( $has_progress_repeat )
          $course_progress_repeat_sql .= ' AND post_join.`ID` IN (' . implode( ',', array_map( 'intval', $args[ 'post_ids' ] ) ) . ')';
      }
    }

    if( isset( $args['time_from' ] ) ) {
      $courses_started_sql      .= ' AND CAST( cud.`value` AS UNSIGNED ) >= ' . absint( $args['time_from'] );
      $courses_completed_at_sql .= ' AND CAST( cud.`value` AS UNSIGNED ) >= ' . absint( $args['time_from'] );
      $lessons_completed_sql    .= ' AND lc.`created_at` >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '"';

      if( $has_progress_repeat )
        $course_progress_repeat_sql .= ' AND CAST( cud.`value` AS UNSIGNED ) >= ' . absint( $args['time_from'] );
    }

    if( isset( $args['time_to' ] ) ) {
      $courses_started_sql .= ' AND CAST( cud.`value` AS UNSIGNED ) < ' . absint( $args['time_to'] );
      $courses_completed_at_sql .= ' AND CAST( cud.`value` AS UNSIGNED ) < ' . absint( $args['time_to'] );
      $lessons_completed_sql    .= ' AND lc.`created_at` < "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';

      if( $has_progress_repeat )
        $course_progress_repeat_sql .= ' AND CAST( cud.`value` AS UNSIGNED ) < ' . absint( $args['time_to'] );
    }

    $started_count   = absint( wpep_wpdb()->get_var( $courses_started_sql ) );
    $completed_count = absint( wpep_wpdb()->get_var( $courses_completed_at_sql ) );
    $lesson_completed_count = absint( wpep_wpdb()->get_var( $lessons_completed_sql ) );

    if( $started_count != 0 )
      $response[ $this->get_singular_name() . ' - ' . __( "Started", "wpep" ) ]   = $started_count;

    if( $completed_count != 0 )
      $response[ $this->get_singular_name() . ' - ' . __( "Completed", "wpep" ) ] = $completed_count;

    if( $lesson_completed_count != 0 )
      $response[ $this->get_singular_name() . ' - ' . __( "Lessons Completed", "wpep" ) ] = $lesson_completed_count;

    if( !$has_progress_repeat )
      return $response;

    $progress_repeat_count = absint( wpep_wpdb()->get_var( $course_progress_repeat_sql ) );

    if( $progress_repeat_count != 0 )
      $response[ $this->get_singular_name() . ' - ' . __( "Progress Repeat", "wpep" ) ] = $progress_repeat_count;

    return $response;
  }

  public function data_user_post_activity_report( $args ) {
    $response = parent::data_user_post_activity_report( $args );

    $has_progress_repeat = wpep_get_setting_flag( 'enable-progress-repeat-support' );

    $courses_started_sql = 'SELECT COUNT( cud.id ) as entries, 
                                   cud.course_id as post_id,
                                   p.post_title as post_title
                              FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA . ' cud 
                        INNER JOIN ' . wpep_wpdb()->posts . ' p ON p.ID = cud.course_id
                             WHERE `key` = "' . WPEP_USER_COURSE_STARTED . '"';

    $courses_completed_at_sql = 'SELECT COUNT( cud.id ) as entries, 
                                        cud.course_id as post_id,
                                        p.post_title as post_title
                                  FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA . ' cud
                            INNER JOIN ' . wpep_wpdb()->posts . ' p ON p.ID = cud.course_id
                                 WHERE `key` = "' . WPEP_USER_COURSE_COMPLETED_AT . '"';
    $lessons_completed_sql    = 'SELECT COUNT( lc.id ) as entries, 
                                        l.post_id as post_id,
                                        p.post_title as post_title 
                                   FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . ' lc
                             INNER JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . ' l ON l.id = lc.lesson_id
                             INNER JOIN ' . wpep_wpdb()->posts . ' p ON p.ID = l.post_id
                                  WHERE 1 = 1';

    if( $has_progress_repeat )
      $course_progress_repeat_sql = 'SELECT COUNT( cud.id ) as entries, 
                                            cud.course_id as post_id,
                                            p.post_title as post_title
                                      FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA . ' cud
                                INNER JOIN ' . wpep_wpdb()->posts . ' p ON p.ID = cud.course_id
                                     WHERE `key` = "' . WPEP_USER_COURSE_REPEAT_TRACKER . '"';

    if( isset( $args[ 'user_ids' ] ) ) {
      if( is_numeric( $args[ 'user_ids' ] ) ) {
        $courses_started_sql .= ' AND cud.`user_id` = ' . intval( $args[ 'user_ids' ] );
        $courses_completed_at_sql .= ' AND cud.`user_id` = ' . intval( $args[ 'user_ids' ] );
        $lessons_completed_sql .= ' AND lc.`user_id` = ' . intval( $args[ 'user_ids' ] );

        if( $has_progress_repeat )
          $course_progress_repeat_sql .= ' AND cud.`user_id` = ' . intval( $args[ 'user_ids' ] );

      } else if( is_array( $args[ 'user_ids' ] ) ) {
        $courses_started_sql .= ' AND cud.`user_id` IN (' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ')';
        $courses_completed_at_sql .= ' AND cud.`user_id` IN (' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ')';
        $lessons_completed_sql .= ' AND lc.`user_id` IN (' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ')';

        if( $has_progress_repeat )
          $course_progress_repeat_sql .= ' AND cud.`user_id` IN (' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ')';
      }
    }

    if( isset( $args[ 'post_ids' ] ) ) {
      if( is_numeric( $args[ 'post_ids' ] ) ) {
        $courses_started_sql .= ' AND p.`ID` = ' . intval( $args[ 'post_ids' ] );
        $courses_completed_at_sql .= ' AND p.`ID` = ' . intval( $args[ 'post_ids' ] );
        $lessons_completed_sql .= ' AND p.`ID` = ' . intval( $args[ 'post_ids' ] );

        if( $has_progress_repeat )
          $course_progress_repeat_sql .= ' AND p.`ID` = ' . intval( $args[ 'post_ids' ] );
      } else if( is_array( $args[ 'post_ids' ] ) ) {
        $courses_started_sql .= ' AND p.`ID` IN (' . implode( ',', array_map( 'intval', $args[ 'post_ids' ] ) ) . ')';
        $courses_completed_at_sql .= ' AND p.`ID` IN (' . implode( ',', array_map( 'intval', $args[ 'post_ids' ] ) ) . ')';
        $lessons_completed_sql .= ' AND p.`ID` IN (' . implode( ',', array_map( 'intval', $args[ 'post_ids' ] ) ) . ')';

        if( $has_progress_repeat )
          $course_progress_repeat_sql .= ' AND p.`ID` IN (' . implode( ',', array_map( 'intval', $args[ 'post_ids' ] ) ) . ')';
      }
    }

    if( isset( $args['time_from' ] ) ) {
      $courses_started_sql        .= ' AND CAST( cud.`value` AS UNSIGNED ) >= ' . absint( $args['time_from'] );
      $courses_completed_at_sql   .= ' AND CAST( cud.`value` AS UNSIGNED ) >= ' . absint( $args['time_from'] );
      $lessons_completed_sql      .= ' AND lc.`created_at` >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '"';

      if( $has_progress_repeat )
        $course_progress_repeat_sql .= ' AND CAST( cud.`value` AS UNSIGNED ) >= ' . absint( $args['time_from'] );
    }

    if( isset( $args['time_to' ] ) ) {
      $courses_started_sql        .= ' AND CAST( cud.`value` AS UNSIGNED ) < ' . absint( $args['time_to'] );
      $courses_completed_at_sql   .= ' AND CAST( cud.`value` AS UNSIGNED ) < ' . absint( $args['time_to'] );
      $lessons_completed_sql      .= ' AND lc.`created_at` < "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';

      if( $has_progress_repeat )
        $course_progress_repeat_sql .= ' AND CAST( cud.`value` AS UNSIGNED ) < ' . absint( $args['time_to'] );
    }

    $courses_started_sql .= ' GROUP BY cud.course_id';
    $courses_completed_at_sql .= ' GROUP BY cud.course_id';
    $lessons_completed_sql .= ' GROUP BY l.post_id';

    if( $has_progress_repeat )
      $course_progress_repeat_sql .= ' GROUP BY cud.course_id';

    $courses_started_map   = wpep_wpdb()->get_results( $courses_started_sql );
    $courses_completed_map = wpep_wpdb()->get_results( $courses_completed_at_sql );
    $lessons_completed_map = wpep_wpdb()->get_results( $lessons_completed_sql );

    foreach( $courses_started_map as $course_started_data )
      $response[ __( "Course Started", "wpep" ) . ' - ' . $course_started_data->post_title ] = $course_started_data->entries;

    foreach( $courses_completed_map as $courses_completed_data )
      $response[ __( "Course Completed", "wpep" ) . ' - ' . $courses_completed_data->post_title ] = $courses_completed_data->entries;

    foreach( $lessons_completed_map as $lessons_completed_data )
      $response[ __( "Lesson Completed", "wpep" ) . ' - ' . $lessons_completed_data->post_title ] = $lessons_completed_data->entries;

    if( !$has_progress_repeat )
      return $response;

    $progress_repeat_map = wpep_wpdb()->get_results( $course_progress_repeat_sql );

    foreach( $progress_repeat_map as $progress_repeat_data )
      $response[ __( "Course Progress Repeat", "wpep" ) . ' - ' . $progress_repeat_data->post_title ] = $progress_repeat_data->entries;

    return $response;
  }

  /**
   * @param [
   *  'current_page'     => '',
   *  'pagination_limit' => '',
   *  'time_from'        => '',
   *  'time_to'          => '',
   *  'user_ids'         => [] <- Needs to not be empty in order to actually limit by this.,
   *  'post_ids'         => [] <- Needs to not be empty in order to actually limit by this.
   * ] $args
   * @return [
   *    'entries_header' => []
   *    'entries'        => []
   *    'pagination'     => []
   * ]
   */
  public function data_user_post_activity_list( $args ) {
    $sql_time_data_fields = [ WPEP_USER_COURSE_STARTED, WPEP_USER_COURSE_LATEST_INTERACTION, WPEP_USER_COURSE_COMPLETED_AT ];
    $has_time_spent = wpep_get_setting_flag( 'enable-course-time-spent-option' );

    if( $has_time_spent )
      $sql_time_data_fields[] = WPEP_USER_COURSE_TIME_SPENT_LAST_SYNC;

    $response = [
      'entries_header'  => [
        'progress_formatted'           => __( "Progress", "wpep" ),
        'course_type_label'            => __( "Course Type", "wpep" ),
        'interaction_at_formatted'     => __( "Latest Interaction", "wpep" ),
        'started_at_formatted'         => __( "Started At", "wpep" ),
        'completed_at_formatted'       => __( "Completed At", "wpep" )
      ],
      'entries'         => [],
      'pagination'      => [
        'current'       => $args[ 'current_page' ],
        'total'         => 0,
        'total_entries' => 0
      ]
    ];

    if( $has_time_spent ) {
      $response[ 'entries_header' ][ 'time_spent_formatted' ] = __( "Time Spent", 'wpep' );
      $response[ 'entries_header' ][ 'time_spent_focused_formatted' ] = __( "Time Viewport", 'wpep' );
      $response[ 'entries_header' ][ 'time_spent_last_sync_formatted' ] = __( "Last Activity", 'wpep' );
    }

    $sql = 'SELECT cud.user_id, 
                   cud.course_id,
                   cud.value as `interaction_timestamp`
              FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA . ' cud 
             WHERE 1 = 1'; // Hacks, leave this like this, we follow with OR statements

    if( isset( $args[ 'user_ids' ] ) && !empty( $args[ 'user_ids' ] ) )
      $sql .= ' AND cud.user_id ' . ( is_array( $args[ 'user_ids' ] ) ? ' IN ( ' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ' )' : ' = ' . intval( $args[ 'user_ids' ] ) );

    if( isset( $args[ 'post_ids' ] ) && !empty( $args[ 'post_ids' ] ) )
      $sql .= ' AND cud.course_id ' . ( is_array( $args[ 'post_ids' ] ) ? ' IN ( ' . implode( ',', array_map( 'intval', $args[ 'post_ids' ] ) ) . ' )' : ' = ' . intval( $args[ 'post_ids' ] ) );

    $sql .= ' AND ( ';

    $i = 0; foreach( $sql_time_data_fields as $current_data_key ) {
      $sql .= ( $i === 0 ? '' : ' OR ' ) . ' ( ';
      $sql .=     'cud.`key` = "' . $current_data_key . '"';

      if( isset( $args[ 'time_from' ] ) )
        $sql .= ' AND CAST( cud.`value` as UNSIGNED ) >= ' . intval( $args[ 'time_from' ] );

      if( isset( $args[ 'time_to' ] ) )
        $sql .= ' AND CAST( cud.`value` as UNSIGNED ) < ' . intval( $args[ 'time_to' ] );

      $sql .= ' ) ';

      $i++;
    }

    $sql .= ')';

    $sql .= ' GROUP BY cud.user_id, cud.course_id';

    $sql = 'SELECT SQL_CALC_FOUND_ROWS inner_cud.* FROM ( ' . $sql . ') inner_cud ORDER BY inner_cud.interaction_timestamp DESC';

    $sql .= ' LIMIT ' . absint( $args[ 'pagination_limit' ] );

    if( isset( $args['current_page'] ) && $args['current_page'] != 1 )
      $sql .= ' OFFSET ' . intval( ( $args[ 'current_page' ] - 1 ) * $args[ 'pagination_limit' ] );

    $results = wpep_wpdb()->get_results( $sql );

    if( empty( $results ) )
      return $response;

    $response[ 'pagination' ][ 'total_entries' ] = absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) );
    $response[ 'pagination' ][ 'total' ] = ceil( $response[ 'pagination' ][ 'total_entries' ] / $args[ 'pagination_limit' ] );

    foreach( $results as $result ) {
      $course_type  = wpep_course_type_from_id( $result->course_id );
      $started_at   = wpep_user_course_started_at( $result->course_id, $result->user_id );
      $completed_at = wpep_user_course_completed_at( $result->course_id, $result->user_id );
      $progress = ( $completed_at === 0 ? wpep_user_course_progress_percentage( $result->course_id, $result->user_id ) : 100 );

      $current_entry_row = [
        'user_id'                => $result->user_id,
        'post_id'                => $result->course_id,
        'course_type'            => $course_type,
        'course_type_label'      => wpep_course_layout_type_label( $course_type ),

        'progress'               => $progress,
        'progress_formatted'     => wpep_get_percentage( $progress ),

        'interaction_at'             => $result->interaction_timestamp,
        'interaction_at_formatted'   => wpep_format_date_with_time( $result->interaction_timestamp ),

        'started_at'             => $started_at,
        'started_at_formatted'   => wpep_format_date_with_time( $started_at ),

        'completed_at'           => $completed_at,
        'completed_at_formatted' => ( $completed_at === 0 ? '-' : wpep_format_date_with_time( $completed_at ) )
      ];

      if( $has_time_spent ) {
        $time_spent = intval( wpep_content_type_course_instance()->user->data->get_value(
          WPEP_USER_COURSE_TIME_SPENT,
          $result->course_id, false, false, $result->user_id, true
        ) );

        $time_spent_focused = intval( wpep_content_type_course_instance()->user->data->get_value(
          WPEP_USER_COURSE_TIME_SPENT_FOCUSED,
          $result->course_id, false, false, $result->user_id, true
        ) );

        $time_spent_last_sync = intval( wpep_content_type_course_instance()->user->data->get_value(
          WPEP_USER_COURSE_TIME_SPENT_LAST_SYNC,
          $result->course_id, false, false, $result->user_id, true
        ) );

        $current_entry_row[ 'time_spent' ] = $time_spent;
        $current_entry_row[ 'time_spent_focused' ] = $time_spent_focused;
        $current_entry_row[ 'time_spent_last_sync' ] = $time_spent_last_sync;

        $current_entry_row[ 'time_spent_formatted' ] = ( $time_spent === 0 ? '-' : wpep_time_to_dhms_string( $time_spent ) );
        $current_entry_row[ 'time_spent_focused_formatted' ] = ( $time_spent_focused === 0 ? '-' : wpep_time_to_dhms_string( $time_spent_focused ) );
        $current_entry_row[ 'time_spent_last_sync_formatted' ] = ( $time_spent_last_sync === 0 ? '-' : wpep_format_date_with_time( $time_spent_last_sync ) );

      }

      $response[ 'entries' ][] = $current_entry_row;
    }

    return $response;
  }

  public function data_asset_map() {
    $response = [
      'meta'  => [
        'wpep_course_description',
        'wpep_thumb_image',
        'wpep_thumb_image_hover',
        'wpep_icon_image',
        'wpep_member_magnet_option_text_before',
        'wpep_member_magnet_option_text_after'
      ],
      'section_list' => [
        'content',
        'lessons' => [ 'content' ],
      ]
    ];

    foreach( $this->_post_dependencies as $post_dependency )
      $response = $post_dependency->attach_data_asset_map_for_course( $response );

    return $response;
  }

  public function data_post_connection() {
    return [
      'meta_wpep_prerequisite' => [
        'type'       => 'meta',
        'key'        => 'wpep_prerequisite',
        'value_type' => 'post_id',
        'location'   => 'internal'
      ]
    ];
  }

  public function data_export_single_post_array( $post, $include_user_data = true ) {
    $response = parent::data_export_single_post_array( $post, $include_user_data );

    $response = $this->_data_export_array_attach_course_data( $response, $post, $include_user_data );

    if( empty( $this->_post_dependencies ) )
      return $response;

    foreach( $this->_post_dependencies as $post_dependency ) {
      $response = $post_dependency->attach_data_export_array_for_course( $response, $include_user_data );
    }

    return $response;
  }

  public function data_import_single_post_array( $post, $id_mapping = [] ) {
    $new_post_id = parent::data_import_single_post_array( $post, $id_mapping );

    if( isset( $post['section_list'] ) && !empty( $post['section_list'] ) ) {
      $course_data_map = $this->_data_import_single_post_section_list( $post['section_list'], $new_post_id );

      if( isset( $post['student_activity'] ) && is_array( $post['student_activity'] ) && !empty( $post['student_activity'] ) ) {
        $student_list = wpep_data_system()->get_user_ids_mapped_by_email();

        foreach( $post['student_activity'] as $student_activity ) {
          if( !isset( $student_list[ $student_activity['user_email_address'] ] ) )
            continue;

          $this->user->data->add(
            $student_activity['key'],
            $student_activity['value'],
            $new_post_id,
            ( isset( $course_data_map['course_section'][ $student_activity['section_id'] ] ) ? $course_data_map['course_section'][ $student_activity['section_id'] ] : 0 ),
            ( isset( $course_data_map['course_lesson'][ $student_activity['lesson_id'] ] )   ? $course_data_map['course_lesson'][ $student_activity['lesson_id'] ]   : 0 ),
            $student_list[ $student_activity['user_email_address'] ]
          );
        }
      }
    }

    if( isset( $post['assessment'] ) && isset( $this->_post_dependencies[ WPEP_POST_TYPE_ASSESSMENT ] ) ) {
      $assessment = $post['assessment'];

      if( !isset( $assessment['meta'] ) || !is_array( $assessment['meta'] ) )
        $assessment['meta'] = [];

      $assessment['meta']['wpep_course_id']  = $new_post_id;
      $assessment['meta']['wpep_section_id'] = 0;

      $this->_post_dependencies[ WPEP_POST_TYPE_ASSESSMENT ]->data_import_single_post_array( $assessment, $id_mapping );
    }

    if( isset( $post['discussions'] ) && !empty( $post['discussions'] ) ) {
      foreach( $post['discussions'] as $discussion_data ) {
        if( !isset( $discussion_data['identifier_args'] ) || !is_array( $discussion_data['identifier_args'] ) )
          continue;

        if( isset( $discussion_data['identifier_args']['course_id'] ) )
          $discussion_data['identifier_args']['course_id'] = $new_post_id;

        if( isset( $discussion_data['identifier_args']['section_id'] ) ) {
          if( !isset( $course_data_map[ 'course_section' ][ $discussion_data['identifier_args']['section_id'] ] ) )
            continue;

          $discussion_data['identifier_args']['section_id'] = $course_data_map[ 'course_section' ][ $discussion_data['identifier_args']['section_id'] ];
        }

        if( isset( $discussion_data['identifier_args']['lesson_id'] ) ) {
          if( !isset( $course_data_map[ 'course_lesson' ][ $discussion_data['identifier_args']['lesson_id'] ] ) )
            continue;

          $discussion_data['identifier_args']['lesson_id'] = $course_data_map[ 'course_lesson' ][ $discussion_data['identifier_args']['lesson_id'] ];
        }

        $this->data_import_single_post_discussion( $new_post_id, $discussion_data );
      }
    }

    return $new_post_id;
  }

  private function _data_import_single_post_section_list( $section_list, $post_id ) {
    $response = [];

    foreach( $section_list as $section_information ) {
      $section_options = [];

      if( isset( $section_information['options'] ) ) {
        $section_options = $section_information['options'];

        if( is_object( $section_options ) )
          $section_options = get_object_vars( $section_options );
        else if( is_string( $section_options ) )
          $section_options = json_decode( $section_options, true );

        if( !is_array( $section_options ) )
          $section_options = [];
      }

      $current_information = [
        'post_id'                    => $post_id,
        'title'                      => $section_information['title'],
        'content'                    => str_replace('\"', '"', ( isset( $section_information['content'] ) ? $section_information['content'] : '' ) ),
        'journey_access_type'        => ( isset( $section_information['journey_access_type'] )  ? $section_information['journey_access_type']  : '' ),
        'journey_access_segment'     => ( isset( $section_information['journey_access_segment'] )  ? $section_information['journey_access_segment']  : '' ),
        'journey_expiration_type'    => ( isset( $section_information['journey_expiration_type'] )  ? $section_information['journey_expiration_type']  : '' ),
        'journey_expiration_segment' => ( isset( $section_information['journey_expiration_segment'] )  ? $section_information['journey_expiration_segment']  : '' ),
        'journey_execution_type'     => ( isset( $section_information['journey_execution_type'] )  ? $section_information['journey_execution_type']  : '' ),
        'journey_execution_segment'  => ( isset( $section_information['journey_execution_segment'] )  ? $section_information['journey_execution_segment']  : '' ),
        'order'                      => $section_information['order'],
        'options'                    => json_encode( $section_options )
      ];

      wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION, $current_information );
      $section_id = wpep_wpdb()->insert_id;

      if( isset( $section_information['id'] ) )
        $response['course_section'][ $section_information['id'] ] = $section_id;

      if( isset( $section_information['assessment'] ) && isset( $this->_post_dependencies[ WPEP_POST_TYPE_ASSESSMENT ] ) ) {
        $assessment = $section_information['assessment'];

        if( !isset( $assessment['meta'] ) || !is_array( $assessment['meta'] ) )
          $assessment['meta'] = [];

        $assessment['meta']['wpep_course_id']  = $post_id;
        $assessment['meta']['wpep_section_id'] = $section_id;

        $this->_post_dependencies[ WPEP_POST_TYPE_ASSESSMENT ]->data_import_single_post_array( $assessment );
      }

      if( !isset( $section_information['lessons'] ) )
        continue;

      if( !is_array( $section_information['lessons'] ) )
        continue;

      foreach( $section_information['lessons'] as $lesson_information ) {
        $lesson_options = [];

        if( isset( $section_information['options'] ) ) {
          $lesson_options = $lesson_information['options'];

          if( is_object( $lesson_options ) )
            $lesson_options = get_object_vars( $lesson_options );
          else if( is_string( $lesson_options ) )
            $lesson_options = json_decode( $lesson_options, true );

          if( !is_array( $lesson_options ) )
            $lesson_options = [];
        }

        $current_information = [
          'post_id'                   => $post_id,
          'section_id'                => $section_id,
          'title'                     => $lesson_information['title'],
          'content'                   => str_replace('\"', '"', $lesson_information['content']),
          'journey_access_type'        => ( isset( $section_information['journey_access_type'] )  ? $section_information['journey_access_type']  : '' ),
          'journey_access_segment'     => ( isset( $section_information['journey_access_segment'] )  ? $section_information['journey_access_segment']  : '' ),
          'journey_expiration_type'    => ( isset( $section_information['journey_expiration_type'] )  ? $section_information['journey_expiration_type']  : '' ),
          'journey_expiration_segment' => ( isset( $section_information['journey_expiration_segment'] )  ? $section_information['journey_expiration_segment']  : '' ),
          'journey_execution_type'     => ( isset( $section_information['journey_execution_type'] )  ? $section_information['journey_execution_type']  : '' ),
          'journey_execution_segment'  => ( isset( $section_information['journey_execution_segment'] )  ? $section_information['journey_execution_segment']  : '' ),
          'order'                     => $lesson_information['order'],
          'options'                   => json_encode( $lesson_options )
        ];

        wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON, $current_information );

        $lesson_id = wpep_wpdb()->insert_id;

        if( isset( $lesson_information['id'] ) )
          $response['course_lesson'][ $lesson_information['id'] ] = $lesson_id;

        if( !isset( $lesson_information['students_completed'] ) )
          continue;

        if( !is_array( $lesson_information['students_completed'] )
            || empty( $lesson_information['students_completed'] ) )
          continue;

        foreach( $lesson_information['students_completed'] as $student_completed ) {
          if( !isset( wpep_data_system()->get_user_ids_mapped_by_email()[ $student_completed['user_email_address'] ] ) )
            continue;

          wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED, [
            'user_id'    => wpep_data_system()->get_user_ids_mapped_by_email()[ $student_completed['user_email_address'] ],
            'lesson_id'  => $lesson_id,
            'created_at' => $student_completed['created_at']
          ] );
        }
      }
    }

    return $response;
  }

  private function _data_export_array_attach_course_data( $response, $post, $include_user_data = true ) {
    $user_list = wpep_data_system()->get_users();

    $course_id = $post->ID;

    $section_list = Entity_Course::instance()->get_with_lessons( $course_id );

    $response['section_list'] = [];

    if( $include_user_data ) {
      $response['student_activity'] = [];

      foreach( $this->user->data->get( false, $course_id, false, false, false, false ) as $course_user_data_row ) {
        if( !isset( $user_list[ $course_user_data_row->user_id ] ) )
          continue;

        $response['student_activity'][] = [
          'user_id'             => $course_user_data_row->user_id,
          'user_email_address'  => $user_list[ $course_user_data_row->user_id ]['email_address'],
          'section_id'          => $course_user_data_row->section_id,
          'lesson_id'           => $course_user_data_row->lesson_id,
          'key'                 => $course_user_data_row->key,
          'value'               => $course_user_data_row->value
        ];
      }
    }


    foreach( $section_list as $section_key => $section_information ) {
      $export_section = [
        'id'                         => $section_information->id, // normally we don't care, but in case of the User Activity Retain we want this.
        'title'                      => $section_information->title,
        'content'                    => ( isset( $section_information->content ) ? $section_information->content : '' ),
        'journey_access_type'        => ( isset( $section_information->journey_access_type )  ? $section_information->journey_access_type  : '' ),
        'journey_access_segment'     => ( isset( $section_information->journey_access_segment )  ? $section_information->journey_access_segment  : '' ),
        'journey_expiration_type'    => ( isset( $section_information->journey_expiration_type )  ? $section_information->journey_expiration_type  : '' ),
        'journey_expiration_segment' => ( isset( $section_information->journey_expiration_segment )  ? $section_information->journey_expiration_segment  : '' ),
        'journey_execution_type'     => ( isset( $section_information->journey_execution_type )  ? $section_information->journey_execution_type  : '' ),
        'journey_execution_segment'  => ( isset( $section_information->journey_execution_segment )  ? $section_information->journey_execution_segment  : '' ),
        'order'                      => $section_information->order,
        'options'                    => ( is_object( $section_information->options ) ? get_object_vars( $section_information->options ) : $section_information->options ),
        'lessons'                    => []
      ];



      if( !empty( $this->_post_dependencies ) ) {
        foreach( $this->_post_dependencies as $post_dependency ) {
          $export_section = $post_dependency->attach_data_export_array_for_course_section( $export_section, $course_id, $section_information->id, $include_user_data );
        }
      }

      foreach( $section_information->lessons as $lesson_ley => $lesson_information ) {
        $export_section['lessons'][$lesson_information->id] = [
          'id'                         => $lesson_information->id, // normally we don't care, but in case of the User Activity Retain we want this.
          'title'                      => $lesson_information->title,
          'content'                    => $lesson_information->content,
          'journey_access_type'        => ( isset( $section_information->journey_access_type )  ? $section_information->journey_access_type  : '' ),
          'journey_access_segment'     => ( isset( $section_information->journey_access_segment )  ? $section_information->journey_access_segment  : '' ),
          'journey_expiration_type'    => ( isset( $section_information->journey_expiration_type )  ? $section_information->journey_expiration_type  : '' ),
          'journey_expiration_segment' => ( isset( $section_information->journey_expiration_segment )  ? $section_information->journey_expiration_segment  : '' ),
          'journey_execution_type'     => ( isset( $section_information->journey_execution_type )  ? $section_information->journey_execution_type  : '' ),
          'journey_execution_segment'  => ( isset( $section_information->journey_execution_segment )  ? $section_information->journey_execution_segment  : '' ),
          'order'                      => $lesson_information->order,
          'options'                    => ( is_object( $lesson_information->options ) ? get_object_vars( $lesson_information->options ) : $lesson_information->options ),
          'students_completed'         => []
        ];

        if( !$include_user_data )
          continue;

        $students_completed = wpep_wpdb()->get_results( 'SELECT user_id, created_at FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . ' WHERE lesson_id = ' . $lesson_information->id, OBJECT );

        foreach( $students_completed as $student_completed ) {
          if( !isset( $user_list[ $student_completed->user_id ] ) )
            continue;

          $export_section['lessons'][$lesson_information->id]['students_completed'][$student_completed->user_id] = [
            'user_id'            => $student_completed->user_id,
            'user_email_address' => $user_list[ $student_completed->user_id ]['email_address'],
            'created_at'         => $student_completed->created_at
          ];
        }
      }

      $response['section_list'][$section_key] = $export_section;
    }

    return $response;
  }

}
