<?php

namespace WPEP\ContentType\Course\Container;

trait Library {

  public function get_grid_content_options() {
    return [
      'courses_per_row' => [
        'name'    => __( "Courses Per Row", "wpep" ),
        'type'    => 'select',
        'options' => [ '' => __( "Default", "wpep" ) ] + wpep_content_primary_row_options( WPEP_POST_TYPE_COURSE ) + [ 0 => __( "Do not display", "wpep" ) ]
      ],
      'courses'         => [
        'name'          => __( "Courses", "wpep" ),
        'type'          => 'multiple_select',
        'options'       => [],
        'description'   => __( "If you want to Display specific Course Tabs only leave the Categories field empty and disable Category Navigation.", "wpep" ),
        'attributes'    => [
          'data-ec-library'      => 'select2',
          "data-ec-library-mode" => 'post-search-map|post_type:' . $this->get_post_type()
        ]
      ],
      'course_progress_bars' => [
        'name'    => __( "Course Progress Bars", "wpep" ),
        'type'    => 'select',
        'options' => [
            ''  => __( "Default", "wpep")
          ] + wpep_flag_setting_field_options(),
      ]
    ];
  }

  public function process_grid_content_options( $filter_proxy_list, $attributes ) {
    $courses_per_row      = isset($attributes['courses_per_row'])      ? $attributes['courses_per_row']  : '';
    $course_progress_bars = isset($attributes['course_progress_bars']) ? intval( $attributes['course_progress_bars'] )  : '';
    $course_list_ids      = isset($attributes['courses'])              ? $attributes['courses'] : '';

    if( $course_progress_bars !== '' )
      $filter_proxy_list['wpep_primary_content_grid_courses_progress_bars'] = $course_progress_bars;

    if( $courses_per_row !== '' )
      $filter_proxy_list['wpep_primary_content_grid_courses_per_row'] = ( $courses_per_row === WPEP_ITEMS_PER_ROW_DYNAMIC ? WPEP_ITEMS_PER_ROW_DYNAMIC : intval( $courses_per_row ) );

    if( $course_list_ids !== '' ) {
      $filter_proxy_list['wpep_primary_content_grid_categories'] = [];

      $filter_proxy_list['wpep_primary_content_' . WPEP_POST_TYPE_COURSE . '_list_args'] = function ($args) use ($course_list_ids) {
        $args['post__in'] = wp_parse_id_list( $course_list_ids );

        if( isset( $args['meta_query'] ) && isset( $args['meta_query']['wpep_show_on_index'] ) )
          unset( $args['meta_query']['wpep_show_on_index'] );

        return $args;
      };
    }

    return $filter_proxy_list;
  }

  public function library_output_grid_content_html( $course_categories = [], $post_list = [] ) {
    $items_per_row = apply_filters( 'wpep_primary_content_grid_courses_per_row', wpep_get_setting( 'index-page-courses-per-row' ) );

    if( $items_per_row == 0 )
      return;

    if( empty( $post_list ) ) {
      $post_list = $this->library_get_grid_post_list( $course_categories );

      if( empty( $post_list ) )
        return;
    }

    $post_list = wpep_utilities_map_object( $post_list, 'ID' );

    if( get_current_user_id() !== 0 ) {
      wpep_content_type_course_instance()->user->data->preload(
        ( wpep_course_has_bookmarks() ? [ WPEP_USER_COURSE_IS_BOOKMARKED, WPEP_USER_COURSE_PROGRESS ] : WPEP_USER_COURSE_PROGRESS ),
        array_keys( $post_list ),
        get_current_user_id()
      );
    }

    $data = ' data-wpep-content-type="' . $this->get_post_type() . '"';

    if( $items_per_row == WPEP_ITEMS_PER_ROW_DYNAMIC ) {
      $data .= ' data-items-per-row="3" data-items-per-row-dynamic="' . WPEP_COURSE_PRESENTATION_CARD_MIN_WIDTH . '"';
    } else {
      $data .= 'data-items-per-row="' . $items_per_row . '"';
    }

    echo '<div class="wpep-course-grid-items-container"' . $data . '>';

    foreach( $post_list as $post_item )
      echo    $this->get_grid_item_content( $post_item, false );

    echo    '<div class="wpep-clearfix"></div>';
    echo '</div>';
  }

  public function library_allow_content() {
    return apply_filters( 'wpep_primary_content_grid_courses_per_row', wpep_get_setting( 'index-page-courses-per-row' ) ) != 0;
  }

  public function get_grid_item_content( $course_object, $short_format = true ) {
    $data = [
      'item'                  => $course_object,
      'short_description'     => wpep_course_excerpt( $course_object->ID, $short_format ),
      'course_link'           => apply_filters( 'wpep_primary_content_item_link', get_permalink( $course_object->ID ), $course_object->ID ),
      'thumbnail_img'         => wpep_course_thumbnail_url( $course_object->ID ),
      'thumbnail_img_hover'   => '',
      'thumbnail_ribbon_text' => do_shortcode( get_post_meta( $course_object->ID, 'wpep_course_ribbon_text', true ) ),
      'user_id'               => get_current_user_id(),
      'card_tags'             => []
    ];

    if( wpep_get_setting_flag( 'index-page-enable-hover-image' ) )
      $data['thumbnail_img_hover'] = wpep_course_thumbnail_hover_url( $course_object->ID );

    if( wpep_content_primary_has_category_card_labels() ) {
      $post_labels = wp_get_post_terms( $course_object->ID, WPEP_TAXONOMY_CATEGORY, [
        "fields"        => "id=>name",
        "meta_key"      => "wpep_show_as_tag",
        "meta_compare"  => "=",
        "meta_value"    => "1"
      ] );

      foreach( $post_labels as $post_label_id => $post_label_name ) {
        $icon = get_term_meta( $post_label_id, 'wpep_tag_icon', true );

        $data['card_tags'][ $post_label_name ] = ( is_array( $icon ) ? ( isset( $icon['url'] ) ? $icon['url'] : '' ) : $icon );
      }
    }

    return wpep_controller()->template->get_template( 'primary/item-course.php', $data );
  }

  public function library_get_search_details( $current_details, $post ) {
    $current_details['permalink'] = apply_filters( 'wpep_primary_content_item_link', get_permalink( $post ) , $post->ID );
    $current_details['thumbnail'] = wpep_course_thumbnail_url( $post->ID );

    if( wpep_get_setting_flag( 'search-image-ribbon' ) )
      $current_details['thumbnail_ribbon_text'] = do_shortcode( get_post_meta( $post->ID, 'wpep_course_ribbon_text', true ) );

    return $current_details;
  }

}
