<?php

namespace WPEP\ContentType\Course;

use WPEP\Controller as WPEP_Controller;

class Events {

  public function __construct() {
    add_action( 'delete_post', [ $this, "_on_delete_post" ] );
    add_action( 'delete_user', [ $this, "_on_delete_user" ] );

    add_action( 'wpep_user_set_course_data', [ $this, '_on_course_started' ], 20, 7 );
  }

  public function _on_delete_post( $post_id ) {
    if( get_post_type( $post_id ) != WPEP_POST_TYPE_COURSE )
      return;

    $lesson_ids = wpep_wpdb()->get_col(
      'SELECT `id` FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` WHERE `post_id` = ' . $post_id,
      0
    );

    if( !empty ( $lesson_ids ) ) {
      wpep_wpdb()->query( "DELETE FROM `" . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` WHERE `id` IN ( ' . implode( ',', $lesson_ids ). ' ) ' );
      wpep_wpdb()->query( "DELETE FROM `" . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . '` WHERE `lesson_id` IN ( ' . implode( ',', $lesson_ids ). ' ) ' );
    };

    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION, [ 'post_id' => $post_id ] );
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA, [ 'course_id' => $post_id ] );
  }

  public function _on_delete_user( $user_id ) {
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED, [ 'user_id' => $user_id ] );
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA, [ 'user_id' => $user_id ] );
  }

  /**
   * @param $key
   * @param $value
   * @param $course_id
   * @param $section_id
   * @param $lesson_id
   * @param $user_id
   * @param $existent_entry
   */
  public function _on_course_started( $key, $value, $course_id, $section_id, $lesson_id, $user_id, $existent_entry ) {
    if ( $key != WPEP_USER_COURSE_STARTED )
      return;

    if( $existent_entry !== false
        && $existent_entry !== null )
      return;

    $on_boarding_status = get_post_meta( $course_id, 'wpep_course_on_boarding_status', true );

    if( $on_boarding_status === 'email-notification' ) {
      WPEP_Controller::instance()->userEngagement->send_email(
        $user_id,
        get_post_meta( $course_id, 'wpep_course_on_boarding_title', true ),
        get_post_meta( $course_id, 'wpep_course_on_boarding_content', true ),
        true
      );

      return;
    }
  }

}