<?php

namespace WPEP\ContentType\Course\Integration\Builder;

use WPEP\ContentType\Course\Integration\Builder as Course_Builder;

class Lesson {

  /**
   * @var Course_Builder
   */
  public $builderInstance;

  /**
   * Lesson constructor.
   * @param Course_Builder $courseBuilderInstance
   */
  public function __construct( $courseBuilderInstance ) {
    $this->builderInstance = $courseBuilderInstance;
  }

  public function display_lesson( $section_id, $section_lesson = null, $course_section = null, $status = 'visible', $field_prefix = 'wpep_course', $editor_preload = null ) {
    $section_lesson = ( $section_lesson === null ? new \stdClass() : $section_lesson );
    $course_section = ( $course_section === null ? new \stdClass() : $course_section );
    $editor_preload = ( $editor_preload === null ? wpep_get_setting_flag( 'lesson-wp-editor-preload' ) : $editor_preload );

    if( !isset( $section_lesson->options ) )
      $section_lesson->options = new \stdClass();
    else if( is_array( $section_lesson->options ) )
      $section_lesson->options = (object) $section_lesson->options;

    $current_lesson_id     = ( isset( $section_lesson->id ) ? $section_lesson->id : uniqid('lesson_') );
    $current_lesson_prefix = $field_prefix . '[' . $section_id . '][lesson]' . '[' . $current_lesson_id . ']';
    $has_learning_path     = ( isset( $course_section->options->question_answers ) && is_array( $course_section->options->question_answers ) && count( $course_section->options->question_answers ) >= 2 );

    $section_lesson->options->question_answer = ( isset( $section_lesson->options->question_answer ) ? $section_lesson->options->question_answer : -1 );

    echo '<fieldset data-wpep-builder-entity="section_lesson" 
                    data-wpep-builder-entity-status="' . $status . '" 
                    data-wpep-builder-entity-id="' . $current_lesson_id . '" 
                    data-field-group-prefix="' . $current_lesson_prefix . '">';
    echo    '<div class="header-section" data-wpep-builder-entity-triggers="' . ( 3 + ( $this->builderInstance->enable_timing_settings ? 1 : 0 ) + ( $this->builderInstance->enable_downloads ? 1 : 0 ) ) . '">';
    echo      '<span class="element-move"></span>';
    echo      '<span class="wpep-builder-entity-children-branching-twig-label wpep-builder-entity-children-branching-twig" 
                     style="' . ( $has_learning_path ? 'display:inline-block;' : 'display:none' ) . '">' .
      ( $has_learning_path ? ( $section_lesson->options->question_answer == -1 ? 'All' : ( "Q : " . ( $section_lesson->options->question_answer + 1 ) ) ) : '' ) .
      '</span>';
    echo        '<input type="text"
                        name="' . $current_lesson_prefix . '[title]' . '"
                        class="wpep-lesson-title"
                        value="' . esc_attr( isset( $section_lesson->title ) ? $section_lesson->title : '' ) . '"
                        placeholder="' . __( "New Lesson", "wpep" ) . '">';
    echo      '<span data-wpep-builder-entity-trigger="delete_entity" data-wpep-builder-entity-trigger-icon="trash"></span>';
    echo      '<span data-wpep-builder-entity-trigger="duplicate_entity" data-wpep-builder-entity-trigger-icon="duplicate"></span>';

    if( $this->builderInstance->enable_timing_settings )
      echo '<span data-wpep-builder-entity-settings-container-trigger="lesson_timing_options" data-wpep-builder-entity-trigger-icon="calendar"></span>';

    if( $this->builderInstance->enable_downloads )
      echo '<span data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type['lesson_downloads'] . '" data-wpep-builder-entity-settings-container-trigger="lesson_download_options" data-wpep-builder-entity-trigger-icon="downloads"></span>';

    echo      '<span data-wpep-builder-entity-settings-container-trigger="lesson_content" data-wpep-builder-entity-trigger-icon="eye" data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type['lesson_content'] . '"></span>';
    echo      '<div class="wpep-clearfix"></div>';
    echo    '</div>';
    echo    '<div class="content-section">';

    if( $this->builderInstance->enable_timing_settings ) {
      echo      '<div data-wpep-builder-entity-settings-container="lesson_timing_options">';
      $this->builderInstance->display_learning_journey_for_field_map( $current_lesson_prefix, wpep_learning_journey_course_tokens_map(), $section_lesson );
      echo      '</div>';
    }

    if( $this->builderInstance->enable_downloads ) {
      echo      '<div data-wpep-builder-entity-settings-container="lesson_download_options">';
      echo        '<div class="wpep-form-group wpep-course-downloads-section">';
      echo          '<div class="wpep-form-group-row">';
      $this->builderInstance->display_attributes_field_downloads(
        'course_lesson_downloads',
        $current_lesson_prefix . '[option][downloads_list]',
        isset( $section_lesson->options->downloads_list ) ? $section_lesson->options->downloads_list : []
      );
      echo          '</div>';
      echo        '</div>';
      echo      '</div>';
    }

    echo      '<div data-wpep-builder-entity-settings-container="lesson_content">';

    do_action( "wpep_administration_lesson_option_field", $current_lesson_prefix . '[option]', ( isset( $section_lesson->options ) ? $section_lesson->options : [] ) );

    echo      '<div data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type['lesson_icon'] . '" class="wpep-form-group wpep-form-group-icon-details" style="display:none;">';

    echo        '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Lesson Icon", "wpep" ) . '</label>';

    $this->builderInstance->display_select_field(
      'icon_pack_alias',
      [
        'document'   => __( "Document Content", "wpep" ),
        'video'      => __( "Video Content", "wpep" ),
        'sound'      => __( "Audio Content", "wpep" )
      ],
      $current_lesson_prefix . '[option][icon_pack_alias]' ,
      ( isset( $section_lesson->options->icon_pack_alias ) ? $section_lesson->options->icon_pack_alias : '' )
    );

    echo       '</div>';
    echo      '</div>';


    echo      '<div data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type['lesson_video'] . '" class="wpep-form-group wpep-form-group-video-details" style="display:none;">';

    echo        '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Video URL", "wpep" ) . '</label>';

    $this->builderInstance->display_text_field(
      'video_embed_source',
      $current_lesson_prefix . '[option][video_embed_source]' ,
      ( isset( $section_lesson->options->video_embed_source ) ? $section_lesson->options->video_embed_source : '' )
    );

    echo       '</div>';
    echo       '<div class="wpep-form-group-row wpep-form-group-row-video-image">';

    $this->builderInstance->display_image_field(
      'video_embed_image',
      __( "Video Image", "wpep" ),
      $current_lesson_prefix . '[option][video_embed_image]' ,
      ( isset( $section_lesson->options->video_embed_image ) ? $section_lesson->options->video_embed_image : '' )
    );

    echo          '<p class="description">' . __( "Chose an image for this video that will appear on the navigation bar of videos.", "wpep" ) . '</p>';
    echo        '</div>';

    echo        '<h3>' . __( "Optional Course Description") . '</h3>';
    echo        '<p>' . __( "Display instructions under the video, in case you want to offer file downloads or extra guidance, this is where you should place it.", "wpep" ) . '</p>';
    echo      '</div>';

    echo      '<div class="wpep-builder-entity-children-branching-twig-container" data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type[ 'learning_path' ] . '">';
    echo        '<div class="wpep-form-group wpep-builder-entity-children-branching-twig" style="display:' . ( $has_learning_path ? 'block' : 'none' ). '">';
    echo          '<div class="wpep-form-group-row">';
    echo            '<label>' . __( "Learning Path", "wpep" ) . '</label>';

    $this->builderInstance->display_select_field(
      'question_answer',
      (
        [
          -1 => __( "Available for any answer", "wpep" )
        ] +
        ( isset( $course_section->options->question_answers ) && is_array( $course_section->options->question_answers ) ? $course_section->options->question_answers : [] )
      ),
      $current_lesson_prefix . '[option][question_answer]',
      $section_lesson->options->question_answer
    );

    echo            '<p class="description">' . __( "This section has learning path configured, chose which learning path is required to display this lesson.", "wpep" ) . '</p>';
    echo        '</div>';
    echo        '</div>';
    echo      '</div>';

    echo      '<div class="wpep-form-group textarea primary-content-editor">';

    if( $editor_preload ) {
      wpep_editor(
        ( isset( $section_lesson->content ) ? $section_lesson->content : '' ),
        'lesson-' . ( isset( $section_lesson->id ) ? $section_lesson->id : str_replace( 'lesson_', '', $current_lesson_id ) ),
        $current_lesson_prefix . '[content]'
      );
    }

    echo      '</div>';
    echo      '</div>';
    echo    '</div>';
    echo '</fieldset>';
  }

}