<?php

namespace WPEP\ContentType\Course\Integration\Builder;

use WPEP\ContentType\Course\Integration\Builder as Course_Builder;
use WPEP\Entity\Course as Entity_Course;

class Section {

  /**
   * @var Course_Builder
   */
  public $builderInstance;

  /**
   * Lesson constructor.
   * @param Course_Builder $courseBuilderInstance
   */
  public function __construct( $courseBuilderInstance ) {
    $this->builderInstance = $courseBuilderInstance;
  }

  public function display_management() {
    echo    '<div data-wpep-builder-entity-container="section" 
                  data-wpep-builder-entity-deleted-input="wpep_deleted_section">';

    $course_sections = Entity_Course::instance()->get_with_lessons( get_the_ID() );

    if( !empty( $course_sections ) ) {
      foreach( $course_sections as $course_section )
        $this->_display_section( $course_section );
    } else {
      for( $i = 1; $i <= $this->builderInstance->new_course_sections; $i++ )
        $this->_display_section();
    }

    $this->_display_section( null, 'template', 'wpep_course', false );

    echo    '</div>';

    echo    '<div class="wpep-button-row">';
    echo      '<button type="button" data-wpep-builder-entity-trigger="add_entity" class="button button-primary">' . __( "Add Section", "wpep") . '</button>';
    echo    '</div>';
  }

  private function _display_section( $course_section = null, $status = 'visible', $field_prefix = 'wpep_course', $editor_preload = null ) {
    $editor_preload = ( $editor_preload === null ? wpep_get_setting_flag( 'lesson-wp-editor-preload' ) : $editor_preload );
    $course_section = ( $course_section === null ? new \stdClass() : $course_section );

    if( !isset( $course_section->options ) )
      $course_section->options = new \stdClass();

    $course_section_id     = isset( $course_section->id ) ? $course_section->id : uniqid('section_');
    $current_course_prefix = $field_prefix . '[' . $course_section_id . ']';

    echo '<fieldset data-wpep-builder-entity="section" 
                    data-wpep-builder-entity-status="' . $status . '" 
                    data-wpep-builder-entity-id="' . $course_section_id . '" 
                    data-field-group-prefix="' . $current_course_prefix . '">';

    echo    '<div class="header-section" data-wpep-builder-entity-triggers="' . ( 2 + ( $this->builderInstance->enable_timing_settings ? 1 : 0 ) + ( $this->builderInstance->enable_assessments ? 1 : 0 ) ) . '">';
    echo      '<span class="element-move"></span>';
    echo      '<input type="text" name="' . $current_course_prefix . '[title]" class="wpep-section-title" placeholder="' . __( 'New Section', 'wpep' ) . '" value="' . esc_attr( isset( $course_section->title ) ? $course_section->title : '' ) . '">';
    echo      '<span data-wpep-builder-entity-trigger="delete_entity" data-wpep-builder-entity-trigger-icon="trash"></span>';
    echo      '<span data-wpep-builder-entity-settings-container-trigger="section_settings" data-wpep-builder-entity-trigger-icon="settings"></span>';

    if( $this->builderInstance->enable_timing_settings )
      echo '<span data-wpep-builder-entity-settings-container-trigger="section_timing_options" data-wpep-builder-entity-trigger-icon="calendar"></span>';

    if( $this->builderInstance->enable_assessments )
      echo '<span data-wpep-builder-entity-settings-container-trigger="section_assessment" data-wpep-builder-entity-trigger-icon="university" data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type[ 'assessment_management' ] . '"></span>';

    echo      '<div class="wpep-clearfix"></div>';
    echo    '</div>';
    echo    '<div class="content-section">';

    if( $this->builderInstance->enable_timing_settings ) {
      echo      '<div data-wpep-builder-entity-settings-container="section_timing_options">';
      $this->builderInstance->display_learning_journey_for_field_map( $current_course_prefix, wpep_learning_journey_course_tokens_map(), $course_section );
      echo      '</div>';

    }

    echo      '<div data-wpep-builder-entity-settings-container="section_settings">';

    echo      '<div data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type[ 'learning_path' ] . '" class="wpep-form-group" style="display:none;">';

    echo        '<h3>' . __( "Section Description", "wpep" ) . '</h3>';
    echo      '</div>';

    echo      '<div class="wpep-form-group textarea primary-content-editor">';

    if( $editor_preload ) {
      wpep_editor(
        ( isset( $course_section->content ) ? $course_section->content : '' ),
        'wpep-section-' . ( isset( $course_section->id ) ? $course_section->id : str_replace( 'section_', '', $course_section_id ) ),
        $current_course_prefix . '[content]'
      );
    }

    echo      '</div>';


    echo      '<div data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type[ 'learning_path' ] . '" class="wpep-form-group" style="display:none;">';

    if( $this->builderInstance->enable_learning_paths )
      $this->_display_learning_path_configuration( $course_section, $current_course_prefix );

    echo          '<div class="wpep-form-group-row" data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type['section_checkbox'] . '">';
    echo            '<h3>' . __( "Misc Options", "wpep" ) . '</h3>';
    echo            '<label>' . __( "Section Checkbox", "wpep" ) . '</label>';

    $this->builderInstance->display_checkbox_field(
      'display_checkbox',
      ( isset( $course_section->options->display_checkbox ) ? $course_section->options->display_checkbox : 1 ),
      $current_course_prefix . '[option][display_checkbox]'
    );

    echo            '<p class="description">' . __( "Optionally hide the section checkbox, this will work perfectly if you want to connect sections based on learning paths.", "wpep" ) . '</p>';
    echo        '</div>';

    echo     '</div>';

    do_action( "wpep_administration_section_option_field", $current_course_prefix . '[option]', ( isset( $course_section->options ) ? $course_section->options : [] ) );

    echo      '</div>';

    if( $this->builderInstance->enable_assessments ) {
      echo      '<div data-wpep-builder-entity-settings-container="section_assessment" data-field-group-prefix="wpep_assessment[' . $course_section_id . ']">';
      wpep_content_type_assessment_instance()->admin->display_assessment_for_course( $course_section_id, ( isset( $course_section->post_id ) && $course_section->post_id != 0 ? $course_section->post_id : get_the_ID() ) );
      echo      '</div>';
    }

    echo      '<div data-wpep-builder-entity-container="section_lesson" 
                    data-field-group-prefix="' . $current_course_prefix . '[lesson]"
                    data-wpep-builder-entity-container-allow-global-sortable="1" 
                    data-wpep-builder-entity-parent="section" 
                    data-wpep-builder-entity-deleted-input="wpep_deleted_lesson">';

    if( !is_numeric( $course_section_id ) ) {
      for( $i = 1; $i <= $this->builderInstance->new_course_lessons; $i++ )
        $this->builderInstance->builderLesson->display_lesson( $course_section_id, null, null, 'visible', $field_prefix );

    } else if( isset( $course_section->lessons ) && !empty( $course_section->lessons ) ) {
      foreach( $course_section->lessons as $section_lesson )
        $this->builderInstance->builderLesson->display_lesson( $course_section_id, $section_lesson, $course_section, 'visible', $field_prefix );
    }

    $this->builderInstance->builderLesson->display_lesson(
      $course_section_id, null, null,
      'template',
      'wpep_course',
      false
    );

    echo      '</div>';
    echo      '<div data-wpep-builder-entity-footer="section_lesson">';
    echo        '<span data-wpep-builder-entity-trigger="add_entity" data-wpep-builder-entity-trigger-icon="plus" data-wpep-builder-entity-trigger-open="' . intval( wpep_get_setting_flag( 'lesson-wp-editor-enabled-new' ) ) . '"></span>';
    echo      '</div>';
    echo    '</div>';
    echo '</fieldset>';
  }

  private function _display_learning_path_configuration( $course_section, $current_course_prefix ) {
    echo '<h3>' . __("Learning Path Options") . '</h3>';

    echo '<div class="wpep-form-group-row">';
    if (isset($course_section->options->question_answers) && is_array($course_section->options->question_answers) && !empty($course_section->options->question_answers)) {

      foreach ($course_section->options->question_answers as $conditional_logic_answer)
        $this->builderInstance->display_text_field(
          'question_answers',
          $current_course_prefix . '[option][question_answers][]',
          $conditional_logic_answer,
          'wpep-builder-entity-children-branching-group',
          __("Leave empty to ignore", "wpep")
        );

      if (count($course_section->options->question_answers) == 1)
        $this->builderInstance->display_text_field(
          'question_answers',
          $current_course_prefix . '[option][question_answers][]',
          '',
          'wpep-builder-entity-children-branching-group',
          __("Please complete...", "wpep")
        );

    } else {
      $this->builderInstance->display_text_field(
        'question_answers',
        $current_course_prefix . '[option][question_answers][]',
        '',
        'wpep-builder-entity-children-branching-group',
        __("Leave empty to ignore", "wpep")
      );

      $this->builderInstance->display_text_field(
        'question_answers',
        $current_course_prefix . '[option][question_answers][]',
        '',
        'wpep-builder-entity-children-branching-group',
        __("Leave empty to ignore", "wpep")
      );
    }
    echo '</div>';
    echo '<div class="wpep-button-row">';
    echo  '<button class="button button-primary wpep-builder-entity-children-branching-add-twig">' . __("Add Learning Path", "wpep") . '</button>';
    echo '</div>';
  }

}