<?php

namespace WPEP\ContentType\Course\ThirdPartyImport;

class LearnDash extends Skeleton {

  public $import_entity_sub_type = 'sfwd-courses';
  public $taxonomy_term_map = [
    'ld_course_category' => WPEP_TAXONOMY_CATEGORY
  ];

  public function get_name() {
    return 'LearnDash 2.x';
  }

  public function get_alias() {
    return 'learndash';
  }

  public function get_content_types_list() {
    return [ WPEP_POST_TYPE_COURSE ];
  }

  public function implementation_import_single_content_map( $import_entity_id ) {
    $current_lessons    = get_posts( [
      'post_status' => wpep_post_monitored_post_status_list(),
      'numberposts' => -1,
      'post_type'   => 'sfwd-lessons',
      'meta_key'    => 'course_id',
      'meta_value'  => $import_entity_id,
      'orderby'     => 'menu_order',
      'order'       => 'ASC'
    ] );
    $current_topics     = get_posts( [
      'post_status' => wpep_post_monitored_post_status_list(),
      'numberposts' => -1,
      'post_type'   => 'sfwd-topic',
      'meta_key'    => 'course_id',
      'meta_value'  => $import_entity_id,
      'orderby'     => 'menu_order',
      'order'       => 'ASC'
    ] );

    $response = [
      'lessons'   => [],
      'sections'  => [],
    ];

    if( empty( $current_lessons ) )
      return [];

    foreach( $current_lessons as $current_lesson ) {
      $import_lesson_information = get_post_meta( $current_lesson->ID, '_sfwd-lessons', true );

      $response[ 'sections' ][ $current_lesson->ID ] = [
        'title'             => $current_lesson->post_title,
        'content'           => $current_lesson->post_content
      ];

      if( isset( $import_lesson_information['sfwd-lessons_lesson_materials'] ) && $import_lesson_information['sfwd-lessons_lesson_materials'] != '' )
        $response[ 'sections' ][ $current_lesson->ID ]['downloads'] = wpep_utilities_html_links_to_array( html_entity_decode( $import_lesson_information['sfwd-lessons_lesson_materials'] ) );
    }

    // Okay, there's no topics, that we don't need to map any topics into lessons, and the sections become the Lessons.
    if( empty( $current_topics ) )
      return [
        'sections'  => [],
        'lessons'   => $response[ 'sections' ]
      ];

    foreach( $current_topics as $current_topic ) {
      $import_topic_information = get_post_meta( $current_topic->ID, '_sfwd-topic', true );

      $response[ 'lessons' ][ $current_topic->ID ] = [
        'import_section_id' => get_post_meta( $current_topic->ID, 'lesson_id', true ),
        'title'             => $current_topic->post_title,
        'content'           => $current_topic->post_content
      ];

      if( isset( $import_topic_information['sfwd-topic_topic_materials'] ) && $import_topic_information['sfwd-topic_topic_materials'] != '' )
        $response[ 'lessons' ][ $current_topic->ID ]['downloads'] = wpep_utilities_html_links_to_array( html_entity_decode( $import_topic_information['sfwd-topic_topic_materials'] ) );
    }

    return $response;
  }

  public function implementation_import_single_remove( $import_entity_id ) {
    $to_delete = get_posts( [
      'fields'      => 'ids',
      'numberposts' => -1,
      'post_type'   => [ 'sfwd-lessons', 'sfwd-topic' ],
      'meta_key'    => 'course_id',
      'meta_value'  => $import_entity_id
    ] );

    foreach( $to_delete as $delete_post_id )
      wp_delete_post( $delete_post_id );

    wp_delete_post( $import_entity_id );
  }

}