<?php

namespace WPEP\ContentType\Course\ThirdPartyImport;

class LifterLMS extends Skeleton {

  public $import_entity_sub_type = 'course';
  public $import_entity_sub_type_meta_key = "_llms_length";

  public $taxonomy_term_map = [
    'course_cat' => WPEP_TAXONOMY_CATEGORY
  ];

  public function get_name() {
    return 'LifterLMS';
  }

  public function get_alias() {
    return 'lifter_lms';
  }

  public function get_default_preferences() {
    return [
      'default_course_type'   => WPEP_COURSE_TYPE_EXECUTION_PLAN
    ];
  }

  public function get_preferences_field_map() {
    return [
      'default_course_type'   => [
        'name'        => __( "Course Type", "wpep" ),
        'type'        => 'select',
        'options'     => wpep_course_layout_types( [ WPEP_COURSE_TYPE_EXECUTION_PLAN, WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED, WPEP_COURSE_TYPE_LEARN_MAP, WPEP_COURSE_TYPE_STANDARD ] ),
        'description' => sprintf( __( "When a the %s course can be split into multiple lessons and sections, this is the course type that will be used.", "wpep" ), $this->get_name() ) . '<br/>' .
          sprintf( __( "If the system cannot determine multiple lessons or sections in %s, it will be migrated as an One-Off Course.", "wpep" ), $this->get_name() )
      ]
    ];
  }

  public function get_content_types_list() {
    return [ WPEP_POST_TYPE_COURSE ];
  }

  public function implementation_import_single_content_map( $import_entity_id ) {
    $response = [
      'lessons'   => [],
      'sections'  => [],
    ];

    $llms_sections   = get_posts( [
      'post_status' => wpep_post_monitored_post_status_list(),
      'numberposts' => -1,
      'post_type'   => 'section',
      'meta_key'    => '_llms_order',
      'orderby'     => 'meta_value_num',
      'order'       => 'ASC',
      'meta_query' => [
        [
          'key'         => '_llms_parent_course',
          'value'       => $import_entity_id,
        ]
      ]
    ] );
    $llms_lessons    = get_posts( [
      'post_status' => wpep_post_monitored_post_status_list(),
      'numberposts' => -1,
      'post_type'   => 'lesson',
      'meta_key'    => '_llms_order',
      'orderby'     => 'meta_value_num',
      'order'       => 'ASC',
      'meta_query' => [
        [
          'key'         => '_llms_parent_course',
          'value'       => $import_entity_id,
        ]
      ]
    ] );

    if( empty( $llms_lessons ) || empty( $llms_sections ) )
      return [];

    foreach( $llms_sections as $llms_section )
      $response[ 'sections' ][ $llms_section->ID ] = [
        'title'   => $llms_section->post_title,
        'content' => $llms_section->post_content
      ];

    foreach( $llms_lessons as $llms_lesson ) {
      $response[ 'lessons' ][ $llms_lesson->ID ] = [
        'import_section_id' => get_post_meta( $llms_lesson->ID, '_llms_parent_section', true ),
        'title'             => $llms_lesson->post_title,
        'content'           => $llms_lesson->post_content
      ];
    }

    return $response;
  }

  public function implementation_import_single_remove( $import_entity_id ) {
    $to_delete = get_posts( [
      'fields'      => 'ids',
      'numberposts' => -1,
      'post_type'   => [ 'section', 'lesson' ],
      'meta_key'    => '_llms_parent_course',
      'meta_value'  => $import_entity_id
    ] );

    foreach( $to_delete as $delete_post_id )
      wp_delete_post( $delete_post_id );

    wp_delete_post( $import_entity_id );
  }

}