<?php

namespace WPEP\ContentType\Course\ThirdPartyImport;

class ProgressAlly extends Skeleton {

  public $import_entity_type              = "post";
  public $import_entity_sub_type          = "page";
  public $import_entity_sub_type_meta_key = "_progressally_post_meta";

  public function get_name() {
    return 'ProgressAlly';
  }

  public function get_alias() {
    return 'progress_ally';
  }

  public function get_description() {
    return '<p>' . sprintf( __( "Import your %s Objectives into %s", "wpep" ), $this->get_name(), WPEP_PLUGIN_NAME ) . '</p>';
  }

  public function get_default_preferences() {
    return [
      'default_section_title' => __( "Start Here", "wpep" ),
      'default_course_type'   => WPEP_COURSE_TYPE_EXECUTION_PLAN
    ];
  }

  public function get_preferences_field_map() {
    return [
      'default_section_title' => [
        'name'        => __( "Section Title", "wpep" ),
        'type'        => 'text',
        'description' => sprintf( __( "ProgressAlly offers objectives, without being organized in sections %s requires Lessons to be part of a section.", "wpep" ), WPEP_PLUGIN_NAME )
      ],
      'default_course_type'   => [
        'name'        => __( "Course Type", "wpep" ),
        'type'        => 'select',
        'options'     => wpep_course_layout_types( [ WPEP_COURSE_TYPE_EXECUTION_PLAN, WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED, WPEP_COURSE_TYPE_LEARN_MAP ] ),
        'description' => sprintf( __( "The Course Type you want your ProgressAlly Objectives to be migrated to in %s.", "wpep" ), WPEP_PLUGIN_NAME )
      ]
    ];
  }

  public function get_content_types_list() {
    return [ WPEP_POST_TYPE_COURSE ];
  }

  public function implementation_get_import_entity_ids() {
    $import_posts = parent::implementation_get_import_entity_ids();

    if( empty( $import_posts ) )
      return [];

    $progress_ally_objective_table = wpep_wpdb()->prefix . 'pa_post_objective';

    return wpep_wpdb()->get_col(
      'SELECT DISTINCT post_id FROM ' . $progress_ally_objective_table . ' WHERE post_id != 0 ORDER BY post_id ASC'
    );
  }

  public function implementation_import_single_content_map( $import_entity_id ) {
    $response = [
      'lessons'   => [],
      'sections'  => [],
    ];

    $progress_ally_objective_table = wpep_wpdb()->prefix . 'pa_post_objective';

    $progress_ally_objectives = wpep_wpdb()->get_results(
      'SELECT * FROM ' . $progress_ally_objective_table . ' WHERE post_id = ' . intval( $import_entity_id ) . ' ORDER BY id ASC'
    );

    if( empty( $progress_ally_objectives ) )
      return [];

    foreach( $progress_ally_objectives as $progress_ally_objective ) {
      $progress_ally_objective_information = unserialize( $progress_ally_objective->meta );

      $lesson_content = '';

      if( isset( $progress_ally_objective_information['ref-post-id'] )
          && $progress_ally_objective_information['ref-post-id'] != 0 ) {
        $ref_post = get_post( $progress_ally_objective_information['ref-post-id'] );

        $lesson_content .= $ref_post->post_title . "\r\n";
        $lesson_content .= trim( strip_shortcodes ( $ref_post->post_content ) );
      }

      $response['lessons'][ $progress_ally_objective->objective_id ] = [
        'title'   => $progress_ally_objective_information['description'],
        'content' => $lesson_content
      ];
    }

    return $response;
  }

  public function implementation_import_single_remove( $import_entity_id ) {
    wpep_wpdb()->delete( wpep_wpdb()->prefix . 'pa_post_objective', [ 'post_id' => $import_entity_id ] );
  }

}