<?php

namespace WPEP\ContentType\Course\User;

class Section {

  private $_course_section_current_time = 0;
  private $_course_sections_user_started_at = 0;

  /**
   * @param $course_id
   * @param bool $user_id
   * @param bool $include_lessons
   * @return array
   */
  public function get_all_by_course_id( $course_id, $user_id = false, $include_lessons = true ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $response = [];

    $sql = 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . ' WHERE post_id = ' . intval( $course_id ) . ' ORDER BY `order` ASC';

    $course_sections = wpep_wpdb()->get_results($sql);

    foreach( $course_sections as $course_section ) {
      $response[$course_section->id] = $course_section;
      $response[$course_section->id]->user_data = wpep_utility_map_to_object_assoc ( wpep_content_type_course_instance()->user->data->get( false, $course_id, $course_section->id, false, $user_id, false ), 'key', 'value' );
      $response[$course_section->id]->options = ( $course_section->options == '' ? new \stdClass() : json_decode( $course_section->options ) );

      if( $include_lessons )
        $response[$course_section->id]->lessons = wpep_content_type_course_instance()->user->lesson->get_all_by_section_id( $course_section->id, $user_id );
    }

    return $response;
  }

  /**
   * This function will return content as needed based on the Timing Options if enabled.
   * @param $course_id
   * @param bool $user_id
   * @return array
   */
  public function get_all_by_course_id_mapped( $course_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $user_sections = $this->get_all_by_course_id( $course_id, $user_id, true );

    $response      = [];

    $remaining_sections = count( $user_sections );

    $args  = apply_filters( 'wpep_user_activity_course_sections_args', [
      'available_sections' => [],
      'available_lessons'  => []
    ] );

    $this->_course_section_current_time = time();

    try {
      if( $user_id === 0 )
        $user_started_at = $this->_course_section_current_time;
      else
        $user_started_at = intval( wpep_content_type_course_instance()->user->data->get_value( WPEP_USER_COURSE_STARTED, $course_id, false, false, $user_id, true ) );

      $this->_course_sections_user_started_at = $user_started_at;
    } catch ( \Exception $e ) {
      $this->_course_sections_user_started_at = $this->_course_section_current_time;
    }

    $_allow_learning_journey = wpep_get_setting_flag( 'enable-course-timing-option' );

    if( $_allow_learning_journey ) {
      if( get_post( $course_id )->post_author == get_current_user_id() || current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
        $_allow_learning_journey = false;
    }

    foreach( $user_sections as $user_section ) {
      $section_locked = ( !empty( $args['available_sections' ] ) && !in_array( $user_section->id, $args['available_sections' ] ) );

      if( $_allow_learning_journey && !$section_locked ) {
        $journey_status = $this->_get_learning_journey_status( $user_section, $user_id );

        if( $journey_status === 'lock' )
          $section_locked = true;
      }

      $response[$user_section->id] = [
        'title'        => $user_section->title,
        'content'      => $user_section->content,
        'lessons'      => [],
        'options'      => ( isset( $user_section->options )   ? ( is_object( $user_section->options ) ? get_object_vars( $user_section->options ) : $user_section->options ) : [] ),
        'user_data'    => ( isset( $user_section->user_data ) ? ( is_object( $user_section->user_data ) ? get_object_vars( $user_section->user_data ) : $user_section->user_data ) : [] ),
        'is_final'     => ( $remaining_sections == 1 )
      ];

      if( wpep_assessments_enabled() )
        $response[$user_section->id]['assessment_id'] = wpep_user_assessment_id_by_course_id( $course_id, $user_section->id, $user_id );

      $response[$user_section->id] = $this->_attach_lessons_to_section_data( $course_id, $response[$user_section->id], $user_section->lessons, $user_id, [
        'remaining_sections' => $remaining_sections,
        'available_lessons'  => ( $section_locked ? false : $args['available_lessons'] )
      ] );

      $remaining_sections--;
    }

    $response = apply_filters( 'wpep_user_activity_course_sections', $response, $course_id );

    return $response;
  }

  private function _attach_lessons_to_section_data( $course_id, $current_section, $lessons, $user_id, $args = [ 'remaining_sections' => 1, 'available_lessons' => [] ] ) {
    $learning_path     = isset( $current_section['user_data']['learning_path'] ) ? $current_section['user_data']['learning_path'] : false;
    $remaining_section_lessons = count( $lessons );

    $_allow_learning_journey = wpep_get_setting_flag( 'enable-course-timing-option' ) && empty( $args['available_lessons' ] );

    if( $_allow_learning_journey ) {
      if( get_post( $course_id )->post_author == get_current_user_id() || current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
        $_allow_learning_journey = false;
    }

    $_indicator_section_completed = true;

    $current_section['options']['disabled_lessons'] = 0;

    foreach( $lessons as $lesson_key => $lesson_information ) {
      $is_completed = apply_filters( 'wpep_user_activity_lesson_is_completed', $lesson_information->is_completed, $lesson_information->id, $user_id, $lesson_information );

      $lesson_options = ( isset( $lesson_information->options ) ? ( is_object( $lesson_information->options ) ? get_object_vars( $lesson_information->options ) : $lesson_information->options ) : [] );
      $lesson_locked   = $args['available_lessons' ] === false ? false : ( empty( $args['available_lessons' ] ) || in_array( $lesson_information->id, $args['available_lessons'] ) );

      if( $lesson_locked && !$is_completed && $_allow_learning_journey ) {
        $journey_status = $this->_get_learning_journey_status( $lesson_information, $user_id );

        if( $journey_status === 'lock' )
          $lesson_locked = false;
      }

      $current_section['lessons'][$lesson_information->id] = [
        'title'           => $lesson_information->title,
        'content'         => '',
        'is_completed'    => false,
        'completed_at'    => $lesson_information->completed_at,
        'options'         => $lesson_options,
        'is_final'        => ( $args['remaining_sections'] == 1 && $remaining_section_lessons == 1 )
      ];

      if( !$lesson_locked ) {
        $current_section['lessons'][$lesson_information->id]['options']['is_disabled'] = true;

        if( isset( $current_section['lessons'][$lesson_information->id]['options']['downloads_list'] ) )
          unset( $current_section['lessons'][$lesson_information->id]['options']['downloads_list'] );


        $current_section['options']['disabled_lessons']++;

        $_indicator_section_completed = false;

        continue;
      }

      $current_section['lessons'][$lesson_information->id]['content'] = $lesson_information->content;

      if( $is_completed )
        $current_section['lessons'][$lesson_information->id]['is_completed'] = $is_completed;

      $remaining_section_lessons--;

      if( $is_completed == true )
        continue;

      if( $learning_path !== false ) {
        if( isset( $lesson_options['question_answer'] )
          && intval( $lesson_options['question_answer'] ) !== -1
          && $lesson_options['question_answer'] !== $learning_path
        )
          continue;
      }

      $_indicator_section_completed = false;
    }

    if( $current_section['options']['disabled_lessons'] == count( $current_section['lessons'] ) ) {
      $current_section['options']['is_disabled'] = true;
      $current_section['content'] = '';

      unset( $current_section['options']['downloads_list'] );
    }

    $current_section['is_completed'] = $_indicator_section_completed;

    return $current_section;
  }

  /**
   * @param $object
   * @param $user_id
   * @return string
   */
  private function _get_learning_journey_status( $object, $user_id ) {
    if( !$this->_is_learning_journey_condition_met( $object, 'journey_access', WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_INSTANT ) )
      return 'lock';

    if( $this->_is_learning_journey_condition_met( $object, 'journey_expiration', WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_NONE ) )
      return 'lock';

    return 'ok';
  }

  private function _is_learning_journey_condition_met( $object, $type_field_prefix, $default_type = WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_INSTANT ) {
    $type_field_key    = $type_field_prefix . '_type';
    $segment_field_key = $type_field_prefix . '_segment';

    if( $object->$type_field_key === '' )
      $object->$type_field_key = $default_type;

    if( $object->$type_field_key === WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_NONE )
      return false;

    if( $object->$type_field_key === WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_INSTANT )
      return true;

    if( in_array( $object->$type_field_key, [ WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_SPECIFIC_DATE, WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_DRIPPED, WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_DRIPPED_SECONDS ] )
      && $object->$segment_field_key === '' )
      return true;

    if( $this->_course_sections_user_started_at === 0 )
      return false;

    if( $object->$type_field_key === WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_DRIPPED ) {
      try {
        $current_allow_action = new \DateTime( '@' . $this->_course_sections_user_started_at );
      } catch ( \Exception $e ) {
        return false;
      }

      $current_allow_action->modify( $object->$segment_field_key );

      return ( $this->_course_section_current_time > $current_allow_action->setTime(0,0 )->getTimestamp() );
    }

    if( $object->$type_field_key === WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_SPECIFIC_DATE ) {
      try {
        $current_allow_action = new \DateTime( $object->$segment_field_key );
      } catch ( \Exception $e ) {
        return false;
      }

      return ( $this->_course_section_current_time > $current_allow_action->getTimestamp() );
    }

    if( $object->$type_field_key === WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_DRIPPED_SECONDS )
      return $this->_course_sections_user_started_at + intval( $object->$segment_field_key ) < time();

    return false;
  }

}