<?php

/**
 * @return \WPEP\ContentType\Course
 */
function wpep_content_type_course_instance() {
  return \WPEP\ContentType::instance()->contentTypes[ WPEP_POST_TYPE_COURSE ];
}

/**
 * @param $course_id
 * @return bool|int
 */
function wpep_course_assessments_required( $course_id ) {
  $assessments_required = get_post_meta( $course_id, 'wpep_course_assessments_required', true );

  if( $assessments_required === '' )
    return true;

  return intval( $assessments_required );
}

/**
 * @return array|mixed|string
 */
function wpep_course_has_bookmarks() {
  return wpep_get_setting_flag( 'enable-course-bookmarks' );
}

function wpep_course_type_from_id( $current_course_id ) {
  $current_course_type = get_post_meta( $current_course_id , WPEP_COURSE_TYPE_POST_META, true );

  $current_course_type = ( $current_course_type == false ? WPEP_COURSE_TYPE_EXECUTION_PLAN : $current_course_type );
  $current_course_type = apply_filters( 'wpep_single_content_course_type', $current_course_type, $current_course_id );

  return $current_course_type;
}

function wpep_course_type_has_lessons( $course_type ) {
  if( in_array( $course_type, [
    WPEP_COURSE_TYPE_EXECUTION_PLAN,
    WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED,
    WPEP_COURSE_TYPE_STANDARD,
    WPEP_COURSE_TYPE_VIDEO,
    WPEP_COURSE_TYPE_FOCUS,
    WPEP_COURSE_TYPE_LEARN_MAP,
    WPEP_COURSE_TYPE_CHALLENGE,
    WPEP_COURSE_TYPE_SCORM
  ] ) )
    return true;

  return false;
}

/**
 * @param bool|array $include
 * @param bool|array $exclude
 * @return array|string
 */
function wpep_course_layout_types( $include = false, $exclude = false ) {
  $layout_types = [
    WPEP_COURSE_TYPE_EXECUTION_PLAN          => __( 'Execution Plan', "wpep"),
    WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED => __( 'Execution Plan Unfolded', "wpep"),
    WPEP_COURSE_TYPE_STANDARD                => __( 'Standard', "wpep" ),
    WPEP_COURSE_TYPE_VIDEO                   => __( 'Video', "wpep" ),
    WPEP_COURSE_TYPE_FOCUS                   => __( 'Focus', "wpep" ),
    WPEP_COURSE_TYPE_ONE_OFF                 => __( 'One Off', "wpep" ),
    WPEP_COURSE_TYPE_LEARN_MAP               => __( 'Learn Map', "wpep" ),
    WPEP_COURSE_TYPE_CHALLENGE               => __( 'Challenge', "wpep" ),
    WPEP_COURSE_TYPE_SCORM                   => __( 'Scorm', "wpep" )
  ];

  $layout_types = apply_filters( 'wpep_course_layout_types', $layout_types );

  if( is_array( $exclude ) ) {
    foreach( $layout_types as $layout_type_key => $layout_type_label ) {
      if( !in_array( $layout_type_key, $exclude ) )
        continue;

      unset( $layout_types[ $layout_type_key ] );
    }
  } else if( is_array( $include ) ) {
    foreach( $layout_types as $layout_type_key => $layout_type_label ) {
      if( in_array( $layout_type_key, $include ) )
        continue;

      unset( $layout_types[ $layout_type_key ] );
    }
  }

  return $layout_types;
}

function wpep_course_has_the_content_filter() {
  return apply_filters( 'wpep_course_has_the_content_filter', wpep_get_setting_flag( 'course-use-the-content-filter' ) );
}

/**
 * @param bool|array $include
 * @param bool|array $exclude
 * @param bool|string $glue
 * @return array|string
 */
function wpep_course_layout_types_string( $include = false, $exclude = false, $glue = " " ) {
  return implode( $glue, array_keys( wpep_course_layout_types( $include, $exclude ) ) );
}

/**
 * @param $course_type
 * @return mixed|string
 */
function wpep_course_layout_type_label( $course_type ) {
  $layout_types = wpep_course_layout_types();

  return isset( $layout_types[ $course_type ] ) ? $layout_types[ $course_type ] : wpep_alias_to_label( $course_type );
}

function wpep_course_has_discussion( $course_id ) {
  if( !wpep_get_setting_flag( 'enable-course-discussions-option' ) )
    return false;

  return absint( get_post_meta( $course_id, 'wpep_course_enable_discussions', true ) );
}

/**
 * @param $course_id
 * @return string
 */
function wpep_course_thumbnail_url( $course_id ) {
  $thumbnail = get_post_meta( $course_id, 'wpep_thumb_image', true );

  if( $thumbnail === '' )
    $thumbnail = wpep_get_placeholder_image();

  return $thumbnail;
}

/**
 * @param $course_id
 * @return string
 */
function wpep_course_thumbnail_hover_url( $course_id ) {
  return get_post_meta( $course_id, 'wpep_thumb_image_hover', true );
}

/**
 * @param $course_id
 * @param bool $short_format
 * @return mixed|string
 */
function wpep_course_excerpt( $course_id, $short_format = false ) {
  $excerpt = get_post_meta( $course_id, 'wpep_course_excerpt', true );

  if( $excerpt == '' ) {
    $excerpt = get_post_meta( $course_id, 'wpep_course_description', true );
    $excerpt = strip_shortcodes( $excerpt );
    $excerpt = strip_tags( $excerpt );
    $excerpt = wp_trim_words( $excerpt, 15 );
  } else {
    if( $short_format ) {
      $short_excerpt = wp_trim_words( $excerpt, 20 );

      if( $excerpt != $short_excerpt )
        $excerpt = $short_excerpt . '...';
    } else {
      $excerpt = do_shortcode( $excerpt );
    }
  }

  return $excerpt;
}

/**
 * @param $course_id
 * @param bool|null $read_only
 * @return int
 */
function wpep_course_has_progress_bar( $course_id, $read_only = null ) {
  $read_only = ( $read_only === null ? wpep_post_read_only( $course_id ) : $read_only );

  if( $read_only != 1 ) {
    $show_progress_bar = get_post_meta( $course_id, 'wpep_show_progress_bar', true );

    if( $show_progress_bar == '' || $show_progress_bar == 'default' ) {
      $show_progress_bar = wpep_get_setting_flag( 'opt-course-percentage-bar' );
    } else {
      $show_progress_bar = ( $show_progress_bar == 'show' ? 1 : 0 );
    }
  } else {
    $show_progress_bar = 0;
  }

  return apply_filters( 'wpep_single_course_has_progress_bar', $show_progress_bar );
}

function wpep_course_has_progress_bar_percentage_number( $course_id ) {
  return wpep_get_setting_flag( 'opt-course-percentage-bar-number' );
}

/**
 * @param int|WP_Post $post - This function aim to work as the get_permalink function.
 * @return string
 */
function wpep_course_permalink( $post ) {
  return apply_filters( 'wpep_course_permalink', get_permalink( $post ), $post );
}

/**
 * @param $lesson_id
 * @param \WP_Post|int|bool $post
 * @param string $slug
 * @return string
 */
function wpep_course_lesson_permalink( $lesson_id, $post = false, $slug = 'lesson' ) {
  if( $post == false )
    $post = get_post();

  return wpep_permalink_extend( wpep_course_permalink( $post ), $slug, $lesson_id );
}

/**
 * This function is going to be used by the Focus Course Type, and soon also the Standard Course Type, to preserve the Sidebar within assessments for better visibility.
 *
 * @param $assessment_id
 * @param \WP_Post|int|bool $post
 * @param string $slug
 * @return string
 */
function wpep_course_assessment_permalink( $assessment_id, $post = false, $slug = 'assessment' ) {
  if( $post == false )
    $post = get_post();

  return wpep_permalink_extend( wpep_course_permalink( $post ), $slug, $assessment_id );
}

/**
 * @param $id
 * @return int
 */
function wpep_get_prerequisite_mode( $id ) {
  $types = wpep_get_prerequisite_modes();

  return ( isset( $types[ $id ] ) ? $types[ $id ] : ( isset( $types[WPEP_PREREQUISITE_MODE_NOTIFY] ) ? $types[WPEP_PREREQUISITE_MODE_NOTIFY] : 0 ) );
}

/**
 * @return array
 */
function wpep_get_prerequisite_modes() {
  return [
    WPEP_PREREQUISITE_MODE_NOTIFY  => [
      WPEP_PREREQUISITE_FIELD_ID                     => WPEP_PREREQUISITE_MODE_NOTIFY,
      WPEP_PREREQUISITE_FIELD_NAME                   => __( "Recommend Prerequisite(s) Completion.", "wpep" ),
      WPEP_PREREQUISITE_FIELD_SHOW_NOTIFICATION      => true,
      WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT => 0,
      WPEP_PREREQUISITE_FIELD_SHOW_NEXT_COURSES      => true
    ],
    WPEP_PREREQUISITE_MODE_REQUIRE_ONE => [
      WPEP_PREREQUISITE_FIELD_ID                     => WPEP_PREREQUISITE_MODE_REQUIRE_ONE,
      WPEP_PREREQUISITE_FIELD_NAME                   => __( "Student must complete 1 Prerequisite", "wpep" ),
      WPEP_PREREQUISITE_FIELD_SHOW_NOTIFICATION      => false,
      WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT => 1,
      WPEP_PREREQUISITE_FIELD_SHOW_NEXT_COURSES      => true
    ],
    WPEP_PREREQUISITE_MODE_REQUIRE_ONE_NOTIFY => [
      WPEP_PREREQUISITE_FIELD_ID                     => WPEP_PREREQUISITE_MODE_REQUIRE_ONE_NOTIFY,
      WPEP_PREREQUISITE_FIELD_NAME                   => __( "Student must complete 1 Prerequisite before being suggested to complete other Prerequisites", "wpep" ),
      WPEP_PREREQUISITE_FIELD_SHOW_NOTIFICATION      => true,
      WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT => 1,
      WPEP_PREREQUISITE_FIELD_SHOW_NEXT_COURSES      => true
    ],
    WPEP_PREREQUISITE_MODE_REQUIRE_ALL => [
      WPEP_PREREQUISITE_FIELD_ID                     => WPEP_PREREQUISITE_MODE_REQUIRE_ALL,
      WPEP_PREREQUISITE_FIELD_NAME                   => __( "The student must complete all the prerequisites.", "wpep" ),
      WPEP_PREREQUISITE_FIELD_SHOW_NOTIFICATION      => false,
      WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT => 'all',
      WPEP_PREREQUISITE_FIELD_SHOW_NEXT_COURSES      => true
    ]
  ];
}

/**
 * @return array
 */
function wpep_get_prerequisite_mode_list() {
  return wpep_utility_map_to_array_assoc( wpep_get_prerequisite_modes(), 'id', 'name' );
}

/**
 * @return array
 */
function wpep_course_type_member_magnet_integrated() {
  return wpep_get_setting_flag( 'enable-post-member-magnet-interaction' ) ? [ WPEP_COURSE_TYPE_EXECUTION_PLAN, WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED, WPEP_COURSE_TYPE_LEARN_MAP ] : [];
}

/**
 * @param $course_id
 * @param bool $user_id
 * @return int|bool
 */
function wpep_user_course_is_bookmarked( $course_id, $user_id = false ) {
  return wpep_content_type_course_instance()->user->flag->get_bookmark_status( $course_id, $user_id );
}

/**
 * @param $course_id
 * @param bool $user_id
 * @return int
 */
function wpep_user_course_started_at( $course_id, $user_id = false ) {
  $user_id = ( $user_id == false ? get_current_user_id() : $user_id );

  return intval( wpep_content_type_course_instance()->user->data->get_value( WPEP_USER_COURSE_STARTED, $course_id, false, false, $user_id, true ) );
}

/**
 * @param $course_id
 * @param bool $user_id
 * @return int
 */
function wpep_user_course_completed_at( $course_id, $user_id = false ) {
  $user_id = ( $user_id == false ? get_current_user_id() : $user_id );

  return intval( wpep_content_type_course_instance()->user->data->get_value( WPEP_USER_COURSE_COMPLETED_AT, $course_id, false, false, $user_id, true ) );
}

/**
 * @param $course_id
 * @param bool $user_id
 * @return int
 */
function wpep_user_course_started_at_seconds( $course_id, $user_id = false ) {
  $started_at = wpep_user_course_started_at( $course_id, $user_id );

  return intval( $started_at == 0 ? 0 : ( time() - $started_at ) );
}

/**
 * @param $course_id
 * @param bool $user_id
 * @param bool $including_assessments
 * @return bool
 */
function wpep_user_course_is_completed( $course_id, $user_id = false, $including_assessments = false ) {
  return wpep_content_type_course_instance()->user->flag->has_completed_course( $course_id, $user_id, $including_assessments );
}

/**
 * @param $course_id
 * @param bool $user_id
 * @param bool|int $option_section_id
 * @return int
 */
function wpep_user_course_progress_percentage( $course_id, $user_id = false, $option_section_id = false ) {
  return wpep_content_type_course_instance()->user->get_progress_percent( $course_id, $user_id, $option_section_id );
}

/**
 * @param $course_id
 * @param bool $user_id
 * @return int
 */
function wpep_user_course_started_event( $course_id, $user_id = false ) {
  $user_id = ( $user_id == false ? get_current_user_id() : $user_id );

  $started_at = wpep_user_course_started_at( $course_id, $user_id );

  if( $started_at == 0 ) {
    $started_at = time();

    wpep_content_type_course_instance()->user->data->set(
      WPEP_USER_COURSE_STARTED,
      $started_at,
      $course_id,
      0,
      0,
      $user_id
    );

    if( !wpep_post_read_only( $course_id ) ) {
      wpep_content_type_course_instance()->user->data->set(
        WPEP_USER_COURSE_PROGRESS,
        0,
        $course_id,
        0,
        0,
        $user_id
      );
    }
  }

  return $started_at;
}

function wpep_course_scorm_student_data_mapping() {
  return [
    'cmi.core.lesson_status'   => 'scorm_lesson_status',
    'cmi.core.lesson_location' => 'scorm_lesson_location',
    'cmi.core.score.raw'       => 'scorm_score_raw',
    'cmi.core.session_time'    => 'scorm_session_time',
    'cmi.core.credit'          => 'scorm_credit',
    'cmi.core.lesson_mode'     => 'scorm_lesson_mode',
  ];
}

function wpep_course_scorm_directory_path( $course_id, $scorm_file_path = false, $return_type = 'baseurl' ) {
  $upload_directory = wp_upload_dir();
  $scorm_file_path  = ( $scorm_file_path === false ? get_post_meta( $course_id, 'wpep_scorm_file', true ) : $scorm_file_path );
  $directory_path   = get_post_meta( $course_id, 'wpep_scorm_directory', true );

  if( $scorm_file_path === '' && $directory_path === '' )
    return '';

  if( $scorm_file_path === '' )
    return ( is_bool( $directory_path ) ? boolval( $directory_path ) : $upload_directory[ $return_type ] . '/' . $directory_path );

  if( wpep_string_ends_with( $directory_path, md5( $scorm_file_path ) ) )
    return $upload_directory[ $return_type ] . '/' . $directory_path;

  if( !class_exists( 'ZipArchive' ) )
    return false;

  $scorm_file_path    = str_replace( $upload_directory['baseurl'], $upload_directory['basedir'], $scorm_file_path );

  $directory_path     = 'wpep-scorm/' . md5( $scorm_file_path );
  $abs_directory_path = $upload_directory['basedir'] . DIRECTORY_SEPARATOR . $directory_path;

  if( !file_exists( $scorm_file_path ) )
    return false;

  if( is_dir( $abs_directory_path ) )
    wpep_file_system_remove_dir( $abs_directory_path );

  mkdir( $abs_directory_path, 0755, true );

  $zip = new \ZipArchive();

  if( $zip->open( $scorm_file_path ) != true )
    return false;

  $zip->extractTo( $abs_directory_path );
  $zip->close();

  if( !file_exists( $abs_directory_path . '/imsmanifest.xml' ) )
    return false;

  $manifest = simplexml_load_file( $abs_directory_path . '/imsmanifest.xml' );

  if( !isset( $manifest->resources->resource ) )
    return false;

  update_post_meta( $course_id, 'wpep_scorm_directory', $directory_path );
  update_post_meta( $course_id, 'wpep_scorm_dispatch', (string) $manifest->resources->resource['href'] );
  update_post_meta( $course_id, 'wpep_scorm_title', wpep_alias_to_label( basename( $scorm_file_path ) ) );

  return $upload_directory[ $return_type ] . '/' . $directory_path;
}