<?php

namespace WPEP;

/**
 * Class Controller
 * @package WPEP
 * @author Robert Rusu
 */
class Controller {

  const SHOP_PRODUCT_ID = 35;

  protected static $instance;

  /**
   * @var Coordinator\ActionTracking
   */
  public $actionTracking;

  /**
   * @var Coordinator\AddOns
   */
  public $addOns;

  /**
   * @var Coordinator\Assets
   */
  public $assets;

  /**
   * @var Coordinator\Authentication
   */
  public $authentication;

  /**
   * @var Coordinator\Capabilities
   */
  public $capabilities;

  /**
   * @var Coordinator\Content
   */
  public $content;

  /**
   * @var Coordinator\ContentLibrary
   */
  public $contentLibrary;

  /**
   * @var Coordinator\Discussion
   */
  public $discussion;

  /**
   * @var Coordinator\Email
   */
  public $email;

  /**
   * @var Coordinator\EmailMarketing
   */
  public $emailMarketing;

  /**
   * @var Coordinator\Hooks
   */
  public $hooks;

  /**
   * @var Coordinator\LicenseLibrary
   */
  public $licenseLibrary;

  /**
   * @var Coordinator\Menu
   */
  public $menu;

  /**
   * @var Coordinator\Migration
   */
  public $migration;

  /**
   * @var Coordinator\MigrationLibrary
   */
  public $migrationLibrary;

  /**
   * @var Coordinator\Notifications
   */
  public $notifications;

  /**
   * @var Coordinator\SQL
   */
  public $sql;

  /**
   * @var Coordinator\Template
   */
  public $template;

  /**
   * @var Coordinator\TemplateStyle
   */
  public $templateStyle;

  /**
   * @var Coordinator\UserEngagement
   */
  public $userEngagement;

  /**
   * @var Admin\Controller
   */
  public $adminController;

  public static function instance() {
    if (!isset(self::$instance))
      self::$instance = new self();

    return self::$instance;

  }

  public function __construct() {
    require_once(WPEP_BASE_PATH . "/app/_functions/general.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/utilities.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/administration.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/auth.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/db.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/discussion.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/file-system.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/post.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/primary.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/user.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/user-profile.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/toolkit.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/shortcode-builder.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/image-builder.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/learning-journey.php" );
  }

  public function setup() {
    $this->register_coordinators();

    // Migration Process need to be early.
    $this->migration->setup();

    do_action( "wpep_migration_setup" );

    add_action( 'plugins_loaded', [ $this, '_setup' ] );
  }

  public function _setup() {
    $response = load_plugin_textdomain( 'wpep', false, basename( WPEP_BASE_PATH ) . '/languages/' );;

    $this->setup_coordinators();

    do_action( 'wpep_register_addon', $this->addOns );

    do_action( 'wpep_init_addon_before', $this->addOns );

    $this->addOns->_init_add_ons();

    do_action( 'wpep_init_addon_after', $this->addOns );

    do_action( 'wpep_loaded' );
  }

  public function register_coordinators() {
    $this->migrationLibrary = new Coordinator\MigrationLibrary();
    $this->migration        = new Coordinator\Migration();
    $this->notifications  = new Coordinator\Notifications();

    $this->addOns    = new Coordinator\AddOns();

    $this->actionTracking = new Coordinator\ActionTracking();
    $this->capabilities   = new Coordinator\Capabilities();
    $this->contentLibrary = new Coordinator\ContentLibrary();
    $this->hooks          = new Coordinator\Hooks();
    $this->licenseLibrary = new Coordinator\LicenseLibrary();
    $this->menu           = new Coordinator\Menu();
    $this->template       = new Coordinator\Template();
    $this->templateStyle  = new Coordinator\TemplateStyle();
    $this->assets         = new Coordinator\Assets();
    $this->authentication = new Coordinator\Authentication();
    $this->content        = new Coordinator\Content();
    $this->sql            = new Coordinator\SQL();
    $this->email          = new Coordinator\Email();
    $this->emailMarketing = new Coordinator\EmailMarketing();
    $this->discussion     = new Coordinator\Discussion();
    $this->userEngagement = new Coordinator\UserEngagement();

    $this->adminController   = new Admin\Controller();
  }

  public function setup_coordinators() {
    $this->notifications->setup();
    $this->actionTracking->setup();
    $this->authentication->setup();
    $this->contentLibrary->setup();
    $this->email->setup();
    $this->hooks->setup();
    $this->licenseLibrary->setup();
    $this->menu->setup();
    $this->template->setup();
    $this->templateStyle->setup();
    $this->assets->setup();
    $this->content->setup();
    $this->discussion->setup();

    $this->adminController->dispatch();
  }

}
