<?php

namespace WPEP\Coordinator\Content;

class Miscellaneous {

  public function __construct() {
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'post_content', [ $this, 'post_content' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'center', [ $this, 'center_section' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'notification', [ $this, 'notification' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'link', [ $this, 'link' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'video', [ $this, 'video' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'hero_section', [ $this, 'hero_section' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'debug_request', [ $this, 'debug_request' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'kb_external_embed', [ $this, 'kb_external_embed' ] );
  }

  public function post_content($attributes, $post_id) {
    $post_content_post = get_post( intval( $post_id ) );

    if( ! ( $post_content_post instanceof \WP_Post ) ) {
      $response  = '<div class="wpep-shortcode-post-content">';
      $response .=    __( "Post ID Provided is not valid. ( " . $post_id . ' ) ' );
      $response .= '</div>';

      return $response;
    }

    global $post;
    $previous_post = $post;

    $post = $post_content_post;

    setup_postdata( $post_content_post );

    $response  = '<div class="wpep-shortcode-post-content">';
    $response .=    apply_filters( 'the_content', $post->post_content );
    $response .= '</div>';

    $post = $previous_post;
    setup_postdata( $post );

    return $response;
  }

  public function center_section($attributes, $content) {
    $response  = '<div class="wpep-center-section">';
    $response .=    do_shortcode( $content );
    $response .= '</div>';

    return $response;
  }

  public function notification( $attributes, $content ) {
    $type = isset( $attributes['type'] ) ? $attributes['type'] : 'standard';

    return '<div class="wpep-notification wpep-notification-' . $type . '">' . do_shortcode( $content ) . '</div>';
  }

  public function link($attributes, $link = null) {
    return wpep_frontend_button( $link, $attributes );
  }

  public function video( $attributes, $content ) {
    $content = strip_tags( $content );
    $content = trim( $content );

    return wpep_get_video_source_output( $content );
  }

  public function hero_section( $attributes, $content ) {
    return '<div class="wpep-hero-section">' . do_shortcode( $content ) . '</div>';
  }

  public function debug_request() {
    if( wpep_request()->is_request_type( 'admin' ) )
      return '';
    
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return '<div class="wpep-notification wpep-notification-danger">' . sprintf( __( "%s shortcode requires %s permission", "wpep" ), WPEP_SHORT_CODE_PREFIX . 'debug_request', WPEP_CAP_MANAGE_SETTINGS ) . '</div>';

    @ini_set('display_errors', 1);
    @ini_set('display_startup_errors', 1);
    @error_reporting(E_ALL);

    return '<div class="wpep-notification wpep-notification-success">' . sprintf( __( "%s debug request activated", "wpep" ), WPEP_PLUGIN_NAME ) . '</div>';
  }

  public function kb_external_embed( $attributes, $content ) {
    if( !isset( $attributes['course_link'] )
        || !isset( $attributes['course_id'] )
        || !isset( $attributes['script_link'] ) )
      return ( current_user_can( WPEP_CAP_MANAGE_SETTINGS )
                  ? '<div class="wpep-notification wpep-notification-danger">' . sprintf( __( "Admin Notification : Missing one of the shortcode parameters for %s", "wpep" ), WPEP_SHORT_CODE_PREFIX . 'kb_external_embed' ) . '</div>'
                  : ''
             );

    $response = '';

    $response .= '<iframe id="wpep-embed-' . $attributes['course_id' ] . '" class="wpep-external-embed" src="' . add_query_arg( 'wpep-embed', 'true', $attributes[ 'course_link' ] ) . '"></iframe>';
    $response .= '<script type="text/javascript" src="' . $attributes['script_link'] . '"></script>';

    return $response;
  }

}