<?php

namespace WPEP\Coordinator\Content;

use WP_User_Query;

class Users {

  public function __construct() {
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'user_directory', [ $this, 'user_directory' ] );
  }

  public function user_directory( $atts, $content ) {
    $users_per_page = ( isset( $atts[ 'users_per_page' ] )           ? $atts[ 'users_per_page' ] : WPEP_USER_DIRECTORY_PER_PAGE );
    $has_search_bar = ( isset( $atts[ 'has_search_bar' ] )           ? $atts[ 'has_search_bar' ] : WPEP_USER_DIRECTORY_HAS_SEARCH_BAR );
    $current_page   = ( isset( $_GET[ 'wpep-user-directory-page' ] ) ? absint( $_GET[ 'wpep-user-directory-page' ] ) : 1 );
    $search         = ( isset( $_GET[ 'wpep-user-directory-search']) ? sanitize_text_field( $_GET[ 'wpep-user-directory-search'] ) : '' );

    $args = [
      'count_total' => 1,
      'paged'       => 1,
      'offset'      => ( $users_per_page * ( $current_page - 1 ) ),
      'number'      => $users_per_page,
      'orderby'     => 'user_registered'
    ];

    if( isset( $atts['user_roles'] ) && !empty( $atts[ 'user_roles' ] ) )
      $args[ 'role__in' ] = array_map( 'trim', explode( ",", $atts['user_roles'] ) );

    if( $search !== '' ) {
      $args[ 'search' ]         = '*' . $search . '*';
      $args[ 'search_columns' ] = [ 'user_login', 'user_nicename', 'user_email' ];
    }

    $user_list_query = new WP_User_Query( $args );
    $user_list = $user_list_query->get_results();

    if( empty( $user_list ) )
      return '';

    return wpep_template_get( 'user/directory.php', [
      'base_url'              => wpep_get_current_url_user_directory(),
      'paginate_total_pages'  => ceil( $user_list_query->get_total() / $users_per_page ),
      'paginate_current_page' => $current_page,
      'user_list'             => $user_list,
      'view_profile_text'     => isset( $atts[ 'view_profile_text' ] ) ? $atts[ 'view_profile_text' ] : false,
      'search_user_text'      => isset( $atts[ 'search_users_text' ] ) ? $atts[ 'search_users_text' ] : __( 'Search User Directory', "wpep" ),
      'has_search_bar'        => $has_search_bar,
      'current_search'        => $search
    ] );
  }

}
