<?php

namespace WPEP\Coordinator\Discussion;

use WPEP\Entity\Discussion as WPEP_Entity_Discussion;
use WPEP\Controller as WPEP_Controller;

class DirectURL {

  public $pagename_identifier = 'wpep-discussion-direct-url';
  public $discussion_identifier = 'wpep-discussion';

  protected $_current_discussion;

  public function setup() {
    if( !wpep_discussion_direct_url_enabled() )
      return;

    $setting_slug = wpep_get_setting( 'comments-direct-url-slug' );

    if( $setting_slug !== '' )
      $this->discussion_identifier = $setting_slug;

    add_action( 'init', [ $this, '_add_rewrite_rules' ] );
    add_filter( 'query_vars', [ $this, '_add_query_vars' ] );
    add_filter( 'status_header', [ $this, '_status_header' ], 50 );
    add_filter( 'body_class', [ $this, '_body_class' ], 50 );

    add_filter( 'wpep_current_request_template', [ $this, '_wpep_current_request_template' ] );
  }

  public function get_discussion() {
    return $this->_current_discussion;
  }

  public function _add_query_vars( $query_vars ) {
    $query_vars[] = $this->discussion_identifier;

    return $query_vars;
  }

  public function _add_rewrite_rules() {
    add_rewrite_rule(
      $this->discussion_identifier . '/([^/]+)/?',
      'index.php?pagename=' . $this->pagename_identifier . '&' . $this->discussion_identifier . '=$matches[1]',
      'top'
    );

    add_rewrite_rule(
      $this->discussion_identifier,
      'index.php?pagename=' . $this->pagename_identifier,
      'top'
    );
  }

  public function _status_header( $response ) {
    if( !$this->_is_valid_request() )
      return $response;

    return 'HTTP/1.1 200 OK';
  }

  public function _body_class( $class ) {
    if( !$this->_is_valid_request() )
      return $class;

    $class = array_diff( $class, [ 'error404', 'error-404' ] );

    $class[] = 'wpep-discussion-direct-url';

    return $class;
  }

  public function _wpep_current_request_template( $template ) {
    if( !$this->_is_valid_request() )
      return $template;

    add_filter( 'wp_title', [ $this, 'wp_title' ], 50 );
    add_filter( 'pre_get_document_title', [ $this, 'wp_title' ], 50 );

    // Remove Yoast
    if( class_exists( '\WPSEO_Frontend' ) ) {
      $wp_thing = \WPSEO_Frontend::get_instance();
      remove_action('wp_head',array($wp_thing,'head'),1);
    }

    return 'discussion/direct-url.php';
  }

  public function wp_title( $title ) {
    return $this->_get_title();
  }

  private function _get_title() {
    if( $this->_current_discussion === null )
      return wpep_get_setting_translated( 'comments-direct-url-list-title' );

    return sprintf(
      wpep_get_setting_translated( 'comments-direct-url-title' ),
      WPEP_Controller::instance()->discussion->get( $this->_current_discussion->namespace )->get_discussion_name( $this->_current_discussion )
    );
  }

  private function _get_current_request_identifier() {
    return urldecode( get_query_var( $this->discussion_identifier ) );
  }

  private function _is_valid_request() {
    global $wp_query;

    if( $wp_query === null )
      return false;

    if( get_query_var( 'pagename' ) != $this->pagename_identifier )
      return false;

    if( !is_user_logged_in() )
      return false;

    if( $this->_get_current_request_identifier() == '' )
      return true;

    if( $this->_current_discussion === null ) {
      $this->_current_discussion = WPEP_Entity_Discussion::instance()->get_discussion( [
        'identifier'  => $this->_get_current_request_identifier()
      ] );

      if( $this->_current_discussion === null ) {
        $this->_current_discussion = false;

        return false;
      }

      if( !WPEP_Controller::instance()->discussion->get( $this->_current_discussion->namespace )->current_user_can_view( $this->_current_discussion ) ) {
        $this->_current_discussion = false;

        return false;
      }
    }

    if( $this->_current_discussion === false )
      return false;

    return true;
  }

}