<?php
namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\ContentType as WPEP_ContentType;
use WPEP\Posts;

class Hooks {

  public function setup() {
    add_action( 'widgets_init', [ $this, 'register_widget_area' ], 10 );
    add_filter( 'pre_get_avatar', [ $this, 'pre_get_avatar' ], 20, 3 );

    if( !is_admin() && wpep_get_setting_flag( 'disable-admin-bar-for-students' ) )
      add_action( 'after_setup_theme', [ $this, '_disable_admin_bar_for_students' ] );

    if( wpep_get_setting_flag( 'disable-xmlrpc' ) )
      add_filter( 'xmlrpc_enabled', '__return_false', 100 );

    if( wpep_get_setting_flag( 'api-response-scu-crd-status' ) )
      add_filter( 'wpep_api_request_response_system/statistics-current-user', [ $this, '_api_filter_response_statistics_current_user' ], 999, 3 );
  }

  public function register_widget_area() {
    register_sidebar( [
      'name'          => sprintf( __('%s Sidebar', 'wpep'), WPEP_PLUGIN_NAME ),
      'id'            => 'wpep-sidebar',
      'before_widget' => '<div id="%1$s" class="wpep-widget %2$s">',
      'after_widget'  => '</div>',
      'before_title'  => '<h1 class="wpep-widget-title">',
      'after_title'   => '</h1>',
    ] );
  }

  public function pre_get_avatar( $default_avatar, $id_or_email, $args ) {
    if( !is_numeric( $id_or_email ) )
      return $default_avatar;

    $profile_picture = get_user_meta( $id_or_email, WPEP_USER_META_PROFILE_PICTURE, true );

    if( $profile_picture == '' )
      return $default_avatar;

    return sprintf(
      "<img alt='%s' src='%s' class='avatar' height='%d' width='%d' %s/>",
      esc_attr( $args['alt'] ),
      esc_url( $profile_picture ),
      (int) $args['height'],
      (int) $args['width'],
      $args['extra_attr']
    );
  }

  public function _disable_admin_bar_for_students() {
    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return;
    
    show_admin_bar(false);
  }

  public function _api_filter_response_statistics_current_user( $response, $action, $request_data ) {
    if( $action !== 'configuration-report' )
      return $response;

    if( isset( $request_data[ '_ignore_crd_sources' ] ) && current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return $response;

    $settings = wpep_get_setting( 'api-response-scu-crd-sources' );

    if( empty( $settings ) ) {
      $response[ 'report_data_sources' ] = [];

      return $response;
    }

    foreach( $response[ 'report_data_sources' ] as $report_data_source_key => $report_data_source ) {
      if( $report_data_source[ 'type' ] !== 'group'  ) {
        if( !isset( $settings[ $report_data_source_key ] ) || intval( $settings[ $report_data_source_key ] ) == false )
          unset( $response[ 'report_data_sources' ][ $report_data_source_key ] );

        continue;
      }

      foreach( $report_data_source[ 'group' ] as $rdsg_key => $rdsg_value ) {
        if( !isset( $settings[ $report_data_source_key ] ) ) {
          unset( $report_data_source[ 'group' ][ $rdsg_key ] );
          continue;
        }

        if( !in_array( $rdsg_key, $settings[ $report_data_source_key ] ) )
          unset( $report_data_source[ 'group' ][ $rdsg_key ] );
      }

      if( empty( $report_data_source[ 'group'] ) ) {
        unset( $response[ 'report_data_sources' ][ $report_data_source_key ] );
      } else {
        $response[ 'report_data_sources' ][ $report_data_source_key ] = $report_data_source;
      }
    }

    return $response;
  }

}