<?php

namespace WPEP\Coordinator\UserEngagement;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\SystemLog as WPEP_Entity_SystemLog;

class WeeklyRecap {

  public function __construct() {
    if( wpep_get_setting_flag( 'email-weekly-recap-status' )  ) {
      add_action( 'wpep_cron_weekly', [ $this, 'weekly_cron_hook' ] );
      add_action( 'wpep_cron_five_minutes', [ $this, 'five_minute_cron_hook' ] );
    }
  }

  public function five_minute_cron_hook() {
    $user_ids = get_option( 'wpep_weekly_recap_que', [] );

    if( empty( $user_ids ) )
      return;

    $current_que_user_ids = array_slice( $user_ids, 0, WPEP_EMAIL_NOTIFICATION_ENTRIES_PER_JOB );

    WPEP_Controller::instance()->userEngagement->send_email(
      $current_que_user_ids,
      wpep_get_setting( 'email-weekly-recap-title' ),
      wpep_get_setting( 'email-weekly-recap-content' )
    );

    $remaining_users = array_diff( $user_ids, $current_que_user_ids );

    update_option( 'wpep_weekly_recap_que', $remaining_users );
  }
  
  public function weekly_cron_hook() {
    $user_ids = WPEP_Controller::instance()->userEngagement->get_user_list();

    if( empty( $user_ids ) )
      return;

    $skipped_user_ids = [];

    $system_log = WPEP_Entity_SystemLog::instance( 'cron', 'weekly_recap_que', time() );

    foreach( $user_ids as $position => $user_id ) {
      if( wpep_user_has_notification( $user_id, 'weekly_recap' ) )
        continue;

      unset( $user_ids[$position] );

      $skipped_user_ids[] = $user_id;
    }

    $system_log->completed_arguments = [
      'user_ids'         => $user_ids,
      'skipped_user_ids' => $skipped_user_ids
    ];
    $system_log->completed_at = time();
    $system_log->save();

    update_option( 'wpep_weekly_recap_que', $user_ids );
  }

}
