<?php

namespace WPEP\Entity;

class Course {

  protected static $_instance;

  public static function instance() {
    if( self::$_instance === null )
      self::$_instance = new self();

    return self::$_instance;
  }

  public function get_section( $section_id ) {
    $sql = 'SELECT *
              FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . '`
             WHERE id = ' . $section_id;

    $section = wpep_wpdb()->get_row($sql);

    if( $section === null )
      return null;

    $section->options = ( !isset( $section->options ) || $section->options == '' ? new \stdClass() : json_decode( $section->options ) );

    return $section;
  }

  public function get_section_by_lesson_id( $lesson_id ) {
    $sql = 'SELECT * FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . '`
                     WHERE id = ( SELECT section_id FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` WHERE id = ' . intval( $lesson_id ) . ' )';

    $section = wpep_wpdb()->get_row($sql);

    if( $section === null )
      return null;

    $section->options = ( !isset( $section->options ) || $section->options == '' ? new \stdClass() : json_decode( $section->options ) );

    return $section;
  }

  public function get_sections_by_course_id( $course_id ) {
    $sql = 'SELECT *
              FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . '`
             WHERE post_id = ' . $course_id . ' ORDER BY `order` ASC';

    $sections = wpep_wpdb()->get_results($sql);

    foreach( $sections as $key => $section )
      $sections[$key]->options = ( $section->options == '' ? new \stdClass() : json_decode( $section->options ) );

    return $sections;
  }

  public function get_with_lessons( $id = false ) {
    if( $id == false )
      $id = get_the_ID();

    $course_sections = $this->get_list( $id );

    if( empty( $course_sections ) )
      return [];

    $lessons = $this->get_lessons( $id, 'post_id' );

    if( empty( $lessons ) ) {
      foreach( $course_sections as $k => $course_section )
        $course_sections[$k]->lessons = [];

      return $course_sections;
    }

    $lessons = wpep_utilities_map_object( $lessons, 'section_id', true );

    foreach( $course_sections as $k => $course_section )
      $course_sections[$k]->lessons = ( isset( $lessons[ $course_section->id ] ) ? $lessons[ $course_section->id ] : [] );

    return $course_sections;
  }

  public function get_list( $id = false ) {
    if( $id == false )
      $id = get_the_ID();

    return $this->get_sections_by_course_id( $id );
  }

  public function get_course_id_by_lesson_id( $lesson_id ) {
    $sql = 'SELECT post_id FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` WHERE id = ' . intval( $lesson_id );

    return wpep_wpdb()->get_var($sql);
  }

  public function get_course_id_by_section_id( $section_id ) {
    $sql = 'SELECT post_id FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . '` WHERE id = ' . intval( $section_id ) . ' )';

    return wpep_wpdb()->get_var($sql);
  }

  public function get_lesson( $lesson_id ) {
    $sql = 'SELECT *
              FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '`
             WHERE id = ' . $lesson_id;

    $lesson = wpep_wpdb()->get_row($sql);

    if( $lesson === null )
      return null;

    $lesson->options = ( $lesson->options == '' ? new \stdClass() : json_decode( $lesson->options ) );

    return $lesson;
  }

  public function get_lesson_title_map( $post_id ) {
    $sql = 'SELECT id, title
                FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '`
               WHERE post_id = ' . $post_id . '
               ORDER BY `order` ASC';

    $lessons = wpep_wpdb()->get_results($sql);

    if( empty( $lessons ) )
      return [];

    return wpep_utility_map_to_array_assoc( $lessons, 'id', 'title' );
  }

  public function get_lessons( $id, $field = 'section_id' ) {
    $sql = 'SELECT *
                FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '`
               WHERE ' . ( $field == 'section_id' ? 'section_id' : 'post_id' ) . ' = ' . $id . '
               ORDER BY `order` ASC';

    $lessons = wpep_wpdb()->get_results($sql);

    foreach( $lessons as $lesson_key => $lesson )
      $lessons[$lesson_key]->options = ( $lesson->options == '' ? new \stdClass() : json_decode( $lesson->options ) );

    return $lessons;
  }

  public function get_lesson_count( $course_id ) {
    $sql = 'SELECT COUNT(*)
                FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '`
               WHERE post_id = ' . $course_id;

    return intval( wpep_wpdb()->get_var($sql) );
  }

}