<?php

namespace WPEP\Entity\CourseType;

use WPEP\Controller as WPEP_Controller;

class PDF extends Skeleton\Standard {

  public $has_next_suggestions = false;
  public $has_next_suggestions_after_completion = false;

  private $_course_sections = [];

  public function __construct( $current_course_id ) {
    $this->_skeleton_init( $current_course_id );

    $this->read_only = true;
  }

  public function init() {
    parent::init();

    $this->_course_sections = wpep_content_type_course_instance()->user->section->get_all_by_course_id_mapped( $this->course_id, get_current_user_id() );

    add_filter( 'wpep_course_single_content_lesson_content', [ $this, 'lesson_content_filter' ], 10, 2 );
    add_filter( 'wpep_course_single_content_section_content', [ $this, 'section_content_filter' ], 10 , 2 );
  }

  public function close() {
    parent::close();

    $this->_course_sections = [];

    remove_filter( 'wpep_course_single_content_lesson_content', [ $this, 'lesson_content_filter' ], 10 );
    remove_filter( 'wpep_course_single_content_section_content', [ $this, 'section_content_filter' ], 10 );
  }

  public function content() {
    return WPEP_Controller::instance()->template->get_template( 'course-single/pdf.php', [
      'course_id'        => $this->course_id,
      'course_sections'  => $this->_course_sections,
    ] );
  }

  public function get_alias() {
    return WPEP_COURSE_TYPE_PDF;
  }

  public function lesson_content_filter( $content, $lesson_information ) {
    if( $content === false )
      return '';

    $content = wpautop( $content );

    if( isset($lesson_information['options'])
        && isset( $lesson_information['options']['downloads_list'] )
        && !empty( $lesson_information['options']['downloads_list'] )
        && strpos( $content, 'wpep-downloads-list-container' ) === false ) {
      $downloads_content = WPEP_Controller::instance()->template->get_template( 'utilities/downloads-list.php', [
        'downloads_list'  => $lesson_information['options']['downloads_list']
      ] );

      if( wpep_get_setting( 'course-downloads-position' ) == 'top' )
        $content = $downloads_content . $content;
      else
        $content .= $downloads_content;

      unset( $downloads_content );
    }

    return do_shortcode( $content );
  }

  public function section_content_filter( $content, $section ) {
    if( $content === false )
      return '';

    $content = wpep_format_content( $content );

    return $content;
  }

}