<?php

namespace WPEP\Entity\CourseType\Skeleton;

use WPEP\Controller as WPEP_Controller;

abstract class Navigation extends Standard {

  public $top_level_request = false;
  public $course_id;
  public $course_has_overview_page = false;
  public $course_has_completion_redirect = false;
  public $read_only = false;

  public $current_item_object = null;
  public $current_item    = null;
  public $current_item_id = null;
  public $lesson_slug     = 'lesson';
  public $section_slug    = 'section';
  public $assessment_slug = 'assessment';

  public $has_next_suggestions = true;
  public $has_next_suggestions_after_completion = true;

  public $course_sections_map = [];
  public $_template_args = [
    'downloads_list' => []
  ];

  public function init() {
    parent::init();

    if( $this->current_item === null ) {
      if( isset( $_GET[$this->lesson_slug] ) ) {
        $this->current_item     = $this->lesson_slug;
        $this->current_item_id  = intval( $_GET[ $this->lesson_slug ] );
      } else if( isset( $_GET[$this->assessment_slug] ) ) {
        $this->current_item     = $this->assessment_slug;
        $this->current_item_id  = intval( $_GET[ $this->assessment_slug ] );
      } else if( ! $this->course_has_overview_page ) {
        $this->current_item     = $this->lesson_slug;
        $this->current_item_id  = wpep_content_type_course_instance()->user->get_delivered_lesson_id( $this->course_id, get_current_user_id() );
      }
    }

    if( $this->current_item != $this->lesson_slug ) {
      $this->course_sections_map = wpep_content_type_course_instance()->user->section->get_all_by_course_id_mapped( $this->course_id, get_current_user_id() );
      return;
    }

    if( intval( get_post_meta( $this->course_id, 'wpep_sequential_completion', true ) ) ) {
      $course_lesson = wpep_content_type_course_instance()->user->lesson->get( $this->current_item_id, get_current_user_id(),true );

      if( isset( $course_lesson->previous_lesson_id )
          && $course_lesson->previous_lesson_id != null
          && $course_lesson->previous_lesson_id != 0
          && intval( $course_lesson->previous_lesson_is_completed ) == 0 ) {
        wpep_redirect( wpep_course_lesson_permalink( wpep_content_type_course_instance()->user->get_delivered_lesson_id( $this->course_id, get_current_user_id() ), $this->course_id ) );
        exit;
      }
    }

    if( isset( $_GET['complete-lesson'] ) ) {
      if( !wp_verify_nonce( $_GET['complete-lesson'], 'complete-lesson' ) )
        wp_nonce_ays( 'complete-lesson' );

      wpep_content_type_course_instance()->user->lesson->set_status(
        $this->current_item_id, 1, get_current_user_id()
      );

      if( $this->course_has_completion_redirect ) {
        $next_lesson_id = wpep_content_type_course_instance()->user->lesson->get_next_lesson_id( $this->current_item_id, get_current_user_id() );

        if( $next_lesson_id != 0 ) {
          wpep_redirect( wpep_course_lesson_permalink( $next_lesson_id, $this->course_id ) );
          exit;
        }
      }
    } elseif( isset( $_GET['incomplete-lesson'] ) && wpep_get_setting_flag( 'enable-lesson-incomplete-option' ) ) {
      if( !wp_verify_nonce( $_GET['incomplete-lesson'], 'incomplete-lesson' ) )
        wp_nonce_ays( 'incomplete-lesson' );

      wpep_content_type_course_instance()->user->lesson->set_status( $this->current_item_id, 0, get_current_user_id() );
    }

    $this->course_sections_map = wpep_content_type_course_instance()->user->section->get_all_by_course_id_mapped( $this->course_id, get_current_user_id() );

    if( !wpep_assessments_enabled() )
      return;

    $assessment_requirements = wpep_user_assessment_requirements_for_lesson( $this->current_item_id );

    if( !empty( $assessment_requirements ) ) {
      $assessment_permalink = get_permalink( array_shift( $assessment_requirements ) );

      wpep_redirect(
        add_query_arg(
          'redirect-from-lesson',
          '1',
          $assessment_permalink
        )
      );
      exit;
    }
  }

  public function close() {
    parent::close();

    $this->course_sections_map = [];
  }

  public function _skeleton_init( $course_id ) {
    $this->course_id = $course_id;
    $this->read_only = wpep_post_read_only( $this->course_id );
  }

  public function _content_attach_discussion_for_section( $response ) {
    if( !is_user_logged_in() )
      return $response;

    $discussion_id = $this->_content_get_discussion_id_by_args( [
      'course_id'   => $this->course_id,
      'section_id'  => $this->current_item_id,
      'user_id'     => get_current_user_id()
    ] );

    if( $discussion_id === 0 )
      return $response;

    if( !wpep_content_type_course_instance()->discussionIntegration->current_user_can_view( $discussion_id ) )
      return $response . '<!-- The user_id ' . get_current_user_id() . ' does not have access to view the discussion_id : ' . $discussion_id . ' for course : ' . $this->course_id . ' -->';

    $response .= $this->_content_attach_discussion_by_id( $discussion_id );

    return $response;
  }

  public function _content_attach_discussion_for_lesson( $response ) {
    if( !is_user_logged_in() )
      return $response;

    $discussion_id = $this->_content_get_discussion_id_by_args( [
      'course_id'   => $this->course_id,
      'lesson_id'   => $this->current_item_id,
      'user_id'     => get_current_user_id()
    ] );

    if( $discussion_id === 0 )
      return $response;

    if( !wpep_content_type_course_instance()->discussionIntegration->current_user_can_view( $discussion_id ) )
      return $response . '<!-- The user_id ' . get_current_user_id() . ' does not have access to view the discussion_id : ' . $discussion_id . ' for course : ' . $this->course_id . ' -->';

    $response .= $this->_content_attach_discussion_by_id( $discussion_id );

    return $response;
  }

  public function course_permalink( $course_permalink, $course_id ) {
    if( $this->top_level_request == true )
      return $course_permalink;

    if( $this->course_id != $course_id )
      return $course_permalink;

    $link = wpep_get_current_url();

    $link = remove_query_arg( $this->lesson_slug, $link );
    $link = remove_query_arg( $this->section_slug, $link );
    $link = remove_query_arg( $this->assessment_slug, $link );
    $link = remove_query_arg( 'complete-lesson', $link );
    $link = remove_query_arg( 'incomplete-lesson', $link );

    return $link;
  }

  public function skeleton_navigation_current_lesson() {
    $course_lesson = wpep_content_type_course_instance()->user->lesson->get( $this->current_item_id, get_current_user_id(), true );;

    if( $course_lesson === null )
      return null;

    if( isset( $this->course_sections_map[ $course_lesson->section_id ][ 'lessons' ][ $course_lesson->id ][ 'options' ][ 'is_disabled' ] )
        && $this->course_sections_map[ $course_lesson->section_id ][ 'lessons' ][ $course_lesson->id ][ 'options' ][ 'is_disabled' ] )
      return null;

    $course_lesson->content = wpautop( $course_lesson->content );

    if( !wpep_course_has_the_content_filter() || $this->top_level_request == false )
      $course_lesson->content = do_shortcode( $course_lesson->content );

    if( isset( $course_lesson->options ) ) {
      $course_lesson->options = ( is_string( $course_lesson->options ) ? json_decode( $course_lesson->options, true ) : $course_lesson->options );

      if( isset( $course_lesson->options['downloads_list'] ) && !empty( $course_lesson->options['downloads_list'] ) && strpos( $course_lesson->content, 'wpep-downloads-list-container' ) === false ) {
        $downloads_template_args = ( isset( $this->_template_args[ 'downloads_list'] ) && is_array( $this->_template_args[ 'downloads_list'] ) ? $this->_template_args[ 'downloads_list'] : [] );

        $downloads_template_args[ 'downloads_list' ] = $course_lesson->options['downloads_list'];

        $downloads_content = WPEP_Controller::instance()->template->get_template( 'utilities/downloads-list.php', $downloads_template_args );


        if( wpep_get_setting( 'course-downloads-position' ) == 'top' )
          $course_lesson->content = $downloads_content . $course_lesson->content;
        else
          $course_lesson->content = $course_lesson->content . $downloads_content;

        unset( $downloads_template_args );
        unset( $downloads_content );
      }
    }

    if( !is_array( $course_lesson->options ) )
      $course_lesson->options = [];

    if( $course_lesson->previous_lesson_id !== null
      && isset( $this->course_sections_map[ $course_lesson->previous_lesson_section_id ][ 'lessons' ][ $course_lesson->previous_lesson_id ][ 'options' ][ 'is_disabled' ] )
      && $this->course_sections_map[ $course_lesson->previous_lesson_section_id ][ 'lessons' ][ $course_lesson->previous_lesson_id ][ 'options' ][ 'is_disabled' ]
    )
      $course_lesson->previous_lesson_id = null;

    if( $course_lesson->next_lesson_id !== null
      && isset( $this->course_sections_map[ $course_lesson->next_lesson_section_id ][ 'lessons' ][ $course_lesson->next_lesson_id ][ 'options' ][ 'is_disabled' ] )
      && $this->course_sections_map[ $course_lesson->next_lesson_section_id ][ 'lessons' ][ $course_lesson->next_lesson_id ][ 'options' ][ 'is_disabled' ]
    )
      $course_lesson->next_lesson_id = null;

    return $course_lesson;
  }

  public function skeleton_navigation_current_item_content_prepare( $content, $args = [] ) {
    if( !wpep_course_has_discussion( $this->course_id ) )
      return $content;

    if( isset( $args[ 'discussion_title' ] ) )
      $content .= '<h2 class="wpep-discussion-title">' . $args[ 'discussion_title' ] . '</h2>';

    $course_discussion_segmentation = get_post_meta( $this->course_id, 'wpep_course_discussion_segmentation', true );

    if( $course_discussion_segmentation === 'course' )
      $content = $this->_content_attach_discussion_for_course( $content );
    elseif( $this->current_item === 'section' )
      $content = $this->_content_attach_discussion_for_section( $content );
    else
      $content = $this->_content_attach_discussion_for_lesson( $content );

    return $content;
  }

}