<?php

namespace WPEP\Entity\CourseType;

use WPEP\Controller as WPEP_Controller;

class Standard extends Skeleton\Navigation {

  /**
   * Standard constructor.
   * @param $current_course_id
   */
  public function __construct( $current_course_id ) {
    $this->_skeleton_init( $current_course_id );

    $this->course_has_overview_page             = wpep_get_setting_flag( 'course-standard-allow-index-page' );
    $this->course_has_completion_redirect = wpep_get_setting_flag( 'course-standard-redirect-on-completion' );

    $course_has_overview_page = get_post_meta( $this->course_id, 'wpep_standard_course_overview_page', true );

    if( $course_has_overview_page != 'default' )
      $this->course_has_overview_page = $course_has_overview_page;
  }

  public function has_sidebar_content() {
    if( $this->current_item == $this->lesson_slug )
      return true;

    if( $this->current_item === null && wpep_get_setting_flag( 'course-standard-allow-index-page-widgets' ) )
      return true;

    return false;
  }

  public function has_default_sidebar_content() {
    if( $this->top_level_request )
      return false;

    if( $this->current_item !== null )
      return true;

    return wpep_get_setting_flag( 'course-standard-allow-index-page-widgets' );
  }

  public function get_alias() {
    return "standard";
  }

  public function init() {
    parent::init();

    if( $this->has_sidebar_content() && $this->current_item == $this->lesson_slug  )
      add_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
  }

  public function close() {
    parent::close();

    if( $this->has_sidebar_content() && $this->current_item == $this->lesson_slug )
      remove_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
  }
  
  public function content() {
    if( $this->current_item == $this->lesson_slug ) {
      $course_lesson = $this->skeleton_navigation_current_lesson();

      if( $course_lesson === null )
        return '';

      $args = [
        'course_id'             => $this->course_id,
        'course_read_only'      => $this->read_only,
        'course_name'           => get_the_title( $this->course_id ),
        'has_overview_page'     => $this->course_has_overview_page,
        'section_id'            => $course_lesson->section_id,
        'section_title'         => $course_lesson->section_title,
        'lesson_id'             => $course_lesson->id,
        'lesson_title'          => $course_lesson->title,
        'lesson_content'        => $course_lesson->content,
        'lesson_is_completed'       => $course_lesson->is_completed,
        'lesson_completed_message'  => wpep_get_setting_translated( 'course-standard-lesson-completed' ),
        'next_lesson_id'            => $course_lesson->next_lesson_id,
        'next_lesson_title'         => $course_lesson->next_lesson_title,
        'previous_lesson_id'        => $course_lesson->previous_lesson_id,
        'previous_lesson_title'     => $course_lesson->previous_lesson_title,
        'complete_lesson_text'      => wpep_get_setting_translated( 'course-standard-complete-lesson' ),
      ];

      return $this->skeleton_navigation_current_item_content_prepare( WPEP_Controller::instance()->template->get_template( 'course-single/standard-lesson-content.php', $args ) );
    }

    $args = [
      'course_id'         => $this->course_id,
      'course_sections'   => $this->course_sections_map,
      'course_title'      => get_the_title( $this->course_id ),
      'course_content'    => wpep_format_content( get_post_meta( $this->course_id , 'wpep_course_description', true) ),
    ];

    return WPEP_Controller::instance()->template->get_template( 'course-single/standard-content-overview.php', $args );
  }

  public function display_sidebar_content( $course_id ) {
    $course_id = ( $course_id == false ? get_the_ID() : $course_id );

    if( $course_id != $this->course_id )
      return;

    echo WPEP_Controller::instance()->template->get_template( 'course-single/standard-lesson-sidebar.php', [
      'course_id'         => $this->course_id,
      'current_lesson_id' => $this->current_item_id,
      'course_sections'   => $this->course_sections_map
    ] );
  }

}