<?php

namespace WPEP\Entity;

class Discussion {

  protected static $_instance;

  public static function instance() {
    if( self::$_instance === null )
      self::$_instance = new self();

    return self::$_instance;
  }

  private $_table_discussion_field_whitelist = [ 'id', 'post_id', 'identifier', 'namespace', 'last_activity' ];
  private $_table_discussion_data_field_whitelist = [ 'id', 'discussion_id', 'user_id', 'key', 'value' ];
  private $_table_discussion_comment_field_whitelist = [ 'id', 'parent_id', 'discussion_id', 'user_id', 'content', 'created_at', 'last_activity', 'is_pinned', 'type' ];

  protected $_discussions_cache = [];

  public function discussion_id_by_identifier( $identifier, $post_id = 0, $namespace = '' ) {
    $discussion = $this->get_discussion( [
      'identifier'  => [ '=' => $identifier ]
    ] );

    if( $discussion === null ) {
      wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION, [
        'post_id'     => $post_id,
        'identifier'  => $identifier,
        'namespace'   => $namespace,
        'created_at'  => date( "Y-m-d H:i:s", time() )
      ] );

      return intval( wpep_wpdb()->insert_id );
    }

    return intval( $discussion->id );
  }

  public function add_discussion_comment( $data, $update_activity_flags = true ) {
    wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT, $data );

    $discussion_comment_id = wpep_wpdb()->insert_id;

    if( $update_activity_flags && isset( $data['discussion_id'] ) ) {
      $this->set_discussion_user_data( WPEP_DISCUSSION_USER_DATA_LAST_VIEW, time(), $data['discussion_id'], $data['user_id'] );

      wpep_wpdb()->update( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION, [
        'last_activity' => date( "Y-m-d H:i:s", time() )
      ], [
        'id' => $data['discussion_id']
      ] );
    }

    return $discussion_comment_id;
  }

  public function update_discussion_comment( $data, $where ) {
    wpep_wpdb()->update( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT, $data, $where );
  }

  public function get_discussion( $where ) {
    if( isset( $where['id'] ) && is_numeric( $where['id'] ) && count( $where ) === 1 ) {
      if( isset( $this->_discussions_cache[ $where['id'] ] )
          && $this->_discussions_cache[ $where['id'] ] !== null )
        return $this->_discussions_cache[ $where['id'] ];
    }

    $sql = 'SELECT *
              FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION;
    $sql .= ' ' . wpep_sql_where( $where, $this->_table_discussion_field_whitelist );

    $response = wpep_wpdb()->get_row( $sql );

    if( $response === null )
      return null;

    $this->_discussions_cache[ $response->id ] = $response;

    return $response;
  }

  public function has_discussion_comments( $where ) {
    $sql = 'SELECT id FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . '`';
    $sql .= ' ' . wpep_sql_where( $where, $this->_table_discussion_comment_field_whitelist );
    $sql .= ' LIMIT 1';

    return wpep_wpdb()->get_var( $sql ) !== null;
  }

  public function get_discussions( $where, $args = [] ) {
    $sql = 'SELECT SQL_CALC_FOUND_ROWS discussion.*';

    $sql .= ' FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION . ' discussion';

    $sql .= ' ' . wpep_sql_where( $where, $this->_table_discussion_field_whitelist, false, 'discussion' );

    $sql .= ' ORDER BY discussion.last_activity DESC';

    if( isset( $args['limit'] ) && $args['limit'] != 0 )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    if( isset( $args['offset'] ) && $args['offset'] != 0 )
      $sql .= ' OFFSET ' . absint( $args[ 'offset' ] );

    if( !isset( $args[ 'count_total' ] ) || $args[ 'count_total' ] == false )
      return $this->_get_discussions_results_pre_response( wpep_wpdb()->get_results( $sql ), $where, $args );

    $results = wpep_wpdb()->get_results( $sql );
    $total   = absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) );

    return [
      'results' => $this->_get_discussions_results_pre_response( $results, $where, $args ),
      'total'   => $total
    ];
  }

  private function _get_discussions_results_pre_response( $entries, $where, $args = [] ) {
    if( !isset( $args[ 'bind_user_information' ] ) || $args[ 'bind_user_information' ] == false )
      return $entries;

    if( empty( $entries ) )
      return $entries;

    $entries = wpep_utilities_map_object( $entries, 'id' );

    foreach( $entries as $entry_key => $entry )
      $entries[ $entry_key ]->user_data = [];

    $sql = 'SELECT * FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA . '` discussion_data
                    WHERE discussion_data.discussion_id IN (' . implode( ',', array_keys( $entries ) ). ')
                      AND discussion_data.user_id = ' . intval( $args[ 'bind_user_information' ] );

    $user_data = wpep_wpdb()->get_results( $sql );

    if( empty( $user_data ) )
      return $entries;

    foreach( $user_data as $user_datum )
      $entries[ $user_datum->discussion_id ]->user_data[ $user_datum->key ] = $user_datum->value;

    return $entries;
  }

  public function get_discussion_comment( $where, $args = [] ) {
    $sql = 'SELECT discussion_comment.*';

    if( isset( $args[ 'bind_user_reaction_data' ] ) && $args[ 'bind_user_reaction_data' ] != false ) {
      $sql .= ' , IFNULL( discussion_comment_data_reaction.value, "" ) as user_reaction';
    }

    $sql .= ' FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . '` discussion_comment';

    if( isset( $args[ 'bind_user_reaction_data' ] ) && $args[ 'bind_user_reaction_data' ] != false ) {
      $sql .= ' LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA . '` discussion_comment_data_reaction
                       ON discussion_comment_data_reaction.`discussion_comment_id` = discussion_comment.`id`
                      AND discussion_comment_data_reaction.`user_id` = ' . intval( $args[ 'bind_user_reaction_data' ] ) . '
                      AND discussion_comment_data_reaction.`key` = "' . WPEP_DISCUSSION_COMMENT_USER_DATA_REACTION . '"';
    }

    $sql .= ' ' . wpep_sql_where(
      $where,
      $this->_table_discussion_comment_field_whitelist,
        wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT,
      'discussion_comment'
      );

    return wpep_wpdb()->get_row( $sql );
  }

  public function delete_discussion_comment_by_id( $discussion_comment_id ) {
    $discussion_comment_id = absint( $discussion_comment_id );

    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA, [ 'discussion_comment_id' => $discussion_comment_id ] );
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT, [ 'parent_id' => $discussion_comment_id ] );
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT, [ 'id' => $discussion_comment_id ] );
  }

  public function pin_discussion_comment_by_id( $discussion_comment_id, $status = 1 ) {
    wpep_wpdb()->update(
      wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT,
      [
        'is_pinned' => absint( $status )
      ],
      [
        'id' => absint( $discussion_comment_id )
      ]
    );
  }

  public function delete_discussion( $where ) {
    foreach( $where as $key => $value ) {
      if( !in_array( $key, $this->_table_discussion_field_whitelist ) )
        unset( $where[ $key ] );

      $where[ $key ] = esc_sql( $value );
    }

    if( empty( $where ) )
      return false;

    /**
    // @todo here

    wpep_wpdb()->delete( Entity_Discussion::instance()->table_discussion_comment, [ 'discussion_id' => $discussion->id ] );
    wpep_wpdb()->delete( Entity_Discussion::instance()->table_discussion_comment_data, [ 'discussion_id' => $discussion->id ] );
    wpep_wpdb()->delete( Entity_Discussion::instance()->table_discussion_data, [ 'discussion_id' => $discussion->id ] );

    wpep_wpdb()->delete( Entity_Discussion::instance()->table_discussion, [ 'id' => $discussion->id ] );
**/
    return wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION, $where );
  }

  public function delete_discussion_data( $where ) {
    foreach( $where as $key => $value ) {
      if( !in_array( $key, $this->_table_discussion_comment_field_whitelist ) )
        unset( $where[ $key ] );

      $where[ $key ] = esc_sql( $value );
    }

    if( empty( $where ) )
      return false;

    return wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA, $where );
  }

  public function delete_discussion_comment( $where ) {
    foreach( $where as $key => $value ) {
      if( !in_array( $key, $this->_table_discussion_comment_field_whitelist ) )
        unset( $where[ $key ] );

      $where[ $key ] = esc_sql( $value );
    }

    if( empty( $where ) )
      return false;

    // @todo here.

    //wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT, [ 'user_id' => $user_id ] );
    //wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA, [ 'user_id' => $user_id ] );

    return wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT, $where );
  }

  /**
   * If WordPress will ever work smoothly with procedures, we want to refactor this.
   * @param $discussion_id
   * @param $search_term
   * @param int|false $filter_by_user_id
   * @return array
   */
  public function search_for_discussion_comment_ids( $discussion_id, $search_term, $filter_by_user_id = false ) {
    $sql = 'SELECT id, 
                   parent_id 
              FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . '`
             WHERE discussion_id = ' . $discussion_id;

    if( $search_term != '' )
      $sql .= ' AND content LIKE "%' . esc_sql( $search_term ) . '%"';

    if( $filter_by_user_id !== false )
      $sql .= ' AND user_id = ' . absint( $filter_by_user_id );

    $results = wpep_wpdb()->get_results( $sql );

    if( empty( $results ) )
      return [];

    $response = [];
    $required_parent_ids = [];

    foreach( $results as $result ) {
      if( !isset( $response[ $result->parent_id ] ) )
        $response[ $result->parent_id ] = [];

      $response[ $result->parent_id ][] = $result->id;

      if( $result->parent_id != 0 )
        $required_parent_ids[] = $result->parent_id;
    }

    if( empty( $required_parent_ids ) )
      return $response;

    while( !empty( $required_parent_ids ) ) {
      $sql = 'SELECT id, 
                     parent_id 
                FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . '`
               WHERE discussion_id = ' . $discussion_id . '
                 AND id IN (' . implode( ",", $required_parent_ids ) . ')';

      $results = wpep_wpdb()->get_results( $sql );

      if( empty( $results ) )
        return $response;

      $required_parent_ids = [];

      foreach( $results as $result ) {
        if( !isset( $response[ $result->parent_id ] ) )
          $response[ $result->parent_id ] = [];

        $response[ $result->parent_id ][] = $result->id;

        if( $result->parent_id != 0 )
          $required_parent_ids[] = $result->parent_id;
      }
    }

    return $response;
  }

  public function get_discussion_comments( $where, $args = [] ) {
    $sql = 'SELECT SQL_CALC_FOUND_ROWS discussion_comment.*';

    if( isset( $args[ 'bind_user_reaction_data' ] ) && $args[ 'bind_user_reaction_data' ] != false ) {
      $sql .= ' , IFNULL( discussion_comment_data_reaction.value, "" ) as user_reaction';
    }

    $sql .= ' FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . '` discussion_comment';

    if( isset( $args[ 'bind_user_reaction_data' ] ) && $args[ 'bind_user_reaction_data' ] != false ) {
      $sql .= ' LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA . '` discussion_comment_data_reaction
                       ON discussion_comment_data_reaction.`discussion_comment_id` = discussion_comment.`id`
                      AND discussion_comment_data_reaction.`user_id` = ' . intval( $args[ 'bind_user_reaction_data' ] ) . '
                      AND discussion_comment_data_reaction.`key` = "' . WPEP_DISCUSSION_COMMENT_USER_DATA_REACTION . '"';
    }

    $sql .= ' ' . wpep_sql_where(
      $where,
      $this->_table_discussion_comment_field_whitelist,
        wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT,
      'discussion_comment'
      );

    $sql .= ' ORDER BY';

    if( !isset( $args[ 'do_not_order_by_pinned'] ) || $args[ 'do_not_order_by_pinned'] == false )
      $sql .= ' discussion_comment.is_pinned DESC,';

    $sql .= ' discussion_comment.' . ( isset( $args['order_by'] ) ? $args['order_by'] : 'last_activity' );
    $sql .= ' ' . ( isset( $args['order'] ) && $args['order'] == 'ASC' ? 'ASC' : 'DESC' );

    if( isset( $args['limit'] ) && $args['limit'] != 0 )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    if( isset( $args['offset'] ) && $args['offset'] != 0 )
      $sql .= ' OFFSET ' . absint( $args[ 'offset' ] );

    if( !isset( $args[ 'count_total' ] ) || $args[ 'count_total' ] == false )
      return wpep_wpdb()->get_results( $sql );

    return [
      'results' => wpep_wpdb()->get_results( $sql ),
      'total'   => absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) )
    ];
  }

  /**
   * @param bool $key
   * @param bool $discussion_id
   * @param bool $user_id
   * @param bool $single_row
   * @return array|null
   */
  public function get_discussion_user_data_value( $key = false, $discussion_id = false, $user_id = false, $single_row = true ) {
    if( $single_row ) {
      $user_data = $this->get_discussion_user_data( $key, $discussion_id, $user_id, true );

      if( $user_data !== null )
        return $user_data->value;

      return null;
    }

    $response = [];

    foreach( $this->get_discussion_user_data( $key, $discussion_id, $user_id, false ) as $user_data )
      $response[$user_data->id] = $user_data->value;

    return $response;
  }

  /**
   * @param bool $key
   * @param bool $discussion_id
   * @param bool $user_id
   * @param bool $single_row
   * @param bool $value
   * @return array|object|null
   */
  public function get_discussion_user_data( $key = false, $discussion_id = false, $user_id = false, $single_row = true, $value = false ) {
    if ( $single_row ) {
      $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

      if ($user_id == 0 )
        return null;
    }

    $sql = 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA . ' dud WHERE 1 = 1 ';

    if( $user_id != 0 )
      $sql .= ' AND dud.user_id = ' . intval( $user_id );

    if( $key !== false )
      $sql .= ' AND dud.key = "' . esc_sql( $key ) . '"';

    if( $discussion_id !== false )
      $sql .= ' AND dud.discussion_id  = ' . intval( $discussion_id );

    if( $value !== false ) {
      if( is_array( $value ) && isset( $value[0] ) && isset( $value[1] )  ) {
        if( $value[0] !== false )
          $sql .= ' AND dud.value  >= ' . intval( $value[0] );
        if( $value[1] !== false )
          $sql .= ' AND dud.value  <= ' . intval( $value[1] );
      } else {
        $sql .= ' AND dud.value  = ' . intval( $value );
      }
    }

    return ( $single_row ? wpep_wpdb()->get_row( $sql ) : wpep_wpdb()->get_results( $sql ) );
  }

  /**
   * @param $key
   * @param $value
   * @param $discussion_id
   * @param bool $user_id
   * @return bool
   */
  public function set_discussion_user_data( $key, $value, $discussion_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return false;

    $discussion_id  = intval( $discussion_id );

    $existent_entry = $this->get_discussion_user_data( $key, $discussion_id, $user_id, true );

    $value_updated = true;

    if( $existent_entry === null )
      $this->add_discussion_user_data( $key, $value, $discussion_id, $user_id );
    else if( $existent_entry->value != $value ) {
      wpep_wpdb()->update(
        wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA,
        [ 'value' => $value ],
        [ 'id'    => $existent_entry->id ]
      );
    } else {
      $value_updated = false;
    }

    if( !$value_updated )
      return true;

    do_action( "wpep_user_set_discussion_data", $key, $value, $discussion_id, $user_id, $existent_entry );

    return true;
  }

  /**
   * @param $key
   * @param $value
   * @param $discussion_id
   * @param bool $user_id
   */
  public function add_discussion_user_data( $key, $value, $discussion_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return;

    $discussion_id  = intval( $discussion_id );

    wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA, [
      'user_id'       => $user_id,
      'discussion_id' => $discussion_id,
      'key'           => $key,
      'value'         => $value,
    ] );

    do_action( "wpep_user_add_discussion_data", $key, $value, $discussion_id, $user_id );
  }

  /**
   * @param string|null $key
   * @param string|null $value
   * @param int|null $discussion_id
   * @param int|bool $user_id
   * @return void
   */
  public function remove_discussion_user_data( $key = null, $value = null, $discussion_id = null, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return null;

    $delete_where = [ 'user_id' => $user_id ];

    if( $key !== null )
      $delete_where['key'] = esc_sql( $key );

    if( $value !== null )
      $delete_where['value'] = esc_sql( $value );

    if( $discussion_id !== null )
      $delete_where['discussion_id'] = intval( $discussion_id );

    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA, $delete_where );

    do_action( "wpep_user_remove_discussion_data", $key, $value, $discussion_id, $user_id );
  }

  /**
   * @param bool $key
   * @param bool $discussion_comment_id
   * @param bool $user_id
   * @param bool $single_row
   * @return array|null
   */
  public function get_discussion_comment_user_data_value( $key = false, $discussion_comment_id = false, $user_id = false, $single_row = true ) {
    if( $single_row ) {
      $user_data = $this->get_discussion_user_data( $key, $discussion_comment_id, $user_id, true );

      if( $user_data !== null )
        return $user_data->value;

      return null;
    }

    $response = [];

    foreach( $this->get_discussion_user_data( $key, $discussion_comment_id, $user_id, false ) as $user_data )
      $response[$user_data->id] = $user_data->value;

    return $response;
  }

  /**
   * @param bool $key
   * @param bool $discussion_comment_id
   * @param bool $user_id
   * @param bool $single_row
   * @param bool $value
   * @return array|object|null
   */
  public function get_discussion_comment_user_data( $key = false, $discussion_comment_id = false, $user_id = false, $single_row = true, $value = false ) {
    if ( $single_row ) {
      $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

      if ($user_id == 0 )
        return null;
    }

    $sql = 'SELECT * FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA . '` dud WHERE 1 = 1 ';

    if( $user_id != 0 )
      $sql .= ' AND dud.user_id = ' . intval( $user_id );

    if( $key !== false )
      $sql .= ' AND dud.key = "' . esc_sql( $key ) . '"';

    if( $discussion_comment_id !== false )
      $sql .= ' AND dud.discussion_comment_id  = ' . intval( $discussion_comment_id );

    if( $value !== false ) {
      if( is_array( $value ) && isset( $value[0] ) && isset( $value[1] )  ) {
        if( $value[0] !== false )
          $sql .= ' AND dud.value  >= ' . intval( $value[0] );
        if( $value[1] !== false )
          $sql .= ' AND dud.value  <= ' . intval( $value[1] );
      } else {
        $sql .= ' AND dud.value  = ' . intval( $value );
      }
    }

    return ( $single_row ? wpep_wpdb()->get_row( $sql ) : wpep_wpdb()->get_results( $sql ) );
  }

  /**
   * @param $key
   * @param $value
   * @param $discussion_comment_id
   * @param bool $user_id
   * @return bool
   */
  public function set_discussion_comment_user_data( $key, $value, $discussion_comment_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return false;

    $discussion_comment_id  = intval( $discussion_comment_id );

    $existent_entry = $this->get_discussion_comment_user_data( $key, $discussion_comment_id, $user_id, true );

    $value_updated = true;

    if( $existent_entry === null )
      $this->add_discussion_comment_user_data( $key, $value, $discussion_comment_id, $user_id );
    else if( $existent_entry->value != $value ) {
      wpep_wpdb()->update(
        wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA,
        [ 'value' => $value, 'last_activity' => date( "Y-m-d H:i:s", time() ) ],
        [ 'id'    => $existent_entry->id ]
      );
    } else {
      $value_updated = false;
    }

    if( !$value_updated )
      return true;

    do_action( "wpep_user_set_discussion_comment_data", $key, $value, $discussion_comment_id, $user_id, $existent_entry );

    return true;
  }

  /**
   * @param $key
   * @param $value
   * @param $discussion_comment_id
   * @param bool $user_id
   * @param bool|string $last_activity
   */
  public function add_discussion_comment_user_data( $key, $value, $discussion_comment_id, $user_id = false, $last_activity = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return;

    $discussion_comment_id  = intval( $discussion_comment_id );

    wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA, [
      'user_id'               => $user_id,
      'discussion_comment_id' => $discussion_comment_id,
      'key'                   => $key,
      'value'                 => $value,
      'last_activity'         => ( $last_activity !== false ? $last_activity : date( "Y-m-d H:i:s", time() ) )
    ] );

    do_action( "wpep_user_add_discussion_comment_data", $key, $value, $discussion_comment_id, $user_id );
  }

  /**
   * @param string|null $key
   * @param string|null $value
   * @param int|null $discussion_comment_id
   * @param int|bool $user_id
   * @return void
   */
  public function remove_discussion_comment_user_data( $key = null, $value = null, $discussion_comment_id = null, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return null;

    $delete_where = [ 'user_id' => $user_id ];

    if( $key !== null )
      $delete_where['key'] = esc_sql( $key );

    if( $value !== null )
      $delete_where['value'] = esc_sql( $value );

    if( $discussion_comment_id !== null )
      $delete_where['discussion_comment_id'] = intval( $discussion_comment_id );

    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA, $delete_where );

    do_action( "wpep_user_remove_discussion_comment_data", $key, $value, $discussion_comment_id, $user_id );
  }

  public function get_discussion_user_ids( $discussion_id, $args = [] ) {
    $sql = 'SELECT SQL_CALC_FOUND_ROWS `user_id`
		          FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA . '`
             WHERE `discussion_id` = ' . absint( $discussion_id ) . '
               AND ( `key` = "' . WPEP_DISCUSSION_USER_DATA_LAST_VIEW . '" OR `key` = "' . WPEP_DISCUSSION_USER_DATA_EMAIL_NOTIFICATION_STATUS . '" )
          GROUP BY `user_id`';

    if( isset( $args[ 'offset' ] ) && isset( $args[ 'limit' ] ) && $args[ 'limit' ] != false )
      $sql .= ' LIMIT ' . absint( $args[ 'offset' ] ) . ',' . absint( $args[ 'limit' ] );
    elseif( isset( $args[ 'limit' ] ) && $args[ 'limit' ] !== false )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    $response = [
      'ids'   => wpep_wpdb()->get_col( $sql ),
      'total' => absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) )
    ];

    return $response;
  }

  public function get_discussion_ids_by_user_id( $user_id, $args = [] ) {
    $count_total = ( isset( $args[ 'count_total' ] ) ? $args[ 'count_total' ] : false );

    // notifications_enabled

    $sql = 'SELECT' . ( $count_total ? ' SQL_CALC_FOUND_ROWS' : '' ) . ' discussion_data.`discussion_id`
		          FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA . '` discussion_data
        INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION . '` discussion
                ON discussion.`id` = discussion_data.`discussion_id`
             WHERE discussion_data.`user_id` = ' . absint( $user_id ) . '
               AND ( discussion_data.`key` = "' . WPEP_DISCUSSION_USER_DATA_LAST_VIEW . '" OR discussion_data.`key` = "' . WPEP_DISCUSSION_USER_DATA_EMAIL_NOTIFICATION_STATUS . '" )
               AND ( discussion.post_id = 0 || IFNULL( ( SELECT posts.post_status FROM ' . wpep_wpdb()->posts . ' posts WHERE posts.`ID` = discussion.post_id LIMIT 1 ), "none" ) = "publish" )
          GROUP BY `discussion_id`';

    if( isset( $args[ 'offset' ] ) && isset( $args[ 'limit' ] ) && $args[ 'limit' ] != false )
      $sql .= ' LIMIT ' . absint( $args[ 'offset' ] ) . ',' . absint( $args[ 'limit' ] );
    elseif( isset( $args[ 'limit' ] ) && $args[ 'limit' ] !== false )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    if( !$count_total )
      return wpep_wpdb()->get_col( $sql );

    $response = [
      'ids'   => wpep_wpdb()->get_col( $sql ),
      'total' => absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) )
    ];

    return $response;
  }

  /**
   * @param \WPEP_Discussion_Integration[] $discussion_integrations
   * @param $limit
   * @return array
   */
  public function get_email_notifications_data( $discussion_integrations, $limit = 20 ) {
    if( empty( $discussion_integrations ) )
      return [];

    $sql  = 'SELECT inner_query.user_id,
                    GROUP_CONCAT( CONCAT( inner_query.id, "=", user_last_activity ) SEPARATOR ",") as discussion_activity_map
               FROM (' . $this->_get_email_notification_data_inner_query( $discussion_integrations ). ') inner_query
               GROUP BY inner_query.user_id
               LIMIT ' . $limit;

    $notifications_list = wpep_wpdb()->get_results( $sql );

    if( empty( $notifications_list ) )
      return [];

    $response = [
      'user_id_map'    => [],
      'discussion_ids' => [],
      'discussions'    => []
    ];

    foreach( $notifications_list as $notification_data ) {
      $discussions_activity_map = explode( ",", $notification_data->discussion_activity_map );

      foreach( $discussions_activity_map as $discussion_activity_map ) {
        $discussion_activity_map = explode( "=", $discussion_activity_map );

        $response['user_id_map'][ $notification_data->user_id ][ $discussion_activity_map[0] ] = $discussion_activity_map[1];
        $response['discussion_ids'][ $discussion_activity_map[0] ] = $discussion_activity_map[0];
      }
    }

    $response['discussions'] = wpep_utilities_map_object( $this->get_discussions( [ 'id' => [ 'IN' => $response['discussion_ids'] ] ] ), 'id' );

    return $response;
  }

  private function _get_email_notification_data_inner_query( $discussion_integrations ) {
    $notifications_interval = intval( wpep_get_setting( 'comments-email-notification-frequency' ) );

    $sql = '';
    $sql .=   'SELECT discussion.id,
                      discussion_dlv.user_id as user_id,
                      IF( discussion_dln.value > discussion_dlv.value, discussion_dln.value, discussion_dlv.value ) as user_last_activity
                 FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION . '` discussion
            LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA . '` discussion_dlv 
                   ON discussion_dlv.discussion_id = discussion.id 
                  AND discussion_dlv.key = "' . WPEP_DISCUSSION_USER_DATA_LAST_VIEW . '"
            LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA . '` discussion_dln 
                   ON discussion_dln.discussion_id = discussion.id 
                  AND discussion_dln.key = "' . WPEP_DISCUSSION_USER_DATA_LAST_EMAIL_NOTIFICATION . '"
                  AND discussion_dln.user_id = discussion_dlv.user_id
            LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA . '` discussion_dns 
                   ON discussion_dns.discussion_id = discussion.id 
                  AND discussion_dns.key = "' . WPEP_DISCUSSION_USER_DATA_EMAIL_NOTIFICATION_STATUS . '"
                  AND discussion_dns.user_id = discussion_dlv.user_id
                WHERE IFNULL( discussion_dns.value, 1 ) != 0
                  AND from_unixtime( IF( discussion_dln.value > discussion_dlv.value, discussion_dln.value, discussion_dlv.value ) ) < discussion.last_activity
                  AND ( IF( CAST( discussion_dln.value AS SIGNED ) > CAST( discussion_dlv.value AS SIGNED ), CAST( discussion_dln.value AS SIGNED ), CAST( discussion_dlv.value AS SIGNED ) ) + ' . $notifications_interval . ' ) < ' . time() . '
                  AND (' . $this->_get_email_notification_data_inner_query_integration_in( $discussion_integrations ) . ')';

    return $sql;
  }

  private function _get_email_notification_data_inner_query_integration_in( $discussion_integrations ) {
    $sql = '';

    $i = 0; foreach( $discussion_integrations as $discussion_integration ) {
      $sql .= ( $i >= 1 ? ' OR ' : ' ' ) . 'discussion.namespace = "' . $discussion_integration->get_namespace() . '"';
      $i++;
    }

    return $sql;
  }

  public function get_notifications_data( $user_id, $args = [] ) {
    $discussion_ids = $this->get_discussion_ids_by_user_id( $user_id, [
      'limit'                 => false,
      'count_total'           => false,
      'notifications_enabled' => true
    ] );

    if( empty( $discussion_ids ) )
      return [];

    $args = wp_parse_args( $args, [
      'limit'                      => 10,
      'include_reactions'          => wpep_get_setting_flag( 'comments-reactions-status' ),
      'include_mentions'           => wpep_get_setting_flag( 'comments-mentions-status' )
    ] );

    $user_id = intval( $user_id );
    $args[ 'limit' ] = intval( $args[ 'limit' ] );

    $sql = '';

    $sql .= 'SELECT discussion_comment.`id` as notification_comment_id,
                    discussion_comment.`discussion_id` as notification_discussion_id,
                    discussion_comment.`user_id` as notification_user_id,
                    discussion_comment.`created_at` as notification_created_at,
                    IFNULL( discussion_data_last_view.`value`, 0 ) as notification_last_user_view_timestamp,
                    "" as notification_icon_emoji_code,
                    IF( 
                      discussion_comment_parent.`user_id` = ' . $user_id . ', 
                      "' . WPEP_DISCUSSION_NOTIFICATION_TYPE_COMMENT_REPLY . '", 
                      IF( discussion_comment_sl_user.`id` IS NOT NULL, "' . WPEP_DISCUSSION_NOTIFICATION_TYPE_COMMENT_REPLY_LEVEL . '", "' . WPEP_DISCUSSION_NOTIFICATION_TYPE_COMMENT . '" ) 
                    ) as notification_type
               FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . '` discussion_comment
         INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION . '` discussion
                 ON discussion.id = discussion_comment.discussion_id
          LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA . '` discussion_data_last_view 
                 ON discussion_data_last_view.`discussion_id` = discussion_comment.discussion_id
                AND discussion_data_last_view.`user_id` = ' . $user_id . '
                AND discussion_data_last_view.`key` = "' . WPEP_DISCUSSION_USER_DATA_LAST_VIEW . '"
          LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . '` discussion_comment_parent 
                 ON discussion_comment.id = discussion_comment_parent.parent_id
          LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . '` discussion_comment_sl_user 
                 ON discussion_comment_parent.`id` = discussion_comment_sl_user.parent_id
                AND discussion_comment_sl_user.`user_id` = ' . $user_id;

    if( $args['include_mentions'] ) {
      $sql .=   ' LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA . '` discussion_comment_data_mention
                         ON discussion_comment_data_mention.`discussion_comment_id` = discussion_comment.`id`
                        AND discussion_comment_data_mention.`key` = "' . WPEP_DISCUSSION_COMMENT_USER_DATA_MENTION . '"
                        AND discussion_comment_data_mention.`user_id` = ' . $user_id;
    }

    $sql .= ' WHERE discussion_comment.`discussion_id` IN (' . implode( ',', $discussion_ids ) . ')
                AND discussion_comment.`user_id` != ' . $user_id . '
                AND ( discussion_comment.`parent_id` = 0
                 OR discussion_comment_parent.`user_id` = ' . $user_id . '
                 OR discussion_comment_sl_user.`id` IS NOT NULL 
                )';

    if( $args['include_mentions'] )
      $sql .= ' AND discussion_comment_data_mention.`id` IS NULL';

    $sql .= ' ORDER BY discussion_comment.`created_at` DESC
                 LIMIT ' . $args[ 'limit' ];

    if( $args['include_reactions'] == false
        && $args['include_mentions'] == false )
      return wpep_wpdb()->get_results( $sql );

    $sql = '(' . $sql . ')';

    if( $args['include_mentions'] ) {
      $sql .= ' UNION ';
      $sql .= '(';
      $sql .= 'SELECT discussion_comment.`id` as notification_comment_id,
                      discussion_comment.`discussion_id` as notification_discussion_id,
                      discussion_comment.`user_id` as notification_user_id,
                      discussion_comment_data_mention.`last_activity` as notification_created_at,
                      IFNULL( discussion_data_last_view.`value`, 0 ) as notification_last_user_view_timestamp,
                      "" as notification_icon_emoji_code,
                      "' . WPEP_DISCUSSION_NOTIFICATION_TYPE_MENTION . '" as notification_type
                 FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA . '` discussion_comment_data_mention
           INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . '` discussion_comment 
                   ON discussion_comment.`id` = discussion_comment_data_mention.`discussion_comment_id`
           INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION . '` discussion
                   ON discussion.id = discussion_comment.`discussion_id`
            LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA . '` discussion_data_last_view 
                   ON discussion_data_last_view.`discussion_id` = discussion_comment.`discussion_id`
                  AND discussion_data_last_view.`user_id` = ' . $user_id . '
                  AND discussion_data_last_view.`key` = "' . WPEP_DISCUSSION_USER_DATA_LAST_VIEW . '"
                WHERE discussion_comment_data_mention.`key` = "' .  WPEP_DISCUSSION_COMMENT_USER_DATA_MENTION . '"
                  AND discussion_comment_data_mention.`user_id` = ' . $user_id . '
                   AND discussion_comment.`user_id` != ' . $user_id . '
                  AND discussion_comment.`discussion_id` IN (' . implode( ',', $discussion_ids ) . ')
             ORDER BY discussion_comment_data_mention.`last_activity` DESC
                LIMIT ' . $args[ 'limit' ];
      $sql .= ')';
    }

    if( $args['include_reactions'] ) {
      $sql .= ' UNION ';
      $sql .= '(';
      $sql .= 'SELECT discussion_comment.`id` as notification_comment_id,
                      discussion_comment.`discussion_id` as notification_discussion_id,
                      discussion_comment_data_reaction.`user_id` as notification_user_id,
                      discussion_comment_data_reaction.`last_activity` as notification_created_at,
                      IFNULL( discussion_data_last_view.`value`, 0 ) as notification_last_user_view_timestamp,
                      discussion_comment_data_reaction.`value` as notification_icon_emoji_code,
                      "' . WPEP_DISCUSSION_NOTIFICATION_TYPE_REACTION . '" as notification_type
                 FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA . '` discussion_comment_data_reaction
           INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . '` discussion_comment 
                   ON discussion_comment.`id` = discussion_comment_data_reaction.`discussion_comment_id`
           INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION . '` discussion
                   ON discussion.id = discussion_comment.`discussion_id`
            LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA . '` discussion_data_last_view 
                   ON discussion_data_last_view.`discussion_id` = discussion_comment.`discussion_id`
                  AND discussion_data_last_view.`user_id` = ' . $user_id . '
                  AND discussion_data_last_view.`key` = "' . WPEP_DISCUSSION_USER_DATA_LAST_VIEW . '"
                WHERE discussion_comment_data_reaction.`key` = "' .  WPEP_DISCUSSION_COMMENT_USER_DATA_REACTION . '"
                  AND discussion_comment_data_reaction.`user_id` != "' . $user_id . '"
                  AND discussion_comment.`user_id` = "' . $user_id . '"
                  AND discussion_comment.`discussion_id` IN (' . implode( ',', $discussion_ids ) . ')
             ORDER BY discussion_comment_data_reaction.`last_activity` DESC
                LIMIT ' . $args[ 'limit' ];
      $sql .= ')';
    }

    $sql = 'SELECT * FROM (' . $sql . ') helper_table 
                  ORDER BY helper_table.notification_created_at DESC
                     LIMIT ' . $args[ 'limit' ];

    return wpep_wpdb()->get_results( $sql );
  }

  public function get_comment_count_by_args( $args = [] ) {
    $sql = 'SELECT COUNT(discussion_comment.id) 
              FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . ' discussion_comment
         LEFT JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION . ' discussion 
                ON discussion.id = discussion_comment.discussion_id
             WHERE 1 = 1';

    if( isset( $args['namespace' ] ) )
      $sql .= ' AND discussion.`namespace` = "' . esc_sql( $args['namespace' ] ) . '"';

    if( isset( $args['parent_id' ] ) )
      $sql .= ' AND discussion_comment.`parent_id` = ' . intval( $args['parent_id' ] );

    if( isset( $args['time_from' ] ) )
      $sql .= ' AND discussion_comment.`created_at` >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '"';

    if( isset( $args['time_to' ] ) )
      $sql .= ' AND discussion_comment.`created_at` < "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';

    if( isset( $args[ 'user_ids' ] ) ) {
      if( is_numeric( $args[ 'user_ids' ] ) )
        $sql .= ' AND discussion_comment.`user_id` = ' . intval( $args[ 'user_ids' ] );
      else if( is_array( $args[ 'user_ids' ] ) )
        $sql .= ' AND discussion_comment.`user_id` IN (' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ')';
    }

    return wpep_wpdb()->get_var( $sql );
  }

  public function get_comment_reply_count_by_args( $args = [] ) {
    $sql = 'SELECT COUNT(discussion_comment.id) 
              FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . ' discussion_comment
         LEFT JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION . ' discussion 
                ON discussion.id = discussion_comment.discussion_id
             WHERE discussion_comment.`parent_id` != 0';

    if( isset( $args['namespace' ] ) )
      $sql .= ' AND discussion.`namespace` = "' . esc_sql( $args['namespace' ] ) . '"';

    if( isset( $args[ 'user_ids' ] ) ) {
      if( is_numeric( $args[ 'user_ids' ] ) )
        $sql .= ' AND discussion_comment.`user_id` = ' . intval( $args[ 'user_ids' ] );
      else if( is_array( $args[ 'user_ids' ] ) )
        $sql .= ' AND discussion_comment.`user_id` IN (' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ')';
    }

    if( isset( $args['time_from' ] ) )
      $sql .= ' AND discussion_comment.`created_at` >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '"';

    if( isset( $args['time_to' ] ) )
      $sql .= ' AND discussion_comment.`created_at` < "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';

    return wpep_wpdb()->get_var( $sql );
  }

  public function get_comment_data_count_by_args( $args = [] ) {
    $sql = 'SELECT COUNT(discussion_comment.id) 
              FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA . ' discussion_comment_data
        INNER JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . ' discussion_comment ON discussion_comment.id = discussion_comment_data.discussion_comment_id
        INNER JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION . ' discussion ON discussion.id = discussion_comment.discussion_id
             WHERE 1 = 1';

    if( isset( $args['namespace' ] ) )
      $sql .= ' AND discussion.`namespace` = "' . esc_sql( $args['namespace' ] ) . '"';

    if( isset( $args[ 'user_ids' ] ) ) {
      if( is_numeric( $args[ 'user_ids' ] ) )
        $sql .= ' AND discussion_comment.`user_id` = ' . intval( $args[ 'user_ids' ] );
      else if( is_array( $args[ 'user_ids' ] ) )
        $sql .= ' AND discussion_comment.`user_id` IN (' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ')';
    }

    if( isset( $args['key'] ) )
      $sql .= ' AND discussion_comment_data.`key` = "' . esc_sql( $args['key'] ) . '"';

    if( isset( $args['value'] ) )
      $sql .= ' AND discussion_comment_data.`value` = "' . esc_sql( $args['value'] ) . '"';

    if( isset( $args['time_from' ] ) )
      $sql .= ' AND discussion_comment_data.`last_activity` >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '"';

    if( isset( $args['time_to' ] ) )
      $sql .= ' AND discussion_comment_data.`last_activity` < "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';

    return wpep_wpdb()->get_var( $sql );
  }

  public function get_discussion_comment_reactions( $discussion_comment_id, $args = [] ) {
    $sql = 'SELECT SQL_CALC_FOUND_ROWS dcd.*';

    $sql .= ' FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA . ' dcd';

    $sql .= ' WHERE dcd.`discussion_comment_id` = ' . intval( $discussion_comment_id );
    $sql .= '   AND dcd.`key` = "' . WPEP_DISCUSSION_COMMENT_USER_DATA_REACTION . '"';

    if( isset( $args[ 'reaction_code' ] ) && $args[ 'reaction_code' ] != '' )
      $sql .= ' AND dcd.`value` = "' . $args[ 'reaction_code' ] . '"';

    $sql .= ' ORDER BY dcd.`last_activity` DESC';

    if( isset( $args['limit'] ) && $args['limit'] != 0 )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    if( isset( $args['offset'] ) && $args['offset'] != 0 )
      $sql .= ' OFFSET ' . absint( $args[ 'offset' ] );

    if( !isset( $args[ 'count_total' ] ) || $args[ 'count_total' ] == false )
      return wpep_wpdb()->get_results( $sql );

    $results = wpep_wpdb()->get_results( $sql );
    $total   = absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) );

    return [
      'results' => $results,
      'total'   => $total
    ];
  }

  /**
   * @param int $discussion_comment_id
   * @return array
   */
  public function get_discussion_comment_reactions_count( $discussion_comment_id ) {
    $sql = 'SELECT COUNT( dcd.`id` ) as reaction_count,
                         dcd.`value` as reaction_code
                    FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA . ' dcd 
                    WHERE dcd.`discussion_comment_id` = ' . intval( $discussion_comment_id ) . '
                      AND dcd.`key` = "' . WPEP_DISCUSSION_COMMENT_USER_DATA_REACTION . '"
                 GROUP BY dcd.`value`';

    $data = wpep_wpdb()->get_results( $sql );

    if( empty( $data ) )
      return [];

    $response = [];

    foreach( $data as $d )
      $response[ $d->reaction_code ] = $d->reaction_count;

    return $response;
  }

}