<?php

namespace WPEP\Entity;

class SystemLog {

  protected static $_instance = [];

  /**
   * @param $namespace
   * @param $event
   * @param $started_at
   * @return SystemLog
   */
  public static function instance( $namespace, $event, $started_at ) {
    if( !isset( self::$_instance[ $namespace . '_' . $event . '_' . $started_at ] ) )
      self::$_instance[ $namespace . '_' . $event . '_' . $started_at ] = new self( $namespace, $event, $started_at );

    return self::$_instance[ $namespace . '_' . $event . '_' . $started_at ];
  }

  public static function get_namespace_list() {
    $sql = 'SELECT DISTINCT namespace FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG;

    return wpep_wpdb()->get_col( $sql );
  }

  public static function get_event_list_by_namespace( $namespace ) {
    $sql = 'SELECT DISTINCT event FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG . ' WHERE `namespace` = "' . esc_sql( $namespace ). '" ';

    return wpep_wpdb()->get_col( $sql );
  }

  public static function get_entries( $args = [] ) {
    $sql = 'SELECT SQL_CALC_FOUND_ROWS *
              FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG .
           ' WHERE 1 = 1';

    if( isset( $args[ 'namespace' ] ) && $args[ 'namespace' ] !== false )
      $sql .= ' AND `namespace` = "' . esc_sql( $args[ 'namespace' ] ) . '"';

    if( isset( $args[ 'event' ] ) && $args[ 'event' ] !== false )
      $sql .= ' AND `event` = "' . esc_sql( $args[ 'event' ] ) . '"';

    if( isset( $args[ 'started_at' ] ) && is_array( $args[ 'started_at' ] ) )
      foreach( $args[ 'started_at' ] as $started_at_key => $started_at_value )
        $sql .= ' AND ( `started_at` >= ' . intval( $started_at_key ) . ' AND `started_at` <= ' . intval( $started_at_value ) . ' )';

    if( isset( $args[ 'completed_at' ] ) )
      $sql .= ' AND `completed_at` = ' . intval( $args[ 'completed_at' ] );

    $sql .= ' ORDER BY started_at DESC';

    if( isset( $args['limit'] ) && $args['limit'] != 0 )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    if( isset( $args['offset'] ) && $args['offset'] != 0 )
      $sql .= ' OFFSET ' . absint( $args[ 'offset' ] );

    if( !isset( $args[ 'count_total' ] ) || $args[ 'count_total' ] == false )
      return wpep_wpdb()->get_results( $sql );

    return [
      'results' => wpep_wpdb()->get_results( $sql ),
      'total'   => absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) )
    ];
  }

  public static function get_reporting_start_at() {
    $sql = 'SELECT system_log.`started_at`';

    $sql .= ' FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG . '` system_log';

    $sql .= ' ORDER BY system_log.`started_at` ASC';
    $sql .= ' LIMIT 1';

    return wpep_wpdb()->get_var( $sql );
  }

  public static function get_old_entries_count() {
    $old_entries_started_at_marker = strtotime( 'today 00:00:00' ) - MONTH_IN_SECONDS;

    return wpep_wpdb()->get_var(
      'SELECT COUNT( system_log.id ) 
               FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG . '` system_log 
              WHERE system_log.started_at < ' . $old_entries_started_at_marker
    );
  }

  public static function get_event_count_by_args( $args ) {
    if( isset( $args['group_by'] ) && !is_array( $args['group_by'] ) )
      $args['group_by'] = [ $args['group_by'] ];

    $sql = 'SELECT COUNT(system_log.id) as entries';

    if( isset( $args['group_by'] ) && in_array( 'namespace', $args['group_by'] ) )
      $sql .= ', system_log.`namespace` as namespace';

    if( isset( $args['group_by'] ) && in_array( 'event', $args['group_by'] ) )
      $sql .= ', system_log.`event` as event';


    $sql .= ' FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG . '` system_log';

    $sql .= ' WHERE 1 = 1';

    if( isset( $args['namespace' ] ) ) {
      if( is_array( $args["namespace"] ) && !empty( $args["namespace"] ) ) {
        $sql .= ' AND system_log.`namespace` IN ("' . implode( '","', array_map( 'esc_sql', $args["namespace"] ) ) . '")';
      } else {
        $sql .= ' AND system_log.`namespace` = "' . esc_sql( $args["namespace"] ) . '"';
      }
    } else {
      if( isset( $args['group_by'] ) && in_array( 'namespace', $args['group_by'] ) )
        $sql .= ' AND system_log.`namespace` != ""';
    }

    if( isset( $args[ 'event' ] ) ) {
      if( is_array( $args["event"] ) && !empty( $args["event"] ) ) {
        $sql .= ' AND system_log.`event` IN ("' . implode( '","', $args["event"] ) . '")';
      } else {
        $sql .= ' AND system_log.`event` = "' . esc_sql( $args["event"] ) . '"';
      }
    } else {
      if( isset( $args['group_by'] ) && in_array( 'segment', $args['group_by'] ) )
        $sql .= ' AND system_log.`segment` != ""';
    }

    if( isset( $args['time_from' ] ) )
      $sql .= ' AND system_log.`started_at` >= ' . intval( $args['time_from'] );

    if( isset( $args['time_to' ] ) )
      $sql .= ' AND system_log.`started_at` < ' . intval( $args['time_to'] );

    if( isset( $args['group_by'] ) ) {
      $sql .= ' GROUP BY';

      $i = 0; foreach( $args['group_by'] as $group_by )
        $sql .= ( $i++ === 0 ? ' ' : ',' ) . 'system_log.`' . $group_by . '`';
    }

    return ( isset( $args['group_by'] ) ? wpep_wpdb()->get_results( $sql ) : absint( wpep_wpdb()->get_var( $sql ) ) );
  }

  public static function delete_old_entries() {
    $old_entries_started_at_marker = strtotime( 'today 00:00:00' ) - MONTH_IN_SECONDS;
    $log = new self( 'maintenance', 'clear_old_logs', time() );
    $log->started_arguments = [
      'started_at_marker' => $old_entries_started_at_marker
    ];
    $log->save();

    $log->completed_arguments = [
      'delete_response' => wpep_wpdb()->query('DELETE FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG . '` WHERE started_at < ' . $old_entries_started_at_marker )
    ];
    $log->completed_at = time();
    $log->save();
  }

  public static function truncate_table() {
    wpep_wpdb()->query( 'TRUNCATE `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG . '`' );
  }

  protected $_table_system_log_field_whitelist = [ 'id', 'namespace', 'event', 'started_at', 'completed_at', 'started_arguments', 'completed_arguments' ];

  public $id = 0;
  public $namespace;
  public $event;
  public $started_at;
  public $completed_at = 0;
  public $started_arguments = [];
  public $completed_arguments = [];

  public function __construct( $namespace, $event, $started_at ) {
    $this->namespace = $namespace;
    $this->event = $event;
    $this->started_at = intval( $started_at );

    $this->_sync_from_db();
  }

  public function save() {
    if( $this->id !== 0 ) {
      wpep_wpdb()->update( wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG, [
        'namespace'           => $this->namespace,
        'event'               => $this->event,
        'started_at'          => $this->started_at,
        'completed_at'        => $this->completed_at,
        'started_arguments'   => json_encode( $this->started_arguments ),
        'completed_arguments' => json_encode( $this->completed_arguments ),
      ], [
        'id'  => $this->id
      ] );

      return;
    }

    wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG, [
      'namespace'           => $this->namespace,
      'event'               => $this->event,
      'started_at'          => $this->started_at,
      'completed_at'        => $this->completed_at,
      'started_arguments'   => json_encode( $this->started_arguments ),
      'completed_arguments' => json_encode( $this->completed_arguments ),
    ] );

    $this->id = wpep_wpdb()->insert_id;
  }

  private function _sync_from_db() {
    $entry_from_db = wpep_wpdb()->get_row( 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG . ' ' . wpep_sql_where( [
      'namespace'  => $this->namespace,
      'event'      => $this->event,
      'started_at' => $this->started_at
    ] ) );

    if( $entry_from_db === null )
      return;

    $this->id                  = $entry_from_db->id;
    $this->completed_at        = intval( $entry_from_db->completed_at );
    $this->started_arguments   = @json_decode( $entry_from_db->started_arguments, true );
    $this->completed_arguments = @json_decode( $entry_from_db->completed_arguments, true );

    if( !is_array( $this->started_arguments ) )
      $this->started_arguments = [];

    if( !is_array( $this->completed_arguments ) )
      $this->completed_arguments = [];
  }

}
