<?php

namespace WPEP;

use WPEP\Entity\SystemLog as WPEP_Entity_SystemLog;

/**
 * Class PlatformHealth
 * @package WPEP
 * @author Robert Rusu
 */
class PlatformHealth {

  protected static $instance;

  public static function instance() {
    if ( !isset( self::$instance ) )
      self::$instance = new self();

    return self::$instance;
  }

  /**
   * @var PlatformHealth\Maintenance
   */
  public $maintenance;
  /**
   * @var PlatformHealth\Report
   */
  public $report;
  /**
   * @var PlatformHealth\Service
   */
  public $service;

  public function setup() {
    $this->maintenance = new PlatformHealth\Maintenance();
    $this->report      = new PlatformHealth\Report();
    $this->service     = new PlatformHealth\Service();

    add_action( 'wpep_cron_data_system_daily_maintenance', [ $this, '_daily_maintenance' ] );
  }

  public function _daily_maintenance() {
    $system_log = WPEP_Entity_SystemLog::instance( 'cron', 'daily_maintenance', time() );

    $tracked_statuses = wpep_get_setting( 'data-system-daily-maintenance' );

    $system_log->started_arguments = [
      'service'         => [],
      'maintenance'     => [],
      'tracked_status'  => $tracked_statuses
    ];

    $system_log->save();

    $platform_health_helpers = [
      'service'     => wpep_platform_health()->service->get_report(),
      'maintenance' => wpep_platform_health()->maintenance->get_report()
    ];

    foreach( $platform_health_helpers as $platform_health_helper_key => $platform_health_helper_data ) {
      foreach( $platform_health_helper_data as $platform_health_helper_data_key => $platform_health_helper_datum ) {
        $system_log->started_arguments[ $platform_health_helper_key ][ $platform_health_helper_data_key ] = $platform_health_helper_datum[ 'status' ];

        if( !in_array( $platform_health_helper_datum[ 'status' ], $tracked_statuses ) || ( !isset( $platform_health_helper_datum[ 'operation' ] ) || !is_callable( $platform_health_helper_datum[ 'operation' ] ) ) ) {
          unset( $platform_health_helpers[ $platform_health_helper_key ][ $platform_health_helper_data_key ] );

          continue;
        }
      }

      if( empty( $platform_health_helpers[ $platform_health_helper_key ] ) )
        unset( $platform_health_helpers[ $platform_health_helper_key ] );
    }

    $system_log->completed_arguments = [];

    if( !empty( $platform_health_helpers ) ) {
      $system_log->save();

      foreach( $platform_health_helpers as $platform_health_helper_key => $platform_health_helper_data )
        foreach( $platform_health_helper_data as $platform_health_helper_data_key => $platform_health_helper_datum )
          $system_log->completed_arguments[ $platform_health_helper_key ][ $platform_health_helper_data_key ] = call_user_func( $platform_health_helper_datum[ 'operation' ] );

      wpep_platform_health()->service->reset_notification_count_transient();
    }

    $system_log->completed_at = time();
    $system_log->save();
  }

}