<?php

namespace WPEP;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\SystemLog as WPEP_Entity_SystemLog;

/**
 * Class Settings
 * @package WPEP
 * @author Robert Rusu
 */
class Settings {

  /**
   * @var Settings
   */
  protected static $_instance;

  /**
   * @return Settings
   */
  public static function instance() {
    if( self::$_instance === null )
      self::$_instance = new self();

    return self::$_instance;
  }

  /**
   * @var array
   */
  public $defaultOptions = [
    'disable-xmlrpc'  => 0,
    'enable-backward-compatibility-short-codes' => 1,
    'enable-student-supervisor'                 => 0,
    'enable-student-supervisor-multiple'        => 0,
    'enable-post-author-field'              => 0,
    'post-list-type-by-individual-author'   => 'all',
    'register-default-categories'           => 0,
    'data-system-daily-maintenance'         => [ 'danger' ],
    'data-system-enable-background-backups' => 0,
    'data-system-backups-cap'               => 7,
    'data-system-api-tracking'              => 0,
    'data-system-settings-tracking'         => 1,
    'configuration-setup-rules-flushed' => 1,
    'menu-position'               => 3,
    'disable-admin-bar-for-students' => 0,
    'opt-course-index'            => 0,
    'opt-course-heading'          => '',
    'opt-course-sub-heading'      => '',
    'opt-course-secondary-filter' => 'Filter By',
    'wpep-placeholder-image'      => WPEP_PLACEHOLDER_IMAGE,
    'wpep-theme'                  => 'legacy',
    'wpep-custom-color'           => '',
    'wpep-custom-color-secondary' => '',
    'wpep-custom-font-size'       => '',
    'wpep-custom-font-family'     => '',
    'wpep-custom-css-by-file'     => 1,
    'wpep-editor-setup'      => 'default',
    'lesson-wp-editor-preload'        => 1,
    'lesson-wp-editor-enabled-new'    => 1,
    'admin-library-conflict-select-2-status'     => 0,
    'admin-library-datetimepicker-simple-format' => 0,
    'admin-course-preset-sections'    => 2,
    'admin-course-preset-lessons'     => 2,
    'admin-course-save-alert'         => 1,
    'admin-editor-integration'        => 1,
    'user-delete-tag-email-marketing'       => '',
    'user-register-tag-email-marketing-map' => [],
    'user-auth-enable-registration'         => 1,
    'user-auth-status-level'                => 0,
    'user-auth-sidebar-marketing-status'   => 0,
    'user-auth-sidebar-marketing-content'  => '',
    'user-auth-form-instructions'         => '',
    'user-auth-field-login-identifier'    => "Username or Email Address",
    'user-auth-field-username'            => "Username",
    'user-auth-field-first-name'          => "First Name",
    'user-auth-field-last-name'           => "Last Name",
    'user-auth-field-location'            => "Location",
    'user-auth-field-website'             => "Website",
    'user-auth-field-sid'                 => "Student Identification",
    'user-auth-field-about-me'            => "About Me",
    'user-auth-field-email-address'       => "Email Address",
    'user-auth-field-password'            => "Password",
    'user-auth-field-password-confirm'    => 'Confirm Password',
    'user-auth-field-password-old'        => 'Existent Password',
    'user-auth-field-remember-me'         => "Remember me",
    'user-auth-open-tab-register'         => "Register Account",
    'user-auth-open-tab-reset-password'   => "Reset Password",
    'user-auth-open-tab-login'            => "I already have an Account",
    'user-auth-return-to-login'           => "Return to Login",
    'user-auth-operation-login'           => "Login",
    'user-auth-operation-register'        => "Register",
    'user-auth-operation-reset-password'  => "Reset Password",
    'user-auth-operation-express'         => "Request Magic Link",
    'user-auth-notification-please-register'        => "Register an account by completing the following fields.",
    'user-auth-notification-page-expired'           => "The page has expired, please check again in a few moments.",
    'user-auth-notification-login-details-required' => "Please enter your username and password",
    'user-auth-notification-password-incorrect'     => "The password you entered is incorrect",
    'user-auth-notification-logged-in'              => "You have been logged in successfully, please wait a moment",
    'user-auth-notification-check-email'            => "Check your email address, a login link has been emailed to you",
    'user-auth-notification-account-registered'     => "Your account has been created, we're logging you in, please wait a moment",
    'user-auth-notification-password-reset'          => "Please check your email address for instructions.",
    'user-auth-notification-already-logged-in'       => "You're already logged in, please wait a few moments.",
    'user-auth-notification-recovery-not-registered' => 'There is no user registered with that email address',
    'user-profile-default-profile-picture'  => WPEP_PROFILE_PLACEHOLDER_IMAGE,
    'user-profile-default-cover-picture'    => WPEP_PROFILE_COVER_PLACEHOLDER_IMAGE,
    'user-profile-cover-picture-status'     => 1,
    'user-profile-heading'                  => "User Profile : %s",
    'user-profile-change-picture-button'    => "Change",
    'user-profile-allow-tab-expanding'      => 0,
    'user-profile-privacy-setting-status'   => 0,
    'user-profile-privacy-setting-default'  => WPEP_USER_PROFILE_PRIVACY_TYPE_PRIVATE,
    'user-profile-privacy-setting-public'   => 'Public Profile',
    'user-profile-privacy-setting-private'  => 'Private Profile',
    'user-profile-privacy-setting-tooltip'  => 'Your Profile is Private, you can switch it to a Profile Public if you want to share it with your friends.',
    'user-profile-public-active'        => 0,
    'user-profile-public-slug'          => '',
    'user-profile-marketing-content-status'         => 0,
    'user-profile-marketing-content'                => '',
    'user-profile-email-notification-open-management'  => 'Manage Notifications',
    'user-profile-email-notification-management-title' => 'Notification Settings',
    'user-profile-email-notification-general'      => 'Platform Email Notifications',
    'user-profile-email-notification-weekly-recap' => 'Weekly Recap',
    'user-profile-update-button' => 'Save',
    'user-profile-change-password-status'         => 0,
    'user-profile-change-password-open-popup'     => 'Change Password',
    'user-profile-change-password-popup-title'    => 'Change your password',
    'user-profile-change-password-popup-button'   => 'Update Password',
    'user-profile-change-password-popup-cancel'   => 'Cancel',
    'user-profile-change-password-notification-wrong-existent'=> 'Your existent password is wrong',
    'user-profile-change-password-notification-not-match'     => 'Passwords do not match',
    'user-profile-change-password-notification-success'       => 'Your Password has been updated',
    'user-profile-student-supervisor-placeholder' => 'Choose your Supervisor',
    'user-profile-student-supervisor-allow-change'=> 0,
    'user-profile-tab-default-tabs-enabled'   => 1,
    'user-profile-tab-my-content-enabled'     => 0,
    'user-profile-tab-my-content'             => 'My Content',
    'user-profile-tab-author-content-enabled' => 0,
    'user-profile-tab-author-content'         => 'Content Created',
    'user-profile-tab-progressing-courses'    => 'Progressing',
    'user-profile-tab-completed-courses'      => 'Completed',
    'user-profile-tab-bookmarked-courses'      => 'My Bookmarks',
    'user-profile-tab-statistics-current-user' => 'Available Analytics',
    'user-profile-courses-per-row'            => WPEP_ITEMS_PER_ROW_DYNAMIC,
    'user-profile-course-public-available'  => 1,
    'user-profile-course-public-progress-bars'=> 0,
    'user-profile-course-public-email-address'=> 0,
    'user-profile-change-picture'             => 'Change Profile Picture',
    'user-profile-change-picture-cover'       => 'Change Cover Picture',
    'user-profile-enabled-social-networks'    => [
      'facebook', 'twitter', 'instagram', 'linkedin'
    ],
    'user-profile-tab-link-format'            => 'label',
    'user-profile-tab-ordering'               => [],
    'user-profile-custom-tabs-enabled'        => 0,
    'user-profile-custom-tabs'                => [],
    'user-profile-custom-tabs-position'       => 'right',
    'index-page-enable-pagination'            => 0,
    'index-page-entries-per-page'             => 12,
    'index-page-enable-filter-all-categories' => 0,
    'index-page-single-category-primary-view' => 0,
    'index-page-enforce-ordering'             => 0,
    'index-page-categories-per-row'           => 0,
    'index-page-category-presentation-labels' => 0,
    'index-page-filter-search'                => 'Search',
    'index-page-categories-filter-require-all'=> 1,
    'index-page-filter-all-categories-title'  => 'All Topics',
    'index-page-filter-all-categories-image'  => '',
    'layout-max-width'                        => 1024,
    'layout-background-color'                 => '',
    'layout-padding'                          => 'default',
    'layout-box-shadow'                       => 'default',
    'rewrite-slug-categories'                 => 'course_category',
    'enable-post-member-magnet-option'        => 0,
    'enable-post-member-magnet-interaction'   => 1,
    'post-member-magnet-type'                 => 'standard',
    'email-weekly-recap-status'                     => 0,
    'email-weekly-recap-title'                      => 'Your weekly recap is here : [date].',
    'email-weekly-recap-content'                    => 'Hello [user_name],
Hope you a great weekend!
[condition_has_activity]
Here is a Quick Summary of your stats from last week :
You have completed [number_lessons_completed] lessons.
[condition_has_completed_courses]
Recently Completed Courses
[list_courses_completed]
[/condition_has_completed_courses]
[condition_has_started_courses]
Courses You Started
[list_courses_started]
[/condition_has_started_courses]
[/condition_has_activity]
[condition_no_activity]

Just a quick reminder for you that you still have some great content waiting for you.

<a href="[site_url]">[site_url]</a>

[/condition_no_activity]

If you want to disable these emails, please update your profile settings, or contact an administrator.',
    'email-logo'                             => '',
    'email-company-name'                     => '',
    'email-company-address'                  => '',
    'email-copyright-text'                   => '',
    'email-from-name'                        => '',
    'email-from-email'                       => '',
    'email-tracking-salt'                    => '',
    'email-notification-disable-button'      => 'Disable Email Notifications',
    'email-notification-disable-title'       => 'Platform Wide Email Notifications Disabled',
    'email-notification-disable-content'     => 'Hello [user_name],
All Platform Email Notifications from [site_url] have been disable.
To change your Email Notification Preferences, check your User Profile.',
    'email-reset-password-override-filter'   => 0,
    'email-reset-password-use-template'      => 1,
    'email-reset-password-title'             => '[%s] Password Reset',
    'email-reset-password-content'           => 'Someone has requested a password reset for the following account:
    [site_url]
    Username : [username]
    If this was a mistake, just ignore this email and nothing will happen.
    To reset your password, visit the following address:
    <a href="[password_reset]">[password_reset]</a>
    
    If the link above is not clickable, copy paste the following link into your browser : 
    <[password_reset]>
    ',
    'email-express-authentication-login-title'   => '[%s] Magic Authentication Link',
    'email-express-authentication-login-content' => 'Someone has requested a login link for the following account:
    [site_url]
    Email Address : [email_address]
    If this was a mistake, just ignore this email and nothing will happen.
    The link is valid for 1 hour
    To login to your account, <a href="[login_link]">Click here</a>
    
    If the link above is not clickable, copy paste the following link into your browser : 
    <[login_link]>
    ',
    'email-express-authentication-register-title'   => '[%s] Magic Account Creation Link',
    'email-express-authentication-register-content' => 'Someone has requested a registration link for the following account:
    [site_url]
    Email Address : [email_address]
    Password : [password] ( this is an automatically generated password for your new account )
    If this was a mistake, just ignore this email and nothing will happen, the account is created only after clicking the link
    To register your new account, <a href="[register_link]">Click here</a>
    
    If the link above is not clickable, copy paste the following link into your browser : 
    <[register_link]>
    ',
    'search-results-number'     => 5,
    'search-no-results-found'   => 'No results found',
    'search-input-placeholder'  => 'Search',
    'search-image-ribbon'       => 1,
    'enable-sid-support'              => 0,
    'enable-auto-login-link'        => 0,
    'auto-login-link-token'         => 'wpep-login-token',
    'auto-login-link-salt'          => '',
    'user-content-embed-domain-verify'    => 0,
    'user-content-embed-domain-list'      => [],
    'user-content-embed-error-notification'      => '<p style="text-align:center;">This content is not available anymore.</p>',
    'user-content-embed-box-inside-embeds'       => 1,
    'user-content-embed-box-enabled'             => 0,
    'user-content-embed-box-course-status'       => 1,
    'user-content-embed-box-title'               => 'Share This Course!',
    'user-content-embed-box-description'         => '<p style="text-align: center;">Embed this course on your own website.</p>
                                                     <p style="text-align: center;">Simply Copy and Paste the embed code onto any Page.</p>',
    'user-content-embed-box-button'              => 'Copy Embed Code',
    'user-content-embed-box-button-clicked'      => 'Copied !',
    'comments-refresh-ms-interval'               => 10,
    'comments-search-label'                      => 'Search',
    'comments-filter-mine'                       => 'Show My Activity',
    'comments-per-page'                          => 5,
    'comment-replies-per-page'                   => 5,
    'comment-replies-show-more'                  => 'View %s more replies',
    'comments-direct-url-status'                 => 0,
    'comments-direct-url-slug'                   => 'wpep-discussion',
    'comments-direct-url-title'                  => 'Discussion : %s',
    'comments-direct-url-list-title'             => 'My Discussions',
    'comments-direct-url-my-discussions'         => 'My Discussions',
    'comments-direct-url-expand-my-discussions'  => 'View My Discussions',
    'comments-direct-url-show-more'              => 'Show more',
    'comments-direct-url-no-discussions'         => '[' . WPEP_SHORT_CODE_PREFIX . 'notification type="info"]You do not have any discussions at the moment.[/' . WPEP_SHORT_CODE_PREFIX . 'notification]',
    'comments-direct-url-menu-item-bubble'            => 0,
    'comments-direct-url-menu-item-bubble-item-click' => 1,
    'comments-direct-url-menu-item-notification-comment'          => '%s has written a comment in %s',
    'comments-direct-url-menu-item-notification-comment-reply'    => '%s has replied to your comment in %s',
    'comments-direct-url-menu-item-notification-comment-reply-on-reply' => '%s has replied to a comment you have replied to in %s',
    'comments-direct-url-menu-item-notification-comment-reaction'       => '%s has reacted to your comment in %s',
    'comments-direct-url-menu-item-notification-comment-mention'  => '%s has mentioned you in a comment in %s',
    'comments-reactions-status'                  => 0,
    'comments-reactions-default-reaction'        => "1F44D",
    'comments-reactions-available-emoji-list'    => [ "1F44D", "1F600", "1F60D", "1F644", "1F973" ],
    'comments-reactions-button-text'             => 'React',
    'comments-reactions-popup-all-reactions-text'=> 'All',
    'comments-reactions-popup-reactions-per-request' => 10,
    'comments-reactions-popup-view-profile'      => 'View Profile',
    'comments-reactions-popup-show-more'         => 'Show More',
    'comments-mentions-status'                   => 0,
    'comments-mentions-notification-title'       => '[mention_source_name] has mentioned you in [discussion_name]',
    'comments-mentions-notification-content'     => '<p>[mention_source_name] mentioned you in a comment in [discussion_name] at [discussion_comment_date_time]: </p>' .
                                                    '[mention_activity_preview]',
    'comments-notification-content-empty'        => 'You cannot send an empty comment.',
    'comments-authority-badge-name'              => 'Admin',
    'comments-button-reply'                      => 'Reply',
    'comments-button-cancel-reply'               => 'Cancel Reply',
    'comments-button-submit'                     => 'Add Post',
    'comments-button-reply-submit'               => 'Add Comment',
    'comments-button-delete'                     => 'Delete',
    'comments-button-confirm-delete'             => 'Confirm Delete',
    'comments-button-feature-comment'            => 'Feature Post',
    'comments-navigation-previous'               => 'Previous',
    'comments-navigation-next'                   => 'Next',
    'comments-user-list-status'                  => 0,
    'comments-user-list-title'                   => "Who's taking part in the conversation :",
    'comments-user-list-view-profile'            => 'View Profile',
    'comments-user-list-show-container'          => 'Show List',
    'comments-email-notification-title'          => "Here's what you've missed : [date]",
    'comments-email-notification-content'        => '<p>Hello [name],</p>
                                                     <p>Here is what has been happening across [discussion_activity_number] conversation(s) you are taking part of.</p> 
                                                     [discussion_activity_preview]',
    'comments-email-notification-button-view'    => 'View Discussion',
    'comments-email-notification-frequency'      => 900,
    'content-type-courses-rewrite-slug'          => 'courses',
    'content-type-courses-enabled'               => 1,
    'content-type-courses-enable-archive'        => 1,
    'content-type-assessment-rewrite-slug'       => 'wpep_assessment',
    'user-agreement-introduction-text'        => 'Before you can continue, you must confirm our Terms of Service & Privacy Policy',
    'user-agreement-button-accept'            => 'Accept',
    'user-agreement-button-decline'           => 'Decline',
    'user-agreement-button-decline-page'      => '',
    'user-agreement-notification-success'     => "Success ! Please wait while we're loading your content...",
    'user-service-agreement-enabled'          => 0,
    'user-service-agreement-title'            => 'I agree to the Terms of Service',
    'user-service-agreement-content'          => '',
    'user-service-agreement-error'            => 'You must agree to the Terms of Service',
    'user-service-agreement-last-change'      => '',
    'user-privacy-agreement-enabled'          => 0,
    'user-privacy-agreement-title'            => 'I agree to the Privacy Policy',
    'user-privacy-agreement-content'          => '',
    'user-privacy-agreement-error'            => 'You must agree to the Privacy Policy',
    'user-privacy-agreement-last-change'      => '',
    'user-account-delete-enabled'             => 0,
    'user-account-delete-button'              => 'Close Account',
    'user-account-delete-confirmation'        => 'This action cannot be undone, your account will be permanently deleted.',
    'user-account-delete-confirmation-button' => 'Delete Account',
    'user-account-delete-confirmation-cancel-button' => "Keep Account",
    'user-account-delete-success-page'               => '',
    'user-account-export-student-information'        => 'Student Information',
    'reporting-start-date'  => '',
    // api-response_%namespace%_%action%-data
    'api-response-scu-crd-status'   => 0,
    'api-response-scu-crd-sources'  => [],
  ];

  public $options;

  public $option_name = 'wpep_opts';

  public function __construct() {
    $this->refresh();
  }

  public function refresh() {
    $this->options = get_option( $this->option_name, $this->defaultOptions );
  }

  /**
   * @param $option_name
   * @return string
   */
  public function get_option_translated( $option_name ) {
    if( isset( $this->options[$option_name] ) )
      return $this->options[$option_name];

    if( isset( $this->defaultOptions[$option_name] ) )
      return __( $this->defaultOptions[$option_name], "wpep" );

    return $option_name;
  }

  /**
   * @param $option_name
   * @param string $default
   * @return array|mixed|string
   */
  public function get_option( $option_name, $default = '' ) {
    if( is_array( $option_name ) ) {
      $response = [];

      foreach( $option_name as $option )
        $response[ $option ] = $this->get_option( $option, $default );

      return $response;
    }

    if( isset( $this->options[$option_name] ) )
      return $this->options[$option_name];

    if( empty( $default ) && isset( $this->defaultOptions[$option_name] ) )
      return $this->defaultOptions[$option_name];

    return $default;
  }

  /**
   * @param array $check_setting
   * @return bool
   */
  public function has_changed_setting( $check_setting = [] ) {
    if( is_string( $check_setting ) )
      return ( !isset( $this->defaultOptions[ $check_setting ] ) ? true : $this->get_option( $check_setting ) != $this->defaultOptions[ $check_setting ] );

    foreach( $check_setting as $current_check )
      if( $this->has_changed_setting( $current_check ) )
        return true;

    return false;
  }

  /**
   * When the settings update is silent, it means it's an update or activation operation.
   * @param $options
   * @param bool $silent
   * @return bool
   */
  public function update_settings( $options, $silent = false ) {
    $flush_rules = false;

    if( intval( $this->get_option( 'data-system-settings-tracking' ) ) ) {
      $current_options = $this->options;

      $system_log = WPEP_Entity_SystemLog::instance( 'settings', ( $silent ? 'silent_update' : 'update' ), time() );
      $system_log->started_arguments = $options;
    }

    foreach( $options as $option_key => $option_value ) {
      if( wpep_string_ends_with( $option_key, '-rewrite-slug' ) ) {
        $option_value = sanitize_title( $option_value );
        $flush_rules  = true;
      } else if( in_array( $option_key, [ 'enable-course-assessments', 'rewrite-slug-categories', 'user-profile-email-notification-weekly-recap', 'comments-direct-url-status', 'comments-direct-url-slug' ] )
                  ||  strpos( $option_key, 'content-type-' ) === 0 ) {
        $flush_rules  = true;
      } else if( in_array( $option_key, [ 'user-service-agreement-last-change', 'user-privacy-agreement-last-change' ] ) ) {
        $option_value = ( $option_value == '' ? $option_value : date( 'Y-m-d', strtotime( $option_value ) ) );
      } else if( in_array( $option_key, [ 'user-content-embed-domain-list' ] ) ) {
        if( !is_array( $option_value ) ) {
          $option_value = [];
        } else {
          foreach( $option_value as $k => $v )
            $option_value[$k] = trim( $v, '/' );
        }
      } else if( in_array( $option_key, [ 'comments-reactions-available-emoji-list', 'data-system-daily-maintenance' ] ) ) {
        $option_value = array_diff( $option_value, [ 'a:0:{}', '' ] );
      } else if( $option_key === 'api-response-scu-crd-sources' ) {
        if( !is_array( $option_value ) ) {
          $option_value = [];
        } else {
          foreach( $option_value as $opt_val_key => $opt_val_val )
            $option_value[ $opt_val_key ] = array_diff( $opt_val_val, [ 'a:0:{}', '' ] );
        }
      }
       
      if( is_string( $option_value ) ) {
        $option_value = stripslashes( $option_value );
      } elseif( is_array( $option_value ) ) {
        foreach( $option_value as $opt_val_key => $opt_val_val )
          $option_value[$opt_val_key] = ( is_string( $opt_val_val ) ? stripslashes( $opt_val_val ) : $opt_val_val );
      }

      if( isset( $this->defaultOptions[ $option_key ] )
          && $this->defaultOptions[ $option_key ] == $option_value
          && strpos( $option_key, 'content-type-' ) !== 0 ) {
        unset( $this->options[$option_key] );
        continue;
      }

      $this->options[$option_key] = $option_value;
    }

    if( $flush_rules )
      $this->options['configuration-setup-rules-flushed'] = 0;

    if( isset( $system_log ) && $this->get_option( 'data-system-settings-tracking' ) ) {
      $system_log->completed_arguments = [];

      foreach( $options as $o_key => $o_v ) {
        if( isset( $this->options[ $o_key ] )
            && ( ( !isset( $current_options[ $o_key ] ) || $this->options[ $o_key ] != $current_options[ $o_key ] )
                  || ( isset( $this->defaultOptions[ $o_key ] ) && !isset( $this->options[ $o_key ] ) )
          ) ) {
          $system_log->completed_arguments[ $o_key ] = $this->options[ $o_key ];
        }
      }

      $system_log->completed_at = time();
      $system_log->save();
    }

    update_option( $this->option_name, $this->options );

    if( $silent )
      return true;

    if( $this->get_option( 'wpep-custom-css-by-file' )
        || isset( $options['wpep-custom-color'] )
        || isset( $options['wpep-custom-color-secondary'] )
        || isset( $options['wpep-custom-font-size'] )
        || isset( $options['wpep-custom-font-family'] )
        || isset( $options['index-page-ribbon-text-color'] )
        || isset( $options['index-page-ribbon-text-color-background'] )
    )
      wpep_refresh_custom_style_file( true );

    return true;
  }

  /**
   * @param array|string $options
   * @return bool
   */
  public function delete_setting( $options ) {
    if( !is_array( $options ) )
      $options = [ $options ];

    foreach( $options as $option_key )
      unset( $this->options[$option_key] );

    update_option( $this->option_name, $this->options );

    if( $this->get_option( 'wpep-custom-css-by-file' ) )
      wpep_refresh_custom_style_file( true );

    return true;
  }

  /**
   * @param array $post_order_list
   */
  public function update_post_order( $post_order_list ) {
    $i = 1; foreach( $post_order_list as $post_id )
      update_post_meta( $post_id, 'wpep_order', $i++ );
  }

  /**
   * @param array $taxonomy_order_list
   */
  public function update_taxonomy_order( $taxonomy_order_list ) {
    $i = 1; foreach( $taxonomy_order_list as $post_id )
      update_term_meta( $post_id, 'wpep_order', $i++ );
  }

  /**
   * @param array $addon_settings_map
   * @return bool
   */
  public function update_addon_settings( $addon_settings_map ) {
    if( !is_array( $addon_settings_map ) )
      return false;

    foreach( $addon_settings_map as $settings_alias => $settings_information ) {

      $settings_instance = WPEP_Controller::instance()->addOns->get_add_on_settings($settings_alias);

      if( $settings_instance == false )
        continue;

      $settings_instance->save_information( $settings_information );
    }

    return true;
  }

  /**
   * Returns options that have no defaults registered.
   * @return array
   */
  public function get_legacy_options() {
    $response = [];

    foreach( $this->options as $option_key => $option_value )
      if( !isset( $this->defaultOptions[ $option_key ] ) )
        $response[] = $option_key;

    return $response;
  }

}
