<?php

function wpep_administration_navigation(
  $navigation_items,
  $current_navigation_item,
  $navigation_base_url,
  $navigation_item_url_key = 'tab',
  $navigation_type     = 'primary',
  $navigation_alias    = 'settings',
  $navigation_tab_attribute_prefix = ''
) {
  $container_type  = ( $navigation_type === 'secondary' ? 'h3' : ( $navigation_type === 'tertiary' ? 'h4' : 'h2') );

  $response = '';
  $response .= '<' . $container_type . ' class="wpep-navigation wpep-navigation-' . $navigation_type . '" data-wpep-navigation="' . $navigation_alias . '">';

  foreach( $navigation_items as $navigation_item_alias => $navigation_item_name ){
    $class = ( $navigation_item_alias == $current_navigation_item ) ? ' wpep-navigation-item-active' : '';
    $response .= '<a data-wpep-tab="' . $navigation_tab_attribute_prefix . $navigation_item_alias . '"';
    $response .=   ' class="wpep-navigation-item' . $class . '"';
    $response .=   ' href="' . add_query_arg( $navigation_item_url_key, $navigation_item_alias, $navigation_base_url ) . '"';
    $response .= '>';
    $response .=    '<span>' . $navigation_item_name . '</span>';
    $response .= '</a>';

  }

  $response .= '</' . $container_type . '>';
  $response .= '<div class="wpep-clearfix"></div>';

  return $response;
}

if( !function_exists( 'wpep_admin_url_dashboard' ) ) {
  function wpep_admin_url_dashboard() {
    $url = admin_url( 'edit.php?post_type=' . WPEP_POST_TYPE_COURSE );

    return $url;
  }
}

if( !function_exists( 'wpep_admin_url' ) ) {
  function wpep_admin_url( $page, $params = [] ) {
    $url = admin_url( 'admin.php?page=' . $page );

    if( is_array( $params ) ) {
      foreach( $params as $key => $value )
        $url = add_query_arg( $key, $value, $url );

    } else {
      $url .= $params;
    }

    return $url;
  }
}

if( !function_exists( 'wpep_admin_url_statistics' ) ) {
  function wpep_admin_url_statistics( $params = [] ) {
    return apply_filters( 'wpep_administration_statistics_url', wpep_admin_url( 'wpep-statistics', $params ), $params );
  }
}

if( !function_exists( 'wpep_admin_url_statistics_content_type' ) ) {
  function wpep_admin_url_statistics_content_type( $content_type, $params = [] ) {
    return wpep_admin_url_statistics( [ 'tab' => $content_type ] + $params );
  }
}

if( !function_exists( 'wpep_admin_url_settings' ) ) {
  function wpep_admin_url_settings( $params = [] ) {
    return apply_filters( 'wpep_administration_settings_url', wpep_admin_url( 'wpep-settings', $params ), $params );
  }
}

if( !function_exists( 'wpep_admin_url_platform_health' ) ) {
  function wpep_admin_url_platform_health( $params = [] ) {
    return apply_filters( 'wpep_administration_platform_health_url', wpep_admin_url( 'wpep-platform-health', $params ), $params );
  }
}

if( !function_exists( 'wpep_admin_url_discussions_management' ) ) {
  function wpep_admin_url_discussions_management( $params = [] ) {
    return apply_filters( 'wpep_administration_discussions_management_url', wpep_admin_url( 'wpep-discussion-management', $params ), $params );
  }
}

if( !function_exists( 'wpep_admin_url_learning_center' ) ) {
  function wpep_admin_url_learning_center( $params = [] ) {
    return apply_filters( 'wpep_administration_learning_center_url', wpep_admin_url( 'wpep-learning-center', $params ), $params );
  }
}

if( !function_exists( 'wpep_admin_url_addons' ) ) {
  function wpep_admin_url_addons( $params = [] ) {
    return apply_filters( 'wpep_administration_addons_url', wpep_admin_url( 'wpep-addons', $params ), $params );
  }
}

if( !function_exists( 'wpep_admin_register_menu' ) ) {

  /**
   * @param $page_title
   * @param $menu_title
   * @param $capability
   * @param $menu_slug
   * @param $function
   * @param int $top_level_position
   */
  function wpep_admin_register_menu( $page_title, $menu_title, $capability, $menu_slug, $function, $top_level_position = 4 ) {
    if( WPEP_ADMIN_BASE_MENU_SLUG === true ) {
      add_menu_page(
        $page_title,
        $menu_title,
        $capability,
        $menu_slug,
        $function,
        '',
        $top_level_position
      );
    } else {
      add_submenu_page(
        WPEP_ADMIN_BASE_MENU_SLUG,
        $page_title,
        $menu_title,
        $capability,
        $menu_slug,
        $function
      );
    }
  }

}