<?php

function wpep_image_builder_imagettfbbox($size, $angle, $fontfile, $text) {
  $bbox = imagettfbbox($size, $angle, $fontfile, $text);

  //calculate x baseline
  if($bbox[0] >= -1) {
    $bbox['x'] = abs($bbox[0] + 1) * -1;
  } else {
    //$bbox['x'] = 0;
    $bbox['x'] = abs($bbox[0] + 2);
  }

  //calculate actual text width
  $bbox['width'] = abs($bbox[2] - $bbox[0]);
  if($bbox[0] < -1) {
    $bbox['width'] = abs($bbox[2]) + abs($bbox[0]) - 1;
  }

  //calculate y baseline
  $bbox['y'] = abs($bbox[5] + 1);

  //calculate actual text height
  $bbox['height'] = abs($bbox[7]) - abs($bbox[1]);
  if($bbox[3] > 0) {
    $bbox['height'] = abs($bbox[7] - $bbox[1]) - 1;
  }

  return $bbox;
}

function wpep_image_builder_get_font_list( $cached = true ) {
  $font_list = ( $cached ? get_transient( 'wpep_image_builder_font_list' ) : false );

  if( $font_list == false ) {
    $font_list = [];
    $font_path = wpep_image_builder_get_font_directory_path();

    if( $font_path === false )
      return apply_filters( 'wpep_image_builder_font_list', [] );

    $font_path = rtrim( $font_path, '/' );

    foreach( scandir( $font_path ) as $font_name ) {
      if(  strpos( $font_name, '.ttf' ) !== false ) {
        $font_list[ $font_name ] = str_replace( '.ttf', '', $font_name );
        $font_list[ $font_name ] = ucwords( str_replace( [ '_', '-' ], ' ', $font_list[ $font_name ] ) );
      }
    }

    set_transient( 'wpep_image_builder_font_list', $font_list, 24 * HOUR_IN_SECONDS );
  }

  return apply_filters( 'wpep_image_builder_font_list', $font_list );
}

/**
 * @return bool|string
 */
function wpep_image_builder_get_font_directory_path() {
  $upload_dir = rtrim( wpep_get_upload_dir(), DIRECTORY_SEPARATOR ) . '/ttf-font-library';

  if( is_dir( $upload_dir ) && is_readable( $upload_dir ) && ( count( scandir( $upload_dir ) ) > 2 ) )
    return $upload_dir . DIRECTORY_SEPARATOR;

  if( wpep_image_builder_import_font_directory_assets() === false )
    return false;

  return $upload_dir . DIRECTORY_SEPARATOR;
}

function wpep_image_builder_import_font_directory_assets( $resource_url = WPEP_RESOURCE_FONT_ZIP ) {
  $upload_dir = rtrim( wpep_get_upload_dir(), ( DIRECTORY_SEPARATOR === '/' ? '/' : '/' . DIRECTORY_SEPARATOR ) ) . DIRECTORY_SEPARATOR . 'ttf-font-library';

  if( !is_dir( $upload_dir ) )
    mkdir($upload_dir, 0755, true);

  try {
    $information = wp_remote_get( $resource_url, [ 'timeout' => 15, 'sslverify' => true ] );

    if( is_wp_error( $information ) ) {
      $information = file_get_contents( $resource_url );
    } else {
      $information = wp_remote_retrieve_body( $information );
    }

    file_put_contents($upload_dir . DIRECTORY_SEPARATOR . "temp.zip", $information );

    $zip = new ZipArchive();
    $res = $zip->open($upload_dir . DIRECTORY_SEPARATOR . "temp.zip" );

    if ($res === TRUE) {
      $zip->extractTo( $upload_dir );
      $zip->close();

      unlink( $upload_dir . DIRECTORY_SEPARATOR . "temp.zip" );

      return $upload_dir;
    } else {
      unlink( $upload_dir . DIRECTORY_SEPARATOR . "temp.zip" );

      return false;
    }
  } catch ( \Exception $e ) {
    if( is_file( $upload_dir . DIRECTORY_SEPARATOR . "temp.zip" ) )
      unlink( $upload_dir . DIRECTORY_SEPARATOR . "temp.zip" );

    if( count( scandir( $upload_dir ) ) <= 2 )
      unlink( $upload_dir );

    return false;
  }
}

/**
 * @param string $font_name
 * @return bool|mixed
 */
function wpep_image_builder_get_font_path( $font_name = 'Arial' ) {
  $font_name = ( wpep_string_ends_with( $font_name, '.ttf' ) ? str_replace( '.ttf', '', $font_name ) : $font_name );
  $font_file = sanitize_title( $font_name ) . '.ttf';
  $font_path = wpep_image_builder_get_font_directory_path();

  if( $font_path === false )
    return false;

  $font_path = $font_path . $font_file;

  if( !file_exists( $font_path ) )
    $font_path = wpep_image_builder_get_font_directory_path() . 'arial.ttf';

  return apply_filters( 'wpep_image_builder_font_path', $font_path , $font_file, $font_name );
}