<?php

use WPEP\Entity\Course as Entity_Course;

/**
 * @param $tag
 * @param array $options
 * @param string $label
 * @param bool $button
 * @return string
 */
function wpep_admin_shortcode_builder( $tag, $options = [], $label = 'Configure ShortCode', $button = true) {
  return '<a href="' . wpep_admin_shortcode_builder_url( $tag, $options ) . '" 
             data-wpep-interaction-trigger="shortcode-builder"
             class="' . ( $button ? ' wpep-button wpep-button-primary' : '' ) . '">' .
            $label .
         '</a>';
}

/**
 * @param $tag
 * @param array $options
 * @return string
 */
function wpep_admin_shortcode_builder_url(
  $tag,
  $options = []
) {
  $url = add_query_arg( 'wpep-global-operation', 'shortcode-builder', admin_url() );
  $url = add_query_arg( 'wpep-shortcode', $tag, $url );

  if( !empty( $options ) ) {
    $url = add_query_arg( 'wpep-sb-options', urlencode( json_encode( $options ) ), $url );
  }
  return $url;
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_index_options() {
  $category_list = _wpep_admin_shortcode_builder_category_select_field_list();

  $wpep_index_options = [
    'categories' => [
      'name'        => __( "Categories", "wpep" ),
      'type'        => 'multiple_select',
      'options'     => $category_list,
      'description' => __( "Display only certain categories within this Index Page", "wpep" )
    ],
    'categories_per_row' => [
      'name'    => __( "Categories Per Row", "wpep" ),
      'type'    => 'select',
      'options' => [ '' => __( "Default", "wpep" ) ] + wpep_content_categories_row_options()
    ],
    'all_categories_filter' => [
      'name'    => __( "All Categories Filter", "wpep" ),
      'type'    => 'select',
      'options' => [
          ''  => sprintf( __( "Default : %s" ), wpep_get_setting_flag( 'index-page-enable-filter-all-categories' ) ? __( 'Enabled', "wpep" ) : __( 'Disabled', "wpep" ) )
        ] + wpep_flag_setting_field_options()
    ],
    'single_category_view' => [
      'name'    => __( "Single Category View", "wpep" ),
      'type'    => 'select',
      'options' => [
        ''  => sprintf( __( "Default : Show %s category navigation" ), wpep_get_setting_flag( 'index-page-single-category-primary-view' ) ? 'primary' : 'secondary' ),
        0   => __( "Show secondary category navigation" ),
        1   => __( "Show primary category navigation" )
      ]
    ],
    'display_header' => [
      'name'    => __( "Show Header", "wpep" ),
      'type'    => 'select',
      'options' => [
        '1' => 'Yes',
        '0' => 'No'
      ]
    ],
    'link_mode'  => [
      'name'    => __( "Content Links", "wpep" ),
      'type'    => 'select',
      'options' => [
        'default' => __( "Default", "wpep" ),
        'new_tab' => __( "New Tab", "wpep" )
      ]
    ],
    'order' => [
      'name'    => __( "Order", "wpep" ),
      'type'    => 'select',
      'options' => wpep_content_primary_order_options()
    ],
    'order_by' => [
      'name'    => __( "Order By", "wpep" ),
      'type'    => 'select',
      'options' => wpep_content_primary_order_by_options()
    ],
  ];

  if( wpep_controller()->contentLibrary->is_integrated() ) {
    $wpep_index_options[ 'display_restriction' ] = [
      'name'    => __( "Display Restriction", "wpep" ),
      'type'    => 'select',
      'options' => [
        'none'    =>  __( "Standard", "wpep" ),
        'access'  =>  __( "Require Access", "wpep" )
      ],
      'description' => __( 'Selecting the "Require Access" option will make it mandatory for student to have access, this functionality may be provided already by your membership plugin.', "wpep" )
    ];
  }

  foreach( wpep_content_type()->libraryContentTypes as $libraryContentType )
    $wpep_index_options += $libraryContentType->get_grid_content_options();

  $wpep_index_options['pagination_per_page'] = [
    'name'        => __( "Pagination Per Page", "wpep" ),
    'type'        => 'text',
    'placeholder' => ( wpep_get_setting_flag( 'index-page-enable-pagination' ) ? sprintf( __( "Using Settings Rule : %s", "wpep" ), wpep_get_setting( 'index-page-entries-per-page' ) ) : __( "Pagination is disabled from settings" ) ),
    'description' => __( "Set to 0 to disable pagination, leave empty for default behaviour.", "wpep" )
  ];

  $wpep_index_options[ 'layout' ] = [
    'name'        => __( "Layout", "wpep" ),
    'type'    => 'select',
    'options' => [
      WPEP_INDEX_PAGE_LAYOUT_TOP_NAVIGATION   => __( "Top Navigation", "wpep" ),
      WPEP_INDEX_PAGE_LAYOUT_LEFT_NAVIGATION  => __( "Left Sidebar Navigation", "wpep" ),
      WPEP_INDEX_PAGE_LAYOUT_RIGHT_NAVIGATION => __( "Right Sidebar Navigation", "wpep" )
    ],
    'description' => __( "Advanced Option : Choose a different Index Page Layout.", "wpep" )
  ];

  $wpep_index_options[ 'layout-category' ] = [
    'name'        => __( "Layout Category", "wpep" ),
    'type'    => 'select',
    'options' => [
      0 =>  __( "Top Level", "wpep" )
    ] + $category_list,
    'description' => __( "Advanced Option : Require all the Content to be part of a Category, but don't offer the filters from it.", "wpep" )
  ];

  return $wpep_index_options;
}

function _wpep_admin_shortcode_builder_category_select_field_list() {
  $category_list = wpep_utility_map_to_array_assoc(
    get_terms( [ 'taxonomy' => WPEP_TAXONOMY_CATEGORY, 'hide_empty' => false, 'parent' => 0 ] ),
    'term_id',
    'name'
  );

  foreach( $category_list as $category_id => $category_name ) {
    $sub_category_list = wpep_utility_map_to_array_assoc(
      get_terms( [ 'taxonomy'   => WPEP_TAXONOMY_CATEGORY, 'hide_empty' => false, 'parent' => $category_id ] ),
      'term_id',
      'name'
    );

    if( empty( $sub_category_list ) )
      continue;

    foreach( $sub_category_list as $k => $v )
      $sub_category_list[ $k ] = ' - ' . $v;

    $category_list[ $category_id ] = [
      'group_title'   => $category_name,
      'group_options' => [ $category_id => $category_name] + $sub_category_list
    ];
  }

  return $category_list;
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_course_grid_options() {
  return [
    'courses'         => [
      'name'    => __( "Courses", "wpep" ),
      'type'    => 'multiple_select',
      'options' => [],
      'description' => __( "Leaving this empty will display all the courses.", "wpep" ),
      'attributes'    => [
        'data-ec-library'      => 'select2',
        "data-ec-library-mode" => 'post-search-map|post_type:' . WPEP_POST_TYPE_COURSE
      ]
    ],
    'container_class' => [
      'name'    => __('Container', "wpep"),
      'type'    => 'select',
      'options' => [
        ''                                    => __( 'Default', "wpep" ),
        'wpep-3d-carousel-container-shadowed' => __( 'Box Shadow on Sides', "wpep" )
      ]
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_user_directory_options() {
  return [
    'user_roles'        => [
      'name'          => __( "User Roles", "wpep" ),
      'type'          => 'multiple_select',
      'options'       => wpep_get_user_role_list_array_assoc(),
      'description'   => __( "Leaving this empty will display all the user roles.", "wpep" ),
    ],
    'users_per_page'    => [
      'name'          => __( "Users Per Page", "wpep" ),
      'type'          => 'text',
      'placeholder'   => WPEP_USER_DIRECTORY_PER_PAGE
    ],
    'has_search_bar'    => [
      'name'          => __( "Has Search", "wpep" ),
      'type'          => 'checkbox',
      'is_checked'    => WPEP_USER_DIRECTORY_HAS_SEARCH_BAR
    ],
    'search_users_text' => [
      'name'        => __( 'Search Text', "wpep" ),
      'type'        => 'text',
      'placeholder' => __( 'Search User Directory', "wpep" )
    ],
    'view_profile_text' => [
      'name'        => __( 'View Profile', "wpep" ),
      'type'        => 'text',
      'placeholder' => __( 'View Profile', "wpep" ),
      'description' => __( "A View Profile button will show up for Users that have public profiles, staff users will see the button for private profiles too.", "wpep" )
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_course_options() {
  $options = [];

  foreach( wpep_course_layout_types() as $course_type => $course_type_label ) {
    $group_options = wpep_get_post_list_assoc( WPEP_POST_TYPE_COURSE, 'ID', 'post_title', 'post_title', [
      [
        'key'   => WPEP_COURSE_TYPE_POST_META,
        'value' => $course_type,
      ]
    ] );

    if( empty( $group_options ) )
      continue;

    $options[] = [
      'group_title'   => $course_type_label,
      'group_options' => $group_options
    ]; ;
  }

  return [
    'id'          => [
      'name'    => __('Course', "wpep"),
      'type'    => 'searchable_select',
      'options' => $options
    ],
    'show_header' => [
      'name'        => __( "Show Header", "wpep" ),
      'type'        => 'checkbox',
      'is_checked'  => 1
    ],
    'read_only'   => [
      'name'    => __( 'Read Only', "wpep"),
      'type'    => 'select',
      'options' => [
        ''    => __( 'Course Default', "wpep" ),
        '0'   => __( 'No', "wpep" ),
        '1'   => __( 'Yes', "wpep" )
      ]
    ],
    'course_type' => [
      'name'    => __('Course Type', "wpep"),
      'type'    => 'select',
      'options' => [
          "" => __( "Option Selected in Course Settings", "wpep" )
        ] + wpep_course_layout_types()
    ]
  ];
}

function wpep_admin_shortcode_builder_wpep_assessment_options() {
  $options = [];

  foreach( wpep_utility_map_to_array_assoc(
             get_posts( [
               'post_type'   => WPEP_POST_TYPE_ASSESSMENT,
               'numberposts' => -1,
               'orderby'     => 'post_title',
               'order'       => 'ASC'
             ] ),
             'ID',
             'post_title'
           ) as $assessment_id => $assessment_title ) {
    $course_id = wpep_get_assessment_course_id( $assessment_id );

    if( !isset( $options[$course_id] ) ) {
      $options[$course_id] = [
        'group_title'   => get_the_title( $course_id ),
        'group_options' => []
      ];
    }

    $options[$course_id]['group_options'][ $assessment_id ] = $assessment_title;
  }

  return [
    'id'          => [
      'name'    => __('Assessment', "wpep"),
      'type'    => 'searchable_select',
      'options' => $options
    ]
  ];
}

function wpep_admin_shortcode_builder_wpep_course_preview_options( $builder_base_options = [] ) {
  if( !isset( $builder_base_options[ 'id' ] ) )
    return [];

  $course_map = Entity_Course::instance()->get_with_lessons( $builder_base_options['id'] );
  $available_lessons_options = [];

  foreach( $course_map as $section_information ) {
    $available_lessons_options[] = [
      'group_title'   => $section_information->title,
      'group_options' => wpep_utility_map_to_array_assoc( $section_information->lessons, 'id', 'title' )
    ]; ;
  }

  return [
    'show_header' => [
      'name'        => __( "Show Header", "wpep" ),
      'type'        => 'checkbox',
      'is_checked'  => 1
    ],
    'course_lessons' => [
      'name'    => __('Available Lessons', "wpep"),
      'type'    => 'multiple_select',
      'options' => $available_lessons_options,
      'desc'    => __( "Select each lesson that will be available under this Course Preview", "wpep" )
    ],
    'course_type'       => [
      'name'    => __('Course Type', "wpep"),
      'type'    => 'select',
      'options' => wpep_course_layout_types( [
          WPEP_COURSE_TYPE_EXECUTION_PLAN,
          WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED,
          WPEP_COURSE_TYPE_LEARN_MAP,
          WPEP_COURSE_TYPE_STANDARD,
          WPEP_COURSE_TYPE_FOCUS,
          WPEP_COURSE_TYPE_VIDEO,
          WPEP_COURSE_TYPE_CHALLENGE
      ] )
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_link_options() {
  return [
    'content' => [
      'name'        => __( "Link", "wpep" ),
      'type'        => 'shortcode_content'
    ],
    'text'  => [
      'name'    => __('Button Text', "wpep"),
      'type'    => 'text'
    ],
    'background_color' => [
      'name'    => __('Button Color', "wpep"),
      'type'    => 'color'
    ],
    'color' => [
      'name'    => __('Text Color', "wpep"),
      'type'    => 'color'
    ],
    'position' => [
      'name'    => __( "Alignment", "wpep" ),
      'type'    => 'select',
      'options' => [
        'left'    => __( "Left", "wpep" ),
        'center'  => __( "Center", "wpep" ),
        'right'   => __( "Right", "wpep" )
      ]
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_notification_options() {
  return [
    'content' => [
      'name'        => __( "Content", "wpep" ),
      'type'        => 'shortcode_content'
    ],
    'type' => [
      'name'    => __( 'Type', "wpep" ),
      'type'    => 'select',
      'options' => [
        "standard" => __( "Standard", "wpep" ),
        "info"     => __( "Informative", "wpep" ),
        "success"  => __( "Success", "wpep" ),
        "warning"  => __( "Warning", "wpep" ),
        "danger"   => __( "Danger", "wpep" ),
      ]
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_course_ids_selection_options() {
  return [
    'content' => [
      'name'        => __( "Content", "wpep" ),
      'type'        => 'shortcode_content',
      'default'     => 'You can add any content here.'
    ],
    'id'          => [
      'name'    => __('Course', "wpep"),
      'type'    => 'multiple_select',
      'options' => wpep_get_post_list_assoc( WPEP_POST_TYPE_COURSE )
    ]
  ];
}

function wpep_admin_shortcode_builder_wpep_search_advanced_options() {
  $category_list = wpep_utility_map_to_array_assoc(
    get_terms( [ 'taxonomy'   => 'course_category', 'hide_empty' => true, 'parent' => 0 ] ),
    'term_id',
    'name'
  );

  return [
    'post_type'         => [
      'name'        => __( "Post Type", "wpep" ),
      'type'        => 'multiple_select',
      'options'     => wpep_content_type()->library_post_type_list(),
      'description' => __( "Display only certain Post Types", "wpep" )
    ],
    'include_categories' => [
      'name'        => __( "Include Categories", "wpep" ),
      'type'        => 'multiple_select',
      'options'     => $category_list,
      'description' => __( "Include only these Categories in this search", "wpep" )
    ],
    'exclude_categories' => [
      'name'        => __( "Exclude Categories", "wpep" ),
      'type'        => 'multiple_select',
      'options'     => $category_list,
      'description' => __( "Exclude these Categories in this search", "wpep" )
    ],

  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_lesson_content_options() {
  $options = [];
  $courses = wpep_get_post_list_assoc( WPEP_POST_TYPE_COURSE );

  $options[0] = __( "Please Select...", "wpep" );

  foreach( $courses as $course_id => $course_title ) {
    if( $course_title === '' )
      continue;

    $options[ $course_id ] = [
      'group_title'   => $course_title,
      'group_options' => Entity_Course::instance()->get_lesson_title_map( $course_id )
    ];
  }

  return [
    'lesson' => [
      'name'        => __( "Lesson", "wpep" ),
      'type'    => 'searchable_select',
      'options' => $options
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_assessment_retake_options() {
  return [
    'assessment_id' => [
      'name'    => __( "Assessment", "wpep" ),
      'type'    => 'searchable_select',
      'options' => [ 0 => __( "Please Select...", "wpep" ) ] + wpep_get_post_list_assoc( WPEP_POST_TYPE_ASSESSMENT )
    ],
    'button_retake' => [
      'name'        => __( "Retake Assessment", "wpep" ),
      'type'        => 'text',
      'placeholder' => wpep_get_setting( 'assessment-retake' )
    ],
    'button_take' => [
      'name'        => __( "Take Assessment", "wpep" ),
      'type'        => 'text',
      'placeholder' => wpep_get_setting( 'assessment-view' )
    ],
    'button_background_color' => [
      'name'    => __('Button Color', "wpep"),
      'type'    => 'color'
    ],
    'button_color' => [
      'name'    => __('Text Color', "wpep"),
      'type'    => 'color'
    ],
    'button_position' => [
      'name'    => __( "Alignment", "wpep" ),
      'type'    => 'select',
      'options' => [
        'left'    => __( "Left", "wpep" ),
        'center'  => __( "Center", "wpep" ),
        'right'   => __( "Right", "wpep" )
      ]
    ]
  ];
}

function wpep_admin_shortcode_builder_wpep_assessment_retake_link_options() {
  return [
    'assessment_link' => [
      'name'        => __( "Link", "wpep" ),
      'type'        => 'text',
      'description' => '<p>' . __( 'Advanced : Link to the Assessment, this will reset the progress for all Assessments if there are more on the same page', "wpep" ) . '</p>' .
                       '<p>' . __( "If you're not using Custom Pages for your Assessments, the recommended way is to use the Assessment Retake shortcode instead of link", "wpep" ) . '</p>'
    ]
  ];
}

function wpep_admin_shortcode_builder_wpep_profile_authentication_options() {
  $options = [
    'refresh'  => __( "Default - Refresh Page", "wpep" ),
    'redirect' => __( "Redirect", "wpep" )
  ];

  if( wpep_public_profiles_enabled() )
    $options[ 'redirect-to-public-profile' ] = __( "Redirect to Public Profile", "wpep" );

  return [
    'default_view' => [
      'name'        => __( "Open View", "wpep" ),
      'type'        => 'select',
      'options'     => [
        ''               => __( "Default", "wpep" ),
        'login'          => __( 'Login', "wpep" ),
        'register'       => __( 'Register', "wpep" ),
        'express'        => __( "Express - Login & Register", "wpep" ),
        'reset_password' => __( 'Reset Password', "wpep" ),
      ]
    ],
    'success_type' => [
      'name'        => __( "Success Type", "wpep" ),
      'type'        => 'select',
      'options'     => $options,
      'description' => __( "What happens after the user is logged in ? This also includes registration.", "wpep" ),
      'placeholder' => __( "Leave this empty to refresh the page instead", "wpep" )
    ],
    'url'          => [
      'name'        => __( "URL", "wpep" ),
      'type'        => 'text',
      'placeholder' => __( "The Page Link where you want to redirect", "wpep" )
    ],
    'lead_segment' => [
      'name'        => __( "Lead Segment", "wpep" ),
      'type'        => 'text',
      'placeholder' => 'This field is optional',
      'description' => __( "Assign a segment to this lead, this will be tracked in the Statistics Panel & Dashboard Report.", "wpep" )
    ],
    'hide_registration' => [
      'name'        => __( "Hide Registration", "wpep" ),
      'type'        => 'checkbox',
      'description' => __( "This option will only hide the registration on this embed, not disable the application code that handles the registration.", "wpep" )
    ],
  ];
}

function wpep_admin_shortcode_builder_wpep_email_marketing_tag_add_or_remove() {
  if( !wpep_controller()->emailMarketing->has_text_based_tag_integrations() )
    return [];

  $options = [
    ''  => __( "All", "wpep" ),
  ];

  foreach( wpep_controller()->emailMarketing->get_text_based_tag_integrations() as $text_based_tag_integration )
    $options[ $text_based_tag_integration->get_alias() ] = $text_based_tag_integration->get_name();

  return [
    'tag'          => [
      'name'        => __( "Tag", "wpep" ),
      'type'        => 'text'
    ],
    'integration' => [
      'name'        => __( "Service Integration", "wpep" ),
      'type'        => 'select',
      'options'     => $options,
    ],
  ];
}