<?php

/**
 * @return array
 */
function wpep_toolkit_list_map() {
  $wpep_search_section_items = [
    WPEP_SHORT_CODE_PREFIX . 'search_content' => [
      'type'        => 'plain',
      'name'        => __( "Content", "wpep" ),
      'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'search]'
    ]
  ];

  foreach( wpep_content_type()->libraryContentTypes as $libraryContentType ) {
    $wpep_search_section_items[ WPEP_SHORT_CODE_PREFIX . 'search_' . $libraryContentType->get_post_type() ] = [
      'type'        => 'plain',
      'name'        => $libraryContentType->get_plural_name(),
      'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'search post_type="' . $libraryContentType->get_post_type() . '"]'
    ];
  }

  $wpep_search_section_items[ WPEP_SHORT_CODE_PREFIX . 'search_advanced' ] = [
    'type'        => 'builder',
    'name'        => __( "Advanced Settings", "wpep" ),
    'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'search', [ 'wpep_admin_shortcode_builder_wpep_search_advanced_options' ] )
  ];

  $response = [];

  $response[  WPEP_SHORT_CODE_PREFIX . 'elements' ] = [
    'type'        => 'menu',
    'name'        => __( "Elements", "wpep" ),
    'items'       => [
      WPEP_SHORT_CODE_PREFIX . 'notification' => [
        'type'        => 'builder',
        'name'        => __( "Notification", "wpep" ),
        'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'notification', [ 'wpep_admin_shortcode_builder_wpep_notification_options' ], 0 )
      ],
      WPEP_SHORT_CODE_PREFIX . 'link'         => [
        'type'        => 'builder',
        'name'        => __( "Link Button", "wpep" ),
        'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'link', [ 'wpep_admin_shortcode_builder_wpep_link_options' ], 0 )
      ],
      WPEP_SHORT_CODE_PREFIX . 'video' => [
        'type'        => 'plain',
        'name'        => __( "Video", "wpep" ),
        'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'video]',
        'placeholder' => __( "video-url-goes-here", "wpep" ),
        'after'       => '[/' . WPEP_SHORT_CODE_PREFIX . 'video]'
      ],
      WPEP_SHORT_CODE_PREFIX . 'hero_section' => [
        'type'        => 'plain',
        'name'        => __( "Hero Section", "wpep" ),
        'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'hero_section]',
        'placeholder' => __( "Hero Section Content", "wpep" ),
        'after'       => '[/' . WPEP_SHORT_CODE_PREFIX . 'hero_section]'
      ]
    ],
  ];

  $response[  WPEP_SHORT_CODE_PREFIX . 'index' ] = [
    'type'        => 'builder',
    'name'        => __( "Index Page", "wpep" ),
    'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'index', [ 'wpep_admin_shortcode_builder_wpep_index_options' ] )
  ];

  if( wpep_content_type()->is_enabled( WPEP_POST_TYPE_COURSE ) ) {
    $response[  WPEP_SHORT_CODE_PREFIX . 'user_course_list' ] = [
      'type'        => 'menu',
      'name'        => __( "Advanced Index Pages", "wpep" ),
      'items'       => [
        WPEP_SHORT_CODE_PREFIX . 'index_courses_recommended' => [
          'type'        => 'plain',
          'name'        => __( "User Courses Recommended", "wpep" ),
          'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'index_courses_recommended]',
        ],
        WPEP_SHORT_CODE_PREFIX . 'index_courses_progressing' => [
          'type'        => 'plain',
          'name'        => __( "User Courses Progressing", "wpep" ),
          'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'index_courses_progressing]',
        ],
        WPEP_SHORT_CODE_PREFIX . 'index_courses_completed' => [
          'type'        => 'plain',
          'name'        => __( "User Courses Completed", "wpep" ),
          'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'index_courses_completed]',
        ]
      ]
    ];

    if( wpep_course_has_bookmarks() )
      $response[  WPEP_SHORT_CODE_PREFIX . 'user_course_list' ][ 'items' ][ WPEP_SHORT_CODE_PREFIX . 'index_courses_bookmarked' ] = [
        'type'        => 'plain',
        'name'        => __( "User Courses Bookmarked", "wpep" ),
        'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'index_courses_bookmarked]',
      ];

    $response[ WPEP_SHORT_CODE_PREFIX . 'course' ] = [
      'type'        => 'builder',
      'name'        => __( "Course Embed", "wpep" ),
      'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'course', [ 'wpep_admin_shortcode_builder_wpep_course_options' ] )
    ];
  }

  if( wpep_content_type()->is_enabled( WPEP_POST_TYPE_ASSESSMENT ) )
    $response[ WPEP_SHORT_CODE_PREFIX . 'assessment' ] = [
      'type'        => 'builder',
      'name'        => __( "Assessment Embed", "wpep" ),
      'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'assessment', [ 'wpep_admin_shortcode_builder_wpep_assessment_options' ] )
    ];

  if( wpep_content_type()->is_enabled( WPEP_POST_TYPE_COURSE ) )
    $response[ WPEP_SHORT_CODE_PREFIX . 'course_grid' ] = [
      'type'        => 'builder',
      'name'        => __( "Course 3D Grid", "wpep" ),
      'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'course_grid', [ 'wpep_admin_shortcode_builder_wpep_course_grid_options' ] )
    ];

  $response[ WPEP_SHORT_CODE_PREFIX . 'profile' ] = [
    'type'        => 'builder',
    'name'        => __( "User Profile Page", "wpep" ),
    'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'profile', [] )
  ];

  $response[ WPEP_SHORT_CODE_PREFIX . 'user_directory' ] = [
    'type'        => 'builder',
    'name'        => __( "User Directory", "wpep" ),
    'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'user_directory', [ 'wpep_admin_shortcode_builder_wpep_user_directory_options' ] )
  ];

  if( wpep_controller()->emailMarketing->has_text_based_tag_integrations() ) {
    $response[ WPEP_SHORT_CODE_PREFIX . 'email_marketing' ] = [
      'type'  => 'menu',
      'name'  => __( "Email Marketing", "wpep" ),
      'items' => [
        WPEP_SHORT_CODE_PREFIX . 'email_marketing_tag_add'        => [
          'type'        => 'builder',
          'name'        => __( "Add Tag", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'email_marketing_tag_add', [ 'wpep_admin_shortcode_builder_wpep_email_marketing_tag_add_or_remove' ], 0 )
        ],
        WPEP_SHORT_CODE_PREFIX . 'email_marketing_tag_remove'     => [
          'type'        => 'builder',
          'name'        => __( "Remove Tag", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'email_marketing_tag_remove', [ 'wpep_admin_shortcode_builder_wpep_email_marketing_tag_add_or_remove' ], 0 )
        ]
      ]
    ];
  }

  $response += [
    WPEP_SHORT_CODE_PREFIX . 'search_section' => [
      'type'        => 'menu',
      'name'        => 'Search Bar',
      'items'       => $wpep_search_section_items
    ],
  ];

  $response[ WPEP_SHORT_CODE_PREFIX . 'misc' ] = [
    'type'  => 'menu',
    'name'  => __( "Miscellaneous", "wpep" ),
    'items' => [
      WPEP_SHORT_CODE_PREFIX . 'center'            => [
        'type'        => 'plain',
        'name'        => __( "Content Center", "wpep" ),
        'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'center]',
        'placeholder' => __( "Content that will be centered", "wpep" ),
        'after'       => '[/' . WPEP_SHORT_CODE_PREFIX . 'center]'
      ],
      WPEP_SHORT_CODE_PREFIX . 'profile_authentication'     => [
        'type'        => 'builder',
        'name'        => __( "Authentication Form", "wpep" ),
        'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'profile_authentication', [ 'wpep_admin_shortcode_builder_wpep_profile_authentication_options' ] )
      ],
      WPEP_SHORT_CODE_PREFIX . 'lesson_content' => [
        'type'        => 'builder',
        'name'        => __( "Lesson Content", "wpep" ),
        'url'         => wpep_toolkit_list_item_url( WPEP_SHORT_CODE_PREFIX . 'lesson_content', [ 'wpep_admin_shortcode_builder_wpep_lesson_content_options' ], 0 )
      ],
      WPEP_SHORT_CODE_PREFIX . 'simple_redirect' => [
        'type'        => 'plain',
        'name'        => __( "Simple Redirect", "wpep" ),
        'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'simple_redirect]',
        'placeholder' => __( "replace-with-your-url", "wpep" ),
        'after'       => '[/' . WPEP_SHORT_CODE_PREFIX . 'simple_redirect]'
      ],
      WPEP_SHORT_CODE_PREFIX . 'user_is_logged_in' => [
        'type'        => 'plain',
        'name'        => __( "User Logged in", "wpep" ),
        'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'user_is_logged_in]',
        'placeholder' => __( "Content visible for logged in users", "wpep" ),
        'after'       => '[/' . WPEP_SHORT_CODE_PREFIX . 'user_is_logged_in]'
      ],
      WPEP_SHORT_CODE_PREFIX . 'user_not_logged_in' => [
        'type'        => 'plain',
        'name'        => __( "User Logged Out", "wpep" ),
        'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'user_not_logged_in]',
        'placeholder' => __( "Content visible for logged out users", "wpep" ),
        'after'       => '[/' . WPEP_SHORT_CODE_PREFIX . 'user_not_logged_in]'
      ],
      WPEP_SHORT_CODE_PREFIX . 'user_name' => [
        'type'        => 'plain',
        'name'        => __( "User Full Name", "wpep" ),
        'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'user_name]'
      ],
      WPEP_SHORT_CODE_PREFIX . 'user_first_name' => [
        'type'        => 'plain',
        'name'        => __( "User First Name", "wpep" ),
        'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'user_first_name]'
      ],
      WPEP_SHORT_CODE_PREFIX . 'user_has_course_recommendation' => [
        'type'        => 'plain',
        'name'        => __( "User Has Course Recommendation", "wpep" ),
        'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'user_has_course_recommendation]',
        'placeholder' => sprintf( __( "Continue where you left off, %s", "wpep" ), '[' . WPEP_SHORT_CODE_PREFIX . 'user_course_recommendation_link][' . WPEP_SHORT_CODE_PREFIX . 'user_course_recommendation_title][/' . WPEP_SHORT_CODE_PREFIX . 'user_course_recommendation_link]'),
        'after'       => '[/' . WPEP_SHORT_CODE_PREFIX . 'user_has_course_recommendation]'
      ],
      WPEP_SHORT_CODE_PREFIX . 'user_course_recommendation_title' => [
        'type'        => 'plain',
        'name'        => __( "User Course Recommendation Title", "wpep" ),
        'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'user_course_recommendation_title]'
      ],
      WPEP_SHORT_CODE_PREFIX . 'user_course_recommendation_link' => [
        'type'        => 'plain',
        'name'        => __( "User Course Recommendation Link", "wpep" ),
        'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'user_course_recommendation_link]'
      ],
      WPEP_SHORT_CODE_PREFIX . 'course_is_completed' => [
        'type'        => 'builder',
        'name'        => __( "Course(s) Is Completed", "wpep" ),
        'url'         => wpep_toolkit_list_item_url(WPEP_SHORT_CODE_PREFIX . 'course_is_completed', [ 'wpep_admin_shortcode_builder_wpep_course_ids_selection_options' ] )
      ],
      WPEP_SHORT_CODE_PREFIX . 'course_not_completed' => [
        'type'        => 'builder',
        'name'        => __( "Course(s) Not Completed", "wpep" ),
        'url'         => wpep_toolkit_list_item_url(WPEP_SHORT_CODE_PREFIX . 'course_not_completed', [ 'wpep_admin_shortcode_builder_wpep_course_ids_selection_options' ] )
      ]
    ]
  ];

  if( wpep_assessments_enabled() ) {
    $response[ WPEP_SHORT_CODE_PREFIX . 'misc' ][ 'items' ][ WPEP_SHORT_CODE_PREFIX . 'assessment_retake' ] = [
      'type'        => 'builder',
      'name'        => __( "Retake Assessment", "wpep" ),
      'url'         => wpep_toolkit_list_item_url(WPEP_SHORT_CODE_PREFIX . 'assessment_retake', [ 'wpep_admin_shortcode_builder_wpep_assessment_retake_options' ] )
    ];

    $response[ WPEP_SHORT_CODE_PREFIX . 'misc' ][ 'items' ][ WPEP_SHORT_CODE_PREFIX . 'assessment_retake_link' ] = [
      'type'        => 'builder',
      'name'        => __( "Retake Assessment Link", "wpep" ),
      'url'         => wpep_toolkit_list_item_url(WPEP_SHORT_CODE_PREFIX . 'assessment_retake_link', [ 'wpep_admin_shortcode_builder_wpep_assessment_retake_link_options' ] )
    ];
  }

  if( wpep_public_profiles_enabled() )
    $response[ WPEP_SHORT_CODE_PREFIX . 'misc' ][ 'items' ][ WPEP_SHORT_CODE_PREFIX . 'public_profile_link' ] = [
      'type'        => 'plain',
      'name'        => __( "Public Profile Link", "wpep" ),
      'before'      => '[' . WPEP_SHORT_CODE_PREFIX . 'public_profile_link]',
      'placeholder' => __( "Link Name", "wpep" ),
      'after'       => '[/' . WPEP_SHORT_CODE_PREFIX . 'public_profile_link]'
    ];

  return apply_filters( 'wpep_toolkit_list_items_map', $response );
}

/**
 * @param $tag
 * @param array $options
 * @param int $layout_options
 * @return string
 */
function wpep_toolkit_list_item_url( $tag, $options = [], $layout_options = 1 ) {
  $url = wpep_admin_shortcode_builder_url( $tag, $options );
  $url = add_query_arg( 'wpep-sb-init', 0, $url );
  $url = add_query_arg( 'wpep-sb-button-text', urlencode( __( "Use ShortCode", "wpep" ) ), $url );
  $url = add_query_arg( 'wpep-sb-button-description', urlencode( __( "The shortcode will be displayed in the editor", "wpep" ) ) , $url );
  $url = add_query_arg( 'wpep-sb-layout-options', intval( $layout_options ) , $url );

  return $url;
}