<?php

if( !function_exists( 'wpep_format_content' ) ) {

  /**
   * @param $content
   * @return string
   */
  function wpep_format_content( $content ) {
    return do_shortcode( wpautop( $content ) );
  }

}

if( !function_exists( 'wpep_debug_dump' ) ) {

  /**
   * Debug helper function.  This is a wrapper for var_dump() that adds
   * the <pre /> tags, cleans up newlines and indents, and runs
   * htmlentities() before output.
   *
   * @param  mixed  $var   The variable to dump.
   * @param  string $label OPTIONAL Label to prepend to output.
   * @param  bool   $echo  OPTIONAL Echo output if true.
   * @return string
   */
  function wpep_debug_dump($var, $label = null, $echo = true) {
    $label = ($label===null) ? '' : rtrim($label) . ' ';

    ob_start();

    var_dump($var);

    $output = ob_get_clean();
    // neaten the newlines and indents
    $output = preg_replace("/\]\=\>\n(\s+)/m", "] => ", $output);

    if ( PHP_SAPI == 'cli') {
      $output = PHP_EOL . $label
        . PHP_EOL . $output
        . PHP_EOL;
    } else {
      if(!extension_loaded('xdebug')) {
        $output = htmlspecialchars($output, ENT_QUOTES);
      }
      $output = '<pre>'
        . $label
        . $output
        . '</pre>';
    }

    if ($echo)
      echo $output;

    return $output;
  }

}

if( !function_exists( 'wpep_format_content_autoembed' ) ) {

  function wpep_format_content_autoembed( $content ) {
    if( !isset( $GLOBALS['wp_embed'] ) )
      return $content;

    $usecache_status = $GLOBALS['wp_embed']->usecache;

    $GLOBALS['wp_embed']->usecache = false;

    $autoembed = $GLOBALS['wp_embed']->autoembed( $content );

    $GLOBALS['wp_embed']->usecache = $usecache_status;

    return $autoembed;
  }

}

if( !function_exists( 'wpep_redirect' ) ) {

  function wpep_redirect( $url ) {
    if( !did_action( 'wp_head' ) ) {
      wp_redirect( $url );

      return;
    }

    echo wpep_spinner_loader();
    echo '<script type="text/javascript">document.location = "' . $url . '"</script>';
  }

}

if( !function_exists( 'wpep_generate_random_token' ) ) {

  function wpep_generate_random_token( $length = 13 ) {
    // I know random bytes is always present in WordPress, or at least should be, but I like to be cautious.

    if (function_exists("random_bytes")) {
      $bytes = random_bytes(ceil($length / 2));
    } elseif (function_exists("openssl_random_pseudo_bytes")) {
      $bytes = openssl_random_pseudo_bytes(ceil($length / 2));
    } else {
      exit( "no cryptographically secure random function available" );
    }

    return substr( bin2hex($bytes), 0, $length );
  }

}

if( !function_exists( 'wpep_utilities_social_share_map' ) ) {

  /**
   * @return array
   */
  function wpep_utilities_social_share_map() {
    $response = [
      'email' => [
        'alias'   => 'email',
        'label'   => "E-mail",
        'url'     => "mailto:{to}?subject={title}&body={url}"
      ],
      'twitter' => [
        'alias'   => 'twitter',
        'label'   => "Tweet",
        'url'     => "https://twitter.com/share?url={url}&text={title}"
      ],
      'facebook' => [
        'alias'   => 'facebook',
        'label'   => "Like",
        'url'     => "https://facebook.com/sharer/sharer.php?u={url}"
      ],
      'linkedin' => [
        'alias'   => 'linkedin',
        'label'   => "Share",
        'url'     => "https://www.linkedin.com/shareArticle?mini=true&url={url}"
      ],
      'pinterest' => [
        'alias'   => 'pinterest',
        'label'   => "Share",
        'url'     => "https://pinterest.com/pin/create/bookmarklet/?media={image}&url={url}"
      ],
      'stumbleupon' => [
        'alias'   => 'stumbleupon',
        'label'   => "Share",
        'url'     => "https://www.stumbleupon.com/submit?url={url}&title={title}"
      ],
      'xing' => [
        'alias'   => 'xing',
        'label'   => "Share",
        'url'     => "https://www.xing.com/spi/shares/new?url={url}"
      ],
      'telegram' => [
        'alias'   => 'telegram',
        'label'   => "Telegram",
        'url'     => "tg://msg?text={url} {title}"
      ],
      'whatsapp' => [
        'alias'   => 'whatsapp',
        'label'   => "WhatsApp",
        'url'     => "whatsapp://send?text={url} {title}"
      ],
      'messenger' => [
        'alias'   => 'messenger',
        'label'   => "Messenger",
        'url'     => "fb-messenger://share?link={url}"
      ],
    ];

    return $response;
  }

}

if( !function_exists( 'wpep_utilities_html_links_to_array' ) ) {

  /**
   * @param string $content
   * @return array
   */
  function wpep_utilities_html_links_to_array( $content ) {
    $response = [];

    $dom = new \DOMDocument();

    //Parse the HTML. The @ is used to suppress any parsing errors
    //that will be thrown if the $html string isn't valid XHTML.
    @$dom->loadHTML( $content );

    $links = $dom->getElementsByTagName('a');

    foreach ($links as $link){
      $response[ $link->getAttribute('href') ] = $link->nodeValue;
    }

    return $response;
  }

}

if( !function_exists( 'wpep_get_video_source_output' ) ) {

  /**
   * @param $video_embed_source
   * @param $enable_js_api
   * @return string
   */
  function wpep_get_video_source_output( $video_embed_source, $enable_js_api = false ) {
    $video_embed_source = trim( $video_embed_source );

    if( wpep_string_ends_with( $video_embed_source, ['.mp4', '.ogv', '.flv'] ) ) {
      $response = '<div class="wpep-video-source-container">' . wp_video_shortcode( [ 'src' => $video_embed_source, 'width' => 640, 'height' => 360 ] ) . '</div>';
    } else {
      $response = wpep_format_content_autoembed( $video_embed_source );

      $response = wpep_get_video_source_output_with_settings( $response );

      if( $enable_js_api )
        $response = wpep_get_video_source_output_with_js_api( $response );
    }

    return $response;
  }

}

if( !function_exists( 'wpep_get_video_source_output_with_settings' ) ) {

  /**
   * @param $html
   * @return mixed
   */
  function wpep_get_video_source_output_with_settings($html) {
    if (strstr($html, 'youtube.com/embed/')) {
      $replace = '?feature=oembed';

      if( !strstr( $html, 'rel=' ) )
        $replace .= '&rel=0';

      if( !strstr( $html, 'autoplay=' ) )
        $replace .= '&autoplay=0';

      $html = str_replace('?feature=oembed', $replace, $html);
    } else if( strpos( $html, 'player.vimeo.com' ) !== false ) {
      $allow_strpos = strpos( $html, 'allow="' );

      if( $allow_strpos === false ) {
        $html = str_replace( "<iframe", '<iframe allow="autoplay"', $html );
      } else if( strpos( $html, "autoplay", $allow_strpos ) === false )
        $html = substr( $html, 0, $allow_strpos + strlen( 'allow="' ) ) . "autoplay;" .  substr( $html,$allow_strpos + strlen( 'allow="' ) );
    }

    return $html;
  }

}

if( !function_exists( 'wpep_get_video_source_output_with_js_api' ) ) {

  /**
   * @param $html
   * @return mixed
   */
  function wpep_get_video_source_output_with_js_api($html) {
    if (strstr($html, 'youtube.com/embed/')) {
      $replace = '?feature=oembed&enablejsapi=1';

      $html = str_replace('?feature=oembed', $replace, $html);
    }

    return $html;
  }

}

if( !function_exists( 'wpep_get_indexed_number_list' ) ) {

  /**
   * @param $from
   * @param $to
   * @param string $format
   * @return array
   */
  function wpep_get_indexed_number_list( $from, $to, $format = '%value%' ) {
    $response = [];

    for( ;$from <= $to; $from++ )
      $response[$from] = str_replace( '%value%', $from, $format );

    return $response;
  }

}

if( !function_exists( 'wpep_implode_associative_array' ) ) {

  /**
   * @param $array
   * @param $inner_separator
   * @param $outer_separator
   * @return string
   */
  function wpep_implode_associative_array( $array, $inner_separator, $outer_separator ) {
    $response = '';

    foreach ($array as $k=>$v) {
      if( $response !== '' )
        $response .= $outer_separator;

      $response .= $k . $inner_separator . $v;
    }

    return $response;
  }

}

if( !function_exists( 'wpep_alias_to_label' ) ) {

  /**
   * @param $alias
   * @return string
   */
  function wpep_alias_to_label( $alias ) {
    if( $alias === 'api' )
      return 'API';

    $response = str_replace( [ '_', '-', '.' ], ' ', $alias );

    $response = preg_replace(array('/\s{2,}/', '/[\t\n]/'), ' ', $response );

    return ucwords( $response );
  }

}

if( !function_exists( 'wpep_label_to_alias' ) ) {

  /**
   * @param $label
   * @return string
   */
  function wpep_label_to_alias( $label ) {
    return preg_replace("/\W+/", '', strtolower( str_replace( ' ', '_', $label ) ) );
  }

}

if( !function_exists( 'wpep_label_to_slug' ) ) {

  /**
   * @param $label
   * @return string
   */
  function wpep_label_to_slug( $label ) {
    $response = str_replace( '_', '-', preg_replace("/\W+/", '', strtolower( str_replace( ' ', '_', $label ) ) ) );

    $response = str_replace( '--', '-', $response );

    return $response;
  }

}

if( !function_exists( 'wpep_label_to_prefix' ) ) {

  /**
   * @param $label
   * @return string
   */
  function wpep_label_to_prefix( $label ) {
    $response = '';
    $tokens   = explode( '_', wpep_label_to_alias( $label ) );

    if( count( $tokens ) <= 1 )
      return str_replace( '_', '-', wpep_label_to_alias( $label ) );

    foreach( $tokens as $token )
      $response .= $token[0];

    return $response;
  }

}

if( !function_exists( 'wpep_string_ends_with' ) ) {

  /**
   * @param $haystack
   * @param $needle
   * @return bool
   */
  function wpep_string_ends_with($haystack, $needle) {
    if( is_array( $needle ) ) {
      foreach( $needle as $current_needle )
        if( wpep_string_ends_with( $haystack, $current_needle ) )
          return true;

      return false;
    }

    // search forward starting from end minus needle length characters
    return $needle === "" || (($temp = strlen($haystack) - strlen($needle)) >= 0 && strpos($haystack, $needle, $temp) !== false);
  }

}

if( !function_exists( 'wpep_utility_selected' ) ) {

  /**
   * @param $selected
   * @param bool $current
   * @param bool $echo
   * @return string
   */
  function wpep_utility_selected( $selected, $current = true, $echo = true ) {
    if( is_array( $selected ) )
      return selected( 1, in_array( $current, $selected ), $echo );

    return selected( $selected, $current, $echo );
  }

}

if( !function_exists( 'wpep_utility_checked' ) ) {

  function wpep_utility_checked( $checked, $current = true, $echo = true ) {
    if( is_array( $current ) )
      return checked( 1, in_array( $checked, $current ), $echo );

    return checked( $checked, $current, $echo );
  }

}

if( !function_exists( 'wpep_utilities_sanitize_input' ) ) {

  function wpep_utilities_sanitize_input( $value ) {
    if( is_array( $value ) ) {
      foreach( $value as $k => $v )
        $value[$k] = wpep_utilities_sanitize_input( $v );

      return $value;
    }

    if( is_numeric( $value ) )
      return $value;

    $value = stripslashes( $value );

    $protocols = wp_allowed_protocols();

    if( !in_array( 'data', $protocols ) )
      $protocols[] = 'data';

    $value = wp_kses( $value, wpep_user_content_allowed_html_tags(), $protocols );

    return $value;
  }

}

if( !function_exists( 'wpep_utilities_indexed_input_name_attr' ) ) {

  function wpep_utilities_indexed_input_name_attr( $field_prefix, $field_name, $field_suffix = '[]' ) {
    $response = $field_prefix;

    if( strpos( $field_name, '[' ) === false ) {
      $response .= '[' . $field_name . ']';
    } else {
      $response .= '[' . substr_replace($field_name, '][', strpos( $field_name, '[' ), 1 );
    }

    return $response . $field_suffix;
  }

}

if( !function_exists( 'wpep_utilities_map_object' ) ) {

  function wpep_utilities_map_object($array, $param = 'ID', $multiple = false ) {
    $ret = [];

    foreach ($array as $a) {
      if( !$multiple ) {
        $ret[ $a->$param ] = $a;
        continue;
      }

      if( !isset( $ret[ $a->$param ] ) )
        $ret[ $a->$param ] = [];

      $ret[ $a->$param ][] = $a;
    }

    return $ret;
  }

}

if( !function_exists( 'wpep_utilities_map_array' ) ) {

  function wpep_utilities_map_array($array, $param = 'ID', $multiple = false ) {
    $ret = [];

    foreach ($array as $a) {
      if( $multiple === false ) {
        $ret[ $a[ $param ] ] = $a;
        continue;
      }

      if( !isset( $ret[ $a[ $param ] ] ) )
        $ret[ $a[ $param ] ] = [];

      if( $multiple === true )
        $ret[ $a[ $param ] ][] = $a;
      else
        $ret[ $a[ $param ] ][ $a[ $multiple ] ] = $a;
    }

    return $ret;
  }

}

if( !function_exists( 'wpep_utilities_filter_object_list_by_params' ) ) {

  function wpep_utilities_filter_object_list_by_params($array, $filter_rules = [], $map_by = false ) {
    $ret = [];

    foreach ($array as $a) {
      foreach( $filter_rules as $param => $value ) {
        if( $a->$param != $value )
          continue 2;
      }

      if( $map_by === false )
        $ret[] = $a;
      else
        $ret[ $a->$map_by ] = $a;
    }

    return $ret;
  }

}

if( !function_exists( 'wpep_utility_extract_params' ) ) {

  function wpep_utility_extract_params( $array, $map ) {
    if( !is_array( $array ) )
      return [];

    $response = [];

    foreach( $map as $current_map_key => $current_map_data  )
      $response[ $current_map_key ] = [];

    foreach ( $array as $a ) {
      if( is_object( $a ) )
        $a = get_object_vars( $a );

      foreach( $map as $current_map_key => $current_map_data ) {
        foreach( $current_map_data as $current_map_inner_key => $current_map_inner_data ) {
          if( is_numeric( $current_map_inner_key ) ) {
            $response[ $current_map_key ][] = $a[ $current_map_inner_data ];
            continue;
          }

          if( is_array( $current_map_inner_data ) ) {
            foreach( $current_map_inner_data as $current_map_inner_inner_key => $current_map_inner_inner_data ) {
              if( ( is_array( $current_map_inner_inner_data ) && !in_array( $current_map_inner_inner_key, $current_map_inner_inner_data ) )
                  || $a[ $current_map_inner_inner_key ] !== $current_map_inner_inner_data )
                continue 2;
            }

            $response[ $current_map_key ][] = $a[ $current_map_inner_key ];
          }
        }
      }
    }

    foreach( $response as $response_key => $response_data )
      $response[ $response_key ] = array_unique( $response_data );

    return $response;
  }

}

if( !function_exists( 'wpep_utility_map_to_array_assoc' ) ) {

  function wpep_utility_map_to_array_assoc( $array, $key_param = 'key', $value_param = 'value', $merge_existent_indexes = false ) {
    if( !is_array( $array ) )
      return [];

    $ret = [];

    foreach ( $array as $a ) {
      if( is_object( $a ) )
        $a = get_object_vars( $a );

      $current_value_param = $a[$value_param];

      if( is_array( $key_param ) ) {
        $current_key_param   = '';

        foreach( $key_param as $key_param_token )
          $current_key_param .= ( isset( $a[ $key_param_token ] ) ? $a[ $key_param_token ] : $key_param_token );
      } else {
        $current_key_param   = $a[$key_param];
      }

      if( isset( $ret[ $current_key_param ] ) && $merge_existent_indexes ) {
        if( !is_array( $ret[ $current_key_param ] ) )
          $ret[ $current_key_param ] = [ $ret[ $current_key_param ] ];

        $ret[ $current_key_param ][] = $current_value_param;

        continue;
      }

      $ret[ $current_key_param ] = $current_value_param;
    }

    return $ret;
  }

}

if( !function_exists( 'wpep_utility_array_sum' ) ) {

  function wpep_utility_array_sum( $array ) {
    if( is_object( $array ) )
      $array = get_object_vars( $array );

    return array_sum( $array );
  }

}

if( !function_exists( 'wpep_utility_array_trim' ) ) {

  function wpep_utility_array_trim( $array ) {
    foreach( $array as $k => $v )
      $array[ $k ] = is_string( $v ) ? trim( $v ) : $v;

    return $array;
  }

}

if( !function_exists( 'wpep_utility_map_to_object_assoc' ) ) {

  function wpep_utility_map_to_object_assoc( $object, $key_param = 'key', $value_param = 'value' ) {
    if( !is_array( $object ) )
      return new stdClass();

    $ret = new stdClass();

    foreach ($object as $o) {
      $index = $o->$key_param;
      $ret->$index = $o->$value_param;
    }

    return $ret;
  }

}

if( !function_exists( 'wpep_utility_time_remaining' ) ) {

  function wpep_utility_time_remaining($seconds) {
    $t = round($seconds);
    return sprintf('%02d:%02d:%02d', ($t/3600),($t/60%60), $t%60);
  }

}

if( !function_exists( 'wpep_get_percentage' ) ) {

  /**
   * @param $percent / goal
   * @param int|float|bool $total
   * @param string $prefix
   * @param string $suffix
   * @param int $decimals
   * @return string
   */
  function wpep_get_percentage( $percent, $total = false, $prefix = '', $suffix = '%', $decimals = 2 ) {
    $percent = floatval( $percent );

    if( $total !== false )
      $percent = ( $total == 0 ? 0 : ( $percent / $total ) * 100 );

    $response = ( $percent != 0 ? number_format_i18n( $percent, $decimals ) : '0.00' );

    return $prefix . $response . $suffix;
  }

}

if( !function_exists( 'wpep_time_to_dhms' ) ) {

  /**
   * @param $time
   * @return array [
  'days'    => 0,
  'hours'   => 0,
  'minutes' => 0,
  'seconds' => 0
  ]
   */
  function wpep_time_to_dhms( $time ) {
    $time = intval( $time );

    $response = [
      'days'    => 0,
      'hours'   => 0,
      'minutes' => 0,
      'seconds' => 0
    ];

    if( $time == 0 )
      return $response;

    $response['days'] = floor( $time / ( 60 * 60 * 24 ) );
    $time %= ( 60 * 60 * 24 );
    $response['hours'] = floor( $time / ( 60 * 60 ) );
    $time %= ( 60 * 60 );
    $response['minutes'] = floor( $time / 60 );
    $time %= 60;
    $response['seconds'] = $time;

    return $response;
  }

}

if( !function_exists( 'wpep_time_to_dhms_string' ) ) {

  /**
   * @param $time
   * @return string
   */
  function wpep_time_to_dhms_string( $time ) {
    $dhms = wpep_time_to_dhms( $time );

    $time_points = 0;
    $response = '';

    if( $dhms[ 'days' ] != 0 ) {
      $response .= ( $dhms[ 'days' ] == 1 ? __( "1 day", "wpep" ) : sprintf( __( "%s days", "wpep" ), $dhms[ 'days' ] ) );

      $time_points++;
    }

    if( $dhms[ 'hours' ] != 0 ) {
      $response .= ( $response === '' ? '' : ' ' ) . ( $dhms[ 'hours' ] == 1 ? __( "1 hour", "wpep" ) : sprintf( __( "%s hours", "wpep" ), $dhms[ 'hours' ] ) );

      $time_points++;
    }

    if( $time_points >= 2 )
      return $response;

    if( $dhms[ 'minutes' ] != 0 ) {
      $response .= ( $response === '' ? '' : ' ' ) . ( $dhms[ 'minutes' ] == 1 ? __( "1 minute", "wpep" ) : sprintf( __( "%s minutes", "wpep" ), $dhms[ 'minutes' ] ) );

      $time_points++;
    }

    if( $time_points >= 2 )
      return $response;

    if( $dhms[ 'seconds' ] != 0 )
      $response .= ( $response === '' ? '' : ' ' ) . ( $dhms[ 'seconds' ] == 1 ? __( "1 second", "wpep" ) : sprintf( __( "%s seconds", "wpep" ), $dhms[ 'seconds' ] ) );

    return $response;
  }

}

if( !function_exists( 'wpep_format_smart_date' ) ) {

  function wpep_format_smart_date( $date = false ) {
    $date = ( is_numeric( $date ) || $date === false ? $date : strtotime( $date ) );

    if( time() - $date < 86400 )
      return sprintf( "%s ago", human_time_diff( time(), $date ) );

    return wpep_format_date_with_time( $date );
  }

}

if( !function_exists( 'wpep_format_date' ) ) {

  function wpep_format_date( $date = false, $date_format = false ) {
    if( $date === '' )
      return '';

    if( $date === false ) {
      $date = time();
    } else {
      $date = ( is_numeric( $date ) ? $date : strtotime( $date ) );
    }

    $date_format = ( $date_format == false ? get_option( 'date_format' ) : $date_format );

    if( function_exists( 'wp_date' ) )
      return wp_date( $date_format, $date );

    return date_i18n( $date_format, $date );
  }

}

if( !function_exists( 'wpep_format_date_with_time' ) ) {

  function wpep_format_date_with_time( $date = false, $type = 'plain' ) {
    if( $date === '' )
      return '';

    $date_format = get_option( 'date_format' );

    if( $type === 'standard' )
      $date_format = 'Y-m-d';
    else if( ( $type === 'datepicker' || $type === 'datetimepicker' ) && wpep_get_setting_flag( 'admin-library-datetimepicker-simple-format' ) )
      $date_format = 'Y-m-d';

    $response = wpep_format_date( $date, $date_format . ( $type === 'log' ? " H:i:s" : ( $type === 'datepicker' ? "" : " H:i" ) ) );

    if( strpos( $response, '00:00' ) !== false )
      $response = str_replace( ' 00:00', '', $response );

    return $response;
  }

}

if( !function_exists( 'wpep_permalink_extend' ) ) {

  function wpep_permalink_extend($permalink, $param, $value ) {
    $permalink = remove_query_arg( $param, $permalink );

    $permalink = add_query_arg( $param, $value, $permalink );

    return $permalink;
  }

}

if( !function_exists( 'wpep_let_to_num' ) ) {

  function wpep_let_to_num( $size ) {
    $l   = substr( $size, -1 );
    $ret = substr( $size, 0, -1 );
    switch ( strtoupper( $l ) ) {
      case 'P':
        $ret *= 1024;
      case 'T':
        $ret *= 1024;
      case 'G':
        $ret *= 1024;
      case 'M':
        $ret *= 1024;
      case 'K':
        $ret *= 1024;
    }
    return $ret;
  }

}

if( !function_exists( 'wpep_link_is_http2' ) ) {

  /**
   * @param $url
   * @return null|bool
   */
  function wpep_link_is_http2( $url ) {
    if( !function_exists( 'curl_init' ) )
      return null;

    $ch = curl_init();
    curl_setopt_array($ch, [
      CURLOPT_URL            => $url,
      CURLOPT_HEADER         => true,
      CURLOPT_NOBODY         => true,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_2_0, // cURL will attempt to make an HTTP/2.0 request (can downgrade to HTTP/1.1)
    ]);

    $response = curl_exec($ch);
    $is_http2 = null;

    if ($response !== false && strpos($response, "HTTP/2.0") === 0) {
      $is_http2 = true;
    } elseif ($response !== false) {
      $is_http2 = false;
    }

    curl_close($ch);

    return $is_http2;
  }

}

if( !function_exists( 'wpep_link_is_404' ) ) {

  function wpep_link_is_404($url) {
    $http_code = wpep_link_http_code( $url );

    if( $http_code == 0 && strpos( $url, get_site_url() ) === 0 )
      return false;

    if( in_array( $http_code, [ 302, 403 ] ) )
      return false;

    if ($http_code >= 200 && $http_code < 300) {
      return false;
    } else {
      return true;
    }
  }

}

if( !function_exists( 'wpep_link_http_code' ) ) {

  function wpep_link_http_code($url) {
    $handle = curl_init($url);
    curl_setopt($handle,  CURLOPT_RETURNTRANSFER, TRUE);
    $response = curl_exec($handle);
    $http_code = curl_getinfo($handle, CURLINFO_HTTP_CODE);
    curl_close($handle);

    return $http_code;
  }

}


if( !function_exists( 'wpep_utility_html_find_hosted_assets' ) ) {

  function wpep_utility_html_find_hosted_assets( $content, $site_url = false ) {
    $site_url = $site_url == false ? get_site_url() : $site_url;
    $content = trim( $content );

    if( strpos( $content, $site_url ) === 0 ) {

      if( wpep_link_is_media_file( $content ) )
        return [ wpep_get_attachment_id_by_url( $content, true ) => $content ];

      return [];
    }

    if( is_numeric( $content ) ) {
      $content = intval( $content );

      $attachment_url = wp_get_attachment_url( $content );

      if( $attachment_url != false )
        return [ $content => $attachment_url ];

      return [];
    }

    $response = [];

    $content = do_shortcode( $content );

    $link_dom_finder = [ '//img' => 'src', '//a' => 'href', '//source' => 'src' ];

    foreach( $link_dom_finder as $element_tag => $element_source ) {
      foreach( wpep_document_dom_extract_element( $content, $element_tag, $element_source ) as $target ) {
        if( wpep_string_ends_with( $target, '?_=1' ) )
          $target = substr( $target, 0, -4 );

        if( strpos( $target, $site_url ) === 0 && wpep_link_is_media_file( $target ) ) {
          $response[ wpep_get_attachment_id_by_url( $target, true ) ] = $target;
        }
      }
    }

    return $response;
  }

}

if( !function_exists( 'wpep_link_is_media_file' ) ) {

  function wpep_link_is_media_file( $url ) {
    $url_headers = @get_headers( $url, 1 );

    if( $url_headers == false
        && strpos( $url, get_site_url( ) ) === 0 ) {
      $attachment_id = wpep_get_attachment_id_by_url( $url, true );

      return is_numeric( $attachment_id ) && $attachment_id != 0;
    }

    if( isset( $url_headers['Content-Type'] ) ){
      if( is_array( $url_headers['Content-Type'] ) )
        $type = $url_headers['Content-Type'][ key( $url_headers['Content-Type'] ) ];
      else
        $type = ( is_string( $url_headers['Content-Type'] ) ? strtolower( $url_headers['Content-Type'] ) : '' );

      if( in_array( $type, [
        'image/png', 'image/jpg', 'image/jpeg', 'image/jpe',
        'image/gif', 'image/tif', 'image/tiff', 'image/svg',
        'image/ico', 'image/icon', 'image/x-icon',
        'application/x-compressed', 'application/x-zip-compressed',
        'application/zip', 'application/octet-stream', 'audio/mpeg',
        'audio/mpeg3', 'audio/x-mpeg-3',
        'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
        'application/msword', 'application/pdf',
        'application/excel', 'application/x-excel',
        'text/css', 'application/javascript'
      ] ) ){
        return true;
      }

    }

    return false;
  }

}

if( !function_exists( 'wpep_document_dom_extract_element_attribute' ) ) {

  function wpep_document_dom_extract_element_attribute( $content, $element = 'a', $element_attribute = 'id' ) {
    if ( empty( $content ) )
      return [];

    $response = [];

    $dom = new \DOMDocument();
    $dom->loadHTML( $content );
    $data_elements = $dom->getElementsByTagName( $element );

    foreach ( $data_elements as $data_element ) {
      if ( ! $data_element->hasAttribute( $element_attribute ) )
        continue;

      $response[] = stripslashes( str_replace( '"' , '' , $data_element->getAttribute( $element_attribute ) ) );
    }

    return $response;
  }

}

if( !function_exists( 'wpep_document_dom_extract_element' ) ) {

  /**
   * @param $content
   * @param string $element_path
   * @param bool|string $element_attribute
   * @return array
   */
  function wpep_document_dom_extract_element( $content, $element_path = '//img', $element_attribute = false ) {
    $doc = new \DOMDocument();
    @$doc->loadHTML('<html><body>' . $content . '</body></html>');

    $xml = simplexml_import_dom($doc);

    $entries = $xml->xpath( $element_path );

    if( empty( $entries ) )
      return [];

    $response = [];

    foreach ( $entries as $entry ) {

      if( $element_attribute == false ) {
        $response[] = $entry;
        continue;
      }

      if( isset( $entry[ $element_attribute ] ) )
        $response[] = (string) $entry[ $element_attribute ]; // Stringify
    }

    return $response;
  }

}

if( !function_exists( 'wpep_get_admin_icon_selection_list' ) ) {

  function wpep_get_admin_icon_selection_list() {
    $response = get_transient( 'wpep_admin_icon_selection_list' );

    if( is_array( $response ) )
      return $response;

    $dashicon_list = wpep_get_dashicons_list();
    sort( $dashicon_list );

    $response = [
      'none'  => __( "No Icon", "wpep" )
    ];

    foreach( $dashicon_list as $dashicon )
      $response[ 'dashicons ' . $dashicon ] = wpep_alias_to_label( str_replace( 'dashicons-', '', $dashicon ) );

    set_transient( 'wpep_admin_icon_selection_list', $response, DAY_IN_SECONDS );

    return $response;
  }

}

if( !function_exists( 'wpep_get_month_list_within_year' ) ) {

  function wpep_get_month_list_within_year( $year ) {
    if( $year > date("Y" ) )
      return [];

    $months_to = ( $year == date( "Y" ) ? intval( date("m" ) ) : 12 );
    $response  = [];

    global $wp_locale;

    for( $i = 1; $i <= $months_to; $i++ )
      $response[ $i ] = $wp_locale->get_month( $i );

    return $response;
  }

}

if( !function_exists( 'wpep_get_dashicons_list' ) ) {
  function wpep_get_dashicons_list() {
    return [
      'dashicons-menu',
      'dashicons-dashboard',
      'dashicons-admin-site',
      'dashicons-admin-media',
      'dashicons-admin-page',
      'dashicons-admin-comments',
      'dashicons-admin-appearance',
      'dashicons-admin-plugins',
      'dashicons-admin-users',
      'dashicons-admin-tools',
      'dashicons-admin-settings',
      'dashicons-admin-network',
      'dashicons-admin-generic',
      'dashicons-admin-home',
      'dashicons-admin-collapse',
      'dashicons-admin-links',
      'dashicons-admin-post',
      'dashicons-format-standard',
      'dashicons-format-image',
      'dashicons-format-gallery',
      'dashicons-format-audio',
      'dashicons-format-video',
      'dashicons-format-links',
      'dashicons-format-chat',
      'dashicons-format-status',
      'dashicons-format-aside',
      'dashicons-format-quote',
      'dashicons-welcome-write-blog',
      'dashicons-welcome-edit-page',
      'dashicons-welcome-add-page',
      'dashicons-welcome-view-site',
      'dashicons-welcome-widgets-menus',
      'dashicons-welcome-comments',
      'dashicons-welcome-learn-more',
      'dashicons-image-crop',
      'dashicons-image-rotate-left',
      'dashicons-image-rotate-right',
      'dashicons-image-flip-vertical',
      'dashicons-image-flip-horizontal',
      'dashicons-undo',
      'dashicons-redo',
      'dashicons-editor-bold',
      'dashicons-editor-italic',
      'dashicons-editor-ul',
      'dashicons-editor-ol',
      'dashicons-editor-quote',
      'dashicons-editor-alignleft',
      'dashicons-editor-aligncenter',
      'dashicons-editor-alignright',
      'dashicons-editor-insertmore',
      'dashicons-editor-spellcheck',
      'dashicons-editor-distractionfree',
      'dashicons-editor-expand',
      'dashicons-editor-contract',
      'dashicons-editor-kitchensink',
      'dashicons-editor-underline',
      'dashicons-editor-justify',
      'dashicons-editor-textcolor',
      'dashicons-editor-paste-word',
      'dashicons-editor-paste-text',
      'dashicons-editor-removeformatting',
      'dashicons-editor-video',
      'dashicons-editor-customchar',
      'dashicons-editor-outdent',
      'dashicons-editor-indent',
      'dashicons-editor-help',
      'dashicons-editor-strikethrough',
      'dashicons-editor-unlink',
      'dashicons-editor-rtl',
      'dashicons-editor-break',
      'dashicons-editor-code',
      'dashicons-editor-paragraph',
      'dashicons-align-left',
      'dashicons-align-right',
      'dashicons-align-center',
      'dashicons-align-none',
      'dashicons-lock',
      'dashicons-calendar',
      'dashicons-visibility',
      'dashicons-post-status',
      'dashicons-edit',
      'dashicons-post-trash',
      'dashicons-trash',
      'dashicons-external',
      'dashicons-arrow-up',
      'dashicons-arrow-down',
      'dashicons-arrow-left',
      'dashicons-arrow-right',
      'dashicons-arrow-up-alt',
      'dashicons-arrow-down-alt',
      'dashicons-arrow-left-alt',
      'dashicons-arrow-right-alt',
      'dashicons-arrow-up-alt2',
      'dashicons-arrow-down-alt2',
      'dashicons-arrow-left-alt2',
      'dashicons-arrow-right-alt2',
      'dashicons-leftright',
      'dashicons-sort',
      'dashicons-randomize',
      'dashicons-list-view',
      'dashicons-exerpt-view',
      'dashicons-hammer',
      'dashicons-art',
      'dashicons-migrate',
      'dashicons-performance',
      'dashicons-universal-access',
      'dashicons-universal-access-alt',
      'dashicons-tickets',
      'dashicons-nametag',
      'dashicons-clipboard',
      'dashicons-heart',
      'dashicons-megaphone',
      'dashicons-schedule',
      'dashicons-wordpress',
      'dashicons-wordpress-alt',
      'dashicons-pressthis,',
      'dashicons-update,',
      'dashicons-screenoptions',
      'dashicons-info',
      'dashicons-cart',
      'dashicons-feedback',
      'dashicons-cloud',
      'dashicons-translation',
      'dashicons-tag',
      'dashicons-category',
      'dashicons-archive',
      'dashicons-tagcloud',
      'dashicons-text',
      'dashicons-media-archive',
      'dashicons-media-audio',
      'dashicons-media-code',
      'dashicons-media-default',
      'dashicons-media-document',
      'dashicons-media-interactive',
      'dashicons-media-spreadsheet',
      'dashicons-media-text',
      'dashicons-media-video',
      'dashicons-playlist-audio',
      'dashicons-playlist-video',
      'dashicons-yes',
      'dashicons-no',
      'dashicons-no-alt',
      'dashicons-plus',
      'dashicons-plus-alt',
      'dashicons-minus',
      'dashicons-dismiss',
      'dashicons-marker',
      'dashicons-star-filled',
      'dashicons-star-half',
      'dashicons-star-empty',
      'dashicons-flag',
      'dashicons-share',
      'dashicons-share1',
      'dashicons-share-alt',
      'dashicons-share-alt2',
      'dashicons-twitter',
      'dashicons-rss',
      'dashicons-email',
      'dashicons-email-alt',
      'dashicons-facebook',
      'dashicons-facebook-alt',
      'dashicons-networking',
      'dashicons-location',
      'dashicons-location-alt',
      'dashicons-camera',
      'dashicons-images-alt',
      'dashicons-images-alt2',
      'dashicons-video-alt',
      'dashicons-video-alt2',
      'dashicons-video-alt3',
      'dashicons-vault',
      'dashicons-shield',
      'dashicons-shield-alt',
      'dashicons-sos',
      'dashicons-search',
      'dashicons-slides',
      'dashicons-analytics',
      'dashicons-chart-pie',
      'dashicons-chart-bar',
      'dashicons-chart-line',
      'dashicons-chart-area',
      'dashicons-groups',
      'dashicons-businessman',
      'dashicons-id',
      'dashicons-id-alt',
      'dashicons-products',
      'dashicons-awards',
      'dashicons-forms',
      'dashicons-testimonial',
      'dashicons-portfolio',
      'dashicons-book',
      'dashicons-book-alt',
      'dashicons-download',
      'dashicons-upload',
      'dashicons-backup',
      'dashicons-clock',
      'dashicons-lightbulb',
      'dashicons-microphone',
      'dashicons-desktop',
      'dashicons-tablet',
      'dashicons-smartphone',
      'dashicons-smiley'
    ];
  }
}

if( !function_exists( 'wpep_get_native_font_list' ) ) {
  function wpep_get_native_font_list() {
    return [
      "Arial",
      "Calibri",
      "Century Gothic",
      "Courier",
      "Geneva",
      "Georgia",
      "Gill Sans",
      "Helvetica",
      "Impact",
      "Lucida",
      "Myriad Pro",
      "Palatino",
      "Tahoma",
      "Times New Roman",
      "Trebuchet",
      "Verdana"
    ];
  }
}

if( !function_exists( 'wpep_get_google_font_list' ) ) {

  function wpep_get_google_font_list() {
    return [
      "ABeeZee", "Abel", "Abhaya Libre", "Abril Fatface", "Aclonica",
      "Acme", "Actor", "Adamina", "Advent Pro", "Aguafina Script",
      "Akronim", "Aladin", "Aldrich", "Alef", "Alegreya",
      "Alegreya SC", "Alegreya Sans", "Alegreya Sans SC", "Alex Brush", "Alfa Slab One",
      "Alice", "Alike", "Alike Angular", "Allan", "Allerta",
      "Allerta Stencil", "Allura", "Almendra", "Almendra Display", "Almendra SC",
      "Amarante", "Amaranth", "Amatic SC", "Amatica SC", "Amethysta",
      "Amiko", "Amiri", "Amita", "Anaheim", "Andada",
      "Andika", "Angkor", "Annie Use Your Telescope", "Anonymous Pro", "Antic",
      "Antic Didone", "Antic Slab", "Anton", "Arapey", "Arbutus",
      "Arbutus Slab", "Architects Daughter", "Archivo Black", "Archivo Narrow", "Aref Ruqaa",
      "Arima Madurai", "Arimo", "Arizonia", "Armata", "Artifika",
      "Arvo", "Arya", "Asap", "Asar", "Asset",
      "Assistant", "Astloch", "Asul", "Athiti", "Atma",
      "Atomic Age", "Aubrey", "Audiowide", "Autour One", "Average",
      "Average Sans", "Averia Gruesa Libre", "Averia Libre", "Averia Sans Libre", "Averia Serif Libre",
      "Bad Script", "Baloo", "Baloo Bhai", "Baloo Bhaina", "Baloo Chettan",
      "Baloo Da", "Baloo Paaji", "Baloo Tamma", "Baloo Thambi", "Balthazar",
      "Bangers", "Basic", "Battambang", "Baumans", "Bayon",
      "Belgrano", "Belleza", "BenchNine", "Bentham", "Berkshire Swash",
      "Bevan", "Bigelow Rules", "Bigshot One", "Bilbo", "Bilbo Swash Caps",
      "BioRhyme", "BioRhyme Expanded", "Biryani", "Bitter", "Black Ops One",
      "Bokor", "Bonbon", "Boogaloo", "Bowlby One", "Bowlby One SC",
      "Brawler", "Bree Serif", "Bubblegum Sans", "Bubbler One", "Buda",
      "Buenard", "Bungee", "Bungee Hairline", "Bungee Inline", "Bungee Outline",
      "Bungee Shade", "Butcherman", "Butterfly Kids", "Cabin", "Cabin Condensed",
      "Cabin Sketch", "Caesar Dressing", "Cagliostro", "Cairo", "Calligraffitti",
      "Cambay", "Cambo", "Candal", "Cantarell", "Cantata One",
      "Cantora One", "Capriola", "Cardo", "Carme", "Carrois Gothic",
      "Carrois Gothic SC", "Carter One", "Catamaran", "Caudex", "Caveat",
      "Caveat Brush", "Cedarville Cursive", "Ceviche One", "Changa", "Changa One",
      "Chango", "Chathura", "Chau Philomene One", "Chela One", "Chelsea Market",
      "Chenla", "Cherry Cream Soda", "Cherry Swash", "Chewy", "Chicle",
      "Chivo", "Chonburi", "Cinzel", "Cinzel Decorative", "Clicker Script",
      "Coda", "Coda Caption", "Codystar", "Coiny", "Combo",
      "Comfortaa", "Coming Soon", "Concert One", "Condiment", "Content",
      "Contrail One", "Convergence", "Cookie", "Copse", "Corben",
      "Cormorant", "Cormorant Garamond", "Cormorant Infant", "Cormorant SC", "Cormorant Unicase",
      "Cormorant Upright", "Courgette", "Cousine", "Coustard", "Covered By Your Grace",
      "Crafty Girls", "Creepster", "Crete Round", "Crimson Text", "Croissant One",
      "Crushed", "Cuprum", "Cutive", "Cutive Mono", "Damion",
      "Dancing Script", "Dangrek", "David Libre", "Dawning of a New Day", "Days One",
      "Dekko", "Delius", "Delius Swash Caps", "Delius Unicase", "Della Respira",
      "Denk One", "Devonshire", "Dhurjati", "Didact Gothic", "Diplomata",
      "Diplomata SC", "Domine", "Donegal One", "Doppio One", "Dorsa",
      "Dosis", "Dr Sugiyama", "Droid Sans", "Droid Sans Mono", "Droid Serif",
      "Duru Sans", "Dynalight", "EB Garamond", "Eagle Lake", "Eater",
      "Economica", "Eczar", "Ek Mukta", "El Messiri", "Electrolize",
      "Elsie", "Elsie Swash Caps", "Emblema One", "Emilys Candy", "Engagement",
      "Englebert", "Enriqueta", "Erica One", "Esteban", "Euphoria Script",
      "Ewert", "Exo", "Exo 2", "Expletus Sans", "Fanwood Text",
      "Farsan", "Fascinate", "Fascinate Inline", "Faster One", "Fasthand",
      "Fauna One", "Federant", "Federo", "Felipa", "Fenix",
      "Finger Paint", "Fira Mono", "Fira Sans", "Fjalla One", "Fjord One",
      "Flamenco", "Flavors", "Fondamento", "Fontdiner Swanky", "Forum",
      "Francois One", "Frank Ruhl Libre", "Freckle Face", "Fredericka the Great", "Fredoka One",
      "Freehand", "Fresca", "Frijole", "Fruktur", "Fugaz One",
      "GFS Didot", "GFS Neohellenic", "Gabriela", "Gafata", "Galada",
      "Galdeano", "Galindo", "Gentium Basic", "Gentium Book Basic", "Geo",
      "Geostar", "Geostar Fill", "Germania One", "Gidugu", "Gilda Display",
      "Give You Glory", "Glass Antiqua", "Glegoo", "Gloria Hallelujah", "Goblin One",
      "Gochi Hand", "Gorditas", "Goudy Bookletter 1911", "Graduate", "Grand Hotel",
      "Gravitas One", "Great Vibes", "Griffy", "Gruppo", "Gudea",
      "Gurajada", "Habibi", "Halant", "Hammersmith One", "Hanalei",
      "Hanalei Fill", "Handlee", "Hanuman", "Happy Monkey", "Harmattan",
      "Headland One", "Heebo", "Henny Penny", "Herr Von Muellerhoff", "Hind",
      "Hind Guntur", "Hind Madurai", "Hind Siliguri", "Hind Vadodara", "Holtwood One SC",
      "Homemade Apple", "Homenaje", "IM Fell DW Pica", "IM Fell DW Pica SC", "IM Fell Double Pica",
      "IM Fell Double Pica SC", "IM Fell English", "IM Fell English SC", "IM Fell French Canon", "IM Fell French Canon SC",
      "IM Fell Great Primer", "IM Fell Great Primer SC", "Iceberg", "Iceland", "Imprima",
      "Inconsolata", "Inder", "Indie Flower", "Inika", "Inknut Antiqua",
      "Irish Grover", "Istok Web", "Italiana", "Italianno", "Itim",
      "Jacques Francois", "Jacques Francois Shadow", "Jaldi", "Jim Nightshade", "Jockey One",
      "Jolly Lodger", "Jomhuria", "Josefin Sans", "Josefin Slab", "Joti One",
      "Judson", "Julee", "Julius Sans One", "Junge", "Jura",
      "Just Another Hand", "Just Me Again Down Here", "Kadwa", "Kalam", "Kameron",
      "Kanit", "Kantumruy", "Karla", "Karma", "Katibeh",
      "Kaushan Script", "Kavivanar", "Kavoon", "Kdam Thmor", "Keania One",
      "Kelly Slab", "Kenia", "Khand", "Khmer", "Khula",
      "Kite One", "Knewave", "Kotta One", "Koulen", "Kranky",
      "Kreon", "Kristi", "Krona One", "Kumar One", "Kumar One Outline",
      "Kurale", "La Belle Aurore", "Laila", "Lakki Reddy", "Lalezar",
      "Lancelot", "Lateef", "Lato", "League Script", "Leckerli One",
      "Ledger", "Lekton", "Lemon", "Lemonada", "Libre Baskerville",
      "Libre Franklin", "Life Savers", "Lilita One", "Lily Script One", "Limelight",
      "Linden Hill", "Lobster", "Lobster Two", "Londrina Outline", "Londrina Shadow",
      "Londrina Sketch", "Londrina Solid", "Lora", "Love Ya Like A Sister", "Loved by the King",
      "Lovers Quarrel", "Luckiest Guy", "Lusitana", "Lustria", "Macondo",
      "Macondo Swash Caps", "Mada", "Magra", "Maiden Orange", "Maitree",
      "Mako", "Mallanna", "Mandali", "Marcellus", "Marcellus SC",
      "Marck Script", "Margarine", "Marko One", "Marmelad", "Martel",
      "Martel Sans", "Marvel", "Mate", "Mate SC", "Maven Pro",
      "McLaren", "Meddon", "MedievalSharp", "Medula One", "Meera Inimai",
      "Megrim", "Meie Script", "Merienda", "Merienda One", "Merriweather",
      "Merriweather Sans", "Metal", "Metal Mania", "Metamorphous", "Metrophobic",
      "Michroma", "Milonga", "Miltonian", "Miltonian Tattoo", "Miniver",
      "Miriam Libre", "Mirza", "Miss Fajardose", "Mitr", "Modak",
      "Modern Antiqua", "Mogra", "Molengo", "Molle", "Monda",
      "Monofett", "Monoton", "Monsieur La Doulaise", "Montaga", "Montez",
      "Montserrat", "Montserrat Alternates", "Montserrat Subrayada", "Moul", "Moulpali",
      "Mountains of Christmas", "Mouse Memoirs", "Mr Bedfort", "Mr Dafoe", "Mr De Haviland",
      "Mrs Saint Delafield", "Mrs Sheppards", "Mukta Vaani", "Muli", "Mystery Quest",
      "NTR", "Neucha", "Neuton", "New Rocker", "News Cycle",
      "Niconne", "Nixie One", "Nobile", "Nokora", "Norican",
      "Nosifer", "Nothing You Could Do", "Noticia Text", "Noto Sans", "Noto Serif",
      "Nova Cut", "Nova Flat", "Nova Mono", "Nova Oval", "Nova Round",
      "Nova Script", "Nova Slim", "Nova Square", "Numans", "Nunito",
      "Nunito Sans", "Odor Mean Chey", "Offside", "Old Standard TT", "Oldenburg",
      "Oleo Script", "Oleo Script Swash Caps", "Open Sans", "Open Sans Condensed", "Oranienbaum",
      "Orbitron", "Oregano", "Orienta", "Original Surfer", "Oswald",
      "Over the Rainbow", "Overlock", "Overlock SC", "Ovo", "Oxygen",
      "Oxygen Mono", "PT Mono", "PT Sans", "PT Sans Caption", "PT Sans Narrow",
      "PT Serif", "PT Serif Caption", "Pacifico", "Palanquin", "Palanquin Dark",
      "Paprika", "Parisienne", "Passero One", "Passion One", "Pathway Gothic One",
      "Patrick Hand", "Patrick Hand SC", "Pattaya", "Patua One", "Pavanam",
      "Paytone One", "Peddana", "Peralta", "Permanent Marker", "Petit Formal Script",
      "Petrona", "Philosopher", "Piedra", "Pinyon Script", "Pirata One",
      "Plaster", "Play", "Playball", "Playfair Display", "Playfair Display SC",
      "Podkova", "Poiret One", "Poller One", "Poly", "Pompiere",
      "Pontano Sans", "Poppins", "Port Lligat Sans", "Port Lligat Slab", "Pragati Narrow",
      "Prata", "Preahvihear", "Press Start 2P", "Pridi", "Princess Sofia",
      "Prociono", "Prompt", "Prosto One", "Proza Libre", "Puritan",
      "Purple Purse", "Quando", "Quantico", "Quattrocento", "Quattrocento Sans",
      "Questrial", "Quicksand", "Quintessential", "Qwigley", "Racing Sans One",
      "Radley", "Rajdhani", "Rakkas", "Raleway", "Raleway Dots",
      "Ramabhadra", "Ramaraja", "Rambla", "Rammetto One", "Ranchers",
      "Rancho", "Ranga", "Rasa", "Rationale", "Ravi Prakash",
      "Redressed", "Reem Kufi", "Reenie Beanie", "Revalia", "Rhodium Libre",
      "Ribeye", "Ribeye Marrow", "Righteous", "Risque", "Roboto",
      "Roboto Condensed", "Roboto Mono", "Roboto Slab", "Rochester", "Rock Salt",
      "Rokkitt", "Romanesco", "Ropa Sans", "Rosario", "Rosarivo",
      "Rouge Script", "Rozha One", "Rubik", "Rubik Mono One", "Rubik One",
      "Ruda", "Rufina", "Ruge Boogie", "Ruluko", "Rum Raisin",
      "Ruslan Display", "Russo One", "Ruthie", "Rye", "Sacramento",
      "Sahitya", "Sail", "Salsa", "Sanchez", "Sancreek",
      "Sansita One", "Sarala", "Sarina", "Sarpanch", "Satisfy",
      "Scada", "Scheherazade", "Schoolbell", "Scope One", "Seaweed Script",
      "Secular One", "Sevillana", "Seymour One", "Shadows Into Light", "Shadows Into Light Two",
      "Shanti", "Share", "Share Tech", "Share Tech Mono", "Shojumaru",
      "Short Stack", "Shrikhand", "Siemreap", "Sigmar One", "Signika",
      "Signika Negative", "Simonetta", "Sintony", "Sirin Stencil", "Six Caps",
      "Skranji", "Slabo 13px", "Slabo 27px", "Slackey", "Smokum",
      "Smythe", "Sniglet", "Snippet", "Snowburst One", "Sofadi One",
      "Sofia", "Sonsie One", "Sorts Mill Goudy", "Source Code Pro", "Source Sans Pro",
      "Source Serif Pro", "Space Mono", "Special Elite", "Spicy Rice", "Spinnaker",
      "Spirax", "Squada One", "Sree Krushnadevaraya", "Sriracha", "Stalemate",
      "Stalinist One", "Stardos Stencil", "Stint Ultra Condensed", "Stint Ultra Expanded", "Stoke",
      "Strait", "Sue Ellen Francisco", "Suez One", "Sumana", "Sunshiney",
      "Supermercado One", "Sura", "Suranna", "Suravaram", "Suwannaphum",
      "Swanky and Moo Moo", "Syncopate", "Tangerine", "Taprom", "Tauri",
      "Taviraj", "Teko", "Telex", "Tenali Ramakrishna", "Tenor Sans",
      "Text Me One", "The Girl Next Door", "Tienne", "Tillana", "Timmana",
      "Tinos", "Titan One", "Titillium Web", "Trade Winds", "Trirong",
      "Trocchi", "Trochut", "Trykker", "Tulpen One", "Ubuntu",
      "Ubuntu Condensed", "Ubuntu Mono", "Ultra", "Uncial Antiqua", "Underdog",
      "Unica One", "UnifrakturCook", "UnifrakturMaguntia", "Unkempt", "Unlock",
      "Unna", "VT323", "Vampiro One", "Varela", "Varela Round",
      "Vast Shadow", "Vesper Libre", "Vibur", "Vidaloka", "Viga",
      "Voces", "Volkhov", "Vollkorn", "Voltaire", "Waiting for the Sunrise",
      "Wallpoet", "Walter Turncoat", "Warnes", "Wellfleet", "Wendy One",
      "Wire One", "Work Sans", "Yanone Kaffeesatz", "Yantramanav", "Yatra One",
      "Yellowtail", "Yeseva One", "Yesteryear", "Yrsa", "Zeyada"
    ];
  }
}

if( !function_exists( 'wpep_get_emoji_list' ) ) {

  /**
   * @return array
   */
  function wpep_get_emoji_list() {
    return [
      "1F44D" => "&#x1F44D thumbs up", "2764" => "&#x2764 love",
      "1F600" => "&#x1F600 grinning face","1F603" => "&#x1F603 grinning face with big eyes",
      "1F604" => "&#x1F604 grinning face with smiling eyes","1F601" => "&#x1F601 beaming face with smiling eyes","1F606" => "&#x1F606 grinning squinting face",
      "1F605" => "&#x1F605 grinning face with sweat","1F923" => "&#x1F923 rolling on the floor laughing","1F602" => "&#x1F602 face with tears of joy",
      "1F642" => "&#x1F642 slightly smiling face","1F643" => "&#x1F643 upside-down face","1F609" => "&#x1F609 winking face",
      "1F60A" => "&#x1F60A smiling face with smiling eyes","1F607" => "&#x1F607 smiling face with halo","1F970" => "&#x1F970 smiling face with hearts",
      "1F60D" => "&#x1F60D smiling face with heart-eyes","1F929" => "&#x1F929 star-struck","1F618" => "&#x1F618 face blowing a kiss",
      "1F617" => "&#x1F617 kissing face","263A" => "&#x263A smiling face","1F61A" => "&#x1F61A kissing face with closed eyes",
      "1F619" => "&#x1F619 kissing face with smiling eyes","1F60B" => "&#x1F60B face savoring food","1F61B" => "&#x1F61B face with tongue",
      "1F61C" => "&#x1F61C winking face with tongue","1F92A" => "&#x1F92A zany face","1F61D" => "&#x1F61D squinting face with tongue",
      "1F911" => "&#x1F911 money-mouth face","1F917" => "&#x1F917 hugging face","1F92D" => "&#x1F92D face with hand over mouth",
      "1F92B" => "&#x1F92B shushing face","1F914" => "&#x1F914 thinking face","1F910" => "&#x1F910 zipper-mouth face",
      "1F928" => "&#x1F928 face with raised eyebrow","1F610" => "&#x1F610 neutral face","1F611" => "&#x1F611 expressionless face",
      "1F636" => "&#x1F636 face without mouth","1F60F" => "&#x1F60F smirking face","1F612" => "&#x1F612 unamused face",
      "1F644" => "&#x1F644 face with rolling eyes","1F62C" => "&#x1F62C grimacing face","1F925" => "&#x1F925 lying face",
      "1F60C" => "&#x1F60C relieved face","1F614" => "&#x1F614 pensive face","1F62A" => "&#x1F62A sleepy face",
      "1F924" => "&#x1F924 drooling face","1F634" => "&#x1F634 sleeping face","1F637" => "&#x1F637 face with medical mask",
      "1F912" => "&#x1F912 face with thermometer","1F915" => "&#x1F915 face with head-bandage","1F922" => "&#x1F922 nauseated face",
      "1F92E" => "&#x1F92E face vomiting","1F927" => "&#x1F927 sneezing face","1F975" => "&#x1F975 hot face",
      "1F976" => "&#x1F976 cold face","1F974" => "&#x1F974 woozy face","1F635" => "&#x1F635 dizzy face",
      "1F92F" => "&#x1F92F exploding head","1F920" => "&#x1F920 cowboy hat face","1F973" => "&#x1F973 partying face",
      "1F60E" => "&#x1F60E smiling face with sunglasses","1F913" => "&#x1F913 nerd face","1F9D0" => "&#x1F9D0 face with monocle",
      "1F615" => "&#x1F615 confused face","1F61F" => "&#x1F61F worried face","1F641" => "&#x1F641 slightly frowning face",
      "2639" => "&#x2639 frowning face","1F62E" => "&#x1F62E face with open mouth","1F62F" => "&#x1F62F hushed face",
      "1F632" => "&#x1F632 astonished face","1F633" => "&#x1F633 flushed face","1F97A" => "&#x1F97A pleading face",
      "1F626" => "&#x1F626 frowning face with open mouth","1F627" => "&#x1F627 anguished face","1F628" => "&#x1F628 fearful face",
      "1F630" => "&#x1F630 anxious face with sweat","1F625" => "&#x1F625 sad but relieved face","1F622" => "&#x1F622 crying face",
      "1F62D" => "&#x1F62D loudly crying face","1F631" => "&#x1F631 face screaming in fear","1F616" => "&#x1F616 confounded face",
      "1F623" => "&#x1F623 persevering face","1F61E" => "&#x1F61E disappointed face","1F613" => "&#x1F613 downcast face with sweat",
      "1F629" => "&#x1F629 weary face","1F62B" => "&#x1F62B tired face","1F971" => "&#x1F971 ⊛ yawning face",
      "1F624" => "&#x1F624 face with steam from nose","1F621" => "&#x1F621 pouting face","1F620" => "&#x1F620 angry face",
      "1F92C" => "&#x1F92C face with symbols on mouth","1F608" => "&#x1F608 smiling face with horns","1F47F" => "&#x1F47F angry face with horns",
      "1F480" => "&#x1F480 skull","2620" => "&#x2620 skull and crossbones","1F4A9" => "&#x1F4A9 pile of poo",
      "1F921" => "&#x1F921 clown face","1F479" => "&#x1F479 ogre","1F47A" => "&#x1F47A goblin", "1F47B" => "&#x1F47B ghost"
    ];
  }
}