<?php

use WPEP\Entity\Course as WPEP_Entity_Course;

$migration_data = get_option( 'wpep_migration_option_194', false );

if( $migration_data === false ) {
  $migration_data = get_posts( [
    'post_type'   => WPEP_POST_TYPE_COURSE,
    'numberposts' => -1,
    'fields'      => 'ids',
    'post_status' => wpep_post_monitored_post_status_list()
  ] );

  update_option( 'wpep_migration_option_194', $migration_data );
}

if( count( $migration_data ) == 0 ) {
  delete_option( "wpep_migration_option_194" );

  echo '<p>' . __( "All courses have been migrated to the new search system.", "wpep" ) . '</p>';
  return;
}

$i = 1; foreach( $migration_data as $migration_data_key => $course_id ) {
  if( $i > 10 )
    break;

  $searchable_post_content = get_post_meta( $course_id, 'wpep_course_description', true ) . "\n";

  if( wpep_course_type_from_id( $course_id ) !== WPEP_COURSE_TYPE_ONE_OFF ) {
    $section_list = WPEP_Entity_Course::instance()->get_with_lessons( $course_id );

    foreach( $section_list as $section ) {
      $searchable_post_content .= "\n" . $section->title;

      if( $section->content != '' )
        $searchable_post_content .= "\n" . $section->content;

      foreach( $section->lessons as $lesson ) {
        $searchable_post_content .= "\n" . $lesson->title;

        if( $section->content != '' )
          $searchable_post_content .= "\n" . $lesson->content;
      }
    }

    $searchable_post_content .= "\n";
  }

  $searchable_post_content = trim( $searchable_post_content );

  wpep_wpdb()->update( wpep_wpdb()->posts, [
    'post_content' => $searchable_post_content
  ], [ 'ID' => $course_id ] );

  unset( $migration_data[ $migration_data_key ] );
  $i++;
}

echo '<p class="wpep-migration-repeat-file-message">';
echo    sprintf( __( "%s remaining courses have to be migrated to the new search system.", "wpep" ), count( $migration_data ) );
echo '</p>';

update_option( 'wpep_migration_option_194', $migration_data );